package iwt.dis;               // our package

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.io.*;
import java.util.*;

import iwt.*;
import org.w3c.dom.*;

public class ModulationType extends PduElement
{
   protected UnsignedShort   spreadSpectrum;	 
   
   protected UnsignedShort     majorModulationType;                        

   protected UnsignedShort     detail;                          

   protected UnsignedShort     system;                       
   
   public static final int sizeOf = 8;  
 
  
public ModulationType()
{
    spreadSpectrum = new UnsignedShort();
	majorModulationType  = new UnsignedShort();
    detail     = new UnsignedShort();
    system = new UnsignedShort();
    return;
}

public Object clone()
{
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.
	ModulationType newModulationType = (ModulationType)super.clone();
    newModulationType.setspreadSpectrum(this.getspreadSpectrum());
	newModulationType.setmajorModulationType(this.getmajorModulationType());
    newModulationType.setdetail(this.getdetail());
    newModulationType.setsystem(this.getsystem());
   return newModulationType;
}

/* serialize the object in DIS format to a stream.
@param outputStream stream the object is being serialized to
*/

public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.
	spreadSpectrum.serialize(outputStream);
    majorModulationType.serialize(outputStream);
    detail.serialize(outputStream);
    system.serialize(outputStream);
    return;
}


public void deSerialize(DataInputStream inputStream)
{
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    // no need to call super, since there are no ivars in the superclass.
	System.out.println("in modulation type deserialize");
    spreadSpectrum.deSerialize(inputStream);
System.out.println("Spread Spectrum: " + spreadSpectrum.intValue());
    
	majorModulationType.deSerialize(inputStream);
    detail.deSerialize(inputStream);
    system.deSerialize(inputStream);
    return;
}

public int length()
{
    return sizeOf;          // ModulationTypes are this long, always.
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

	printStream.println(indent + "Spread Spectrum: " + spreadSpectrum.intValue());
    printStream.println(indent + "majorModulationType: " + majorModulationType.intValue());
    printStream.println(indent + "detail: "    + detail.intValue());
    printStream.println(indent + "system: "+ system.intValue());
    
    return;
}

// Accessor methods
public UnsignedShort getspreadSpectrum()
{   return (UnsignedShort)spreadSpectrum.clone();
}
public void setspreadSpectrum(UnsignedShort pspreadSpectrum)
{spreadSpectrum = pspreadSpectrum;
}
public void setspreadSpectrum(int pspreadSpectrum)
{ spreadSpectrum = new UnsignedShort(pspreadSpectrum);
}


public UnsignedShort getmajorModulationType()
{   return (UnsignedShort)majorModulationType.clone();
}
public void setmajorModulationType(UnsignedShort pmajorModulationType)
{majorModulationType = pmajorModulationType;
}
public void setmajorModulationType(int pmajorModulationType)
{ majorModulationType = new UnsignedShort(pmajorModulationType);
}


public UnsignedShort getdetail()
{   return (UnsignedShort)detail.clone();
}
public void setdetail(UnsignedShort pdetail)
{detail = pdetail;
}
public void setdetail(int pdetail)
{detail = new UnsignedShort(pdetail);
}

public UnsignedShort getsystem()
{   return (UnsignedShort)system.clone();
}
public void setsystem(UnsignedShort psystem)
{system = psystem;
}
public void setsystem(int psystem)
{ system = new UnsignedShort(psystem);
}

public void 
write(org.w3c.dom.Document outDoc)
{
	insertToDocument(outDoc,outDoc);
}


public void 
insert(org.w3c.dom.Node outNode)
{
	org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
	insertToDocument(outDoc,outNode);
} // end of write

void insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
  {
    Element ele = outDoc.createElement(iwt.DIS_TAGS.ModulationType);
	ele.setAttribute(iwt.DIS_TAGS.SpreadSpectrum,spreadSpectrum.toString());
    ele.setAttribute(iwt.DIS_TAGS.MajorModulationType,majorModulationType.toString());
    ele.setAttribute(iwt.DIS_TAGS.Detail,detail.toString());
    ele.setAttribute(iwt.DIS_TAGS.System,system.toString());
    outNode.appendChild(ele);

}

public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(iwt.DIS_TAGS.ModulationType);

    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()


public void
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(iwt.DIS_TAGS.ModulationType) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {	

	   org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.SpreadSpectrum);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.MajorModulationType);
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.Detail);
       org.w3c.dom.Attribute a4 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.System);
       String v1, v2, v3, v4;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();
       v4 = a4.getValue();

       try
       {
         spreadSpectrum = new UnsignedShort(Integer.parseInt(v1));

       } catch(NumberFormatException nfe) {}

       try
       {
         majorModulationType = new UnsignedShort(Integer.parseInt(v2));

       } catch(NumberFormatException nfe) {}

       try
       {
         detail = new UnsignedShort(Integer.parseInt(v3));

       } catch(NumberFormatException nfe) {}

       try
       {
         system = new UnsignedShort(Integer.parseInt(v4));

       } catch(NumberFormatException nfe) {}

	}
}


} 
