/*-----------------------------------------------------------------------

  Class:       MulticastPduSender
  Author:      Mike McCarthy
  Last Update: 6 Jan. 1998

  Program Invocation:  This class is instantiated by other classes
  When this class is instantiated, its methods are used to output 
  PDUs on the specified Multicast group at a 1/10Hz rate. 

  Changed by H. Timucin Ozdemir on October 18, 1998

--------------------------------------------------------------------------*/
package cmsper.comm;

import mil.navy.nps.dis.*;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import cmsper.*;

public class 
MulticastPduSender 
{

   byte     	outputBuffer[]  = new byte[1500];// buffer to hold datagram data
   InetAddress 	group; 
   MulticastSocket s ;                   // socket to spit it out on
   DatagramPacket          dgramPacket;  // one datagram packet
   ByteArrayOutputStream   outputStream;
   DataOutputStream        dataOutputStream;
   byte                    streamArray[];
   int                     pduSentCount = 0;

   String  mcastAddress = monitor.DEFAULT_MCAST;
   int     port         = monitor.DEFAULT_MPORT;


   public static final byte DIS203_PROTOCOL_VERSION_1_0 = (byte)1;
   public static final byte DIS203_PROTOCOL_VERSION_IEEE_1278_1993 = (byte)2;
   public static final byte DIS203_PROTOCOL_VERSION_2_0 = (byte)3;
   public static final byte DIS203_PROTOCOL_VERSION_CURRENT = DIS203_PROTOCOL_VERSION_2_0;
   
   public MulticastPduSender(String mAddress, int _port)
   {
      try
      {
        //group = InetAddress.getByName("224.2.181.145");
        mcastAddress = mAddress; port = _port;
        group = InetAddress.getByName(mcastAddress);
        s = new MulticastSocket(port);
        s.joinGroup(group);

      } catch (UnknownHostException e) { System.err.println(e); }
        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

   }// end of CONSTRUCTOR()

   /**
   */
   public void 
   send (ProtocolDataUnit pdu) { if(pdu!=null) SendMPDU(pdu); } 

   /**
   */
   public void 
   SendMPDU (ProtocolDataUnit pdu)
   {

      pdu.setProtocolVersion(DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();

      outputStream = new ByteArrayOutputStream();
      dataOutputStream = new DataOutputStream(outputStream);
      pdu.serialize(dataOutputStream);
      streamArray = outputStream.toByteArray();

      System.out.println( " data serialized to " + streamArray[0] + 
                          " and " + streamArray[1]);

      dgramPacket = new DatagramPacket( outputStream.toByteArray(), 
                                        outputStream.size(), group, port);


	  try
      {
	     { s.send(dgramPacket, (byte)127);}

         if(++pduSentCount % 100 == 0) 
         { System.out.println("sent " + pduSentCount + " Pdus"); }

	  } catch (SocketException se) { System.err.println(se); }

        catch (UnknownHostException e) { System.err.println(e); }

        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

        catch(NullPointerException nullExp) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }
          
   }//end method

   /**
   */
   public static void 
   send( ProtocolDataUnit pdu, MulticastSocket socket, 
         InetAddress mgroup, int mport )
   {

      pdu.setProtocolVersion(DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();

      ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
      DataOutputStream dataOutputStream = new DataOutputStream(outputStream);
      pdu.serialize(dataOutputStream);
      byte[] streamArray = outputStream.toByteArray();
      int pduSentCount=0;

      System.out.println( " data serialized to " + streamArray[0] + 
                          " and " + streamArray[1]);

      DatagramPacket dgramPacket = new DatagramPacket( outputStream.toByteArray(), 
                                        //outputStream.size(), group, port);
                                        outputStream.size(), mgroup, mport);


	  try
      {
	     { socket.send(dgramPacket, (byte)127);}

         if(++pduSentCount % 100 == 0) 
         { System.out.println("sent " + pduSentCount + " Pdus"); }

	  } catch (SocketException se) { System.err.println(se); }

        catch (UnknownHostException e) { System.err.println(e); }

        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

        catch(NullPointerException nullExp) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }
          
   }//end method

}//endclass

