/**
* @date  October 16, 1998 ( Last Upadated on October 16, 1998 )
*
*/
package cmsper.gui;

import mil.navy.nps.dis.*;

import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import cmsper.gui.forms.*;
import cmsper.apis.*;
import cmsper.*;
import worker.*;

/**
*
* @author H. Timucin Ozdemir
* @date  October 16, 1998 ( Last Updated on October 16, 1998 )
*
*/
public class 
FormPanel extends Panel implements ActionListener, ItemListener, GUI_CONST
{

    /** Varius Form names for different PDU types */
    public static final String DETONATION_FORM = "Detonation";
    public static final String ESPDU_FORM      = "ESPDU";
    public static final String FIRE_FORM       = "Fire";

    /** State of the Tester object */
    public static final int TESTER_STARTED     = 0;
    public static final int TESTER_STOPPED     = 1;

    /** Tester object */
    RunnerObject  tester = null;

    /** State of Tester object */
    int           testerState = TESTER_STOPPED;

    /** GUI Components for User Interaction */
    Label      fre ;
    myTxtField  freTxtFld ;

    /** PDU Specific Panel*/
    Panel         cards;

    /** Common Panel */
    Panel         common;

    /** */
    Hashtable     pduForms;

    String        currFormName;

    Frame mainFrame;

    String mCastAddress   = monitor.DEFAULT_MCAST;
    int    gPortNumber    = monitor.DEFAULT_MPORT;

    
    /**
    */
    public FormPanel(Frame _mainFrame, String _mCast, int _gPNum ) 
    {

      if(_mCast!=null) mCastAddress = _mCast;
      if(_gPNum>10)    gPortNumber = _gPNum;
      mainFrame = _mainFrame;
      registerFrame();

      //setLayout(new BorderLayout());
      GridBagLayout gridbag = new GridBagLayout();
      GridBagConstraints c = new GridBagConstraints();

      setLayout(gridbag);
      setBackground(GUI_CONST.PnlBgClr);
      cards = new Panel();
      cards.setLayout(new CardLayout());

      EspduForm pfr1 = new EspduForm();
      DetonationForm pfr2 = new DetonationForm();
      FireForm pfr3 = new FireForm();
      cards.add(ESPDU_FORM, pfr1);
      cards.add(DETONATION_FORM, pfr2);
      cards.add(FIRE_FORM, pfr3);

      pduForms = new Hashtable();
      pduForms.put(ESPDU_FORM, pfr1);
      pduForms.put(DETONATION_FORM, pfr2);
      pduForms.put(FIRE_FORM, pfr3);
      
      currFormName = ESPDU_FORM;

      //add("Center",cards);
      c.fill = GridBagConstraints.BOTH;
      //c.weightx = 1.0;
      c.gridwidth = GridBagConstraints.REMAINDER;
      //c.gridheight = 5;
      c.anchor =  GridBagConstraints.CENTER;
      gridbag.setConstraints(cards,c);
      add(cards);

      common = new Panel();
      GridBagLayout gbCommon = new GridBagLayout();
      GridBagConstraints gcCommon = new GridBagConstraints();

      common.setLayout(gbCommon);
      common.setBackground(GUI_CONST.PnlBgClr);
      fre = new myLabel("Frequency");
      freTxtFld = new myTxtField(String.valueOf(1),5);
      myButton start = new myButton("Start");
      myButton stop = new myButton("Stop");

      start.addActionListener(this);
      stop.addActionListener(this);

      gcCommon.fill = GridBagConstraints.BOTH;
      //gcCommon.weightx = 1.0;
      //gcCommon.gridwidth = GridBagConstraints.REMAINDER;
      //gcCommon.gridheight = 5;
      //gcCommon.anchor =  GridBagConstraints.CENTER;
      gbCommon.setConstraints(fre,gcCommon);
      common.add(fre);
      gcCommon.gridwidth = GridBagConstraints.REMAINDER;
      gbCommon.setConstraints(freTxtFld,gcCommon);
      common.add(freTxtFld);
      gcCommon.gridwidth = GridBagConstraints.RELATIVE;
      gbCommon.setConstraints(start,gcCommon);
      common.add(start);
      gbCommon.setConstraints(stop,gcCommon);
      common.add(stop);

      //common.validate();

      c.fill = GridBagConstraints.BOTH;
      c.gridwidth = GridBagConstraints.REMAINDER;
      c.gridheight = 1;
      c.anchor =  GridBagConstraints.CENTER;
      gridbag.setConstraints(common,c);

      add(common);

      validate();
      
    }// end of CONSTRUCTOR()

    /**
    * @return How many times a pdu will be broadcasted in 1 second?
    */
    public int
    getFrequency()
    {
       int ret = 1;
       String fStr = freTxtFld.getText();
       try
       {
         ret = Integer.parseInt(fStr);

       } catch(NumberFormatException nfe) { ret=1; }
       return ret;

    }// end of getFrequency()

    /**
    * @return Multicast Session Address
    */
    public String getAddress() { return mCastAddress; }

    /**
    * @return Multicast Session Port Number
    */
    public int getPort() { return gPortNumber; }

    /**
    * @return Multicast Session Port Number
    */
    public ProtocolDataUnit
    getPDU()
    {
       ProtocolDataUnit ret = null;
       PduProducer pro = (PduProducer)pduForms.get(currFormName);
       if(pro!=null) ret = pro.producePDU();
       return ret;

    }// end of getPort()

    /**
    * @params Tester object 
    */
    public void
    setTester( RunnerObject _tester ) { tester = _tester; }

    /**
    * @params ActionEvent object 
    */
    public void
    actionPerformed(ActionEvent evt)
    {
       System.out.println(" I received "+evt.getActionCommand());

       if( evt.getActionCommand().equals("Start") && 
           tester!=null && testerState == TESTER_STOPPED ) 
       {   testerState = TESTER_STARTED; tester.start(); }
       else if( evt.getActionCommand().equals("Stop") && 
                tester!=null && testerState == TESTER_STARTED ) 
            { testerState = TESTER_STOPPED; tester.stop(); }
       else if( evt.getActionCommand().equals(DETONATION_FORM) ||
                evt.getActionCommand().equals(ESPDU_FORM) ||
                evt.getActionCommand().equals(FIRE_FORM) )
                handleCard(evt.getActionCommand());

    }// end of actionPerformed()

    public void
    itemStateChanged(ItemEvent evt)
    {
       System.out.println(" I received "+evt.paramString());

       handleCard((String)evt.getItem());

    }// end of itemStateChanged()

    void handleCard(String _newFormName)
    {
       if(!currFormName.equals(_newFormName))
       {
          currFormName = _newFormName;
          CardLayout cl = (CardLayout)cards.getLayout();
          cl.show(cards,currFormName);

       }// end of if

    }// end of handleCard)

    void registerFrame()
    {
       MenuBar mBar = mainFrame.getMenuBar();
       Menu    test = mBar.getMenu(0);
       MenuItem espdu     = test.getItem(0);
       MenuItem detonpdu  = test.getItem(1);
       MenuItem firepdu   = test.getItem(2);

       espdu.addActionListener(this);
       detonpdu.addActionListener(this);
       firepdu.addActionListener(this);

    }// end of registerFrame()

}// end of Class SelectPanel
