package comm;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;
import jdis.*;

/*******************************/
/**  OWRTI Related Packages.   */
/*******************************/
import RTIcap.*;             // The package containing our stubs.
import npacrti.*;            // The package containing our stubs.
import org.omg.CosNaming.*;  // 
import org.omg.CORBA.*;      // All CORBA applications need these classes.
import owrti.util.*;

/**
*
*  1. Find RTIKernel Object
*  2. Get RTIambassador Object
*  3. Join the given federation execution
*  4. Get the MOP of DIS Class and related attribute 
*  5. Express publish intend
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class DISHLAPublisher implements PDUPublisher 
{

   byte                    streamArray[];
   int                     pduSentCount = 0;

   boolean DEBUG        = false;

   RTIambassador rtiAmb = null;

   /** Without .fed extention */
   String  FederationName    = JDISConstants.DEFAULT_FEDERATION_NAME; 
   String  disInterClassName = JDISConstants.DEFAULT_INTERACTION_CLASS;
   String  disParamName      = JDISConstants.DEFAULT_PARAMETER_NAME;

   String  nServer      = JDISConstants.DEFAULT_SERVER;
   int     nPortNumber  = JDISConstants.DEFAULT_SERVER_PORT;
   short   federateId   = -1;  // Not a valid federate id
   String  federateName = "DISHLAPub";

   short   disInterClassTypeId = -1;
   int     disParamTypeId = -1;

   java.lang.Object guard = new java.lang.Object();

   boolean       advanceGranted = false;
   double        currentTime = 0.0; 
   double        timeStep = 5.0; 
   double        requestTime ; 

   boolean       publishingIsOn   = false;
   boolean       subscriptionIsOn = false;
   boolean       isRegistered     = false;
   int           publisherStat    = PDUPublisher.NOT_CONNECTED; 

   RTIcap.FederateAmbassador fedAmb = null;

   /**
   */
   public DISHLAPublisher( String _nServer, int _nServerPortNumber )
   {
      this( _nServer, _nServerPortNumber, JDISConstants.DEFAULT_FEDERATION_NAME,
            JDISConstants.DEFAULT_INTERACTION_CLASS,
            JDISConstants.DEFAULT_PARAMETER_NAME );

   }// end of CONSTRUCTOR()

   /**
   */
   public DISHLAPublisher( String _nServer, int _nServerPortNumber, 
                           String _FederationName, String _InteractionClass,
                           String _ParamName )
   {

      try
      {
         nServer = _nServer;
         nPortNumber = _nServerPortNumber;
         Date aDate  = new Date();
         federateName += ("_t"+aDate);
         federateName = federateName.trim().replace(' ','_');
         String[] args = null;

         // Create and initialize the ORB
         ORB orb = ORB.init(args, null);

           String strIOR = owrti.util.KernelLocator.askIOR(nServer,nPortNumber,"IDL:npacrti/RTIKernelInterface:1.0");

           if( strIOR != null )
           {
      
              RTIKernelInterface rtiKernel = RTIKernelInterfaceHelper.narrow(orb.string_to_object(strIOR));

              // Get the Version of the Kernel
              String version = rtiKernel.getVersion();
              System.out.println(" Version is : "+version);      

              //if(args.length>2) 
              rtiAmb = rtiKernel.createRTIAmbassador();
              publisherStat    = PDUPublisher.CONNECTED; 

           } else System.out.println(" IOR file is empty| \n "+ 
                                     "I can not find the address of RTIKernelInterface");

     } catch(Exception e) 
       {
         System.out.println("ERROR : " + e);
         e.printStackTrace(System.out);
       }  

   }// end of CONSTRUCTOR()

   /**
   *
   * 1. If it is connected before, then disconnect it.
   * 2. Reconnect to the new server.
   *
   */
   public final void
   reconnect(String _nServer, int _nServerPortNumber)
   {

      if(rtiAmb!=null) closeConnection();

      try
      {
         publisherStat    = PDUPublisher.NOT_CONNECTED; 
         nServer = _nServer;
         nPortNumber = _nServerPortNumber;
         Date aDate  = new Date();
         String[] args = null;

         // Create and initialize the ORB
         ORB orb = ORB.init(args, null);

         String strIOR = owrti.util.KernelLocator.askIOR(nServer,nPortNumber,"IDL:npacrti/RTIKernelInterface:1.0");

         if( strIOR != null )
         {
      
             RTIKernelInterface rtiKernel = RTIKernelInterfaceHelper.narrow(orb.string_to_object(strIOR));

             // Get the Version of the Kernel
             String version = rtiKernel.getVersion();
             System.out.println(" Version is : "+version);      

             //if(args.length>2) 
             rtiAmb = rtiKernel.createRTIAmbassador();
             publisherStat    = PDUPublisher.CONNECTED; 

          } else System.out.println(" IOR file is empty| \n "+ 
                                     "I can not find the address of RTIKernelInterface");

     } catch(Exception e) 
       {
         System.out.println("ERROR : " + e);
         e.printStackTrace(System.out);
       }  

   }// end of reconnect()

   /**
   */
   public final void
   closeConnection()
   {
      if(DEBUG) 
         System.out.println("isRegistered:"+isRegistered+" closeConnection()");

      if(isRegistered) unregisterMe();
      stopPublishing();
      federateId          = -1;  
      disInterClassTypeId = -1;
      disParamTypeId      = -1;
      rtiAmb = null;
      publisherStat    = PDUPublisher.NOT_CONNECTED; 

   }// end of closeConnection()

   /**
   */
   public final void
   startPublishing()
   {
      if(publisherStat==PDUPublisher.CONNECTED)
      {
         if(!isRegistered) 
         { 
             if(DEBUG) System.out.println("isRegistered:"+isRegistered);
             isRegistered = registerMe(); 
             if(DEBUG) System.out.println("isRegistered:"+isRegistered);
         }
         if(isRegistered)  publisherStat = PDUPublisher.CAN_PUBLISH;

      }// end of if
      if(DEBUG) System.out.println("startPublishing()");

   }// end of startPublishing()

   /**
   */
   public final void
   stopPublishing()
   {
      if( publisherStat==PDUPublisher.CAN_PUBLISH )
          publisherStat=PDUPublisher.CONNECTED; 

      if(DEBUG) System.out.println("stopPublishing()");

   }// end of stopPublishing()

   /**
   */
   public void 
   send (ProtocolDataUnit pdu) 
   {   
      if( publisherStat==PDUPublisher.CAN_PUBLISH )
      {
         if(DEBUG) System.out.println("send() of  DISHLAPublisher");
         ByteArrayOutputStream out  = new ByteArrayOutputStream();
         DataOutputStream      dout = new DataOutputStream(out);
         pdu.serialize(dout);
         send(out.toByteArray());

      }// end of if

   }// end of send() 

   /**
   */
   public void 
   send (byte[] data)
   {

      if( rtiAmb != null && publisherStat==PDUPublisher.CAN_PUBLISH )
      {
          if(DEBUG) System.out.println("send() of  DISHLAPublisher");
   
          RTIcap.ParameterHandleValuePair[] eles = new RTIcap.ParameterHandleValuePair[1];
   
          eles[0] = new RTIcap.ParameterHandleValuePair( (short)disParamTypeId, data );

          try
          {
            rtiAmb.sendInteraction(disInterClassTypeId,eles,null);

          } catch (RTIcap.InteractionClassNotDefined onk) {}
            catch (RTIcap.InteractionClassNotPublished onk) {}
            catch (RTIcap.InteractionParameterNotDefined onk) {}
            catch (RTIcap.FederateNotExecutionMember onk) {}
            catch (RTIcap.SaveInProgress onk) {}
            catch (RTIcap.RestoreInProgress onk) {}
            catch (RTIcap.RTIinternalError onk) {}

          try
          { 
            rtiAmb.timeAdvanceRequest( currentTime+1 );
          } catch( RTIcap.TimeAdvanceAlreadyInProgress aa ) {}
            catch( RTIcap.EnableTimeConstrainedPending aa ) {}
            catch( RTIcap.EnableTimeRegulationPending aa ) {}
            catch( RTIcap.InvalidFederationTime aa ) {}
            catch( RTIcap.FederationTimeAlreadyPassed aa ) {}
            catch( RTIcap.FederateNotExecutionMember aa ) {}
            catch( RTIcap.SaveInProgress aa ) {}
            catch( RTIcap.RestoreInProgress aa ) {}
            catch( RTIcap.RTIinternalError aa ) {}

      }// end of if


   }//end of send()

   ///   End of PDUPublisher Interface  ////

   /**
   */
   final boolean
   registerMe()
   {
      boolean ret = false;
      
      if( fedAmb!=null && publisherStat==PDUPublisher.CONNECTED )
      {

         ret = joinFederationExecution(fedAmb) && getRtiIds() && 
               publishDIS() && decideTiming();

         publishingIsOn = ret;

      }// end of if

      return ret;

   }// end of registerMe()

   /**
   */
   final void
   unregisterMe()
   {
      if( isRegistered )
      {
         leaveFederationExecution();
         publisherStat=PDUPublisher.CONNECTED;
         publishingIsOn   = false;
         subscriptionIsOn = false;
         isRegistered     = false;
         
      }// end of if

   }// end of registerMe()

   /**
   */
   public void 
   setFederateAmbassador(RTIcap.FederateAmbassador _fedAmb)
   { fedAmb = _fedAmb; }

   /**
   */
   public void 
   SendMPDU (ProtocolDataUnit pdu)
   {

      /*
      pdu.setProtocolVersion(DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();
      */

      send(pdu);

   }//end method

   /** */
   public short getDISInterClassTypeId() { return disInterClassTypeId; }

   /** */
   public int getDISParamTypeId()      { return disParamTypeId; }

   /**
   *
   * Gets the RTTI (Meta-Object Protocol MOP) handles.
   *
   * Ask RTI about the class indentifier of each object, attribute, 
   * interaction and parameter. So that we can define, subscribe, publish,
   * and interpret these attributes/objects.
   *
   */
   boolean getRtiIds()
   {
      boolean ret = false;

      if( rtiAmb != null )
      {
        
           try
           { 
             disInterClassTypeId = 
                    rtiAmb.getInteractionClassHandle(disInterClassName);
           } catch( RTIcap.NameNotFound icnd ) {}
             catch( RTIcap.FederateNotExecutionMember icnd ) {}
             catch( RTIcap.RTIinternalError icnd ) {}

           try
           { 
             System.out.println("disParamTypeId:"+disParamTypeId);
             disParamTypeId = 
                  rtiAmb.getParameterHandle(disParamName,disInterClassTypeId);
             ret = true;
             System.out.println("Received disParamTypeId:"+disParamTypeId);
           } catch( RTIcap.InteractionClassNotDefined icnd ) {}
             catch( RTIcap.NameNotFound icnd ) {}
             catch( RTIcap.FederateNotExecutionMember icnd ) {}
             catch( RTIcap.RTIinternalError icnd ) {}

      }// end of if
      System.out.println("ret:"+ret);
  
      return ret;

   }// end of getRtiIds()

   /**
   *
   * Describe our interest for publishing DIS interactions.
   *
   */
   boolean publishDIS()
   {

      boolean ret = false;

      if( rtiAmb != null )
      {
           try
           {
             rtiAmb.publishInteractionClass( disInterClassTypeId );
             ret = true;
           } catch( RTIcap.FederateNotExecutionMember fnem) {}
             catch( RTIcap.RestoreInProgress rip) {}
             catch( RTIcap.InteractionClassNotDefined icnd) {}
             catch( RTIcap.SaveInProgress sip) {}
             catch( RTIcap.RTIinternalError ie) {}

      }// end of if

      return ret;

   }// end of publishDIS()

   /**
   *
   * Describe our interest for receiving DIS interactions.
   *
   */
   public boolean subscribeDIS()
   {

      boolean ret = subscriptionIsOn && isRegistered ;

      if( rtiAmb != null && isRegistered && !subscriptionIsOn )
      {
           try
           {
             rtiAmb.subscribeInteractionClass( disInterClassTypeId, RTIcap._Boolean.RTI_TRUE);
             ret = true;
             subscriptionIsOn=true;
             if(DEBUG) System.out.println("subscribeDIS()");
           } catch( RTIcap.FederateLoggingServiceCalls  flsc) {}
             catch( RTIcap.FederateNotExecutionMember  fnem) {}
             catch( RTIcap.RTIinternalError  ie) {}
             catch( RTIcap.RestoreInProgress  rip) {}
             catch( RTIcap.InteractionClassNotDefined  icnd) {}
             catch( RTIcap.SaveInProgress  sip) {}

      }// end of if

      return ret;

   }// end of subscribeDIS()

   /**
   *
   * Describe our loss of interest for receiving DIS interactions.
   *
   */
   public boolean unsubscribeDIS()
   {

      boolean ret = false;

      // If it is registered, subscribed and there is a valid RTI
      // Ambassador
      if( rtiAmb != null && isRegistered && subscriptionIsOn )
      {
           try
           {
             rtiAmb.unsubscribeInteractionClass( disInterClassTypeId );
             ret = true;
             subscriptionIsOn=false;
             if(DEBUG) System.out.println("unsubscribeDIS()");
           } catch( RTIcap.InteractionClassNotDefined  flsc) {}
             catch( RTIcap.InteractionClassNotSubscribed  fnem) {}
             catch( RTIcap.FederateNotExecutionMember  icnd) {}
             catch( RTIcap.RestoreInProgress  rip) {}
             catch( RTIcap.SaveInProgress  sip) {}
             catch( RTIcap.RTIinternalError  ie) {}

      }// end of if

      return ret;

   }// end of unsubscribeDIS()

   /**
   * 
   * We want to join the Federation but we do not know its ID and
   * whether it is on. Therefore, we first try to create this Federation.
   * If this federation is  already exists, we will receive the exception.
   * If not, it will be created for us.
   * Then we need to join this execution and get ourself a federate id.
   * 
   */
   boolean decideTiming()
   {

      boolean ret = false;

      if( rtiAmb != null )
      {
          boolean doTick = false; 
          /****************************************************************/
          /** TimeConstrained : we *do not* want to receive timestamped   */
          /** events in order */
          /****************************************************************/
          resetTimeAdvanceGranted();

          try
          {
            rtiAmb.disableTimeConstrained( );
            doTick = true;
          } catch( RTIcap.TimeConstrainedWasNotEnabled aa ) {}
            catch( RTIcap.FederateNotExecutionMember aa ) {}
            catch( RTIcap.SaveInProgress aa ) {}
            catch( RTIcap.RestoreInProgress aa ) {}
            catch( RTIcap.RTIinternalError aa ) {}

          /***************************************************/
          /** Tick RTI to get timeConstrainedEnabled message */
          /***************************************************/
          if(doTick) tick();

          /**********************************************************/
          /** TimeRegulating : We want to vote for time advancement */
          /**********************************************************/
          resetTimeAdvanceGranted();
          doTick = false; 

          try
          {
            rtiAmb.disableTimeRegulation( );
            doTick = true;
          } catch( RTIcap.TimeRegulationWasNotEnabled aa ) {}
            catch( RTIcap.FederateNotExecutionMember aa ) {}
            catch( RTIcap.SaveInProgress aa ) {}
            catch( RTIcap.RestoreInProgress aa ) {}
            catch( RTIcap.RTIinternalError aa ) {}

          /**************************************************/
          /** Tick RTI to get timeRegulationEnabled message */
          /**************************************************/
          if(doTick) tick();

          try
          {
            rtiAmb.enableAsynchronousDelivery( );
            ret=true;
          } catch( RTIcap.FederateNotExecutionMember aa ) {}
            catch( RTIcap.AsynchronousDeliveryAlreadyEnabled aa ) {}
            catch( RTIcap.ConcurrentAccessAttempted aa ) {}
            catch( RTIcap.SaveInProgress aa ) {}
            catch( RTIcap.RestoreInProgress aa ) {}
            catch( RTIcap.RTIinternalError aa ) {}

      }// end of if

      return ret;

   }// end of decideTiming() 

   /**
   * 
   * We want to join the Federation but we do not know its ID and
   * whether it is on. Therefore, we first try to create this Federation.
   * If this federation is  already exists, we will receive the exception.
   * If not, it will be created for us.
   * Then we need to join this execution and get ourself a federate id.
   * 
   */
   boolean joinFederationExecution(RTIcap.FederateAmbassador fedAmb)
   {

      boolean ret = false;

      if( rtiAmb != null )
      {

         try {
           System.out.println("FederationName:"+FederationName);
           rtiAmb.createFederationExecution(FederationName,FederationName);
           System.out.println(FederationName+" execution created!");
         } catch( RTIcap.FederationExecutionAlreadyExists icnd) {}
           catch( RTIcap.RTIinternalError ie) {}

         try {
           System.out.println(" attempt to joind :"+FederationName);
           federateId = rtiAmb.joinFederationExecution(federateName,FederationName,fedAmb);
             ret = true;
           System.out.println("JOINED!");
         } catch( RTIcap.FederateAlreadyExecutionMember fnem) {}
           catch( RTIcap.FederationExecutionDoesNotExist icnd) {}
           catch( RTIcap.CouldNotOpenFED fnem) {}
           catch( RTIcap.ErrorReadingFED fnem) {}
           catch( RTIcap.SaveInProgress sip) {}
           catch( RTIcap.RestoreInProgress rip) {}
           catch( RTIcap.RTIinternalError ie) {}
           catch( Exception ee) 
           { System.out.println(" Exception in joinFederationExecution ");
             ee.printStackTrace(System.out);
             throw new RuntimeException();
           }

      }// end of if

      return ret;

   }// end of joinFederationExecution() 

   /**
   *
   * Before we leave the Federation
   *  1. Delete all the object that we are suppose(promised) to simulate.
   *  2. resign from the federation.
   *  3. Try to destroy the federation execution. Note that if there 
   *     are still federates in the Federation, the federation execution
   *     won't be stopped. Only, we resign it. 
   *
   */
   void leaveFederationExecution()
   {

      if( rtiAmb != null && isRegistered)
      {

         if(subscriptionIsOn) unsubscribeDIS();

         try 
         {
           rtiAmb.resignFederationExecution(RTIcap.ResignAction.DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);
           isRegistered = false;
         } catch( RTIcap.InvalidResignAction ie) {}
           catch( RTIcap.FederateOwnsAttributes ie) {}
           catch( RTIcap.FederateNotExecutionMember ie) {}
           catch( RTIcap.RTIinternalError ie) {}

         try 
         {
           rtiAmb.destroyFederationExecution(FederationName);
         } catch( RTIcap.FederationExecutionDoesNotExist icnd) {}
           catch( RTIcap.FederatesCurrentlyJoined ie) {}
           catch( RTIcap.RTIinternalError ie) {}

      }// end of if

   }// end of leaveFederationExecution() 
   
   /**
   */
   public void
   resetTimeAdvanceGranted()
   { synchronized(guard) { advanceGranted = false; } 
   }// end of resetTimeAdvanceGranted()

   /**
   * This method will be invoked by HwFededarateAmbassador object whenever
   *
   *   timeAdvanceGrant 
   *   timeConstrainedEnabled
   *   timeRegulationEnabled
   *
   * messages received.
   *
   */
   public void
   setTimeAdvanceGranted()
   { synchronized(guard) { advanceGranted = true; } 
   }// end of setTimeAdvanceGranted()

   /**
   */
   public boolean isTimeAdvanceGranted() { return advanceGranted ; }

   /**
   */
   public void
   setCurrentTime( double _newTime)
   { 
     if( _newTime >= currentTime ) currentTime = _newTime;

   }// end of setCurrentTime()

   /**
   */
   public final void
   mySleep(long aa)
   {  
      //try
      //{ if(runner != null ) runner.sleep(aa);
      //} catch( java.lang.InterruptedException ie) {}

   }// end of mySleep()

   /**
   *
   * Ticks the RTIambassador. This will cause the delivery of waiting 
   * events in the Receive Order Queue.
   *
   */
   public void
   tickRTI( long sleepTime )
   {
      while( !isTimeAdvanceGranted() )
      {
         try { rtiAmb.tick( ); } 
         catch( RTIcap.RTIinternalError aa ) {}

         mySleep(2);

      }// end of while() 

   }// end of tickRTI()

   /**
   *
   * Ticks the RTIambassador. This will cause the delivery of waiting 
   * events in the Receive Order Queue.
   *
   */
   public void
   tick()
   {
      try { rtiAmb.tick( ); } 
      catch( RTIcap.RTIinternalError aa ) {}

   }// end of tick()

}// end class

