/**
* @date  January 27, 1999 ( Last Updated on January 27, 1999 )
*
*/
package comm;

import mil.navy.nps.dis.*;

import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import jdis.*;
import dishelper.*;

import org.omg.CORBA.*;
import RTIcap.*;
//import owrti.util.CDRCoderEncoder; // compatibility between various platforms.

/**
*
* This is callback object to receive RTI messages.
* Collects the PDU from the RTI.
* It is alittle bit twisted since DISHLASubAdmin has an RTIAmbassador handle.
*
* <p>
* <ol>
* <p> <li> discoverObjectInstance
* <p> <li> removeObjectInstance[With Time]
* <p> <li> reflectAttributeValues[WithTime]
* <p> <li> provideAttributeValueUpdate
* <p> <li> turnUpdatesOnForObjectInstance
* <p> <li> turnUpdatesOffForObjectInstance
* <p> <li> timeAdcanceGrant
* <p> <li> timeRegulationEnabled
* <p> <li> timeConstrainedEnabled
* </ol>
*
* <p>
*
* @see DISHLAPublisher
* @see SubscriberAdmin
* @see FederateAmbassador
*
* @author H. Timucin Ozdemir
* @date  January 27, 1999 ( Last Updated on February 9, 1999 )
*
*/
public class 
DISHLASubscriberAdmin extends RTIcap._FederateAmbassadorImplBase 
                      implements SubscriberAdmin 
{

    PushPDUType     inform = null;

    Vector          listeners = new Vector();
    boolean         collecting = false;
    boolean         DEBUG      = false;

    DISHLAPublisher boss=null;

    short  disInterClassTypeId = -1;
    int    disParamTypeId      = -1;
    Ticker runner              = new Ticker();
    int    receiverState       = SubscriberAdmin.NOT_CONNECTED;

    /**
    * @param _inform PDU Type visualizer.
    */
    public DISHLASubscriberAdmin( PushPDUType _inform)
    {
       inform = _inform;

    }// end of CONSTRUCTOR()

    /**
    */
    public DISHLASubscriberAdmin( )
    {
       this(null);
      
    }// end of CONSTRUCTOR()

    /**
    */
    public void setBoss(DISHLAPublisher _boss) 
    { 
       if(_boss!=null) 
       { 
          boss = _boss; 
          disInterClassTypeId = boss.getDISInterClassTypeId();
          disParamTypeId = boss.getDISParamTypeId();
          boss.setFederateAmbassador((RTIcap.FederateAmbassador)this);
          receiverState       = SubscriberAdmin.CONNECTED;

       }// end of if

    }// end of setBoss()

    /**
    * Broadcasts the data to all listeners.
    */
    void broadcast(byte[] data)
    {
       int i=0;

       ProtocolDataUnit pdu = (collecting ? PDUReadWrite.getPDU(data) : null);

       if(pdu!=null) 
       {
          if(inform!=null) inform.push(pdu.getPduType().intValue());
          if(DEBUG) System.out.println("PDU type : "+ pdu.getPduType().intValue());

          Enumeration e = listeners.elements();

          while(e.hasMoreElements())
          {
             PDUSubscriber aSub = (PDUSubscriber) e.nextElement();

             if( aSub!=null ) aSub.receive(pdu);

          }// end of for

       } else System.out.println("PDU is NULL!");

    }// end of broadcast()

    //
    // jdis.SubscriberAdmin 
    //

    /**
    * registers a possible listener.
    */
    public final void 
    register(PDUSubscriber _aSub)
    {
       if( !listeners.contains(_aSub) ) listeners.addElement(_aSub);

    }// end of register()       

    /**
    * deregisters the listener.
    */
    public final void 
    deregister(PDUSubscriber _aSub)
    {
       if( listeners.contains(_aSub) ) listeners.removeElement(_aSub);

    }// end of deregister()       

    /**
    *
    * Starts receiving interactions from the RTI.
    * Then it will receive receiveInteraction call from RTI
    * and will forward to the listeners.
    *
    */
    public final void 
    startCollect()
    {
       if(boss!=null) 
       { 
          collecting=boss.subscribeDIS();
          disInterClassTypeId = boss.getDISInterClassTypeId();
          disParamTypeId = boss.getDISParamTypeId();
          runner.startTicking(boss);
          if(collecting) receiverState = SubscriberAdmin.CAN_RECEIVE;

       }// end of if

    }// end of startCollect()       

    /**
    * Stops collecting.
    */
    public final void 
    stopCollect()
    {
       if(boss!=null) 
       {  
          // we won't be collecting
          collecting=false;

          // stop ticking
          runner.stopTicking();

          // unsubscribe to interactions.
          boss.unsubscribeDIS();

          // change my state.
          receiverState = SubscriberAdmin.CONNECTED;

          // refresh listener list.
          listeners = new Vector();

       }// end of if

    }// end of stopCollect()       

    /**
    * DISHLAPublisher will take care of it.
    */
    public final void 
    reconnect(String a, int p1)
    {
    }// end of reconnect()       

    /**
    * DISHLAPublisher will take care of it.
    */
    public final void 
    closeConnection()
    {
       stopCollect();
       //receiverState = SubscriberAdmin.CONNECTED;

    }// end of closeConnection()       

    //
    // implement RTIcap._FederateAmbassadorImplBase
    //

    public void 
    synchronizationPointRegistrationSucceeded(String theLabel)
        throws RTIcap.FederateInternalError
    { }

    public void 
    synchronizationPointRegistrationFailed(String theLabel)
        throws RTIcap.FederateInternalError
    { }

    public void 
    announceSynchronizationPoint(String theLabel, String theTag)
        throws RTIcap.FederateInternalError
    { }

    public void 
    federationSynchronized(String theLabel) throws RTIcap.FederateInternalError
    { }

    public void 
    initiateFederateSave(String theLabel)
        throws RTIcap.UnableToPerformSave, RTIcap.FederateInternalError
    { }

    public void 
    federationSaved() throws RTIcap.FederateInternalError
    { }

    public void 
    federationNotSaved() throws RTIcap.FederateInternalError
    { }

    public void 
    requestFederationRestoreSucceeded(String theLabel)
        throws RTIcap.FederateInternalError
    { }

    public void 
    requestFederationRestoreFailed(String theLabel)
        throws RTIcap.FederateInternalError
    { }

    public void 
    federationRestoreBegun() throws RTIcap.FederateInternalError
    { }

    public void 
    initiateFederateRestore(String theLabel, short handle)
        throws RTIcap.SpecifiedLabelDoesNotExist, RTIcap.CouldNotRestore, 
               RTIcap.FederateInternalError
    { }

    public void 
    federationRestored() throws RTIcap.FederateInternalError
    { }

    public void 
    federationNotRestored() throws RTIcap.FederateInternalError
    { }

    public void 
    startRegistrationForObjectClass(short theClass)
        throws RTIcap.ObjectClassNotPublished, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    public void 
    stopRegistrationForObjectClass(short theClass)
        throws RTIcap.ObjectClassNotPublished, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    public void 
    turnInteractionsOn(short theHandle)
        throws RTIcap.InteractionClassNotPublished, RTIcap.FederateInternalError
    { }

    public void 
    turnInteractionsOff(short theHandle)
       throws RTIcap.InteractionClassNotPublished, RTIcap.FederateInternalError
    { }

    public void 
    startUpdates(short theClass, short[] theAttributes)
        throws RTIcap.ObjectClassNotPublished, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    public void 
    stopUpdates(short theClass, short[] theAttributes)
        throws RTIcap.ObjectClassNotPublished, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    /**
    * Add this new discovered object to the EntityManager.
    */
    public void 
    discoverObjectInstance(int theObject, short theObjectClass)
        throws RTIcap.CouldNotDiscover, RTIcap.ObjectClassNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    { 
        /*
        Country c = entManager.getEntity(theObject);
        if(c==null) entManager.add(theObject,new Country(theObject));
        */

    }// end of discoverObjectInstance()

    /**
    * Update attributes.
    */
    public void 
    reflectAttributeValuesWithTime(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    { 
        reflectAttributeValues(theObject,theAttributes,theTag); 
    }// end of reflectAttributeValuesWithTime()

    /**
    *
    * Find the objects in the EntityManager and updates their received
    * attributes. Note that, CDR provided us the necessary decode/encode
    * mechanism for various data types.
    *
    */
    public void 
    reflectAttributeValues(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.FederateInternalError
    { 
        /*
        Country c = entManager.getEntity(theObject);

        //System.out.println(" in  reflectAttributeValues("+theObject+")");

        if( c != null ) 
        {
           //System.out.println(theObject+" is found ");

           for(int i=0;i<theAttributes.length;i++)
           {
              //System.out.println(" for attribute ["+i+"] "+theAttributes[i].handle);
              if( theAttributes[i].handle == boss.getNameAttributeId() )
                  c.setName(new String(theAttributes[i].value));
              else if( theAttributes[i].handle == boss.getPopuAttributeId() )
              {
                  //c.setPopulation(Double.valueOf(new String(theAttributes[i].value)).doubleValue());
                  c.setPopulation(CDRCoderEncoder.extractDouble(theAttributes[i].value));
              } 

           }// end of for

        } else System.out.println(theObject+" is not known ");
        */

    }// end of reflectAttributeValues()

    public void 
    receiveInteractionWithTime(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    { 
       if( collecting && theInteraction==disInterClassTypeId)
           receiveInteraction(theInteraction,theParameters,theTag);
    }// end of receiveInteraction()

    public void 
    receiveInteraction(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, String theTag)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, RTIcap.FederateInternalError
    { 
       if(DEBUG) System.out.println(" theInteraction: "+theInteraction);
       if(DEBUG) System.out.println(" disInterClassTypeId: "+disInterClassTypeId);
       if( collecting && theInteraction==disInterClassTypeId)
       {
         if(DEBUG) System.out.println(" theParameters.length: "+theParameters.length);
         for(int i=0;i<theParameters.length;i++)
         {
            if(theParameters[i].handle==disParamTypeId) 
               broadcast(theParameters[i].value);

         }// end of for  

       }// end of if

    }// end of receiveInteraction()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public void 
    removeObjectInstanceWithTime(int theObject, double theTime, String theTag, 
               RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    { 
        /**
        Country c = entManager.getEntity(theObject);
        if(c!=null) entManager.remove(theObject);
        */

    }// end of removeObjectInstanceWithTime()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public void 
    removeObjectInstance(int theObject, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateInternalError
    { 
        /**
        Country c = entManager.getEntity(theObject);
        if(c!=null) entManager.remove(theObject);
        */

    }// end of removeObjectInstanceWithTime()

    public void 
    attributesInScope(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateInternalError
    { }

    public void 
    attributesOutOfScope(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateInternalError
    { }

    /**
    * Ask the corresponding SimCountry object to update its attribute.
    */
    public void 
    provideAttributeValueUpdate(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeNotOwned, RTIcap.FederateInternalError
    { 
        /*
        //System.out.println(" In provideAttributeValueUpdate() ");
        Country c = entManager.getEntity(theObject);

        if(c!=null && c instanceof SimCountry )
        {
           for(int i=0;i<theAttributes.length;i++)
            if( theAttributes[i] == boss.getNameAttributeId() )
                ((SimCountry)c).setNameChanged();
            else if( theAttributes[i] == boss.getPopuAttributeId() )
                ((SimCountry)c).setPopuChanged();

        }// end of if
        //System.out.println(" Out of provideAttributeValueUpdate() ");
        */

    }// end of provideAttributeValueUpdate()

    /**
    * Tell simulated object to send its updates to RTI.
    */
    public void 
    turnUpdatesOnForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { 
        /*
        Country c = entManager.getEntity(theObject);
        if( c != null && c instanceof SimCountry ) 
            ((SimCountry)c).UpdateControls(true,theAttributes); 
        */

    }// end of turnUpdatesOnForObjectInstance()

    /**
    * Tell simulated object *NOT* to send its updates to RTI.
    */
    public void 
    turnUpdatesOffForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { 
        /**
        Country c = entManager.getEntity(theObject);
        if( c != null && c instanceof SimCountry ) 
            ((SimCountry)c).UpdateControls(false,theAttributes); 
        **/
    }// end of turnUpdatesOffForObjectInstance()

    public void 
    requestAttributeOwnershipAssumption(int theObject, 
               short[] offeredAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeAlreadyOwned, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    public void 
    attributeOwnershipDivestitureNotification(int theObject, 
               short[] releasedAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeNotOwned, 
               RTIcap.AttributeDivestitureWasNotRequested, 
               RTIcap.FederateInternalError
    { }

    public void 
    attributeOwnershipAcquisitionNotification(int theObject, 
               short[] securedAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeAcquisitionWasNotRequested, 
               RTIcap.AttributeAlreadyOwned, RTIcap.AttributeNotPublished, 
               RTIcap.FederateInternalError
    { }

    public void 
     attributeOwnershipUnavailable(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeAlreadyOwned, 
               RTIcap.AttributeAcquisitionWasNotRequested, 
               RTIcap.FederateInternalError
    { }

    public void 
    requestAttributeOwnershipRelease(int theObject, 
               short[] candidateAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeNotOwned, RTIcap.FederateInternalError
    { }

    public void 
    confirmAttributeOwnershipAcquisitionCancellation(int theObject, 
               short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeAlreadyOwned, 
               RTIcap.AttributeAcquisitionWasNotCancelled, 
               RTIcap.FederateInternalError
    { }

    public void 
    informAttributeOwnership(int theObject, short theAttribute, short theOwner)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateInternalError
    { }

    public void 
    attributeIsNotOwned(int theObject, short theAttribute)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateInternalError
    { }

    public void 
    attributeIsOwnedByRTI(int theObject, short theAttribute)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateInternalError
    { }

    /**
    * Change the HelloWorld federation's time. And inform that time is
    * advanced.
    */
    public void 
    timeAdvanceGrant(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.TimeAdvanceWasNotInProgress, 
               RTIcap.FederateInternalError
    { 
        if(boss!=null)
        {
           boss.setCurrentTime( theTime );
           boss.setTimeAdvanceGranted();
        }

    }// end of timeAdvanceGrant()

    /**
    * Change the HelloWorld federate's time. And inform that time is
    * advanced.
    */
    public void 
    timeRegulationEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeRegulationWasNotPending, 
               RTIcap.FederateInternalError
    { 
        if(boss!=null)
        {
           boss.setCurrentTime( theTime );
           boss.setTimeAdvanceGranted();
        }

    }// end of timeRegulationEnabled()

    /**
    * Change the HelloWorld federate's time. And inform that time is
    * advanced.
    */
    public void 
    timeConstrainedEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeConstrainedWasNotPending, 
               RTIcap.FederateInternalError
    { 
        if(boss!=null)
        {
           boss.setCurrentTime( theTime );
           boss.setTimeAdvanceGranted();
        }// end of if

    }// end of timeConstrainedEnabled()

    public void 
    requestRetraction(RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.EventNotKnown, RTIcap.FederateInternalError
    { }

    public void 
    changeThresholds(RTIcap.Region theRegion, 
                     RTIcap.ThresholdSetHolder theThresholds)
        throws RTIcap.RegionNotKnown, RTIcap.FederateInternalError
    { }

}// end of Class DISHLASubscriberAdmin

