package comm;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;
import jdis.*;

/**
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MulticastPDUPublisher implements PDUPublisher 
{

   byte     	outputBuffer[]  = new byte[1500];// buffer to hold datagram data

   InetAddress 	group; 
   MulticastSocket s ;                   // socket to spit it out on
   DatagramPacket          dgramPacket;  // one datagram packet
   ByteArrayOutputStream   outputStream;
   DataOutputStream        dataOutputStream;
   byte                    streamArray[];
   int                     pduSentCount = 0;

   String  mcastAddress = JDISConstants.DEFAULT_MCAST;
   int     port         = JDISConstants.DEFAULT_MPORT;
   boolean DEBUG        = false;

   boolean alreadyConnected = false;
   int     publisherState = PDUPublisher.NOT_CONNECTED;
   
   /**
   *
   * Defines a Multicast PDU sender comm. port and connects it to the
   * channel.
   *
   * @param mAddress Multicast Group Address
   * @param _port    Multicast Port Number
   */
   public MulticastPDUPublisher(String mAddress, int _port)
   {

      try
      {
        mcastAddress = mAddress; port = _port;
        group = InetAddress.getByName(mcastAddress);
        s = new MulticastSocket(port);
        s.joinGroup(group);
        alreadyConnected = true;
        publisherState = PDUPublisher.CONNECTED;

      } catch (UnknownHostException e) { System.err.println(e); }
        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

   }// end of CONSTRUCTOR()

   /**
   * @param mAddress Multicast Group Address
   * @param _port    Multicast Port Number
   */
   public void 
   reconnect (String mAddress, int _port)
   {   

      if(!alreadyConnected)
        try
        {
          mcastAddress = mAddress; port = _port;
          group = InetAddress.getByName(mcastAddress);
          s = new MulticastSocket(port);
          s.joinGroup(group);
          alreadyConnected = true;
          publisherState = PDUPublisher.CONNECTED;

        } catch (UnknownHostException e) 
          { if(DEBUG) System.err.println(e.getMessage()); 
            publisherState = PDUPublisher.NOT_CONNECTED;
          }
          catch(IOException ioException) 
          { if(DEBUG) 
               System.out.println("sent a total of " + pduSentCount + "packets"); 
            publisherState = PDUPublisher.NOT_CONNECTED;
          }

   }// end of reconnect()

   /**
   */
   public void 
   closeConnection()
   {   

      if(alreadyConnected)
        try
        {
          if(s!=null) s.leaveGroup(group);
          alreadyConnected = false;
          publisherState = PDUPublisher.NOT_CONNECTED;

        } catch(IOException ioException) 
          { if(DEBUG) System.out.println(" MulPDUPub " + ioException.getMessage() ); }

   }// end of closeConnection()

   /**
   */
   public final boolean 
   isConnected() { return ( publisherState == PDUPublisher.CONNECTED ); } 

   /**
   */
   public final boolean 
   canPublish() { return ( publisherState == PDUPublisher.CAN_PUBLISH ); } 

   /**
   */
   public void 
   send (ProtocolDataUnit pdu) 
   {   
      if( canPublish() )
      {
         ByteArrayOutputStream out  = new ByteArrayOutputStream();
         DataOutputStream      dout = new DataOutputStream(out);
         //pdu.serialize(dout);

         pdu.setProtocolVersion(JDISConstants.DIS203_PROTOCOL_VERSION_CURRENT);
         pdu.setExerciseID(1);
         pdu.makeTimestampCurrent();

         pdu.serialize(dout);

         send(out.toByteArray());

      }// end of if

   }// end of send() 

   /**
   */
   public void 
   send (byte[] data)
   {

      /*
      pdu.setProtocolVersion(JDISConstants.DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();
      */

      if( canPublish() )
      {
         streamArray = data;

         if(DEBUG) System.out.println(" data serialized to "+streamArray[0]+ 
                                      " and " + streamArray[1]);

         dgramPacket = new DatagramPacket( data, data.length, group, port);


	     try
         {
	        { s.send(dgramPacket, (byte)127);}

            if(++pduSentCount % 100 == 0 && DEBUG) 
               System.out.println("sent " + pduSentCount + " Pdus"); 

	     } catch (SocketException se) 
           { if(DEBUG) System.err.println(se); }
           catch (UnknownHostException e) 
           { if(DEBUG) System.err.println(e); }
           catch(IOException ioException) 
           { if(DEBUG) 
               System.out.println("sent a total of " + pduSentCount + "packets");            
           }
           catch(NullPointerException nullExp) 
           { if(DEBUG) System.out.println("sent a total of " + pduSentCount + "packets"); 
           }

      }// end of if
          
   }//end method

   /**
   */
   public final void
   startPublishing()
   {
     if( isConnected() ) publisherState = PDUPublisher.CAN_PUBLISH;

   }// end of startPublishing()

   /**
   */
   public final void
   stopPublishing()
   {
     if( canPublish() ) publisherState = PDUPublisher.CONNECTED;

   }// end of stopPublishing()

   /**
   */
   public void 
   SendMPDU (ProtocolDataUnit pdu)
   {

      pdu.setProtocolVersion(JDISConstants.DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();

      outputStream = new ByteArrayOutputStream();
      dataOutputStream = new DataOutputStream(outputStream);
      pdu.serialize(dataOutputStream);
      streamArray = outputStream.toByteArray();

      System.out.println( " data serialized to " + streamArray[0] + 
                          " and " + streamArray[1]);

      dgramPacket = new DatagramPacket( outputStream.toByteArray(), 
                                        outputStream.size(), group, port);


	  try
      {
	     { s.send(dgramPacket, (byte)127);}

         if(++pduSentCount % 100 == 0) 
         { System.out.println("sent " + pduSentCount + " Pdus"); }

	  } catch (SocketException se) { System.err.println(se); }

        catch (UnknownHostException e) { System.err.println(e); }

        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

        catch(NullPointerException nullExp) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }
          
   }//end method

   /**
   */
   public static void 
   send( ProtocolDataUnit pdu, MulticastSocket socket, 
         InetAddress mgroup, int mport )
   {

      pdu.setProtocolVersion(JDISConstants.DIS203_PROTOCOL_VERSION_CURRENT);
      pdu.setExerciseID(1);
      pdu.makeTimestampCurrent();

      ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
      DataOutputStream dataOutputStream = new DataOutputStream(outputStream);
      pdu.serialize(dataOutputStream);
      byte[] streamArray = outputStream.toByteArray();
      int pduSentCount=0;

      System.out.println( " data serialized to " + streamArray[0] + 
                          " and " + streamArray[1]);

      DatagramPacket dgramPacket = new DatagramPacket( outputStream.toByteArray(), 
                                        //outputStream.size(), group, port);
                                        outputStream.size(), mgroup, mport);


	  try
      {
	     { socket.send(dgramPacket, (byte)127);}

         if(++pduSentCount % 100 == 0) 
         { System.out.println("sent " + pduSentCount + " Pdus"); }

	  } catch (SocketException se) { System.err.println(se); }

        catch (UnknownHostException e) { System.err.println(e); }

        catch(IOException ioException) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }

        catch(NullPointerException nullExp) 
        { System.out.println("sent a total of " + pduSentCount + "packets"); }
          
   }//end method

}//endclass

