/**
* @date  January 26, 1999 ( Last Updated on January 26, 1999 )
*
*/
package comm;

import mil.navy.nps.dis.*;

import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import jdis.*;
import worker.*;

/**
*
* Collects the PDU from the Multicast Socket and pushes them to the
* listeners.
*
* @see RunnerObject
* @see SubscriberAdmin
*
* @author H. Timucin Ozdemir
* @date  January 26, 1999 ( Last Updated on January 26, 1999 )
*
*/
public class 
MulticastPDUSubAdmin extends RunnerObject implements SubscriberAdmin
{

    String mAddress;
    int    mPort;
    PushPDUType inform = null;

    RunnerObject  peer; 

    MulticastSocket socket;

    Vector        listeners = new Vector();
    boolean stay = true;

    /**
    */
    public MulticastPDUSubAdmin( String _mAddress, int _mPort, PushPDUType _inform)
    {
       inform   = _inform;

      try
      {
        mAddress = _mAddress; mPort = _mPort;
        InetAddress group = InetAddress.getByName(mAddress);
        socket = new MulticastSocket(mPort);
        socket.joinGroup(group);
        //alreadyConnected = true;
        //publisherState = PDUPublisher.CONNECTED;

      } catch (UnknownHostException e) { System.err.println(e); }
        catch(IOException ioException)
        { System.out.println("sent a total of packets"); }
      
    }// end of CONSTRUCTOR()

    /**
    */
    public MulticastPDUSubAdmin( String mAddress, int _port)
    {
       this(mAddress,_port,null);
      
    }// end of CONSTRUCTOR()

    /**
    */
    public void setPeer(RunnerObject _peer) { if(_peer!=null) peer = _peer; }

    /**
    */
    public void stop() { super.stop(); if(peer!=null) peer.stop(); }

    /**
    */
    public void 
    run()
    {
       int i=0;

       try
       {

         DatagramPacket packet;

         if(peer!=null) peer.start();

         while(stay)
         {
            byte[] buf = new byte[5500];
            packet = new DatagramPacket(buf, buf.length);
            socket.receive(packet);

            //
            byte[] received = new byte[packet.getLength()];
            System.arraycopy(buf,0,received,0,received.length);

            /*
            ProtocolDataUnit pdu = PDUReadWrite.getPDU(packet.getData());

            if(pdu!=null) 
            {
               if(inform!=null) inform.push(pdu.getPduType().intValue());

               Enumeration e = listeners.elements();

               while(e.hasMoreElements())
               {
                   PDUSubscriber aSub = (PDUSubscriber) e.nextElement();

                   if( aSub!=null ) aSub.receive(packet.getData());

               }// end of for

            }// end of if
            */

            ProtocolDataUnit pdu = PDUReadWrite.getPDU(received);

            if(pdu!=null) 
            {
               if(inform!=null) inform.push(pdu.getPduType().intValue());

               Enumeration e = listeners.elements();

               while(e.hasMoreElements())
               {
                   PDUSubscriber aSub = (PDUSubscriber) e.nextElement();

                   if( aSub!=null ) aSub.receive(received);

               }// end of for

            }// end of if
            
            yield();

         }// end of while

       } catch( java.io.IOException ie) {  }

       stop();

    }// end of run()

    /**
    */
    public final void 
    register(PDUSubscriber _aSub)
    {
       if( !listeners.contains(_aSub) ) listeners.addElement(_aSub);
    }//       

    /**
    */
    public final void 
    deregister(PDUSubscriber _aSub)
    {
       if( listeners.contains(_aSub) ) listeners.removeElement(_aSub);

    }//       

    /**
    */
    public final void 
    startCollect()
    {
       stay=true;
       start();

    }//       

    /**
    */
    public final void 
    stopCollect()
    {
       stay=false;
       stop();

    }//       

    /**
    */
    public void 
    reconnect( String mAddress, int mPortNum )
    {

       // Shut down the socket.
       if( socket!=null ) closeConnection(); 

       try
       {

         InetAddress group = InetAddress.getByName(mAddress);
         socket = new MulticastSocket(mPortNum);
         socket.joinGroup(group);

       } catch (UnknownHostException e) 
         { System.err.println(e); socket=null; }
         catch (IOException ioe) 
        { System.out.println("IOException in reconnect()"); socket=null; }
      
    }// end of reconnect()

    /**
    */
    public void 
    closeConnection ()
    {

       try
       {
         if(socket!=null) socket.leaveGroup(socket.getInterface());

       } catch(java.net.SocketException ioe) {}
         catch(java.io.IOException ioe) {}

       socket=null; 
      
    }// end of closeConnection ()

}// end of Class MulticastPDUSubAdmin

