/**
* @date  October 17, 1998 ( Last Updated on October 17, 1998 )
*
*/
package comm;

import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

// DIS-JAVA-VRML PDU Package
import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

// IWT Mine PDU Package
import iwt.mine.*;

// IWT PDU Package
import iwt.dis.*;

// W3C DOM Package
import org.w3c.dom.*;
import com.docuverse.freedom.*;

// IWT Utilities 
import iwt.domUtil.*;
import iwt.*;

/**
*
* 1. read PDU from DIS format 
* 2. read PDU from XML format (DOM Element object)
*
* @author H. Timucin Ozdemir
* @date  October 17, 1998 ( Last Updated on February 2, 1999 )
*
*/
public class PDUReadWrite 
{
    static final boolean DEBUG = true;
    public static int cnt = 0;

    /**
    */
    public static void 
    putPDU(DataOutputStream outDataStream, ProtocolDataUnit pdu)
    {
        pdu.serialize(outDataStream);

    }// end of putPDU()

    /**
    */
    public static ProtocolDataUnit getPDU(PushbackInputStream pStream)
    {

      // The pdu we return from the datagram packet
      ProtocolDataUnit        newPdu=null;                         
      UnsignedByte            pduVersion = new UnsignedByte();// pdu version

      if(DEBUG)
         System.out.println("ProtocolDataUnit: translating a datagram into a PDU");
    
      try
      {
      
        byte[] buf = new byte[8];
        pStream.read(buf);
        pStream.unread(buf);
        ByteArrayInputStream bin = new ByteArrayInputStream(buf);

        // wrap a data stream around _that_
        DataInputStream initialDataStream = new DataInputStream(bin);  

        // The pdu version is the third byte in the stream. 
        // Read the first two just to throw them away.

        initialDataStream.mark(8);                 

        try
        {
           // Skip first two bytes to get to pdu type
           initialDataStream.skipBytes(2);                 

           // read the PDU type, keep the results
           pduVersion.deSerialize(initialDataStream);      

         } catch (IOException ioError)
           {
             throw new RuntimeException("Exception in ProtocolDataUnit. Error instantiating from datagram.");
           }
    
        try
        {
           // reset the stream to the start
           initialDataStream.reset();          
           // reset the stream to the start
        } catch(IOException ioException)
          {
             throw new RuntimeException("Exception in ProtocolDataUnit. Error resetting stream.");
          }

       // We need to degrade gracefully if this is an unrecognized PDU...
       // we'll probably see a lot of pdus that we haven't implemented yet.

       if(DEBUG) System.out.println("Got PDU of type " + pduVersion);

       switch(pduVersion.shortValue())
       {
        // entity state pdu
        case PDUTypeField.ENTITYSTATE:      // do EntityStatePdu 
                if(DEBUG)
                System.out.println("got a packet, instantiating an ESPDU");
                newPdu = new EntityStatePdu();
                ((EntityStatePdu)newPdu).deSerialize(new DataInputStream(pStream));
                break;


        // Fire pdu
        case PDUTypeField.FIRE:              
                if(DEBUG)
                  System.out.println("got a packet, instantiating a FIRE PDU");
                newPdu = new FirePdu();
                ((FirePdu)newPdu).deSerialize(new DataInputStream(pStream));
                break;
        
        // detonation PDU, the fire PDU's frequent companion.
        case PDUTypeField.DETONATION:
                if(DEBUG)
            System.out.println("got a packet, instantiating a DETONATATION PDU");
            newPdu = new DetonationPdu();
            ((DetonationPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Message PDU, same as CommentPDU, I THINK.
        case PDUTypeField.MESSAGE:   
            if(DEBUG) System.out.println("got a message PDU");
            //newPdu = new CommentPdu();
            //((CommentPdu)newPdu).deSerialize(new DataInputStream(pStream));
            newPdu = null; 
            break;

        // Acknowledge PDU
        case PDUTypeField.ACKNOWLEDGE:   
                if(DEBUG)
            System.out.println("got a ACKNOWLEDGE PDU");
            newPdu = new AcknowledgePdu();
            ((AcknowledgePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // ActionRequest PDU
        case PDUTypeField.ACTIONREQUEST:   
                if(DEBUG)
            System.out.println("got a ActionRequest PDU");
            newPdu = new ActionRequestPdu();
            ((ActionRequestPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // ActionResponse PDU
        case PDUTypeField.ACTIONRESPONSE:   
                if(DEBUG)
            System.out.println("got a ActionResponse PDU");
            newPdu = new ActionResponsePdu();
            ((ActionResponsePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // CreateEntity PDU
        case PDUTypeField.CREATEENTITY:   
                if(DEBUG)
            System.out.println("got a CreateEntity PDU");
            newPdu = new CreateEntityPdu();
            ((CreateEntityPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Data PDU
        case PDUTypeField.DATA:   
                if(DEBUG)
            System.out.println("got a Data PDU");
            newPdu = new DataPdu();
            ((DataPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // DataQuery PDU
        case PDUTypeField.DATAQUERY:   
                if(DEBUG)
            System.out.println("got a DataQuery PDU");
            newPdu = new DataQueryPdu();
            ((DataQueryPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // EventReport PDU
        case PDUTypeField.EVENTREPORT:   
                if(DEBUG)
            System.out.println("got a EventReport PDU");
            newPdu = new EventReportPdu();
            ((EventReportPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // RemoveEntity PDU
        case PDUTypeField.REMOVEENTITY:   
                if(DEBUG)
            System.out.println("got a RemoveEntity PDU");
            newPdu = new RemoveEntityPdu();
            ((RemoveEntityPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // SetData PDU
        case PDUTypeField.SETDATA:   
                if(DEBUG)
            System.out.println("got a SetData PDU");
            newPdu = new SetDataPdu();
            ((SetDataPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // StartResume PDU
        case PDUTypeField.STARTRESUME:   
                if(DEBUG)
            System.out.println("got a StartResume PDU");
            newPdu = new StartResumePdu();
            ((StartResumePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // StopFreeze PDU
        case PDUTypeField.STOPFREEZE:   
                if(DEBUG)
            System.out.println("got a StopFreeze PDU");
            newPdu = new StopFreezePdu();
            ((StopFreezePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Mine PDU
        case PDUTypeField.MINE:   
                if(DEBUG) System.out.println("got a Mine PDU");
            newPdu = new MinePdu();
            ((MinePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield PDU
        case PDUTypeField.MINEFIELD:   
             if(DEBUG) System.out.println("got a Minefield PDU");
             newPdu = new MinefieldPdu();
             ((MinefieldPdu)newPdu).deSerialize(new DataInputStream(pStream));
             break;

        // Minefield Request PDU
        case PDUTypeField.MINEFIELDREQUEST:   
                if(DEBUG)
            System.out.println("got a Minefield Request PDU");
            //newPdu = new MineFieldRequestPdu();
            newPdu = new MineFieldRequestRevPdu();
            ((MineFieldRequestRevPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield Response PDU
        case PDUTypeField.MINEFIELDRESPONSE:   
                if(DEBUG)
            System.out.println("got a Minefield Response PDU");
            //newPdu = new MineFieldResponsePdu();
            newPdu = new MineFieldResponseRevPdu();
            ((MineFieldResponseRevPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield State PDU
        case PDUTypeField.MINEFIELDSTATE:   
                if(DEBUG)
            System.out.println("got a Minefield State PDU");
            newPdu = new MineFieldStatePdu();
            ((MineFieldStatePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield State PDU
        case PDUTypeField.MINEFIELDNACK:   
                if(DEBUG)
            System.out.println("got a Minefield NACK PDU");
            newPdu = new MineFieldNACKPdu();
            ((MineFieldNACKPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Transmitter State PDU
        case PDUTypeField.TRANSMITTER:   
                if(DEBUG)
            System.out.println("got a Transmitter PDU");
            newPdu = new TransmitterPdu();
            ((TransmitterPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        /*
        // SimulationManagement PDU
        case PDUTypeField.SETDATA:   
                if(DEBUG)
            System.out.println("got a SimulationManagement PDU");
            newPdu = new SimulationManagementPdu();
            ((SimulationManagementPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;
        */

        //case PDUTypeField.COLLISION:           return;         // do collision pdu here
    

        default:
            {
                // unrecognized PDU type. 
              System.out.println("Unrecognized PDU type received in packet:"  + pduVersion.shortValue());
                return null;
            }
    }

    if(DEBUG) 
       System.out.println("ProtocolDataUnit: translated a datagram into a PDU");
      
    } catch (IOException ioError) {}
      catch( java.lang.RuntimeException re) 
      { newPdu=null; 
        System.out.println("deserialize error...!!");
      }

    return newPdu;      // may be null if PDU type not recognized

    }// end of getPDU()

    /**
    */
    public static int getPDUType(byte[] data)
    {

       if(data==null || data.length<5) return -1;

       UnsignedByte            pduVersion = new UnsignedByte();// pdu version

       if(DEBUG)
          System.out.println("ProtocolDataUnit: translating a datagram into a PDU");
      

       // wrap a data stream around _that_

       ByteArrayInputStream bin = new ByteArrayInputStream(data);
       DataInputStream initialDataStream = new DataInputStream(bin);

       // The pdu version is the third byte in the stream. 
       // Read the first two just to throw them away.

       int ret=-1; 

       try
       {
          // Skip first two bytes to get to pdu type
          initialDataStream.skipBytes(2);                 

          // read the PDU type, keep the results
          pduVersion.deSerialize(initialDataStream);      
         
          ret = (int) pduVersion.shortValue();

       } catch (IOException ioError) { }
    
       return ret;      

    }// end of getPDUType()

    /**
    */
    public static ProtocolDataUnit getPDU(byte[] data)
    {

      // The pdu we return from the datagram packet
      ProtocolDataUnit        newPdu=null;                         

      try
      {
      
        ByteArrayInputStream pStream = new ByteArrayInputStream(data);
        int pduVersion = getPDUType(data);
        
        switch(pduVersion)
        {
        // entity state pdu
        case PDUTypeField.ENTITYSTATE:      // do EntityStatePdu 
                if(DEBUG)
                System.out.println("got a packet, instantiating an ESPDU");
                newPdu = new EntityStatePdu();
                ((EntityStatePdu)newPdu).deSerialize(new DataInputStream(pStream));
                break;


        // Fire pdu
        case PDUTypeField.FIRE:              
                if(DEBUG)
                  System.out.println("got a packet, instantiating a FIRE PDU");
                newPdu = new FirePdu();
                ((FirePdu)newPdu).deSerialize(new DataInputStream(pStream));
                break;
        
        // detonation PDU, the fire PDU's frequent companion.
        case PDUTypeField.DETONATION:
                if(DEBUG)
            System.out.println("got a packet, instantiating a DETONATATION PDU");
            newPdu = new DetonationPdu();
            ((DetonationPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Message PDU, same as CommentPDU, I THINK.
        case PDUTypeField.MESSAGE:   
             if(DEBUG) System.out.println("got a message PDU!");
             printFile(pduVersion,data);
             //newPdu = new CommentPdu();
             //((CommentPdu)newPdu).deSerialize(new DataInputStream(pStream));
             //newPdu = null;
             break;

        // Acknowledge PDU
        case PDUTypeField.ACKNOWLEDGE:   
                if(DEBUG)
            System.out.println("got a ACKNOWLEDGE PDU");
            newPdu = new AcknowledgePdu();
            ((AcknowledgePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // ActionRequest PDU
        case PDUTypeField.ACTIONREQUEST:   
                if(DEBUG)
            System.out.println("got a ActionRequest PDU");
            newPdu = new ActionRequestPdu();
            ((ActionRequestPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // ActionResponse PDU
        case PDUTypeField.ACTIONRESPONSE:   
                if(DEBUG)
            System.out.println("got a ActionResponse PDU");
            newPdu = new ActionResponsePdu();
            ((ActionResponsePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // CreateEntity PDU
        case PDUTypeField.CREATEENTITY:   
                if(DEBUG)
            System.out.println("got a CreateEntity PDU");
            newPdu = new CreateEntityPdu();
            ((CreateEntityPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Data PDU
        case PDUTypeField.DATA:   
                if(DEBUG)
            System.out.println("got a Data PDU");
            newPdu = new DataPdu();
            ((DataPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // DataQuery PDU
        case PDUTypeField.DATAQUERY:   
                if(DEBUG)
            System.out.println("got a DataQuery PDU");
            newPdu = new DataQueryPdu();
            ((DataQueryPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // EventReport PDU
        case PDUTypeField.EVENTREPORT:   
                if(DEBUG)
            System.out.println("got a EventReport PDU");
            newPdu = new EventReportPdu();
            ((EventReportPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // RemoveEntity PDU
        case PDUTypeField.REMOVEENTITY:   
                if(DEBUG)
            System.out.println("got a RemoveEntity PDU");
            newPdu = new RemoveEntityPdu();
            ((RemoveEntityPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // SetData PDU
        case PDUTypeField.SETDATA:   
                if(DEBUG)
            System.out.println("got a SetData PDU");
            newPdu = new SetDataPdu();
            ((SetDataPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // StartResume PDU
        case PDUTypeField.STARTRESUME:   
                if(DEBUG)
            System.out.println("got a StartResume PDU");
            newPdu = new StartResumePdu();
            ((StartResumePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // StopFreeze PDU
        case PDUTypeField.STOPFREEZE:   
                if(DEBUG)
            System.out.println("got a StopFreeze PDU");
            newPdu = new StopFreezePdu();
            ((StopFreezePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Mine PDU
        case PDUTypeField.MINE:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Mine PDU");
            newPdu = new MinePdu();
            ((MinePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield PDU
        case PDUTypeField.MINEFIELD:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Minefield PDU");
            newPdu = new MinefieldPdu();
            ((MinefieldPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield Request PDU
        case PDUTypeField.MINEFIELDREQUEST:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Minefield Request PDU");
            //newPdu = new MineFieldRequestPdu();
            newPdu = new MineFieldRequestRevPdu();
            ((MineFieldRequestRevPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield Response PDU
        case PDUTypeField.MINEFIELDRESPONSE:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Minefield Response PDU");
            //newPdu = new MineFieldResponsePdu();
            newPdu = new MineFieldResponseRevPdu();
            ((MineFieldResponseRevPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield State PDU
        case PDUTypeField.MINEFIELDSTATE:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Minefield State PDU");
            newPdu = new MineFieldStatePdu();
            ((MineFieldStatePdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Minefield State PDU
        case PDUTypeField.MINEFIELDNACK:   
                printFile(pduVersion,data);
                if(DEBUG)
            System.out.println("got a Minefield NACK PDU");
            newPdu = new MineFieldNACKPdu();
            ((MineFieldNACKPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        // Transmitter State PDU
        case PDUTypeField.TRANSMITTER:   
                if(DEBUG)
            System.out.println("got a Transmitter PDU");
            newPdu = new TransmitterPdu();
            ((TransmitterPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;

        /*
        // SimulationManagement PDU
        case PDUTypeField.SETDATA:   
                if(DEBUG)
            System.out.println("got a SimulationManagement PDU");
            newPdu = new SimulationManagementPdu();
            ((SimulationManagementPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;
        */

        //case PDUTypeField.COLLISION:           return;         // do collision pdu here
    

        default:
            {
                System.out.println("Unrecognized PDU type received in packet:"  + pduVersion);

                printFile(pduVersion,data);

                return null;
            }

    }// end of switch()

    } catch( java.lang.RuntimeException re) 
      { newPdu=null; 
        System.out.println("deserialize error...!!");
      }

    return newPdu;      // may be null if PDU type not recognized

    }// end of getPDU()


    public static void printFile(int pduType, byte[] data)
    {
      try
      {
         PDUReadWrite.cnt = (PDUReadWrite.cnt+1) % 20;  
         char ap = (char)('a'+cnt);

         DataOutputStream out = new DataOutputStream(new FileOutputStream("unk/p"+pduType+ap+".unk"));
         out.write(data,0,data.length);
         out.close();

      } catch (java.io.IOException ioe) {}

    }// end of printFile()

    public static void printFile( ProtocolDataUnit pdata )
    {
      if( pdata!=null )
      {
        int pduType = pdata.getPduType().shortValue();
        PDUReadWrite.cnt = (PDUReadWrite.cnt+1) % 20;  
        char ap = (char)('a'+cnt);
        PDUReadWrite.printFile(pdata,"unk/p"+pduType+ap+".unk");

      }// end of if

    }// end of printFile()
 
    public static void printFile( ProtocolDataUnit pdata, String pduFileName )
    {
      if( pdata!=null && pduFileName!=null )
      try
      {  
         DataOutputStream out = new DataOutputStream(new FileOutputStream(pduFileName));
         pdata.serialize(out);
         out.close();

      } catch (java.io.IOException ioe) {}
        catch (Exception ie) {}

    }// end of printFile()

    /**
    */
    public static ProtocolDataUnit
    getPDU(org.w3c.dom.Element aNode)
    {
      ProtocolDataUnit pdu = new EntityStatePdu();
      ProtocolDataUnit newPdu = null;

      org.w3c.dom.NodeList nodes = aNode.getElementsByTagName(iwt.DIS_TAGS.DIS_PDU_Header);
      if( nodes.getLength()<1 ) throw new InvalidDocumentType();

      org.w3c.dom.Node rt = nodes.item(0);
      pdu.extractHeader(rt);
      
      UnsignedByte pduType = pdu.getPduType();
      
      switch( pduType.shortValue() )
      { 

         // entity state pdu
         case PDUTypeField.ENTITYSTATE:
              newPdu = new EntityStatePdu();
              //System.out.println(" PduType : EntityState");
              //newPdu.extract(aNode);
              break;

         // Fire pdu
         case PDUTypeField.FIRE:
              newPdu = new FirePdu();
              //newPdu.extract(aNode);
              break;

        // detonation PDU, the fire PDU's frequent companion.
        case PDUTypeField.DETONATION:
             if(DEBUG)
                System.out.println("got a packet, instantiating a DETONATATION PDU");
             newPdu = new DetonationPdu();
             //newPdu.extract(aNode);
             break;

        // Message PDU, same as CommentPDU, I THINK.
        case PDUTypeField.MESSAGE:   
             if(DEBUG) System.out.println("got a message PDU");
             newPdu = new CommentPdu();
             //newPdu.extract(aNode);
             break;

        // Acknowledge PDU
        case PDUTypeField.ACKNOWLEDGE:   
             if(DEBUG) System.out.println("got a ACKNOWLEDGE PDU");
             newPdu = new AcknowledgePdu();
             //newPdu.extract(aNode);
             break;

        // ActionRequest PDU
        case PDUTypeField.ACTIONREQUEST:   
             if(DEBUG) System.out.println("got a ActionRequest PDU");
             newPdu = new ActionRequestPdu();
             //newPdu.extract(aNode);
             break;

        // ActionResponse PDU
        case PDUTypeField.ACTIONRESPONSE:   
             if(DEBUG) System.out.println("got a ActionResponse PDU");
             newPdu = new ActionResponsePdu();
             //newPdu.extract(aNode);
             break;

        // CreateEntity PDU
        case PDUTypeField.CREATEENTITY:   
             if(DEBUG) System.out.println("got a CreateEntity PDU");
             newPdu = new CreateEntityPdu();
             //newPdu.extract(aNode);
             break;

        // Data PDU
        case PDUTypeField.DATA:   
             if(DEBUG) System.out.println("got a Data PDU");
             newPdu = new DataPdu();
             //newPdu.extract(aNode);
             break;

        // DataQuery PDU
        case PDUTypeField.DATAQUERY:   
             if(DEBUG) System.out.println("got a DataQuery PDU");
             newPdu = new DataQueryPdu();
             //newPdu.extract(aNode);
             break;

        // EventReport PDU
        case PDUTypeField.EVENTREPORT:   
             if(DEBUG) System.out.println("got a EventReport PDU");
             newPdu = new EventReportPdu();
             //newPdu.extract(aNode);
             break;

        // RemoveEntity PDU
        case PDUTypeField.REMOVEENTITY:   
             if(DEBUG) System.out.println("got a RemoveEntity PDU");
             newPdu = new RemoveEntityPdu();
             //newPdu.extract(aNode);
             break;

        // SetData PDU
        case PDUTypeField.SETDATA:   
             if(DEBUG) System.out.println("got a SetData PDU");
             newPdu = new SetDataPdu();
             //newPdu.extract(aNode);
             break;

        // StartResume PDU
        case PDUTypeField.STARTRESUME:   
             if(DEBUG) System.out.println("got a StartResume PDU");
             newPdu = new StartResumePdu();
             //newPdu.extract(aNode);
             break;

        // StopFreeze PDU
        case PDUTypeField.STOPFREEZE:   
             if(DEBUG) System.out.println("got a StopFreeze PDU");
             newPdu = new StopFreezePdu();
             //newPdu.extract(aNode);
             break;

        // Mine PDU
        case PDUTypeField.MINE:   
             if(DEBUG) System.out.println("got a Mine PDU");
             newPdu = new MinePdu();
             //newPdu.extract(aNode);
             break;

        // Minefield PDU
        case PDUTypeField.MINEFIELD:   
             if(DEBUG) System.out.println("got a Minefield PDU");
             newPdu = new MinefieldPdu();
             //newPdu.extract(aNode);
             break;

        // Minefield Request PDU
        case PDUTypeField.MINEFIELDREQUEST:   
             if(DEBUG) System.out.println("got a Minefield Request PDU");
             //newPdu = new MineFieldRequestPdu();
             newPdu = new MineFieldRequestRevPdu();
             //newPdu.extract(aNode);
             break;

        // Minefield Response PDU
        case PDUTypeField.MINEFIELDRESPONSE:   
             if(DEBUG) System.out.println("got a Minefield Response PDU");
             //newPdu = new MineFieldResponsePdu();
             newPdu = new MineFieldResponseRevPdu();
             //newPdu.extract(aNode);
             break;

        // Minefield State PDU
        case PDUTypeField.MINEFIELDSTATE:   
             if(DEBUG) System.out.println("got a Minefield State PDU");
             newPdu = new MineFieldStatePdu();
             //newPdu.extract(aNode);
             break;

        // Minefield NACK PDU
        case PDUTypeField.MINEFIELDNACK:   
             if(DEBUG) System.out.println("got a Minefield NACK PDU");
             newPdu = new MineFieldNACKPdu();
             //newPdu.extract(aNode);
             break;

        // Transmitter State PDU
        case PDUTypeField.TRANSMITTER:   
             if(DEBUG) System.out.println("got a Transmitter PDU");
             newPdu = new TransmitterPdu();
             //newPdu.extract(aNode);
             break;

        /*
        // SimulationManagement PDU
        case PDUTypeField.SETDATA:   
                if(DEBUG)
            System.out.println("got a SimulationManagement PDU");
            newPdu = new SimulationManagementPdu();
            ((SimulationManagementPdu)newPdu).deSerialize(new DataInputStream(pStream));
            break;
        */

        //case PDUTypeField.COLLISION:           return;         // do collision pdu here
    

        default:
            {
              newPdu = null;     // just to make sure
              // unrecognized PDU type. 
              System.out.print("Unrecognized PDU type received in packet:" );
              System.out.println(pduType.shortValue());
            }

      }// end of switch()

      if(newPdu!=null) 
      { 
         newPdu.extract(aNode);
         PDUTypeField aTypeField = new PDUTypeField();
         System.out.println(" PduType : "+aTypeField.toString(pduType.shortValue()));
      }// end of if 

      return newPdu;
 
     }// end of getPDU()

}// end of Class PDUReaderWriter
