package dishelper;

/**
* CDRCoderEncoder this utility class should be used to put the attributes
* values to the wire and read them from the wire. This coder convers the
* basic data types in BIG_ENDIAN format and supports the following 
* standards:
*
* <table>
* <tr>
* <td> char </td> <td> 8 bits </td>
* </tr>
* <tr>
* <td> byte </td> <td> 8 bits </td>
* </tr>
* <tr>
* <td> short </td> <td> 16 bits </td>
* </tr>
* <tr>
* <td> int </td> <td> 32 bits </td>
* </tr>
* <tr>
* <td> float </td> <td> 32 bits </td>
* </tr>
* <tr>
* <td> double </td> <td> 64 bits </td>
* </tr>
* </table>
*
* @author H. Timucin Ozdemir
* @date  July 22, 1998 ( Upadted on July 22, 1998)
*
*/
public class CDRCoderEncoder
{

   /**
   */
   public static final byte[]
   toByte(char value)
   {
      byte myBytes[] = new byte[1];
      myBytes[0] = (byte) value;

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte[]
   toByte(byte value)
   {
      byte myBytes[] = new byte[1];
      myBytes[0] = value;

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte[]
   toByte(short value)
   {
      byte myBytes[] = new byte[2];
      int val    = value & 0x0000ffff; 
      myBytes[0] = (byte) ( ( val >> 8 ) & 0x000000ff);
      myBytes[1] = (byte) ( ( val      ) & 0x000000ff);

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte[]
   toByte(int val)
   {
      byte myBytes[] = new byte[4];
      myBytes[0] = (byte) ( ( val >> 24 ) & 0x000000ff);
      myBytes[1] = (byte) ( ( val >> 16 ) & 0x000000ff);
      myBytes[2] = (byte) ( ( val >>  8 ) & 0x000000ff);
      myBytes[3] = (byte) ( ( val       ) & 0x000000ff);

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte[]
   toByte(float value)
   {
      int val = Float.floatToIntBits(value);
      byte myBytes[] = new byte[4];
      myBytes[0] = (byte) ( ( val >> 24 ) & 0x000000ff);
      myBytes[1] = (byte) ( ( val >> 16 ) & 0x000000ff);
      myBytes[2] = (byte) ( ( val >>  8 ) & 0x000000ff);
      myBytes[3] = (byte) ( ( val       ) & 0x000000ff);

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte[]
   toByte(double value)
   {
      long val = Double.doubleToLongBits(value);
      byte myBytes[] = new byte[8];
      int val1 = (int) (val&0x00000000ffffffff);
      int val2 = (int) (val>>>32);

      myBytes[7]  = (byte) (       val1 & 0x000000ff);
      myBytes[6]  = (byte) ((val1>>> 8) & 0x000000ff );
      myBytes[5]  = (byte) ((val1>>>16) & 0x000000ff );
      myBytes[4]  = (byte) ((val1>>>24) & 0x000000ff );

      myBytes[3]  = (byte) (       val2 & 0x000000ff );
      myBytes[2]  = (byte) ((val2>>> 8) & 0x000000ff );
      myBytes[1]  = (byte) ((val2>>>16) & 0x000000ff );
      myBytes[0]  = (byte) ((val2>>>24) & 0x000000ff );

      return myBytes;
     
   }// end of toByte()

   /**
   */
   public static final byte
   extractByte(byte[] myBytes)
   {
      byte ret = myBytes[0];

      return ret;

   }// end of extractByte()

   /**
   */
   public static final char
   extractChar(byte[] myBytes)
   {
      return (char) extractByte(myBytes);

   }// end of extractChar()

   /**
   */
   public static final short
   extractShort(byte[] myBytes)
   {
      int ret = 0;
 
      int ai[] = new int[2];
  
      for(int i=0;i<2;i++)
      {
          if(myBytes[i]<0) { ai[i] = (myBytes[i]&0x7f); ai[i]+=128; }
          else ai[i] = myBytes[i];

      }// end of for

      for(int i=0;i<2;i++) ret = (ret<<8) | ai[i];

      return (short) ret;

   }// end of extractShort()

   /**
   */
   public static final int
   extractInt(byte[] myBytes)
   {
      int ret = 0;
 
      int ai[] = new int[4];
  
      for(int i=0;i<4;i++)
      {
          if(myBytes[i]<0) { ai[i] = (myBytes[i]&0x7f); ai[i]+=128; }
          else ai[i] = myBytes[i];

      }// end of for

      for(int i=0;i<4;i++) ret = (ret<<8) | ai[i];

      return ret;

   }// end of extractInt()

   /**
   */
   public static final float
   extractFloat(byte[] myBytes)
   {
      float ret = 0f;
      int vals  = 0;
      int valr  = 0;

      for(int i=0;i<4&&i<myBytes.length;i++)
      {
         vals = (int) (myBytes[i]&0x7f);
         if((int)myBytes[i]<0) vals |= 0x80;
         valr = (valr<<8) | vals;

      }// end of for

      ret = Float.intBitsToFloat(valr);

      return ret;

   }// end of extractFloat()

   /**
   */
   public static final double
   extractDouble(byte[] myBytes)
   {
      double ret = 0f;
      long vals  = 0;
      long valr  = 0;

      for(int i=0;i<8&&i<myBytes.length;i++)
      {
         vals = (long) (myBytes[i]&0x7f);
         if((int)myBytes[i]<0) vals |= 0x80;
         valr = (valr<<8) | vals;

      }// end of for

      ret = Double.longBitsToDouble(valr);

      return ret;

   }// end of extractDouble()

}; // end of CDRCoderEncoder

