package iwt.dis;                                   // package to which we belong

import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS
import mil.navy.nps.dis.*;

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

public class BeamAntennaPattern extends PduElement 
{
    
    protected EulerAngle			beamDirection;             // ID of entity that's doing the shooting

    
    protected float					azimuthBeamwidth;

 
	protected float					elevationBeamwidth;

	protected UnsignedByte			referenceSystem;	

	protected float					Ez;

	protected float					Ex;


	protected float					phase;

     public static final int sizeOf = 32;  
 
public BeamAntennaPattern()
{
    beamDirection  = new EulerAngle();
    azimuthBeamwidth  =  0;
	elevationBeamwidth   =0;
	referenceSystem = new UnsignedByte(0);
    Ez     =0;      
	
	Ex     =0;
    
	phase=0 ;
    
   
    
	 return;
}

public Object clone()
{
    /** make a copy of the object. This requires a deep copy, so we don't have two
        objects sharing pointers to the same data.
    */

 BeamAntennaPattern  newBeamAntennaPattern = (BeamAntennaPattern)super.clone();
 

 newBeamAntennaPattern.setBeamDirection(this.getBeamDirection());
 
 newBeamAntennaPattern.setazimuthBeamwidth(this.getazimuthBeamwidth());
 
 newBeamAntennaPattern.setelevationBeamwidth(this.getelevationBeamwidth());
 
 newBeamAntennaPattern.setreferenceSystem(this.getreferenceSystem());

 newBeamAntennaPattern.setEz(this.getEz());

 newBeamAntennaPattern.setEx(this.getEx());

 newBeamAntennaPattern.setphase(this.getphase());

 
 return newBeamAntennaPattern;
}

public void serialize(DataOutputStream outputStream)
{
    UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte	paddingAdditional = new UnsignedByte();

    try
	{

    beamDirection.serialize(outputStream);

    outputStream.writeFloat(azimuthBeamwidth);
	
	outputStream.writeFloat(elevationBeamwidth);

	referenceSystem.serialize(outputStream);
	
	padding.serialize(outputStream);                // aligns us w/ byte boundary

	paddingAdditional.serialize(outputStream);

    outputStream.writeFloat(Ez);
	
	outputStream.writeFloat(Ex);

    outputStream.writeFloat(phase);
	
    

    
    
}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in BeamAntennaPattern. Error writing to wire.");
    }
}

public void  deSerialize(DataInputStream inputStream)
{
 /**
    deserialize our data from the stream. We first call the superclass,
    so that all the data there is read into the buffer first.
    Then we deserialize each of our ivars to the stream, being careful
    about the order.
   */

    int             idx = 0;
    UnsignedShort   padding = new UnsignedShort(0);
	UnsignedByte	paddingAdditional = new UnsignedByte();

    
    // Do our ivars
	try
	{

    
    beamDirection.deSerialize(inputStream);
	
	azimuthBeamwidth=inputStream.readFloat();
	
	elevationBeamwidth =inputStream.readFloat();
	
	referenceSystem.deSerialize(inputStream);
	
	padding.deSerialize(inputStream);

	paddingAdditional.deSerialize(inputStream);
	
	Ez= inputStream.readFloat();
	
	Ex= inputStream.readFloat();

    phase = inputStream.readFloat();
    }
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in BeamAntennaPattern. Error writing to wire.");
    }
}

public int length()
{
        return sizeOf ;
}



public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Designator PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    beamDirection.printValues(indentLevel, printStream);
   
	printStream.println(indent + "Azimuth Beamwidth: " + azimuthBeamwidth);
    
    printStream.println(indent + "Elevation Beamwidth: " + elevationBeamwidth);

	printStream.println(indent + "referenceSystem: "    + referenceSystem.intValue());
    
	printStream.println(indent + "Ez Field: " + Ez);
    printStream.println(indent + "Ex Field: " + Ex);
    printStream.println(indent + "Phase Field: " + phase);
    

    return;
}

    //Accessor methods

public EulerAngle getBeamDirection()
{   return (EulerAngle)beamDirection.clone();
}
public void setBeamDirection(EulerAngle pbeamDirection)
{beamDirection = pbeamDirection;
}

public float getBeamDirectionPsi()
{ return beamDirection.getPsi();       
}
public void setBeamDirectionPsi(float pPsi)
{ beamDirection.setPsi(pPsi);}

public float getBeamDirectionTheta()
{ return beamDirection.getTheta();     
}
public void setBeamDirectionTheta(float pTheta)
{ beamDirection.setTheta(pTheta);}

public float getBeamDirectionPhi()
{ return beamDirection.getPhi();       
}
public void setBeamDirectionPhi(float pPhi)
{ beamDirection.setPhi(pPhi);}


public float getazimuthBeamwidth()
{   return (float)azimuthBeamwidth;
}
public void setazimuthBeamwidth(float pazimuthBeamwidth)
{azimuthBeamwidth = pazimuthBeamwidth;
}


public float getelevationBeamwidth()
{   return (float)elevationBeamwidth;
}
public void setelevationBeamwidth(float pelevationBeamwidth)
{elevationBeamwidth = pelevationBeamwidth;
}

public UnsignedByte getreferenceSystem()
{   return (UnsignedByte)referenceSystem.clone();
}
public void setreferenceSystem(UnsignedByte preferenceSystem)
{referenceSystem= preferenceSystem;
}



public float getEz()
{   return Ez;
}
public void setEz(float pEz)
{Ez = pEz;
}

public float getphase()
{   return phase;
}
public void setphase(float pphase)
{pphase = pphase;
}
public float getEx()
{   return Ex;
}
public void setEx(float pEx)
{Ex = pEx;
}

} 