package iwt.dis;                                   // package to which we belong

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*; 
                                          // input/output for serialization
import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;

public class DesignatorPdu extends ProtocolDataUnit 
{
    
    protected EntityID				designatingEntityID;             // ID of entity that's doing the shooting

    
    protected UnsignedShort			codeName;

	protected EntityID				designatedEntityID;             // ID of entity that is undergoing darwinian selection
 
	protected UnsignedShort			designatorCode;

	protected float					designatorPower;

	protected float					designatorWavelength;

	protected EntityCoordinate		designatorSpotwrtDesignatedEntity;
    
	
	protected WorldCoordinate		designatorSpotLocation;      
     
	protected UnsignedByte			deadReckoningAlgorithm;	
	protected LinearAcceleration      entityLinearAcceleration;                   // Acceleration of object (x, y, z in floats)
  

    
 
public DesignatorPdu()
{
    super.setPduType(PDUTypeField.DESIGNATOR);

    designatingEntityID  = new EntityID();
    codeName  =  new UnsignedShort();
	designatedEntityID  = new EntityID();
    designatorCode         = new UnsignedShort();

    designatorPower     =0;         // special nerf ammo by default--hits at zero speed.
                                                            // let's play nice!
	designatorWavelength     =0;
    
	designatorSpotwrtDesignatedEntity = new EntityCoordinate(0,0,0);
    
	designatorSpotLocation        = new WorldCoordinate(0, 0, 0);

    
    deadReckoningAlgorithm = new UnsignedByte(0);
    
	entityLinearAcceleration = new LinearAcceleration(0,0,0);
    return;
}

public Object clone()
{
    /** make a copy of the object. This requires a deep copy, so we don't have two
        objects sharing pointers to the same data.
    */

 DesignatorPdu  newDesignatorPdu = (DesignatorPdu)super.clone();
 

 newDesignatorPdu.setdesignatingEntityID(this.getdesignatingEntityID());
 
 newDesignatorPdu.setcodeName(this.getcodeName());
 
 newDesignatorPdu.setdesignatedEntityID(this.getdesignatedEntityID());
 
 newDesignatorPdu.setdesignatorCode(this.getdesignatorCode());

 newDesignatorPdu.setdesignatorPower(this.getdesignatorPower());

 newDesignatorPdu.setdesignatorWavelength(this.getdesignatorWavelength());

 newDesignatorPdu.setdeadReckoningAlgorithm(this.getdeadReckoningAlgorithm());
 
 newDesignatorPdu.setEntityLinearAcceleration(this.getEntityLinearAcceleration());


 return newDesignatorPdu;
}

public void serialize(DataOutputStream outputStream)
{
    UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte	paddingAdditional = new UnsignedByte();

    super.serialize(outputStream);      // write out header info
    try
	{

    designatingEntityID.serialize(outputStream);

    codeName.serialize(outputStream);

	designatedEntityID.serialize(outputStream);
    
    designatorCode.serialize(outputStream);

    outputStream.writeFloat(designatorPower);
	
	outputStream.writeFloat(designatorWavelength);

    designatorSpotwrtDesignatedEntity.serialize(outputStream);

    designatorSpotLocation.serialize(outputStream);

    deadReckoningAlgorithm.serialize(outputStream);

    padding.serialize(outputStream);                // aligns us w/ byte boundary

	paddingAdditional.serialize(outputStream);

    entityLinearAcceleration.serialize(outputStream);
}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in DesignatorPdu. Error writing to wire.");
    }
}

public void  deSerialize(DataInputStream inputStream)
{
 /**
    deserialize our data from the stream. We first call the superclass,
    so that all the data there is read into the buffer first.
    Then we deserialize each of our ivars to the stream, being careful
    about the order.
   */

    int             idx = 0;
    UnsignedShort   padding = new UnsignedShort(0);
	UnsignedByte	paddingAdditional = new UnsignedByte();

    super.deSerialize(inputStream);

    // Do our ivars
	try
	{

    
    designatingEntityID.deSerialize(inputStream);

    codeName.deSerialize(inputStream);

	designatedEntityID.deSerialize(inputStream);
    
    designatorCode.deSerialize(inputStream);

    designatorPower= inputStream.readFloat();
	
	designatorWavelength= inputStream.readFloat();

    designatorSpotwrtDesignatedEntity.deSerialize(inputStream);
    
	designatorSpotLocation.deSerialize(inputStream);
    
	deadReckoningAlgorithm.deSerialize(inputStream);

    padding.deSerialize(inputStream);

	paddingAdditional.deSerialize(inputStream);

    entityLinearAcceleration.deSerialize(inputStream);

    }
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in DesignatorPdu. Error writing to wire.");
    }
}

public int length()
{
        return sizeOf ;
}

public String PDUName()
{
  return new String("Designator PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Designator PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    designatingEntityID.printValues(indentLevel, printStream);
   printStream.println(indent + "Code Name: " + codeName);
  designatedEntityID.printValues(indentLevel, printStream);
    
    printStream.println(indent + "Designator Code: " + designatorCode);

	
	printStream.println(indent + "Designator Power: " + designatorPower);
    printStream.println(indent + "Designator Wavelength: " + designatorWavelength);
    designatorSpotwrtDesignatedEntity.printValues(indentLevel, printStream);
    
	designatorSpotLocation.printValues(indentLevel, printStream);
    printStream.println(indent + "deadReckoningAlgorithm: "    + deadReckoningAlgorithm.intValue());
    
    entityLinearAcceleration.printValues(indentLevel, printStream);
   

    return;
}

    //Accessor methods

public EntityID getdesignatingEntityID()
{   return (EntityID)designatingEntityID.clone();
}
public void setdesignatingEntityID(EntityID pdesignatingEntityID)
{designatingEntityID = pdesignatingEntityID;
}

public UnsignedShort getcodeName()
{   return (UnsignedShort)codeName.clone();
}
public void setcodeName(UnsignedShort pcodeName)
{codeName = pcodeName;
}

public EntityID getdesignatedEntityID()
{   return (EntityID)designatedEntityID.clone();
}
public void setdesignatedEntityID(EntityID pdesignatedEntityID)
{designatedEntityID = pdesignatedEntityID;
}



public UnsignedShort getdesignatorCode()
{   return (UnsignedShort)designatorCode.clone();
}
public void setdesignatorCode(UnsignedShort pdesignatorCode)
{designatorCode = pdesignatorCode;
}


public float getdesignatorPower()
{   return designatorPower;
}
public void setdesignatorPower(float pdesignatorPower)
{designatorPower = pdesignatorPower;
}

public float getdesignatorWavelength()
{   return designatorWavelength;
}
public void setdesignatorWavelength(float pdesignatorWavelength)
{designatorWavelength = pdesignatorWavelength;
}

public EntityCoordinate getdesignatorSpotwrtDesignatedEntity()
{   return (EntityCoordinate)designatorSpotwrtDesignatedEntity.clone();
}
public void setdesignatorSpotwrtDesignatedEntity(EntityCoordinate pdesignatorSpotwrtDesignatedEntity)
{designatorSpotwrtDesignatedEntity = pdesignatorSpotwrtDesignatedEntity;
}
public void setdesignatorSpotwrtDesignatedEntity(float x, float y, float z)
{
    designatorSpotwrtDesignatedEntity = new EntityCoordinate(x, y, z);
}

public WorldCoordinate getdesignatorSpotLocation()
{   return (WorldCoordinate)designatorSpotLocation.clone();
}
public void setdesignatorSpotLocation(WorldCoordinate pLocation)
{designatorSpotLocation = pLocation;
}
public void setdesignatorSpotLocation(double x, double y, double z)
{
    designatorSpotLocation = new WorldCoordinate(x, y, z);
}


public UnsignedByte getdeadReckoningAlgorithm()
{   return (UnsignedByte)deadReckoningAlgorithm.clone();
}
public void setdeadReckoningAlgorithm(UnsignedByte pdeadReckoningAlgorithm)
{deadReckoningAlgorithm = pdeadReckoningAlgorithm;
}
public void setdeadReckoningAlgorithm(int pdeadReckoningAlgorithm)
{ deadReckoningAlgorithm = new UnsignedByte(pdeadReckoningAlgorithm);
}


public LinearAcceleration getEntityLinearAcceleration()
{ return (LinearAcceleration)entityLinearAcceleration.clone();
}


public void setEntityLinearAcceleration(LinearAcceleration pAcceleration)
{ entityLinearAcceleration = pAcceleration;
}

public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()

 /**
   *@exception RuntimeException when io error occurs.
   *fileEncoding ensures we convert string to local system encoding properly.
   */
   public void  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {
  /*   int             idx = 0;
     byte[]          entityMarkingBytes = new byte[11];
     UnsignedByte    articulationParameterCount = new UnsignedByte();
     String          fileEncoding;
     Element         ele;
     NodeList        nodes;

     readHeader(inDoc);

     // Do our ivars

     try     // catch-all for any exceptions in reading from stream
     {
        entityID.read(inDoc);
        
        nodes = inDoc.getElementsByTagName(DIS_TAGS.CodeName);

        if( nodes.getLength()>0 )
        {
          Node anode = nodes.item(0);
          if( anode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
          {
             String val1 = anode.getNodeValue();
             try
             {
               codeName = new UnsignedShort(Integer.parseInt(val1));

             } catch(NumberFormatException nfe) {}
          }
        } else throw new InvalidDocumentType();

        entityID.read(inDoc);

        nodes = inDoc.getElementsByTagName(DIS_TAGS.DesignatorCode);

        if( nodes.getLength()>0 )
        {
          Node anode = nodes.item(0);
          if( anode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
          {
             String val1 = anode.getNodeValue();
             try
             {
               designatorCode = new UnsignedShort(Integer.parseInt(val1));
             } catch(NumberFormatException nfe) {}
          }
        } else throw new InvalidDocumentType();

        //designatorPower.read(inDoc);

        nodes = inDoc.getElementsByTagName(DIS_TAGS.DesignatorPower);

        if( nodes.getLength()>0 )
        {
          Node anode = nodes.item(0);
          if( anode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
          {
             String val1 = anode.getNodeValue();
             try
             {
               designatorPower = (float)(Integer.parseInt(val1));
             } catch(NumberFormatException nfe) {}
          }
        } else throw new InvalidDocumentType();
		
		nodes = inDoc.getElementsByTagName(DIS_TAGS.DesignatorWavelength);

        if( nodes.getLength()>0 )
        {
           Node anode = nodes.item(0);

           if( anode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
           {
              String val1 = anode.getNodeValue();
              try
              {
                designatorWavelength = new float(Integer.parseInt(val1));
              } catch(NumberFormatException nfe) {}

           } else throw new InvalidDocumentType();

        } else throw new InvalidDocumentType();

		// the entity as friendly forces see it
        designatorSpotwrtDesignatedEntity.read(inDoc);

        // entity as other forces see it
        designatorSpotLocation.read(inDoc);
       

        

       	nodes = inDoc.getElementsByTagName(DIS_TAGS.DeadReckoningAlgorithm);

        if( nodes.getLength()>0 )
        {
           Node anode = nodes.item(0);

           if( anode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
           {
              String val1 = anode.getNodeValue();
              try
              {
                deadReckoningAlgorithm = new UnsignedByte(Integer.parseInt(val1));
              } catch(NumberFormatException nfe) {}

           } else throw new InvalidDocumentType();

        } else throw new InvalidDocumentType();
      
		entityLinearAcceleration.read(inDoc);

                
    }
    catch(IOException ioException)      // catch-all for all IO exceptions in the above code
     {
        throw new 
            RuntimeException("Exception in DesignatorPdu. Error deSerializing unit.");
     }
*/
 }  // end of read

public void
  insert(org.w3c.dom.Node outNode)
  {
	
    int             idx;

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);
    
    ele = outDoc.createElement(iwt.DIS_TAGS.DesignatingEntityID);
    ele1.appendChild(ele);
    designatingEntityID.insert(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.CodeName);
	ele.setAttribute(iwt.DIS_TAGS.Value,codeName.toString());
	ele1.appendChild(ele);

    ele = outDoc.createElement(iwt.DIS_TAGS.DesignatedEntityID);
    ele1.appendChild(ele);
    designatedEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.DesignatorCode);
	ele.setAttribute(iwt.DIS_TAGS.Value,designatorCode.toString());
    ele1.appendChild(ele);
    
	ele = outDoc.createElement(iwt.DIS_TAGS.DesignatorPower);
	ele.setAttribute(iwt.DIS_TAGS.Value,Float.toString(designatorPower));
    ele1.appendChild(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.DesignatorWavelength);
	ele.setAttribute(iwt.DIS_TAGS.Value,Float.toString(designatorWavelength));
    ele1.appendChild(ele);
    
    
	designatorSpotwrtDesignatedEntity.insert(ele);

    designatorSpotLocation.insert(ele1);

 ele = outDoc.createElement(iwt.DIS_TAGS.DeadReckoningAlgorithm);
    ele.setAttribute(iwt.DIS_TAGS.Value,deadReckoningAlgorithm.toString());
    ele1.appendChild(ele);

    
    entityLinearAcceleration.insert(ele1);

    
  }// end of insert()

public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    int             idx = 0;
    byte[]          entityMarkingBytes = new byte[11];
    UnsignedByte    articulationParameterCount = new UnsignedByte();
    String          fileEncoding;
    Element         ele;
    NodeList        nodes;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.CodeName) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    codeName = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DesignatingEntityID) )
                   designatingEntityID.extract(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DesignatedEntityID) )
                   designatedEntityID.extract(aNode);

		  
		  
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DesignatorCode) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    designatorCode = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DesignatorPower) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    designatorPower = (float)(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DesignatorWavelength) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                   designatorWavelength = (float)(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

			else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityCoordinate) )
                     designatorSpotwrtDesignatedEntity.extract(aNode);

           else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.WorldCoordinate) )
                   designatorSpotLocation.extract(aNode);

         else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.LinearAcceleration) )
                   entityLinearAcceleration.extract(aNode);
 
           }// end of if

       i++;

    }// end of while() 

 }  // 

} 