package iwt.dis;

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.util.*;                           
import java.io.*; 

import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;

public class ReceiverPdu extends ProtocolDataUnit
{

protected EntityID          EID;

protected UnsignedInt RadioID;

protected UnsignedInt ReceiverState;

protected float ReceivedPower;

protected EntityID TransmitterEntityID;

protected UnsignedInt TransmitterRadioID;
 

public ReceiverPdu()
{
super.setPduType(PDUTypeField.RECEIVER);

EID = new EntityID();
RadioID =  new UnsignedInt();
ReceiverState = new UnsignedInt();
ReceivedPower   = 0;
TransmitterEntityID = new EntityID();
TransmitterRadioID = new UnsignedInt();

}

public Object clone()
{

	ReceiverPdu newReceiverPdu = (ReceiverPdu)super.clone();

	newReceiverPdu.setEID(this.getEID());
	newReceiverPdu.setRadioID(this.getRadioID());
	newReceiverPdu.setReceiverState(this.getReceiverState());
	newReceiverPdu.setReceivedPower(this.getReceivedPower());
	newReceiverPdu.setTransmitterEntityID(this.getTransmitterEntityID());
	newReceiverPdu.setTransmitterRadioID(this.getTransmitterRadioID());
	return newReceiverPdu;
}


public void serialize(DataOutputStream outputStream)
{	
	UnsignedShort   padding = new UnsignedShort(0);
	super.serialize(outputStream);
	try
	{
		EID.serialize(outputStream);
		RadioID.serialize(outputStream);
		ReceiverState.serialize(outputStream);
		padding.serialize(outputStream);                // aligns us w/ byte boundary
		outputStream.writeFloat(ReceivedPower);
		TransmitterEntityID.serialize(outputStream);
		TransmitterRadioID.serialize(outputStream);
	
	
	}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in ReceiverPdu. Error writing to wire.");
    }
}

public void deSerialize(DataInputStream inputStream)
{
	UnsignedShort   padding = new UnsignedShort(0);

	
	super.deSerialize(inputStream);
	try
	{
		EID.deSerialize(inputStream);
		RadioID.deSerialize(inputStream);
		ReceiverState.deSerialize(inputStream);
		padding.deSerialize(inputStream);
		ReceivedPower = inputStream.readFloat();
		TransmitterEntityID.deSerialize(inputStream);
		TransmitterRadioID.deSerialize(inputStream);
	}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in ReceiverPdu. Error writing to wire.");
    }
		

}


public int length()
{
   
    return sizeOf;
}


public String PDUName()
{
  return new String("Receiver PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Receiver PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    EID.printValues(indentLevel, printStream);
    printStream.println(indent + "radio ID: "+ RadioID.longValue());
    printStream.println(indent + "ReceiverState: "    + ReceiverState.longValue());
    printStream.println(indent + "ReceivedPower: " + ReceivedPower);
	TransmitterEntityID.printValues(indentLevel, printStream);
    printStream.println(indent + "TransmitterRadioID: "+ TransmitterRadioID.longValue());

    return;
}

    //Accessor methods

public EntityID getEID()
{   return (EntityID)EID.clone();
}
public void setEID(EntityID pEID)
{EID = pEID;
}

public UnsignedInt getRadioID()
{   return (UnsignedInt)RadioID.clone();
}


public void setRadioID(UnsignedInt pRadioID)
{ RadioID = pRadioID;
}

public UnsignedInt getReceiverState()
{   return (UnsignedInt)ReceiverState.clone();
}
public void setReceiverState(UnsignedInt pReceiverState)
{ReceiverState = pReceiverState;
}

public float getReceivedPower()
{   return ReceivedPower;
}
	
public void setReceivedPower(float pReceivedPower)
{ReceivedPower = pReceivedPower;
}


public EntityID getTransmitterEntityID()
{   return (EntityID)TransmitterEntityID.clone();
}
public void setTransmitterEntityID(EntityID pTransmitterEntityID)
{TransmitterEntityID = pTransmitterEntityID;
}
public UnsignedInt getTransmitterRadioID()
{   return (UnsignedInt)TransmitterRadioID.clone();
}
public void setTransmitterRadioID(UnsignedInt pTransmitterRadioID)
{TransmitterRadioID = pTransmitterRadioID;
}

public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()
 /**
   *@exception RuntimeException when io error occurs.
   *fileEncoding ensures we convert string to local system encoding properly.
   */
   public void  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {
   }  // end of read

public void
  insert(org.w3c.dom.Node outNode)
  {
	UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte    parameterCount;
    int             idx;

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);

	    
    ele = outDoc.createElement(iwt.DIS_TAGS.EntityID);
    ele1.appendChild(ele);
    EID.insert(ele);

 ele = outDoc.createElement(iwt.DIS_TAGS.RadioID);
    ele.setAttribute(iwt.DIS_TAGS.Value,RadioID.toString());
    ele1.appendChild(ele);

 ele = outDoc.createElement(iwt.DIS_TAGS.ReceiverState);
    ele.setAttribute(iwt.DIS_TAGS.Value,ReceiverState.toString());
    ele1.appendChild(ele);


	ele = outDoc.createElement(iwt.DIS_TAGS.ReceivedPower);
	ele.setAttribute(iwt.DIS_TAGS.Value,Float.toString(ReceivedPower));
    ele1.appendChild(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.TransmitterEntityID);
    ele1.appendChild(ele);
    TransmitterEntityID.insert(ele);

ele = outDoc.createElement(iwt.DIS_TAGS.TransmitterRadioID);
    ele.setAttribute(iwt.DIS_TAGS.Value,TransmitterRadioID.toString());
    ele1.appendChild(ele);
    
  }// end of insert()

public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    int             idx = 0;
    byte[]          entityMarkingBytes = new byte[11];
    UnsignedByte    articulationParameterCount = new UnsignedByte();
    String          fileEncoding;
    Element         ele;
    NodeList        nodes;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);
				
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.EntityID) )
                   EID.extract(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RadioID) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    RadioID = new UnsignedInt(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

         else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ReceiverState) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    ReceiverState = new UnsignedInt(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ReceivedPower) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    ReceivedPower = (float)(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

 else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TransmitterEntityID) )
                   TransmitterEntityID.extract(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TransmitterRadioID) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                   TransmitterRadioID = new UnsignedInt(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

			
           }// end of if

       i++;

    }// end of while() 

 }  // 




}          