package iwt.dis;

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.util.*;                           
import java.io.*; 

import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;

public class RepairCompletePdu extends ProtocolDataUnit
{

protected EntityID          receivingEntityID;

protected EntityID          repairingEntityID;

protected UnsignedShort		repair;



public RepairCompletePdu()
{
super.setPduType(PDUTypeField.REPAIRCOMPLETE);

receivingEntityID = new EntityID();
repairingEntityID = new EntityID();
repair = new UnsignedShort();

}

public Object clone()
{

	RepairCompletePdu newRepairCompletePdu = (RepairCompletePdu)super.clone();

	newRepairCompletePdu.setReceivingEntityID(this.getReceivingEntityID());
	newRepairCompletePdu.setRepairingEntityID(this.getRepairingEntityID());
	newRepairCompletePdu.setRepair(this.getRepair());
	return newRepairCompletePdu;
}


public void serialize(DataOutputStream outputStream)
{
	UnsignedShort   padding = new UnsignedShort(0);
	super.serialize(outputStream);

   
		receivingEntityID.serialize(outputStream);
		repairingEntityID.serialize(outputStream);
		repair.serialize(outputStream);
		padding.serialize(outputStream); 
}

public void deSerialize(DataInputStream inputStream)
{
	UnsignedShort   padding = new UnsignedShort(0);
		super.deSerialize(inputStream);
		receivingEntityID.deSerialize(inputStream);
		repairingEntityID.deSerialize(inputStream);
		repair.deSerialize(inputStream);
		padding.deSerialize(inputStream);
	

}


public int length()
{
   
    return sizeOf;
}


public String PDUName()
{
  return new String("RepairComplete PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("RepairComplete PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    receivingEntityID.printValues(indentLevel, printStream);
    repairingEntityID.printValues(indentLevel, printStream);
    printStream.println(indent + "Repair: "+ repair.intValue());

    return;
}

    //Accessor methods

public EntityID getReceivingEntityID()
{   return (EntityID)receivingEntityID.clone();
}
public void setReceivingEntityID(EntityID pReceivingEntityID)
{receivingEntityID = pReceivingEntityID;
}



public EntityID getRepairingEntityID()
{   return (EntityID)repairingEntityID.clone();
}
public void setRepairingEntityID(EntityID pRepairingEntityID)
{repairingEntityID = pRepairingEntityID;
}
public UnsignedShort getRepair()
{   return (UnsignedShort)repair.clone();
}
public void setRepair(UnsignedShort pRepair)
{repair = pRepair;
}

public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()
   public void  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {
   }  // end of read
public void
  insert(org.w3c.dom.Node outNode)
  {
	Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);
  
    ele = outDoc.createElement(iwt.DIS_TAGS.ReceivingEntityID);
    ele1.appendChild(ele);
    receivingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.RepairingEntityID);
    ele1.appendChild(ele);
    repairingEntityID.insert(ele);


	ele = outDoc.createElement(iwt.DIS_TAGS.Repair);
	ele.setAttribute(iwt.DIS_TAGS.Value,repair.toString());
    ele1.appendChild(ele);

	
    
  }// end of insert()


public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    
    Element         ele;
    NodeList        nodes;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);
				
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ReceivingEntityID) )
                   receivingEntityID.extract(aNode);

		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RepairingEntityID) )
                   repairingEntityID.extract(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.Repair) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    repair = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

           }// end of if

       i++;

    }// end of while() 

 }  // 

}          