package iwt.dis;

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.util.*;                           
import java.io.*; 

import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;


public class RepairResponsePdu extends ProtocolDataUnit
{

protected EntityID          receivingEntityID;

protected EntityID          repairingEntityID;

protected UnsignedByte repairResult;



public RepairResponsePdu()
{
super.setPduType(PDUTypeField.REPAIRRESPONSE);

receivingEntityID = new EntityID();
repairingEntityID = new EntityID();
repairResult = new UnsignedByte();

}

public Object clone()
{

	RepairResponsePdu newRepairResponsePdu = (RepairResponsePdu)super.clone();

	newRepairResponsePdu.setReceivingEntityID(this.getReceivingEntityID());
	newRepairResponsePdu.setRepairingEntityID(this.getRepairingEntityID());
	newRepairResponsePdu.setRepair(this.getRepair());
	return newRepairResponsePdu;
}


public void serialize(DataOutputStream outputStream)
{	
	UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte	paddingAdditional = new UnsignedByte();

	super.serialize(outputStream);
	
		receivingEntityID.serialize(outputStream);
		repairingEntityID.serialize(outputStream);
		repairResult.serialize(outputStream);
		padding.serialize(outputStream);                // aligns us w/ byte boundary
		paddingAdditional.serialize(outputStream);

}

public void deSerialize(DataInputStream inputStream)
{
	UnsignedShort   padding = new UnsignedShort(0);
	UnsignedByte	paddingAdditional = new UnsignedByte();
	super.deSerialize(inputStream);

		receivingEntityID.deSerialize(inputStream);
		repairingEntityID.deSerialize(inputStream);
		repairResult.deSerialize(inputStream);
        padding.deSerialize(inputStream);
		paddingAdditional.deSerialize(inputStream);


}


public int length()
{
   
    return sizeOf;
}


public String PDUName()
{
  return new String("RepairResponse PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("RepairResponse PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    receivingEntityID.printValues(indentLevel, printStream);
    repairingEntityID.printValues(indentLevel, printStream);
    printStream.println(indent + "RepairResult: "+ repairResult.intValue());

    return;
}

    //Accessor methods

public EntityID getReceivingEntityID()
{   return (EntityID)receivingEntityID.clone();
}
public void setReceivingEntityID(EntityID pReceivingEntityID)
{receivingEntityID = pReceivingEntityID;
}



public EntityID getRepairingEntityID()
{   return (EntityID)repairingEntityID.clone();
}
public void setRepairingEntityID(EntityID pRepairingEntityID)
{repairingEntityID = pRepairingEntityID;
}
public UnsignedByte getRepair()
{   return (UnsignedByte)repairResult.clone();
}
public void setRepair(UnsignedByte pRepairResult)
{repairResult = pRepairResult;
}


public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()

   public void  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {
   }  // end of read
public void

  insert(org.w3c.dom.Node outNode)
  {
	Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);
  
    ele = outDoc.createElement(iwt.DIS_TAGS.ReceivingEntityID);
    ele1.appendChild(ele);
    receivingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.RepairingEntityID);
    ele1.appendChild(ele);
    repairingEntityID.insert(ele);


	ele = outDoc.createElement(iwt.DIS_TAGS.RepairResult);
	ele.setAttribute(iwt.DIS_TAGS.Value,repairResult.toString());
    ele1.appendChild(ele);

	
    
  }// end of insert()


public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    
    Element         ele;
    NodeList        nodes;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);
				
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ReceivingEntityID) )
                   receivingEntityID.extract(aNode);

		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RepairingEntityID) )
                   repairingEntityID.extract(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RepairResult) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    repairResult = new UnsignedByte(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

           }// end of if

       i++;

    }// end of while() 

 }  // 



}          