package iwt.dis;
	
import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS


import java.util.*;                           
import java.io.*; 

import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;

public class ResupplyOfferPdu extends ProtocolDataUnit
{

protected EntityID          receivingEntityID;

protected EntityID          supplyingEntityID;

protected UnsignedByte		noOfSupplyTypes;

protected Vector			supplies;


public ResupplyOfferPdu()
{
super.setPduType(PDUTypeField.RESUPPLYOFFER);

receivingEntityID = new EntityID();
supplyingEntityID = new EntityID();
supplies = new Vector();
}

public Object clone()
{

	ResupplyOfferPdu newResupplyOfferPdu = (ResupplyOfferPdu)super.clone();
	int            idx = 0;
	newResupplyOfferPdu.setReceivingEntityID(this.getReceivingEntityID());
	newResupplyOfferPdu.setSupplyingEntityID(this.getSupplyingEntityID());
		
for(idx = 0; idx < supplies.size(); idx++)
 {
    newResupplyOfferPdu.addSupplies(this.getSupplies(idx));
 }
	return newResupplyOfferPdu;
}


public void serialize(DataOutputStream outputStream)
{	
	int             idx;
	UnsignedByte noOfSupplyTypes;
	UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte	paddingAdditional = new UnsignedByte();


	super.serialize(outputStream);
		

		receivingEntityID.serialize(outputStream);
		supplyingEntityID.serialize(outputStream);
		noOfSupplyTypes= new UnsignedByte(supplies.size());

		noOfSupplyTypes.serialize(outputStream);
		padding.serialize(outputStream);                // aligns us w/ byte boundary
		paddingAdditional.serialize(outputStream);

for(idx = 0; idx < supplies.size(); idx++)
    {
        SupplyQuantity  supply;

        supply= (SupplyQuantity)supplies.elementAt(idx);
        supply.serialize(outputStream);
    }

      


}

public void deSerialize(DataInputStream inputStream)
{
		int idx;
		UnsignedByte		noOfSupplyTypes = new UnsignedByte();
		UnsignedShort   padding = new UnsignedShort(0);
		UnsignedByte	paddingAdditional = new UnsignedByte();

	super.deSerialize(inputStream);
		receivingEntityID.deSerialize(inputStream);
		supplyingEntityID.deSerialize(inputStream);
		noOfSupplyTypes.deSerialize(inputStream);
		padding.deSerialize(inputStream);
		paddingAdditional.deSerialize(inputStream);
		
	for(idx = 0; idx < noOfSupplyTypes.intValue(); idx++)
    {
        SupplyQuantity  supply = new SupplyQuantity();
		supply.deSerialize(inputStream);
        supplies.addElement(supply);
    }

}


public int length()
{
   
    return sizeOf;
}


public String PDUName()
{
  return new String("ResupplyOffer PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("ResupplyOffer PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    receivingEntityID.printValues(indentLevel, printStream);
    supplyingEntityID.printValues(indentLevel, printStream);
    printStream.println(indent + "noOfSupplyTypes: "+ supplies.size());
	for(idx = 0; idx < supplies.size(); idx ++)
     {
        SupplyQuantity   suppl;

        suppl = (SupplyQuantity )supplies.elementAt(idx);
        suppl.printValues(indentLevel + 4, printStream);                // indent the parts of us a bit
     }
    return;
}

    //Accessor methods

public EntityID getReceivingEntityID()
{   return (EntityID)receivingEntityID.clone();
}
public void setReceivingEntityID(EntityID pReceivingEntityID)
{receivingEntityID = pReceivingEntityID;
}



public EntityID getSupplyingEntityID()
{   return (EntityID)supplyingEntityID.clone();
}
public void setSupplyingEntityID(EntityID pSupplyingEntityID)
{supplyingEntityID = pSupplyingEntityID;
}

public SupplyQuantity getSupplies(int pIdx)
{   
	SupplyQuantity supply;
	supply = (SupplyQuantity)supplies.elementAt(pIdx);
	return (SupplyQuantity)supply.clone();
}
public void addSupplies(SupplyQuantity pSupplies)
{supplies.addElement(pSupplies);
}


public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()

   public void  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {
   }  // end of read

public void
  insert(org.w3c.dom.Node outNode)
  {
	Element         ele;
    Element         ele1;
    Text            txt1;
	int				idx;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);
  
    ele = outDoc.createElement(iwt.DIS_TAGS.ReceivingEntityID);
    ele1.appendChild(ele);
    receivingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.SupplyingEntityID);
    ele1.appendChild(ele);
    supplyingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.NoOfSupplyTypes);
	ele.setAttribute(iwt.DIS_TAGS.Value,noOfSupplyTypes.toString());
	ele1.appendChild(ele);
	
	for(idx=0; idx<supplies.size();idx++)
	{
	SupplyQuantity supplyQ;
	supplyQ = (SupplyQuantity)supplies.elementAt(idx);
	supplyQ.insert(ele);
	}

	 
  }// end of insert()


public void  
extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    
    Element         ele;
    NodeList        nodes;
	NodeList        aList;
	int				k;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);
				
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ReceivingEntityID) )
                   receivingEntityID.extract(aNode);

		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.SupplyingEntityID) )
                   supplyingEntityID.extract(aNode);

          
		  else if (aNode.getNodeType() == Node.ELEMENT_NODE &&
					aNode.getNodeName().equals(iwt.DIS_TAGS.NoOfSupplyTypes))
		  {
			  String val1 =((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
			  if (val1==null) throw new InvalidDocumentType();
			  try
			  {
				noOfSupplyTypes = new UnsignedByte(Integer.parseInt(val1));
			  }
			  catch(NumberFormatException nfe)
			  {throw new InvalidDocumentType();}
		  }
		  else if (aNode.getNodeType() == Node.ELEMENT_NODE &&
					aNode.getNodeName().equals(iwt.DIS_TAGS.Supplies))
		  {
			  aList = ((Element)aNode).getElementsByTagName(iwt.DIS_TAGS.SupplyQuantity);
				 
			  supplies = new Vector();
			  
			  if(aList != null)
			  for( k = 0; k < aList.getLength();k++)
			  {
				  Node bNode = nodes.item(k);
				  if(bNode !=null &&
					  bNode.getNodeType() ==Node.ELEMENT_NODE &&
					  bNode.getNodeName().equals(iwt.DIS_TAGS.SupplyQuantity))
				  {
					  SupplyQuantity spq= new SupplyQuantity();
					  spq.extract(bNode);
					  supplies.addElement(spq);
				  }
			  }
		  				
		  }
	   }// end of if

       i++;

    }// end of while() 

 }  // 


}          