package iwt.dis;

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS


import java.util.*;                           
import java.io.*; 

import iwt.*;
import org.w3c.dom.*;
import iwt.domUtil.*;



public class ResupplyReceivedPdu extends ProtocolDataUnit
{

protected EntityID          receivingEntityID;

protected EntityID          supplyingEntityID;

protected UnsignedByte		noOfSupplyTypes;


protected Vector			supplies;


public ResupplyReceivedPdu()
{
super.setPduType(PDUTypeField.RESUPPLYRECEIVED);

receivingEntityID = new EntityID();
supplyingEntityID = new EntityID();
supplies = new Vector();
}

public Object clone()
{

	ResupplyReceivedPdu newResupplyReceivedPdu = (ResupplyReceivedPdu)super.clone();
	int            idx = 0;
	newResupplyReceivedPdu.setReceivingEntityID(this.getReceivingEntityID());
	newResupplyReceivedPdu.setSupplyingEntityID(this.getSupplyingEntityID());
		
for(idx = 0; idx < supplies.size(); idx++)
 {
    newResupplyReceivedPdu.addSupplies(this.getSupplies(idx));
 }
	return newResupplyReceivedPdu;
}


public void serialize(DataOutputStream outputStream)
{	
	int             idx;
	UnsignedByte NoOfSupplyTypes;
	super.serialize(outputStream);
		

		receivingEntityID.serialize(outputStream);
		supplyingEntityID.serialize(outputStream);
		NoOfSupplyTypes= new UnsignedByte(supplies.size());

		NoOfSupplyTypes.serialize(outputStream);
for(idx = 0; idx < supplies.size(); idx++)
    {
        SupplyQuantity  supply;

        supply= (SupplyQuantity)supplies.elementAt(idx);
        supply.serialize(outputStream);
    }

      


}

public void deSerialize(DataInputStream inputStream)
{
		int idx;
		UnsignedByte		NoOfSupplyTypes = new UnsignedByte();

	super.deSerialize(inputStream);
		receivingEntityID.deSerialize(inputStream);
		supplyingEntityID.deSerialize(inputStream);
		NoOfSupplyTypes.deSerialize(inputStream);
	
		
	for(idx = 0; idx < NoOfSupplyTypes.intValue(); idx++)
    {
        SupplyQuantity  supply = new SupplyQuantity();
		supply.deSerialize(inputStream);
        supplies.addElement(supply);
    }

}


public int length()
{
   
    return sizeOf;
}


public String PDUName()
{
  return new String("ResupplyReceived PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("ResupplyReceived PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    receivingEntityID.printValues(indentLevel, printStream);
    supplyingEntityID.printValues(indentLevel, printStream);
    printStream.println(indent + "NoOfSupplyTypes: "+ supplies.size());
	for(idx = 0; idx < supplies.size(); idx ++)
     {
        SupplyQuantity   suppl;

        suppl = (SupplyQuantity )supplies.elementAt(idx);
        suppl.printValues(indentLevel + 4, printStream);                // indent the parts of us a bit
     }
    return;
}

    //Accessor methods

public EntityID getReceivingEntityID()
{   return (EntityID)receivingEntityID.clone();
}
public void setReceivingEntityID(EntityID pReceivingEntityID)
{receivingEntityID = pReceivingEntityID;
}



public EntityID getSupplyingEntityID()
{   return (EntityID)supplyingEntityID.clone();
}
public void setSupplyingEntityID(EntityID pSupplyingEntityID)
{supplyingEntityID = pSupplyingEntityID;
}

public SupplyQuantity getSupplies(int pIdx)
{   
	SupplyQuantity supply;
	supply = (SupplyQuantity)supplies.elementAt(pIdx);
	return (SupplyQuantity)supply.clone();
}
public void addSupplies(SupplyQuantity pSupplies)
{supplies.addElement(pSupplies);
}

public void write(org.w3c.dom.Document outDoc)
{
	insert(outDoc);
}

public void read(org.w3c.dom.Document inDoc)
{
}

public void insert(org.w3c.dom.Node outNode)throws InvalidDocumentType
  
{
	Element ele;
	Element ele1;
	int idx;

	Document outDoc = outNode.getOwnerDocument();
	ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
	outNode.appendChild(ele1);

	insertHeader(ele1);

	ele = outDoc.createElement(iwt.DIS_TAGS.ReceivingEntityID);
	ele1.appendChild(ele);
	receivingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.SupplyingEntityID);
	ele1.appendChild(ele);
	supplyingEntityID.insert(ele);

	ele = outDoc.createElement(iwt.DIS_TAGS.NoOfSupplyTypes);
	ele.setAttribute(iwt.DIS_TAGS.Value,(Integer.toString(supplies.size())));
	ele1.appendChild(ele);
	
	for (idx = 0; idx< supplies.size(); idx++)
	{
		SupplyQuantity spq;
		spq= (SupplyQuantity)supplies.elementAt(idx);
		spq.insert(ele);
	}
}// end of insert


public void extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
	Element         ele;
    NodeList        nodes;
	NodeList        aList;
	int				k;
	
	if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

	nodes = inNode.getChildNodes();
	int i=0;
	while(i<nodes.getLength())
	{
		Node aNode = nodes.item(i);

		if(aNode!=null)
		{
			if(aNode.getNodeType() ==Node.ELEMENT_NODE &&
			   aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header))
				extractHeader(aNode);
			
			else if(aNode.getNodeType()==Node.ELEMENT_NODE &&
				aNode.getNodeName().equals(iwt.DIS_TAGS.ReceivingEntityID))
				receivingEntityID.extract(aNode);

			else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.SupplyingEntityID) )
                   supplyingEntityID.extract(aNode);

			else if(aNode.getNodeType() == Node.ELEMENT_NODE &&
					aNode.getNodeName().equals(iwt.DIS_TAGS.NoOfSupplyTypes))
			{
				String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
				if (val1==null) throw new InvalidDocumentType();
			  try
			  {
				noOfSupplyTypes = new UnsignedByte(Integer.parseInt(val1));
			  }
			  catch(NumberFormatException nfe)
			  {throw new InvalidDocumentType();}
			}
		  else if (aNode.getNodeType() == Node.ELEMENT_NODE &&
					aNode.getNodeName().equals(iwt.DIS_TAGS.Supplies))
		  {
			aList = ((Element)aNode).getElementsByTagName(iwt.DIS_TAGS.SupplyQuantity);
			
			supplies= new Vector();

			if(aList!=null)
			for(k=0;k<aList.getLength();k++)
			{
				Node bNode = nodes.item(k);
				if(bNode !=null &&
					  bNode.getNodeType() ==Node.ELEMENT_NODE &&
					  bNode.getNodeName().equals(iwt.DIS_TAGS.SupplyQuantity))
				  {
					  SupplyQuantity spq= new SupplyQuantity();
					  spq.extract(bNode);
					  supplies.addElement(spq);
				  }
			  }
		  				
		  }
	   }// end of if

       i++;

    }// end of while() 

 }  // 








}          