package iwt.dis;               // our package

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.io.*;
import java.util.*;

import iwt.*;
import org.w3c.dom.*;


import iwt.domUtil.*;

public class SignalPdu extends ProtocolDataUnit
{
    
    protected EntityID			entity_ID;                       // describes country & type of munition

    protected UnsignedShort     radioID;                        // type of warhead

   protected UnsignedShort		encodingScheme;                           // type of fuse

   protected UnsignedShort		tdlType;                       // how much was fired in the burst
 
   protected UnsignedInt		sampleRate;
   
   protected UnsignedShort		dataLength;                           // rate of fire for the burst

   protected UnsignedShort		samples;
    
   public static final int		sizeOf = 34;                    // 16 bytes long when written to wire

/**
 *Default constructor. Initializes everything to zero, basically.
 */
public SignalPdu()
{
    entity_ID = new EntityID();
    radioID  = new UnsignedShort();
    encodingScheme     = new UnsignedShort();
    tdlType = new UnsignedShort();
    sampleRate =  new UnsignedInt();
	dataLength     = new UnsignedShort();
	samples		= new UnsignedShort();
    return;
}

public Object clone()
{
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    SignalPdu newSignalPdu = new SignalPdu();

    newSignalPdu.setEntity_ID(this.getEntity_ID());
    newSignalPdu.setRadioID(this.getRadioID());
    newSignalPdu.setEncodingScheme(this.getEncodingScheme());
    newSignalPdu.setTdlType(this.getTdlType());
	newSignalPdu.setSampleRate(this.getSampleRate());
    newSignalPdu.setDataLength(this.getDataLength());
	newSignalPdu.setSamples(this.getSamples());

    return newSignalPdu;
}

/* serialize the object in DIS format to a stream.
@param outputStream stream the object is being serialized to
*/
public String PDUName()
{
  return new String("Signal PDU");
}
public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    //super.serialize(outputStream);        // No need to call super, since no ivars are in our direct superclass
    
    entity_ID.serialize(outputStream);
    radioID.serialize(outputStream);
    encodingScheme.serialize(outputStream);
    tdlType.serialize(outputStream);
    sampleRate.serialize(outputStream);
	dataLength.serialize(outputStream);
	samples.serialize(outputStream);
    return;
}


public void deSerialize(DataInputStream inputStream)
{
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    // no need to call super, since there are no ivars in the superclass.

    entity_ID.deSerialize(inputStream);
    radioID.deSerialize(inputStream);
    encodingScheme.deSerialize(inputStream);
    tdlType.deSerialize(inputStream);
    sampleRate.deSerialize(inputStream);
	dataLength.deSerialize(inputStream);
	samples.deSerialize(inputStream);
    return;
}

public int length()
{
    return sizeOf;          // SignalPdus are this long, always.
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    entity_ID.printValues(indentLevel, printStream);
    printStream.println(indent + "radioID: " + radioID.intValue());
    printStream.println(indent + "encodingScheme: "    + encodingScheme.intValue());
    printStream.println(indent + "tdlType: "+ tdlType.intValue());
    printStream.println(indent + "Sample Rate: "+ sampleRate.longValue());
    printStream.println(indent + "dataLength: "    + dataLength.intValue());
	printStream.println(indent + "Samples: "    + samples.intValue());

    return;
}

// Accessor methods

public EntityID getEntity_ID()
{ return (EntityID)entity_ID.clone();
}
public void setEntity_ID(EntityID pEntity_ID)
{ entity_ID = pEntity_ID;
}

public UnsignedShort getRadioID()
{   return (UnsignedShort)radioID.clone();
}
public void setRadioID(UnsignedShort pradioID)
{radioID = pradioID;
}
public void setRadioID(int pradioID)
{ radioID = new UnsignedShort(pradioID);
}


public UnsignedShort getEncodingScheme()
{   return (UnsignedShort)encodingScheme.clone();
}
public void setEncodingScheme(UnsignedShort pencodingScheme)
{encodingScheme = pencodingScheme;
}
public void setEncodingScheme(int pencodingScheme)
{encodingScheme = new UnsignedShort(pencodingScheme);
}

public UnsignedShort getTdlType()
{   return (UnsignedShort)tdlType.clone();
}
public void setTdlType(UnsignedShort ptdlType)
{tdlType = ptdlType;
}
public void setTdlType(int ptdlType)
{ tdlType = new UnsignedShort(ptdlType);
}
public UnsignedInt getSampleRate()
{   return (UnsignedInt)sampleRate.clone();
}
public void setSampleRate(UnsignedInt psampleRate)
{sampleRate = sampleRate;
}

public UnsignedShort getDataLength()
{   return (UnsignedShort)dataLength.clone();
}
public void setDataLength(UnsignedShort pdataLength)
{dataLength = pdataLength;
}
public void setDataLength(int pdataLength)
{ dataLength = new UnsignedShort(pdataLength);
}
public UnsignedShort getSamples()
{   return (UnsignedShort)samples.clone();
}
public void setSamples(UnsignedShort psamples)
{samples = psamples;
}

public void setSamples(int psamples)
{ samples = new UnsignedShort(psamples);
}


public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()

public void read(org.w3c.dom.Document inDoc)
{
}// end of read

public void
  insert(org.w3c.dom.Node outNode)
  {
	int             idx;

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Fire serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    // write out header info
    insertHeader(ele1);
	ele = outDoc.createElement(iwt.DIS_TAGS.Entity_ID);
    ele1.appendChild(ele);
    entity_ID.insert(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.RadioID);
	ele.setAttribute(iwt.DIS_TAGS.Value,radioID.toString());
	ele1.appendChild(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.EncodingScheme);
	ele.setAttribute(iwt.DIS_TAGS.Value,encodingScheme.toString());
	ele1.appendChild(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.TdlType);
	ele.setAttribute(iwt.DIS_TAGS.Value,tdlType.toString());
	ele1.appendChild(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.SampleRate);
	ele.setAttribute(iwt.DIS_TAGS.Value,sampleRate.toString());
	ele1.appendChild(ele);
	
	ele= outDoc.createElement(iwt.DIS_TAGS.DataLength);
	ele.setAttribute(iwt.DIS_TAGS.Value,dataLength.toString());
	ele1.appendChild(ele);

	ele= outDoc.createElement(iwt.DIS_TAGS.Samples);
	ele.setAttribute(iwt.DIS_TAGS.Value,samples.toString());
	ele1.appendChild(ele);

}

public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    int             idx = 0;
    byte[]          entityMarkingBytes = new byte[11];
    UnsignedByte    articulationParameterCount = new UnsignedByte();
    String          fileEncoding;
    Element         ele;
    NodeList        nodes;

    if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();

    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.Entity_ID) )
				entity_ID.extract(aNode);
		  
         else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RadioID) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    radioID = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.EncodingScheme) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    encodingScheme = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		
		 else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TdlType) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    tdlType = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.SampleRate) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    sampleRate = new UnsignedInt(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		
		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.DataLength) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    dataLength = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		
		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.Samples) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    samples = new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }
		
	   }
	   }
  }
}


