package iwt.dis;                                   // package to which we belong

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;  

import iwt.*;
import org.w3c.dom.*;
                                        

public class TransmitterPdu extends ProtocolDataUnit 
{
    
    protected EntityID				entityId;             

    
    protected UnsignedShort			radioId;

	protected EntityType			radioEntityType;             
 
	protected UnsignedByte			transmitState;
	
	protected UnsignedByte			inputSource;

	protected AntennaLocation		antennaLocation;
		
	protected UnsignedShort			antennaPatternType;

	protected UnsignedShort			antennaPatternLength;


	protected long					transmissionFrequency;

	protected float					transmissionFrequencyBandwidth;
	protected float					power;
	protected ModulationType		modulationType;
	
	protected UnsignedShort			cryptoSystem;

	protected UnsignedShort			cryptoKeyId;

  	protected Vector				modulationParameters;

	protected Vector				antennaPatternParameterVarients;	
    
 
public TransmitterPdu()
{
    
	super.setPduType(PDUTypeField.TRANSMITTER);

    entityId  = new EntityID();
    radioId  =  new UnsignedShort();
	radioEntityType  = new EntityType();
	transmitState = new UnsignedByte();
	inputSource = new UnsignedByte();
    antennaLocation  = new AntennaLocation();
	
	
	antennaPatternType         = new UnsignedShort();
	antennaPatternLength         = new UnsignedShort();

    transmissionFrequency     =0;         
                                                            
	transmissionFrequencyBandwidth     =0;
   	power     =0;
	modulationType = new ModulationType();
	cryptoSystem         = new UnsignedShort();
	cryptoKeyId         = new UnsignedShort();

	
	modulationParameters         = new Vector();
	antennaPatternParameterVarients = new Vector();	
    


   return;
}

public Object clone()
{
    /** make a copy of the object. This requires a deep copy, so we don't have two
        objects sharing pointers to the same data.
    */

 TransmitterPdu  newTransmitterPdu = (TransmitterPdu)super.clone();
 int modulationParametersLength ;
 int idx;

 newTransmitterPdu.setEntityId(this.getEntityId());
 
 newTransmitterPdu.setRadioId(this.getRadioId());
 
 newTransmitterPdu.setRadioEntityType(this.getRadioEntityType());

 newTransmitterPdu.setTransmitState(this.getTransmitState());
newTransmitterPdu.setInputSource(this.getInputSource());
 newTransmitterPdu.setAntennaLocation(this.getAntennaLocation());

 newTransmitterPdu.setAntennaPatternType(this.getAntennaPatternType());
 newTransmitterPdu.setAntennaPatternLength(this.getAntennaPatternLength());

 newTransmitterPdu.setTransmissionFrequency(this.getTransmissionFrequency());

 newTransmitterPdu.setTransmissionFrequencyBandwidth(this.gettransmissionFrequencyBandwidth());
 newTransmitterPdu.setPower(this.getPower());
 newTransmitterPdu.setModulationType(this.getModulationType());
 newTransmitterPdu.setCryptoSystem(this.getcryptoSystem());
 newTransmitterPdu.setCryptoKeyId(this.getCryptoKeyId());

 for(idx=0;idx < modulationParameters.size();idx++)
 {
 newTransmitterPdu.addModulationParameter(this.getModulationParameterAt(idx));
}

for(idx=0;idx < modulationParameters.size();idx++)
 {
 newTransmitterPdu.addAntennaPatternParameterVarient(this.getAntennaPatternParameterVarient(idx));
}
 
 return newTransmitterPdu;
}




public void serialize(DataOutputStream outputStream)
{
    UnsignedShort   padding = new UnsignedShort(0);
    UnsignedByte     additionalPadding = new UnsignedByte();
    UnsignedByte	modulationParameterCount= new UnsignedByte();
	int idx;
	
	super.serialize(outputStream);      // write out header info
    try
	{

    entityId.serialize(outputStream);

    radioId.serialize(outputStream);

	radioEntityType.serialize(outputStream);
    
	transmitState.serialize(outputStream);
	inputSource.serialize(outputStream);
	padding.serialize(outputStream); 
    antennaLocation.serialize(outputStream);
    antennaPatternType.serialize(outputStream);
    antennaPatternLength.serialize(outputStream);

    outputStream.writeLong(transmissionFrequency);
	
	outputStream.writeFloat(transmissionFrequencyBandwidth);
	outputStream.writeFloat(power);

    modulationType.serialize(outputStream);

    cryptoSystem.serialize(outputStream);
    cryptoKeyId.serialize(outputStream);
	modulationParameterCount.serialize(outputStream);
	padding.serialize(outputStream); 
	additionalPadding.serialize(outputStream); 
    
	for(idx=0;idx< modulationParameterCount.intValue(); idx++)
	{
		UnsignedShort	mParameter;
		mParameter = (UnsignedShort)modulationParameters.elementAt(idx);
		mParameter.serialize(outputStream);
	}
	
	for(idx=0;idx< modulationParameterCount.intValue(); idx++)
	{
		AntennaPatternParameterVarient	mParameter;
		mParameter = (AntennaPatternParameterVarient)antennaPatternParameterVarients.elementAt(idx);
		mParameter.serialize(outputStream);
	}

}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in TransmitterPdu. Error writing to wire.");
    }
	return;

}

public void  deSerialize(DataInputStream inputStream)
{
 /**
    deserialize our data from the stream. We first call the superclass,
    so that all the data there is read into the buffer first.
    Then we deserialize each of our ivars to the stream, being careful
    about the order.
   */
	int indentLevel =0;
char[]  indent = new char[indentLevel];
    int             idx = 0;
    UnsignedShort   padding = new UnsignedShort(0);
	UnsignedByte    additionalPadding = new UnsignedByte(0);
    UnsignedByte	modulationParameterCount=new UnsignedByte();
	//public static final String printStream = "System.out";
	
	PrintStream printStream = System.out;
	super.deSerialize(inputStream);

    // Do our ivars
	try
	{

    entityId.deSerialize(inputStream);
	//entityId.printValues(0,System.out);
    
	radioId.deSerialize(inputStream);
	
	radioEntityType.deSerialize(inputStream);
    //radioEntityType.printValues(indentLevel, printStream);

	transmitState.deSerialize(inputStream);
	inputSource.deSerialize(inputStream);
	padding.deSerialize(inputStream);
    antennaLocation.deSerialize(inputStream);
	//antennaLocation.printValues(indentLevel, printStream);
	
    antennaPatternType.deSerialize(inputStream);
	
	antennaPatternLength.deSerialize(inputStream);
	
	transmissionFrequency= inputStream.readLong();
	
	transmissionFrequencyBandwidth= inputStream.readFloat();
	power= inputStream.readFloat();

    modulationType.deSerialize(inputStream);
    //modulationType.printValues(indentLevel, printStream);
    cryptoSystem.deSerialize(inputStream);
    cryptoKeyId.deSerialize(inputStream);

	modulationParameterCount.deSerialize(inputStream);
	padding.deSerialize(inputStream);
	additionalPadding.deSerialize(inputStream);
    
	for(idx=0;idx<modulationParameterCount.intValue();idx++)
	{
		UnsignedShort	mParameter = new UnsignedShort();
		mParameter.deSerialize(inputStream);
		modulationParameters.addElement(mParameter);
	}
    
	for(idx=0;idx<modulationParameterCount.intValue();idx++)
	{
		AntennaPatternParameterVarient	mParameter = new AntennaPatternParameterVarient();
		mParameter.deSerialize(inputStream);
		antennaPatternParameterVarients.addElement(mParameter);
	}
	
	}
	catch (IOException ioError)
    {
        throw new 
            RuntimeException("Exception in TransmitterPdu. Error writing to wire.");
    }
	return;
}

public int length()
{
        return sizeOf ;
}

public String PDUName()
{
  return new String("Transmitter PDU");
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Designator PDU-");

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    entityId.printValues(indentLevel, printStream);
   printStream.println(indent + "Radio ID: " + radioId);
  radioEntityType.printValues(indentLevel, printStream);
  printStream.println(indent + "Transmit State: " + transmitState);

  printStream.println(indent + "Input Source: " + inputSource);

    antennaLocation.printValues(indentLevel, printStream);

	
	printStream.println(indent + "antennaPatternType: " + antennaPatternType);
	printStream.println(indent + "antennaPatternLength: " + antennaPatternLength);

	
	printStream.println(indent + "transmissionFrequency: " + transmissionFrequency);
    printStream.println(indent + "transmissionFrequency Bandwidth: " + transmissionFrequencyBandwidth);
    
	printStream.println(indent + "power: " + power);
       
 
	
	modulationType.printValues(indentLevel, printStream);
    
	printStream.println(indent + "cryptoSystem: " + cryptoSystem);
	printStream.println(indent + "cryptoKeyId: " + cryptoKeyId);
    printStream.println(indent + "Length of Modulation Parameters: " + modulationParameters.size());
   
	for(idx=0;idx <modulationParameters.size();idx++)
	{
		UnsignedShort	mParameter;
		mParameter = (UnsignedShort)modulationParameters.elementAt(idx);
		printStream.println(indent+"Modulation Parameters " + mParameter);
	}
	for(idx=0;idx <modulationParameters.size();idx++)
	{
		AntennaPatternParameterVarient	mParameter;
		mParameter = (AntennaPatternParameterVarient)antennaPatternParameterVarients.elementAt(idx);
		printStream.println(indent+"AntennaPatternParameterVarients " + mParameter);
	}
	
    return;
}

    //Accessor methods

public EntityID getEntityId()
{   return (EntityID)entityId.clone();
}
public void setEntityId(EntityID pentityId)
{entityId = pentityId;
}

public UnsignedShort getRadioId()
{   return (UnsignedShort)radioId.clone();
}
public void setRadioId(UnsignedShort pradioId)
{radioId = pradioId;
}

public EntityType getRadioEntityType()
{   return (EntityType)radioEntityType.clone();
}
public void setRadioEntityType(EntityType pradioEntityType)
{radioEntityType = pradioEntityType;
}

public UnsignedByte getTransmitState()
{   return (UnsignedByte)transmitState.clone();
}
public void setTransmitState(UnsignedByte ptransmitState)
{transmitState= ptransmitState;
}

public UnsignedByte getInputSource()
{   return (UnsignedByte)inputSource.clone();
}
public void setInputSource(UnsignedByte pinputSource)
{inputSource= pinputSource;
}
public AntennaLocation getAntennaLocation()
{   return (AntennaLocation)antennaLocation.clone();
}

public void setAntennaLocation(AntennaLocation pantennaLocation)
{ antennaLocation = pantennaLocation;
}


public UnsignedShort getAntennaPatternType()
{   return (UnsignedShort)antennaPatternType.clone();
}
public void setAntennaPatternType(UnsignedShort pantennaPatternType)
{antennaPatternType = pantennaPatternType;
}

public UnsignedShort getAntennaPatternLength()
{   return (UnsignedShort)antennaPatternLength.clone();
}
public void setAntennaPatternLength(UnsignedShort pantennaPatternLength)
{antennaPatternLength = pantennaPatternLength;
}
public long getTransmissionFrequency()
{   return transmissionFrequency;
}
public void setTransmissionFrequency(long ptransmissionFrequency)
{transmissionFrequency = ptransmissionFrequency;
}

public float gettransmissionFrequencyBandwidth()
{   return transmissionFrequencyBandwidth;
}
public void setTransmissionFrequencyBandwidth(float ptransmissionFrequencyBandwidth)
{transmissionFrequencyBandwidth = ptransmissionFrequencyBandwidth;
}

public float getPower()
{   return power;
}
public void setPower(float ppower)
{power = ppower;
}

public ModulationType getModulationType()
{   return (ModulationType)modulationType.clone();
}
public void setModulationType(ModulationType pmodulationType)
{modulationType = pmodulationType;
}




public UnsignedShort getcryptoSystem()
{   return (UnsignedShort)cryptoSystem.clone();
}
public void setCryptoSystem(UnsignedShort pcryptoSystem)
{cryptoSystem = pcryptoSystem;
}

public UnsignedShort getCryptoKeyId()
{   return (UnsignedShort)cryptoKeyId.clone();
}
public void setCryptoKeyId(UnsignedShort pcryptoKeyId)
{cryptoKeyId = pcryptoKeyId;
}

public UnsignedShort getModulationParameterAt(int pIdx)
{   
	UnsignedShort mParameter;
	mParameter =(UnsignedShort)modulationParameters.elementAt(pIdx);

return (UnsignedShort)mParameter.clone();
}

public void addModulationParameter(UnsignedShort pmodulationParameter)
{
	modulationParameters.addElement(pmodulationParameter);
}

public AntennaPatternParameterVarient getAntennaPatternParameterVarient(int pIdx)
{   
	AntennaPatternParameterVarient mParameter;
	mParameter =(AntennaPatternParameterVarient)antennaPatternParameterVarients.elementAt(pIdx);

return (AntennaPatternParameterVarient)mParameter.clone();
}

public void addAntennaPatternParameterVarient(AntennaPatternParameterVarient pmodulationParameter)
{
	antennaPatternParameterVarients.addElement(pmodulationParameter);
}



public void  write(org.w3c.dom.Document outDoc)
  {
     insert(outDoc);

   }// end of write()

public void read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {

}

  public void  insert(org.w3c.dom.Node outNode)
  {
 

    int             idx = 0;
    Element         ele;
    Element         ele1;
    Text            txt1;
	UnsignedByte	modulationParameterLength;
	
	printValues(0,System.out);
    System.out.println("Entering TransmitterPDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU); 
    outNode.appendChild(ele1);
    insertHeader(ele1);

    entityId.insert(ele1);

    ele = outDoc.createElement(DIS_TAGS.RadioID);
    ele.setAttribute(DIS_TAGS.Value,radioId.toString());
    ele1.appendChild(ele);

	radioEntityType.insert(ele1);
    
	ele = outDoc.createElement(DIS_TAGS.TransmitState);
	ele.setAttribute(DIS_TAGS.Value,transmitState.toString());
	ele1.appendChild(ele);

	ele = outDoc.createElement(DIS_TAGS.InputSource);
	ele.setAttribute(DIS_TAGS.Value,inputSource.toString());
	ele1.appendChild(ele);
	
/*	ele = outDoc.createElement(DIS_TAGS.AntennaLocation);
    ele1.appendChild(ele);
    antennaLocation.insert(ele1);*/
    
	ele = outDoc.createElement(DIS_TAGS.AntennaPatternType);
	ele.setAttribute(DIS_TAGS.Value,antennaPatternType.toString());
	ele1.appendChild(ele);

	ele = outDoc.createElement(DIS_TAGS.AntennaPatternLength);
	ele.setAttribute(DIS_TAGS.Value,antennaPatternLength.toString());
	ele1.appendChild(ele);

	
	ele = outDoc.createElement(DIS_TAGS.TransmissionFrequency);
	ele.setAttribute(DIS_TAGS.Value,Long.toString(transmissionFrequency));
	ele1.appendChild(ele);
	
	ele = outDoc.createElement(DIS_TAGS.TransmissionFrequencyBandwidth);
	ele.setAttribute(DIS_TAGS.Value,Float.toString(transmissionFrequencyBandwidth));
	ele1.appendChild(ele);
	
	ele = outDoc.createElement(DIS_TAGS.Power);
	ele.setAttribute(DIS_TAGS.Value,Float.toString(power));
	ele1.appendChild(ele);

	modulationType.insert(ele1);

	ele = outDoc.createElement(DIS_TAGS.CryptoSystem);
	ele.setAttribute(DIS_TAGS.Value,cryptoSystem.toString());
	ele1.appendChild(ele);

	ele = outDoc.createElement(DIS_TAGS.CryptoKeyId);
	ele.setAttribute(DIS_TAGS.Value,cryptoKeyId.toString());
	ele1.appendChild(ele);

	modulationParameterLength = new UnsignedByte(modulationParameters.size());

	ele = outDoc.createElement(DIS_TAGS.ModulationParameter);
	ele.setAttribute(DIS_TAGS.Value,modulationParameterLength.toString());
	ele1.appendChild(ele);
		
	

 } // end of insert

  /**
  *@exception RuntimeException when io error occurs.
  *fileEncoding ensures we convert string to local system encoding properly.
  */
  public void  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {
    int             idx = 0;
    Element         ele;
    NodeList        nodes;


	int indent =0;
	PrintStream printStream= System.out;

    if( !inNode.getNodeName().equals(DIS_TAGS.PDU) ) 
        throw new InvalidDocumentType();
	
    nodes = inNode.getChildNodes();
    int i=0;

    while( i<nodes.getLength() )
    {
       Node aNode = nodes.item(i);

       if( aNode!=null )
       {
          if( aNode.getNodeType() == Node.ELEMENT_NODE &&
              aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
              extractHeader(aNode);
				
           else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.EntityID) )
		    entityId.extract(aNode);

		    else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.RadioID) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    radioId = new UnsignedShort(Integer.parseInt(val1));
           		  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

         else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.EntityType))
		 		 radioEntityType.extract(aNode);
		
          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TransmitState) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    transmitState = new UnsignedByte(Integer.parseInt(val1));
           		  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.InputSource) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    inputSource = new UnsignedByte(Integer.parseInt(val1));
		
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }


		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.AntennaLocation) )
                   antennaLocation.extract(aNode);


          
		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.AntennaPatternLength) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    antennaPatternLength= new UnsignedShort(Integer.parseInt(val1));
        
				  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }


		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TransmissionFrequency) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    transmissionFrequency= (long)(Integer.parseInt(val1));
				   } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }


		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.TransmissionFrequencyBandwidth) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    transmissionFrequencyBandwidth=Float.valueOf(val1).floatValue();            
			
				  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

		else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.Power) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    power= Float.valueOf(val1).floatValue();
					} catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

	

          else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.ModulationType) )
		   modulationType.extract(aNode);
		
		
		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.CryptoSystem) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    cryptoSystem= new UnsignedShort(Integer.parseInt(val1));
                  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

		  else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                   aNode.getNodeName().equals(iwt.DIS_TAGS.CryptoKeyId) )
               {
                  String val1 = ((Element)aNode).getAttribute(iwt.DIS_TAGS.Value);
                  if( val1 == null ) throw new InvalidDocumentType();
                  try
                  {
                    cryptoKeyId= new UnsignedShort(Integer.parseInt(val1));
               	  } catch(NumberFormatException nfe) 
                    { throw new InvalidDocumentType(); }
               }

		

       }// end of if

       i++;

    }// end of while() 

 }  




} 
