package iwt.mine;

import mil.navy.nps.dis.*;         
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import org.w3c.dom.*;

/** 
* <pre>
* struct EMfAppearance_REV 
* {
*    unsigned MinefieldType : 2;
*    unsigned activeStatus  : 1;
*    unsigned Lane          : 1;
*    unsigned _unused_1     : 9;
*    unsigned State         : 1;
*    unsigned _unknown_     : 2;
* };
* </pre>
* @see mil.navy.nps.dis.PduElement 
* @see SerializationInterface
*
* @author Hasan Timucin Ozdemir
* @date   January 25, 1999
*
*/
public class EMAppearance extends PduElement
{

   protected UnsignedInt   minefieldType;
   protected UnsignedInt   activeStatus;
   protected UnsignedInt   lane;
   protected UnsignedInt   state;
   
   /**
   * Constant value--size of a EMAppearance when written out; 
   * here :<code>sizeOf = 24 bytes</code>.
   */
   public final int sizeOf = 24;   // object is 24 bytes long 


   /**
   */
   public EMAppearance()
   {
     // default constructor

     minefieldType = new UnsignedInt(2); 
     activeStatus = new UnsignedInt(1);
     lane = new UnsignedInt(1); 
     state = new UnsignedInt(1);

     return;
   }

   /**
   * Constructs a
   * @param  
   * @param   
   * @param 
   * @param 
   */
   public EMAppearance(int _minefieldType, int _activeStatus, int _lane, int _state)
   {
     minefieldType = new UnsignedInt(_minefieldType); 
     activeStatus = new UnsignedInt(_activeStatus);
     lane = new UnsignedInt(_lane); 
     state = new UnsignedInt(_state);

   }// end of CONSTRUCTOR
  
   /**
   * Constructs a
   * @param  
   * @param   
   * @param 
   * @param 
   */
   public EMAppearance(UnsignedInt _minefieldType, UnsignedInt _activeStatus, 
                       UnsignedInt _lane, UnsignedInt _state)
   {
     minefieldType = _minefieldType; 
     activeStatus = _activeStatus;
     lane = _lane; 
     state = _state;

   }// end of CONSTRUCTOR

   public Object clone()
   {
     EMAppearance  newEMAppearance = new EMAppearance(minefieldType, activeStatus, lane, state);

     return newEMAppearance;

   }// end of clone()

   /**
   */
   public void serialize(DataOutputStream outputStream)
   {
     // write out the data to an output stream. Order is important here, since
     // it needs to conform to the DIS standard.

     UnsignedInt  unused_1 = new UnsignedInt(1);
     UnsignedInt  unknown2 = new UnsignedInt(2);
    
     minefieldType.serialize(outputStream); 
     activeStatus.serialize(outputStream);  
     lane.serialize(outputStream); 
     unused_1.serialize(outputStream);
     state.serialize(outputStream);  
     unknown2.serialize(outputStream);
          
   }// end of serialize()

   /**
   */
   public void deSerialize(DataInputStream pInputStream)
   {
     UnsignedInt unused_1 = new UnsignedInt(1);
     UnsignedInt unknown1 = new UnsignedInt(1);

     minefieldType.deSerialize(pInputStream); 
     activeStatus.deSerialize(pInputStream);  
     lane.deSerialize(pInputStream); 
     unused_1.deSerialize(pInputStream);
     state.deSerialize(pInputStream);  
     unknown1.deSerialize(pInputStream);

   }// end of deSerialize()

   /**
   */
   public int length() { return sizeOf; }

   /**
   */
   public void printValues(int indentLevel, PrintStream printStream)
   {
     // print the values of the object out, with correct level of
     // indentation on the page.

     char[]  indent = new char[indentLevel];
     int     idx = 0;
     String  spacing;

     for(idx = 0; idx < indentLevel; idx++) indent[idx] = ' ';

     spacing = new String(indent);

     printStream.println(indent + "EMAppearance minefieldType: "   + minefieldType.intValue());
     printStream.println(indent + "EMAppearance  activeStatus: "   + activeStatus.intValue());
     printStream.println(indent + "EMAppearance          lane: "   + lane.intValue());
     printStream.println(indent + "EMAppearance         state: "   + state.intValue());
     return;

   }// end of printValues()


  // Accessor methods

  public int getMinefieldType() { return minefieldType.intValue(); }

  public void setMinefieldType(int _minefieldType) 
  { minefieldType = new UnsignedInt(_minefieldType); }

  public int getActiveStatus() { return activeStatus.intValue(); }

  public void setActiveStatus(int _activeStatus) 
  { activeStatus = new UnsignedInt(_activeStatus); }

  public int getLane() { return lane.intValue(); }

  public void setLane(int _lane) 
  { lane = new UnsignedInt(_lane); }

  public int getState() { return state.intValue(); }

  public void setState(int _state) 
  { state = new UnsignedInt(_state); }

  /**
  */
  public void 
  write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

  /**
  */
  public void 
  insert( org.w3c.dom.Node outNode )
  {
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode);

  }// end of insert()

  /**
  */
  public void 
  insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
  {
     org.w3c.dom.Element ele = outDoc.createElement(DIS_TAGS.EMAppearance);
     ele.setAttribute(DIS_TAGS.MinefieldType,minefieldType.toString());
     ele.setAttribute(DIS_TAGS.ActiveState,activeStatus.toString());
     ele.setAttribute(DIS_TAGS.Lane,lane.toString());
     ele.setAttribute(DIS_TAGS.State,state.toString());
     outNode.appendChild(ele);

  }// end of insertToDocument

  /**
  */
  public void 
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(DIS_TAGS.EMAppearance);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(DIS_TAGS.EMAppearance) && 
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.MinefieldType);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.ActiveState);
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Lane);
       org.w3c.dom.Attribute a4 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.State);

       String v1, v2, v3, v4;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();
       v4 = a4.getValue();

       try
       {
         minefieldType = new UnsignedInt(Integer.parseInt(v1));

       } catch(NumberFormatException nfe) {}

       try
       {
         activeStatus = new UnsignedInt(Integer.parseInt(v2));

       } catch(NumberFormatException nfe) {}

       try
       {
         lane = new UnsignedInt(Integer.parseInt(v3));

       } catch(NumberFormatException nfe) {}

       try
       {
         state = new UnsignedInt(Integer.parseInt(v4));

       } catch(NumberFormatException nfe) {}

    } else throw new InvalidDocumentType();

  }// end of extract()

} // end of class EMAppearance
