package iwt.mine;

import mil.navy.nps.dis.*;         
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import org.w3c.dom.*;

/** 
*
* <pre>
* struct TFixedObjectOrientedLoc {
*   DIS203_VECTOR_RECORD       fLocation;
*   DIS203_EULER_ANGLES_RECORD fOrientation;
*   float                      fBurialDepth;
*   unsigned int               fId;
*   unsigned int               padding;
* };
* </pre>
*
* @see PduElement 
* @see SerializationInterface
* @see EntityCoordinate
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class FixedObjectOrientedLocation extends PduElement
{
   protected VectorRecord  fLocation;      // 4*3
   protected EulerAngle    fOrientation;   // 4*3
   protected float         fBurialDepth;   // 4
   protected UnsignedShort fId;            // 2
   protected UnsignedShort padding;        // 2
   
   /**
   * Constant value--size of a FixedObjectOrientedLocation when written out; 
   * here :<code>sizeOf = 32 bytes</code>.
   */
   public final int sizeOf = 32;   // object is 32 bytes long 

   /**
   */
   public FixedObjectOrientedLocation()
   {
     // default constructor

     fLocation    = new VectorRecord(); 
     fOrientation = new EulerAngle();
     fId     = new UnsignedShort(0); 
     padding = new UnsignedShort(0);
     fBurialDepth = 0;

     return;
   }

   /**
   * Constructs 
   * @param _fLocation  
   * @param _fOrientation  
   * @param _fBurialDepth
   * @param _fId 
   */
   public FixedObjectOrientedLocation(VectorRecord _fLocation, EulerAngle _fOrientation,
                                      float _fBurialDepth, UnsignedShort _fId) 
   {
     fLocation    = _fLocation;
     fOrientation = _fOrientation;
     fBurialDepth = _fBurialDepth;
     fId     = _fId;
   }// end of CONSTRUCTOR
  
   public Object clone()
   {
     FixedObjectOrientedLocation  newFixedObjectOrientedLocation = 
        new FixedObjectOrientedLocation(fLocation, fOrientation, fBurialDepth, fId);

     return newFixedObjectOrientedLocation;

   }// end of clone()

   /**
   * @exception RuntimeException when IO error occurs.
   */
   public void serialize(DataOutputStream outputStream)
   {
     // write out the data to an output stream. Order is important here, since
     // it needs to conform to the DIS standard.

     try
     {
        fLocation.serialize(outputStream); 
        fOrientation.serialize(outputStream);  
        outputStream.writeFloat(fBurialDepth);
        fId.serialize(outputStream);
        padding.serialize(outputStream);

     } catch( java.io.IOException ioe) {}
     
   }// end of serialize()

   /**
   * @exception RuntimeException when IO error occurs.
   */
   public void deSerialize(DataInputStream pInputStream)
   {
     try
     {
        fLocation.deSerialize(pInputStream); 
        fOrientation.deSerialize(pInputStream);  
        fBurialDepth = pInputStream.readFloat();
        fId.deSerialize(pInputStream);
        padding.deSerialize(pInputStream);

     } catch( java.io.IOException ioe) {}

   }// end of deSerialize()

   public int length() { return sizeOf; }

   public void printValues(int indentLevel, PrintStream printStream)
   {
     // print the values of the object out, with correct level of
     // indentation on the page.

     char[]  indent = new char[indentLevel];
     int     idx = 0;
     String  spacing;

     for(idx = 0; idx < indentLevel; idx++) indent[idx] = ' ';

     spacing = new String(indent);
     fLocation.printValues(indentLevel,printStream);
     fOrientation.printValues(indentLevel,printStream);
     printStream.println(indent + "BurialDepth: "   + fBurialDepth);
     printStream.println(indent + "fId: "   + fId.intValue());
     return;

   }// end of printValues()


  // Accessor methods

  public VectorRecord getLocation() { return fLocation; }

  public void setLocation(VectorRecord _fLocation) 
  { fLocation = _fLocation; }

  public EulerAngle getOrientation() { return fOrientation; }

  public void setOrientation(EulerAngle _fOrientation) 
  { fOrientation = _fOrientation; }

  public float getBurialDepth() { return fBurialDepth; }

  public void setBurialDepth(float _fBurialDepth) 
  { fBurialDepth = _fBurialDepth; }

  public UnsignedShort getID() { return fId; }

  public void setID(UnsignedShort _fid ) 
  { fId = _fid; }

  /**
  */
  public void 
  write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

  /**
  */
  public void 
  insert( org.w3c.dom.Node outNode )
  {
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode);

  }// end of insert()

  /**
  */
  public void 
  insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
  {
     org.w3c.dom.Element ele = outDoc.createElement(DIS_TAGS.FixedObjectOrientedLocation);
     ele.setAttribute(DIS_TAGS.ID,fId.toString());
     ele.setAttribute(DIS_TAGS.BurialDepth,Float.toString(fBurialDepth));
     outNode.appendChild(ele);
     fLocation.insert(ele);
     fOrientation.insert(ele);

  }// end of insertToDocument

  /**
  */
  public void 
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(DIS_TAGS.FixedObjectOrientedLocation);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(DIS_TAGS.FixedObjectOrientedLocation) && 
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.ID);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.BurialDepth);


       String v1, v2;

       v1 = a1.getValue();
       v2 = a2.getValue();

       try
       {
         fId = new UnsignedShort(Integer.parseInt(v1));

       } catch(NumberFormatException nfe) {}

       try
       {
         fBurialDepth = Float.valueOf(v2).floatValue();

       } catch(NumberFormatException nfe) {}

       org.w3c.dom.NodeList nodes = ((Element)rt).getElementsByTagName(DIS_TAGS.VectorRecord);
       if( nodes.getLength()<1 ) throw new InvalidDocumentType();

       org.w3c.dom.Node rt1 = nodes.item(0);

       fLocation.extract(rt1);

       nodes = ((Element)rt).getElementsByTagName(DIS_TAGS.EulerAngle);
       if( nodes.getLength()<1 ) throw new InvalidDocumentType();
       rt1 = nodes.item(0);
       fOrientation.extract(rt1);

    } else throw new InvalidDocumentType();

  }// end of extract()

} // end of class FixedObjectOrientedLocation
