
package iwt.mine;

import mil.navy.nps.dis.*;         
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import iwt.util.*;
import org.w3c.dom.*;

/** 
* 
* <pre> 
* struct TMineDataFlag {
*  unsigned int orientation_flag  : 1;
*  unsigned int burial_depth_flag : 1;
*  unsigned int temperature_flag  : 1;
*  unsigned int _unused_          : 29;
* };
* </pre> 
*
* @see PduElement 
* @see SerializationInterface
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MineDataFlag extends PduElement
{
   /** */
   protected boolean  bOrientation = false;     

   /** */
   protected boolean  bBurialDepth = false;     

   /** */
   protected boolean  bTemperature = false;     

   /** */
   protected int      bunused = 0;     

   /**
   * Constant value--size of a MineDataFlag Record when written out; 
   * here :<code>sizeOf = 4 bytes</code>.
   */
   public final int sizeOf = 4;   // object is 4 bytes long 

   /**
   * Constructs an new MineDataFlag
   */
   public MineDataFlag()  {  }

   /**
   * Constructs a new MineDataFlag with coordinate values passed in parameters.
   *
   * @param 
   * @param 
   * @param 
   *
   */
   public MineDataFlag( boolean _fOrientation, boolean _fBurialDepth, 
                        boolean _fTemperature, int _unused )
   {
      bOrientation = _fOrientation; 
      bBurialDepth = _fBurialDepth;
      bTemperature = _fTemperature;
      bunused      = _unused;
   }

   /** */
   public Object clone()
   {
     MineDataFlag   newMineDataFlag = new MineDataFlag(bOrientation,bBurialDepth,bTemperature,bunused);

     return newMineDataFlag;
   }

   /** */
   public void serialize(DataOutputStream outputStream)
   {
     
     try
     {
       int val = (bunused & 0x1fff) | (bOrientation?0x8000:0x0000) | 
                 (bBurialDepth?0x4000:0x0000) | (bTemperature?0x2000:0x0000) ;

       outputStream.writeInt(val);

     } catch(java.io.IOException ioe)
       { 
         String str = "Exception in MineDataFlag. "+
                      "Error writing to file."; 
         throw new RuntimeException(str);
       }

   }// end of serialize()

   /** */
   public void deSerialize(DataInputStream pInputStream)
   {
     try
     {
       int val = pInputStream.readInt();
       bunused = val & 0x1fff;
       bOrientation = (val & 0x8000)>0 ;
       bBurialDepth = (val & 0x4000)>0 ;
       bTemperature = (val & 0x2000)>0 ;    
     } catch(java.io.IOException ioe)
       {  throw new 
                RuntimeException("Exception in MineDataFlag. Error writing to file.");
       }

   }// end of deSerialize()

   /** */
   public int length() { return sizeOf;    }

   /** */
   public void printValues(int indentLevel, PrintStream printStream)
   {
      // print the values of the object out, with correct level of
      // indentation on the page.

      char[]  indent = new char[indentLevel];
      int     idx = 0;
      String  spacing;

      for(idx = 0; idx < indentLevel; idx++) indent[idx] = ' ';

      spacing = new String(indent);

      printStream.println(indent + "Orientation Flag  : " + bOrientation);
      printStream.println(indent + "Burial Depth Flag : " + bBurialDepth);
      printStream.println(indent + "Temperature Flag  : " + bTemperature);

   }

   // Accessor methods
   public boolean getOrientationFlag() { return bOrientation;}
   public void setOrientationFlag()    { bOrientation=true;}
   public void resetOrientationFlag()  { bOrientation=false;}

   public boolean getBurialDepthFlag() { return bBurialDepth;}
   public void setBurialDepthFlag()    { bBurialDepth=true;}
   public void resetBurialDepthFlag()  { bBurialDepth=false;}

   public boolean getTemperature()     { return bTemperature;}
   public void setTemperature()        { bTemperature=true;}
   public void resetTemperature()      { bTemperature=false;}

   public int getUnused()              { return bunused;}
   public void setUnused(int _val)     { bunused=_val;}

   /**
   */
   public void 
   write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

   /**
   */
   public void 
   insert(org.w3c.dom.Node outNode)
   {
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode);

   }// end of insert()

   /**
   */
   public void 
   insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
   {
      org.w3c.dom.Element ele = outDoc.createElement(iwt.DIS_TAGS.MineDataField);
      ele.setAttribute(iwt.DIS_TAGS.OrientationFlag,(bOrientation?"true":"false"));
      ele.setAttribute(iwt.DIS_TAGS.BurialDepthFlag,(bBurialDepth?"true":"false"));
      ele.setAttribute(iwt.DIS_TAGS.TemperatureFlag,(bTemperature?"true":"false"));
      outNode.appendChild(ele);

   }// end of insertToDocument()

   /**
   */
   public void 
   read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
   {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(iwt.DIS_TAGS.MineDataFlag);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(iwt.DIS_TAGS.MineDataFlag) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.OrientationFlag);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.BurialDepthFlag);
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.TemperatureFlag);
       String v1, v2, v3;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();

       bOrientation = v1.equals("true");
       bBurialDepth = v2.equals("true");
       bTemperature = v3.equals("true");
       
    } else throw new InvalidDocumentType();

  }// end of read()

} // end of class MineDataFlag
