/*
*/
package iwt.mine;                                   

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
*
*<pre>
*struct cpduMinefieldNACK {
*   DIS203_PDU_HEADER_RECORD            fHeader;
*   DIS203_ENTITY_ID_RECORD             fMinefieldId;
*   DIS203_ENTITY_ID_RECORD             fRequesterId;
*   unsigned char                          fRequestId;
*   unsigned char              fNumMissingPackets;
*   unsigned char*             fpMissingPacketNum;
*  
*   int GetStructSize();     // without variable sized arrays
*
*};
*</pre>
*
*
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MineFieldNACKPdu extends ProtocolDataUnit 
{

   /**     */
   protected EntityID          fMineFieldId;         // 6 bytes    
 
   /**     */
   protected EntityID          fRequesterId;         // 6 bytes    

   /**
   */
   protected UnsignedByte      fRequestId;           // 1 byte

   /**
   */
   protected UnsignedByte      fNumOfMissingPackets; // 1 byte               

   /**
   */
   protected UnsignedByte[]     fMissingPacketIndex;   // 2 bytes

   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 24 bytes</code>
   */
   public final static int     sizeOf = 14;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MineFieldNACKPdu()
   {
      super.setPduType(PDUTypeField.MINEFIELDNACK); // XXX

      fMineFieldId     = new EntityID();
      fRequesterId     = new EntityID();
      fRequestId       = new UnsignedByte(0); 
      fNumOfMissingPackets = new UnsignedByte(0);
            
      return;
   }


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFieldNACKPdu   newPdu = new MineFieldNACKPdu();

    newPdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newPdu.setRequesterID(this.getRequesterID());
    newPdu.setRequestId(this.getRequestId());
    newPdu.setNumOfMissingPackets(this.getNumOfMissingPackets());
    for(int i=0;i<fNumOfMissingPackets.intValue();i++)
       newPdu.setMissingPacketIndexAt(i,this.getMissingPacketIndexAt(i));

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    

    fMineFieldId.serialize(outputStream);  
    fRequesterId.serialize(outputStream);
    fRequestId.serialize(outputStream); 
    fNumOfMissingPackets.serialize(outputStream);   
    for(int i=0;i<fNumOfMissingPackets.intValue();i++)
        getMissingPacketIndexAt(i).serialize(outputStream);   

    return;
  }

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info
    fMineFieldId.deSerialize(inputStream);  
    fRequesterId.deSerialize(inputStream);
    fRequestId.deSerialize(inputStream);         
    fNumOfMissingPackets.deSerialize(inputStream);   
    setNumOfMissingPackets(fNumOfMissingPackets);
    for(int i=0;i<fNumOfMissingPackets.intValue();i++)
        getMissingPacketIndexAt(i).deSerialize(inputStream);   
   
  }// 

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Minefield NACK");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield State PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    fRequesterId.printValues(indentLevel, printStream);
    printStream.println(indent + "Request Id: "+fRequestId.intValue());
    printStream.println(indent + "NumOfMissingPackets: "+fNumOfMissingPackets.intValue());
    
    for(int i=0;i<fNumOfMissingPackets.intValue();i++)
        printStream.println(indent + "MPIndex["+i+"] : "+getMissingPacketIndexAt(i).intValue());
        
    return;
  }

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public EntityID getRequesterID()
  { return (EntityID)fRequesterId.clone();}

  public void setRequesterID(EntityID _fRequesterId)
  {fRequesterId = _fRequesterId; }

  public UnsignedByte getRequestId()
  { return (UnsignedByte)fRequestId.clone();}

  public void setRequestId(UnsignedByte _fRequestId)
  { fRequestId = _fRequestId;}

  public UnsignedByte getNumOfMissingPackets()
  { return (UnsignedByte)fNumOfMissingPackets.clone();}

  public void setNumOfMissingPackets(UnsignedByte _fNumOfMissingPackets)
  { fNumOfMissingPackets = _fNumOfMissingPackets;
    if(fNumOfMissingPackets.intValue()>0)
       fMissingPacketIndex = new UnsignedByte[fNumOfMissingPackets.intValue()];
  }
  
  public UnsignedByte getMissingPacketIndexAt(int i)
  { 
    UnsignedByte ret = null;
    if( i>=0 && i<fMissingPacketIndex.length)
        ret = fMissingPacketIndex[i];
    return (UnsignedByte)(ret!=null?ret.clone():null);
  }

  public void setMissingPacketIndexAt(int i, UnsignedByte _fMissingPacketIndex)
  {
    if( i>=0 && i<fNumOfMissingPackets.intValue())
        fMissingPacketIndex[i] = _fMissingPacketIndex;

  }

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield NACK PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MineFieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequesterID);
    ele1.appendChild(ele);
    fRequesterId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequestID);
    ele.setAttribute(DIS_TAGS.Value,fRequestId.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfMissingPackages);
    ele.setAttribute(DIS_TAGS.Value,fNumOfMissingPackets.toString());
    ele1.appendChild(ele);
    
    for(int i=0;i<fNumOfMissingPackets.intValue();i++)
    {    Element elet = outDoc.createElement(DIS_TAGS.MissingPackageIndex);
         elet.setAttribute(DIS_TAGS.Value,getMissingPacketIndexAt(i).toString());
         ele.appendChild(elet);
    }

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

            if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                extractHeader(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequestID) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fRequestId = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfMissingPackages) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumOfMissingPackets = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                     
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.MissingPackageIndex);

                     Node bNode ;

                     for(int j=0;j<aList.getLength()&&j<fNumOfMissingPackets.intValue();j++)
                     {
                        bNode = aList.item(j);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE && 
                            bNode.getNodeName().equals(iwt.DIS_TAGS.MissingPackageIndex) ) 
                        { 
                           String v2 = ((Element)bNode).getAttribute(DIS_TAGS.Value);                        
                           try
                           {  fMissingPacketIndex[j]  = new UnsignedByte(Integer.parseInt(v2));
                           } catch(NumberFormatException nfe) {}
                        }
                     
                     }// end of for

                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineFieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequesterID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fRequesterId.extract(bNode);

                     }// end of if

                 }// end of if
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldNACKPdu
