/*
*/
package iwt.mine;                                   

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
* 
*<pre>
*struct cpduMinefieldRequest {
*   DIS203_PDU_HEADER_RECORD            fHeader;
*   DIS203_ENTITY_ID_RECORD             fMinefieldId;
*   DIS203_ENTITY_ID_RECORD             fRequesterId;
*   unsigned char                          fRequestId;
*   unsigned char                          fNumPerimeterPts;
*  
*   TMinefieldObjectFilter                 fFilter;
*   DIS203_ENTITY_TYPE_RECORD           fObjType;
*
*//**
*   TEntCoord                              fPerimeter[4];
*};
*</pre>
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
* 
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MineFieldRequestPdu extends ProtocolDataUnit 
{

   /**     */
   protected EntityID          fMineFieldId;        // 6 bytes    
 
   /**     */
   protected EntityID          fRequesterId;        // 6 bytes    

   /**
   */
   protected UnsignedByte      fRequestId;          // 1 byte

   /**
   */
   protected UnsignedByte      fNumPerimeterPts;    // 1 byte               

   /**
   */
   protected UnsignedShort     fFilter;             // 2 bytes

   /**
   */
   protected EntityType        fObjType;            // 8 bytes     

   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 24 bytes</code>
   */
   public final static int     sizeOf = 24;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MineFieldRequestPdu()
   {
      super.setPduType(PDUTypeField.MINEFIELDREQUEST); // 188

      fMineFieldId     = new EntityID();
      fRequesterId     = new EntityID();
      fRequestId       = new UnsignedByte(0); 
      fNumPerimeterPts = new UnsignedByte(0);
      fFilter          = new UnsignedShort(0);
      fObjType         = new EntityType();
      
      return;
   }


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFieldRequestPdu   newPdu = new MineFieldRequestPdu();

    newPdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newPdu.setRequesterID(this.getRequesterID());
    newPdu.setRequestId(this.getRequestId());
    newPdu.setNumPerimeterPts(this.getNumPerimeterPts());
    newPdu.setFilter(this.getFilter());
    newPdu.setObjType(this.getObjType());

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    

    fMineFieldId.serialize(outputStream);  
    fRequesterId.serialize(outputStream);
    fRequestId.serialize(outputStream);         
    fNumPerimeterPts.serialize(outputStream);   
    fFilter.serialize(outputStream);              
    fObjType.serialize(outputStream);           

    return;
  }

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info
    fMineFieldId.deSerialize(inputStream);  
    fRequesterId.deSerialize(inputStream);
    fRequestId.deSerialize(inputStream);         
    fNumPerimeterPts.deSerialize(inputStream);   
    fFilter.deSerialize(inputStream);              
    fObjType.deSerialize(inputStream);           

  }// 

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Minefield Request");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield State PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    fRequesterId.printValues(indentLevel, printStream);
    printStream.println(indent + "Request Id: "+fRequestId.intValue());
    printStream.println(indent + "NumPerimeterPts: "+fNumPerimeterPts.intValue());
    
    printStream.println(indent + "Filter : "+fFilter.intValue());
    fObjType.printValues(indentLevel, printStream);
    


    return;
  }

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public EntityID getRequesterID()
  { return (EntityID)fRequesterId.clone();}

  public void setRequesterID(EntityID _fRequesterId)
  {fRequesterId = _fRequesterId; }

  public UnsignedByte getRequestId()
  { return (UnsignedByte)fRequestId.clone();}

  public void setRequestId(UnsignedByte _fRequestId)
  { fRequestId = _fRequestId;}

  public UnsignedByte getNumPerimeterPts()
  { return (UnsignedByte)fNumPerimeterPts.clone();}

  public void setNumPerimeterPts(UnsignedByte _fNumPerimeterPts)
  { fNumPerimeterPts = _fNumPerimeterPts;}
  
  public UnsignedShort getFilter()
  { return (UnsignedShort)fFilter.clone();}

  public void setFilter(UnsignedShort _fFilter)
  { fFilter = _fFilter;}

  public EntityType getObjType()
  { return (EntityType)fObjType.clone();}

  public void setObjType(EntityType _fObjType)
  { fObjType = _fObjType;}

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield Request PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MineFieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequesterID);
    ele1.appendChild(ele);
    fRequesterId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequestID);
    ele.setAttribute(DIS_TAGS.Value,fRequestId.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfPerimeterPts);
    ele.setAttribute(DIS_TAGS.Value,fNumPerimeterPts.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.Filter);
    ele.setAttribute(DIS_TAGS.Value,fFilter.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.ObjectType);
    ele1.appendChild(ele);
    fObjType.insert(ele);

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

            if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                extractHeader(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequestID) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fRequestId = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfPerimeterPts) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumPerimeterPts = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.Filter) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fFilter = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineFieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.ObjectType) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityType);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fObjType.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequesterID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fRequesterId.extract(bNode);

                     }// end of if

                 }// end of if
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldStatePdu
