/* */
package iwt.mine;                                   

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
* 
*<pre>
*struct cpduMinefieldResponse {
*   DIS203_PDU_HEADER_RECORD            fHeader;
*   DIS203_ENTITY_ID_RECORD             fMinefieldId;
*   DIS203_ENTITY_ID_RECORD             fRequesterId;
*   unsigned char                          fRequestId;
*   unsigned char                          fPacketNumber;
*   unsigned char                         fTotalPackets;
*   unsigned char                         fNumberOfObjects;
*
*   DIS203_ENTITY_TYPE_RECORD           fObjType;
*  
*   TFixedObjectOrientedLoc                fObjOrientedLoc[1];
*  
*   int    GetStructSize();         // without variable sized array
*   void   WriteMfResponsePdu();
*};
*</pre>
*
* 
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
* 
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
* 
*/
public class MineFieldResponsePdu extends ProtocolDataUnit 
{

   /**     */
   protected EntityID          fMineFieldId;        // 6 bytes    
 
   /**     */
   protected EntityID          fRequesterId;        // 6 bytes    

   /**
   */
   protected UnsignedByte      fRequestId;          // 1 byte

   /**
   */
   protected UnsignedByte      fPacketNumber;       // 1 byte               

   /**
   */
   protected UnsignedByte      fTotalPackets;       // 1 bytes
   
   /**
   */
   protected UnsignedByte      fNumberOfObjects;    // 1 byte               
   
   /**
   */
   protected EntityType        fObjType;            // 8 bytes     
  
   /**
   */
   protected FixedObjectOrientedLocation   fObjOrientedLoc;            
   
   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 24 bytes</code>
   */
   public final static int     sizeOf = 24;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MineFieldResponsePdu()
   {
      super.setPduType(PDUTypeField.MINEFIELDRESPONSE); // 189

      fMineFieldId     = new EntityID();
      fRequesterId     = new EntityID();
      fRequestId       = new UnsignedByte(0); 
      fPacketNumber    = new UnsignedByte(0);
      fTotalPackets    = new UnsignedByte(0);
      fNumberOfObjects = new UnsignedByte(0);
      fObjType         = new EntityType();
      fObjOrientedLoc  = new FixedObjectOrientedLocation();     
 
      return;
   }


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFieldResponsePdu   newPdu = new MineFieldResponsePdu();

    newPdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newPdu.setRequesterID(this.getRequesterID());
    newPdu.setRequestId(this.getRequestId());
    newPdu.setPacketNumber(this.getPacketNumber());
    newPdu.setTotalPackets(this.getTotalPackets());
    newPdu.setNumberOfObjects(this.getNumberOfObjects());
    newPdu.setObjType(this.getObjType());
    newPdu.setObjOrientedLoc(this.getObjOrientedLoc());

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    
    fMineFieldId.serialize(outputStream);  
    fRequesterId.serialize(outputStream);
    fRequestId.serialize(outputStream);         
    fPacketNumber.serialize(outputStream);   
    fTotalPackets.serialize(outputStream);   
    fNumberOfObjects.serialize(outputStream);              
    fObjType.serialize(outputStream);           
    fObjOrientedLoc.serialize(outputStream);           

    return;

  }// end of serialize()

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info
   
    fMineFieldId.deSerialize(inputStream);  
    fRequesterId.deSerialize(inputStream);
    fRequestId.deSerialize(inputStream);         
    fPacketNumber.deSerialize(inputStream);   
    fTotalPackets.deSerialize(inputStream);   
    fNumberOfObjects.deSerialize(inputStream);              
    fObjType.deSerialize(inputStream);           
    fObjOrientedLoc.deSerialize(inputStream);           

  }// end of deSerialize()

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Minefield Response");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield Response PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0) superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    fRequesterId.printValues(indentLevel, printStream);

    printStream.println(indent + "Request Id: "+fRequestId.intValue());
    printStream.println(indent + "PacketNumber: "+fPacketNumber.intValue());
    printStream.println(indent + "TotalPackets: "+fTotalPackets.intValue()); 
    printStream.println(indent + "NumberOfObjects: "+fNumberOfObjects.intValue()); 

    fObjType.printValues(indentLevel, printStream);
    fObjOrientedLoc.printValues(indentLevel, printStream);

    return;
  }

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public EntityID getRequesterID()
  { return (EntityID)fRequesterId.clone();}

  public void setRequesterID(EntityID _fRequesterId)
  {fRequesterId = _fRequesterId; }

  public UnsignedByte getRequestId()
  { return (UnsignedByte)fRequestId.clone();}

  public void setRequestId(UnsignedByte _fRequestId)
  { fRequestId = _fRequestId;}

  public UnsignedByte getPacketNumber()
  { return (UnsignedByte)fPacketNumber.clone();}

  public void setPacketNumber(UnsignedByte _fPacketNumber)
  { fPacketNumber = _fPacketNumber;}
  
  public UnsignedByte getTotalPackets()
  { return (UnsignedByte)fTotalPackets.clone();}

  public void setTotalPackets(UnsignedByte _fTotalPackets)
  { fTotalPackets = _fTotalPackets;}
 
  public UnsignedByte getNumberOfObjects()
  { return (UnsignedByte)fTotalPackets.clone();}

  public void setNumberOfObjects(UnsignedByte _fNumberOfObjects)
  { fNumberOfObjects = _fNumberOfObjects;}
     
  public EntityType getObjType()
  { return (EntityType)fObjType.clone();}

  public void setObjType(EntityType _fObjType)
  { fObjType = _fObjType;}
     
  public FixedObjectOrientedLocation getObjOrientedLoc()
  { return (FixedObjectOrientedLocation)fObjOrientedLoc.clone();}

  public void setObjOrientedLoc(FixedObjectOrientedLocation _fObjOrientedLoc)
  { fObjOrientedLoc = _fObjOrientedLoc;}

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield Response PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MineFieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequesterID);
    ele1.appendChild(ele);
    fRequesterId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequestID);
    ele.setAttribute(DIS_TAGS.Value,fRequestId.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.PacketNumber);
    ele.setAttribute(DIS_TAGS.Value,fPacketNumber.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.TotalPackets);
    ele.setAttribute(DIS_TAGS.Value,fTotalPackets.toString());
    ele1.appendChild(ele);
   
    ele = outDoc.createElement(DIS_TAGS.NumberOfObjects);
    ele.setAttribute(DIS_TAGS.Value,fNumberOfObjects.toString());
    ele1.appendChild(ele);

    fObjType.insert(ele1);
    fObjOrientedLoc.insert(ele1);

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

                 if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                     extractHeader(aNode);
            
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityType) )
                     fObjType.extract(aNode);
             
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.FixedObjectOrientedLocation) )
                     fObjOrientedLoc.extract(aNode);
                                   
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequestID) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fRequestId = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfObjects) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumberOfObjects = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.TotalPackets) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fTotalPackets = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
               
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.PacketNumber) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fPacketNumber = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
                  
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineFieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequesterID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fRequesterId.extract(bNode);

                     }// end of if

                 }// end of if
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldResponsePdu
