/* */
package iwt.mine;                                   

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
*
*<pre>
*struct cpduMinefieldResponse_REV {
*   DIS203_PDU_HEADER_RECORD            fHeader;
*   DIS203_ENTITY_ID_RECORD             fMinefieldId;
*   DIS203_ENTITY_ID_RECORD             fRequesterId;
*   unsigned               _unused_1 : 1;
*   unsigned               fMfSeqNum : 15;
*   unsigned char                          fRequestId;
*   unsigned char                          fPacketNumber;
*   unsigned char                         fTotalPackets;
*   unsigned char                         fNumberOfObjects;
*
*   unsigned char             fNumReqSensors;
*   unsigned char                 _unused_2;
*   unsigned int                  fFilter;
*
*   DIS203_ENTITY_TYPE_RECORD           fObjType;
*
*   void*                 pVarRec;
*
*   //The below line was commented out.  May cause error later.
*   //TFixedObjectOrientedLoc                fObjOrientedLoc[1];
*
*   int    GetStructSize();         // without variable sized array
*   void   WriteMfResponsePdu(VARPTRS* VariableRec);
*};
*</pre>
*
*
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
* 
* @author Hasan Timucin Ozdemir
* @date   January 26, 19999
* 
*/
public class MineFieldResponseRevPdu extends ProtocolDataUnit 
{

   /**     */
   protected EntityID          fMineFieldId;        // 6 bytes    
 
   /**     */
   protected EntityID          fRequesterId;        // 6 bytes    

   /**
   */
   protected UnsignedByte      fRequestId;          // 1 byte

   /**
   */
   protected UnsignedInt       fMFSeqNum;          // 1 byte

  
   /**
   */
   protected UnsignedByte      fPacketNumber;       // 1 byte               

   /**
   */
   protected UnsignedByte      fTotalPackets;       // 1 bytes
   
   /**
   */
   protected UnsignedByte      fNumberOfObjects;    // 1 byte               
     
   /**
   */
   protected UnsignedByte      fNumOfReqSensors;    // 1 byte               
      
   /**
   */
   protected UnsignedInt       fFilter;    // 1 byte               
       
   /**
   */
   protected EntityType        fObjType;            // 8 bytes     
  
   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 24 bytes</code>
   */
   public final static int     sizeOf = 24;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MineFieldResponseRevPdu()
   {
      super.setPduType(PDUTypeField.MINEFIELDRESPONSE); // 189

      fMineFieldId     = new EntityID();
      fRequesterId     = new EntityID();
      fMFSeqNum        = new UnsignedInt(0); 
      fRequestId       = new UnsignedByte(0); 
      fPacketNumber    = new UnsignedByte(0);
      fTotalPackets    = new UnsignedByte(0);
      fNumberOfObjects = new UnsignedByte(0);
      fNumOfReqSensors = new UnsignedByte(0);
      fFilter          = new UnsignedInt(0); 
      fObjType         = new EntityType();
 
      return;
   }

  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFieldResponseRevPdu   newPdu = new MineFieldResponseRevPdu();

    newPdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newPdu.setRequesterID(this.getRequesterID());
    newPdu.setMFSeqNum(this.getMFSeqNum());
    newPdu.setRequestId(this.getRequestId());
    newPdu.setPacketNumber(this.getPacketNumber());
    newPdu.setTotalPackets(this.getTotalPackets());
    newPdu.setNumberOfObjects(this.getNumberOfObjects());
    newPdu.setNumReqSensors(this.getNumReqSensors());
    newPdu.setFilter(this.getFilter());
    newPdu.setObjType(this.getObjType());

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.
    UnsignedInt  pad  = new UnsignedInt(0);
    UnsignedByte pad2 = new UnsignedByte(0);
    
    super.serialize(outputStream);      // write out header info
    
    fMineFieldId.serialize(outputStream);  
    fRequesterId.serialize(outputStream);  
    pad.serialize(outputStream);
    fMFSeqNum.serialize(outputStream);         
    fRequestId.serialize(outputStream);         
    fPacketNumber.serialize(outputStream);   
    fTotalPackets.serialize(outputStream);   
    fNumberOfObjects.serialize(outputStream);              
    fNumOfReqSensors.serialize(outputStream);              
    pad2.serialize(outputStream);
    fFilter.serialize(outputStream);              
    fObjType.serialize(outputStream);           

    return;

  }// end of serialize()

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    UnsignedInt  pad  = new UnsignedInt(0);
    UnsignedByte pad2 = new UnsignedByte(0);
   
    super.deSerialize(inputStream);     // read in all the header info
    
    fMineFieldId.deSerialize(inputStream);  
    fRequesterId.deSerialize(inputStream);
    pad.deSerialize(inputStream);
    fMFSeqNum.deSerialize(inputStream);         
    fRequestId.deSerialize(inputStream);         
    fPacketNumber.deSerialize(inputStream);   
    fTotalPackets.deSerialize(inputStream);   
    fNumberOfObjects.deSerialize(inputStream);              
    fNumOfReqSensors.deSerialize(inputStream);              
    pad2.deSerialize(inputStream);
    fFilter.deSerialize(inputStream);              
    fObjType.deSerialize(inputStream);           

  }// end of deSerialize()

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Minefield Response");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield Response PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0) superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    fRequesterId.printValues(indentLevel, printStream);
    
    printStream.println(indent + "MFSeqNum: "+fMFSeqNum.intValue());
    printStream.println(indent + "Request Id: "+fRequestId.intValue());
    printStream.println(indent + "PacketNumber: "+fPacketNumber.intValue());
    printStream.println(indent + "TotalPackets: "+fTotalPackets.intValue()); 
    printStream.println(indent + "NumberOfObjects: "+fNumberOfObjects.intValue()); 
    printStream.println(indent + "NumReqSensors: "+fNumOfReqSensors.intValue()); 
    printStream.println(indent + "Filter: "+fFilter.intValue()); 

    fObjType.printValues(indentLevel, printStream);

    return;

  }// end of printValues()

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public EntityID getRequesterID()
  { return (EntityID)fRequesterId.clone();}

  public void setRequesterID(EntityID _fRequesterId)
  {fRequesterId = _fRequesterId; }

  public UnsignedInt getMFSeqNum()
  { return (UnsignedInt)fMFSeqNum.clone();}

  public void setMFSeqNum(UnsignedInt _fMFSeqNum)
  { fMFSeqNum = _fMFSeqNum;}

  public UnsignedByte getRequestId()
  { return (UnsignedByte)fRequestId.clone();}

  public void setRequestId(UnsignedByte _fRequestId)
  { fRequestId = _fRequestId;}

  public UnsignedByte getPacketNumber()
  { return (UnsignedByte)fPacketNumber.clone();}

  public void setPacketNumber(UnsignedByte _fPacketNumber)
  { fPacketNumber = _fPacketNumber;}
  
  public UnsignedByte getTotalPackets()
  { return (UnsignedByte)fTotalPackets.clone();}

  public void setTotalPackets(UnsignedByte _fTotalPackets)
  { fTotalPackets = _fTotalPackets;}
 
  public UnsignedByte getNumberOfObjects()
  { return (UnsignedByte)fNumberOfObjects.clone();}

  public void setNumberOfObjects(UnsignedByte _fNumberOfObjects)
  { fNumberOfObjects = _fNumberOfObjects;}

  public UnsignedByte getNumReqSensors()
  { return (UnsignedByte)fNumOfReqSensors.clone();}

  public void setNumReqSensors(UnsignedByte _fNumReqSensors)
  { fNumOfReqSensors = _fNumReqSensors;}
  
  public UnsignedInt getFilter()
  { return (UnsignedInt)fFilter.clone();}

  public void setFilter(UnsignedInt _fFilter)
  { fFilter = _fFilter;}
                   
  public EntityType getObjType()
  { return (EntityType)fObjType.clone();}

  public void setObjType(EntityType _fObjType)
  { fObjType = _fObjType;}
     
  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield Response PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MineFieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.RequesterID);
    ele1.appendChild(ele);
    fRequesterId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.MFSeqNumber);
    ele.setAttribute(DIS_TAGS.Value,fMFSeqNum.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.RequestID);
    ele.setAttribute(DIS_TAGS.Value,fRequestId.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.PacketNumber);
    ele.setAttribute(DIS_TAGS.Value,fPacketNumber.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.TotalPackets);
    ele.setAttribute(DIS_TAGS.Value,fTotalPackets.toString());
    ele1.appendChild(ele);
   
    ele = outDoc.createElement(DIS_TAGS.NumberOfObjects);
    ele.setAttribute(DIS_TAGS.Value,fNumberOfObjects.toString());
    ele1.appendChild(ele);
   
    ele = outDoc.createElement(DIS_TAGS.NumReqSensors);
    ele.setAttribute(DIS_TAGS.Value,fNumOfReqSensors.toString());
    ele1.appendChild(ele);
   
    ele = outDoc.createElement(DIS_TAGS.Filter);
    ele.setAttribute(DIS_TAGS.Value,fFilter.toString());
    ele1.appendChild(ele);

    fObjType.insert(ele1);

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

                 if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                     extractHeader(aNode);
            
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityType) )
                     fObjType.extract(aNode);
             
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequestID) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fRequestId = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfObjects) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumberOfObjects = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.TotalPackets) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fTotalPackets = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
               
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.PacketNumber) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fPacketNumber = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.Filter) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fFilter = new UnsignedInt(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
            
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumReqSensors) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumOfReqSensors = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
                 
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MFSeqNumber) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fMFSeqNum = new UnsignedInt(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
                                                                                                                             
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineFieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.RequesterID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fRequesterId.extract(bNode);

                     }// end of if

                 }// end of if
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldResponsePdu
