/*
*/

package iwt.mine;                                   

import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**

 *@see ProtocolDataUnit
 *@see PduElement 
 *@see SerializationInterface
 *@see DetonationPdu
 */
public class MineFieldStatePdu extends ProtocolDataUnit 
{

   /**     */
   protected EntityID          fMineFieldId;        // 6 bytes    

   /**
   */
   protected UnsignedByte      fForce;              // 1 byte

   /**
   */
   protected UnsignedByte      fNumPerimeterPts;    // 1 byte               

   /**
   */
   protected EntityType        fType;               // 8 bytes     

   /**
   */
   protected EntityType        fAltType;            // 8 bytes

   /**
   */
   protected UnsignedShort     fNumMineTypes;       // 2 bytes

   /**
   */
   protected UnsignedShort     fNumOfFixObjTypes;   // 2 bytes


   /**
   */
   protected WorldCoordinate   fLocation;      // 24 bytes

   /**
   */
   protected EulerAngle        fOrientation;   // 12 byte

    /**
     *Constant value--size of Fire PDU with header. Here:
     *<code>sizeOf = 64 bytes</code>
     */
    public final static int     sizeOf = 64;                // size of object as written to wire

/**
 *Default constructor--fills with zeros for all values.
 */
public MineFieldStatePdu()
{
    super.setPduType(PDUTypeField.MINEFIELDSTATE); // 190

    fMineFieldId  = new EntityID();
    fForce        = new UnsignedByte(0); 
    fNumPerimeterPts = new UnsignedByte(0);
    fType            = new EntityType();
    fAltType         = new EntityType();
    fNumMineTypes    = new UnsignedShort(0);
    fNumOfFixObjTypes = new UnsignedShort(0);

    fLocation        = new WorldCoordinate(0, 0, 0);
    fOrientation     = new EulerAngle(0, 0, 0);

    return;
}


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFielStatePdu   newMineFielStatePdu = new MineFielStatePdu();

    newMineFielStatePdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newMineFielStatePdu.setForce(this.getForce());
    newMineFielStatePdu.setNumPerimeterPts(this.getNumPerimeterPts());
    newMineFielStatePdu.setFieldType(this.getFieldType());
    newMineFielStatePdu.setFieldAltType(this.getFieldAltType());

    newMineFielStatePdu.setNumMineTypes(this.getNumMineTypes());
    newMineFielStatePdu.setNumOfFixObjTypes(this.getNumOfFixObjTypes());
 
    newMineFielStatePdu.setLocationInWorldCoordinate(this.getLocationInWorldCoordinate());

    newMineFielStatePdu.setOrientation(this.getOrientation());
 
    return newMineFielStatePdu;

  }// end of clone()

  /**
  *@exception RuntimeException when IO error occurs.
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    
    try
    {
      fMineFieldId.serialize(outputStream);  
      fForce.serialize(outputStream);          
      fNumPerimeterPts.serialize(outputStream);   
      fType.serialize(outputStream);              
      fAltType.serialize(outputStream);           
      fNumMineTypes.serialize(outputStream);     
      fNumOfFixObjTypes.serialize(outputStream);   
      fLocation.serialize(outputStream);          
      fOrientation.serialize(outputStream);        

    } catch (IOException ioError)
      {
        throw new 
            RuntimeException("Exception in FirePdu. Error writing to wire.");
      }

    return;
}

/**
 *@exception RuntimeException when IO error occurs.
 */
public void deSerialize(DataInputStream inputStream)
{
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info

    try
    { 
      fMineFieldId.deSerialize(inputStream);  
      fForce.deSerialize(inputStream);          
      fNumPerimeterPts.deSerialize(inputStream);   
      fType.deSerialize(inputStream);              
      fAltType.deSerialize(inputStream);           
      fNumMineTypes.deSerialize(inputStream);     
      fNumOfFixObjTypes.deSerialize(inputStream);   
      fLocation.deSerialize(inputStream);          
      fOrientation.deSerialize(inputStream);        

    } catch (IOException ioError)
      {
            throw new 
                RuntimeException("Exception in FirePdu. Error reading from wire.");
      }

}// 

public int length()
{
    return sizeOf;          // EntityTypes are this long, always.
}

public String PDUName()
{
  return new String("Minefield State");
}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield State PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    printStream.println(indent + "Force: "+fForce.intValue());
    printStream.println(indent + "NumPerimeterPts: "+fNumPerimeterPts.intValue());
    
    fType.printValues(indentLevel, printStream);
    fAltType.printValues(indentLevel, printStream);
    
    printStream.println(indent + "NumMineTypes: "+fNumMineTypes.intValue());
    printStream.println(indent + "NumOfFixObjTypes: "+fNumOfFixObjTypes.intValue());

    printStream.println(indent + "fireMissionIndex: "    + fireMissionIndex.longValue());

    fLocation.printValues(indentLevel, printStream);
    
    fOrientation.printValues(indentLevel, printStream);

    return;
  }

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public UnsignedByte getForce()
  { return (UnsignedByte)fForce.clone();}

  public void setForce(UnsignedByte _fForce)
  { fForce = _fForce;}

  public UnsignedByte getNumPerimeterPts()
  { return (UnsignedByte)fNumPerimeterPts.clone();}

  public void setNumPerimeterPts(UnsignedByte _fNumPerimeterPts)
  { fNumPerimeterPts = _fNumPerimeterPts;}

  public EntityType getFieldType()
  { return (EntityType)fType.clone();}

  public void setFieldType(EntityType _fType)
  { fType = _fType;}

  public EntityType getFieldAltType()
  { return (EntityType)fAltType.clone();}

  public void setFieldAltType(EntityType _fAltType)
  { fAltType = _fAltType;}
  
  public UnsignedShort getNumMineTypes()
  { return (UnsignedShort)fNumMineTypes.clone();}

  public void setNumMineTypes(UnsignedShort _fNumMineTypes)
  { fNumMineTypes = _fNumMineTypes;}

  public UnsignedShort getNumOfFixObjTypes()
  { return (UnsignedShort)fNumOfFixObjTypes.clone();}

  public void setNumOfFixObjTypes(UnsignedShort _fNumOfFixObjTypes)
  { fNumOfFixObjTypes = _fNumOfFixObjTypes;}

  public WorldCoordinate getLocationInWorldCoordinate()
  { return (WorldCoordinate)fLocation.clone();}

  public void setLocationInWorldCoordinate(WorldCoordinate pWorldCoordinate)
  { fLocation = pWorldCoordinate;}

  public void setLocationInWorldCoordinate(double pX, double pY, double pZ)
  { fLocation = new WorldCoordinate(pX, pY, pZ); }
  
  public EulerAngle getOrientation()
  { return (EulerAngle)fOrientation.clone();}

  public void setOrientation(EulerAngle pWorldCoordinate)
  { fOrientation = pWorldCoordinate;}

  public void setOrientation(double pX, double pY, double pZ)
  { fOrientation = new EulerAngle(pX, pY, pZ); }

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield State PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MinefieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);

    ele = outDoc.createElement(DIS_TAGS.Force);
    ele.setAttribute(DIS_TAGS.Value,fForce.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfPerimeterPts);
    ele.setAttribute(DIS_TAGS.Value,fNumPerimeterPts.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.FieldType);
    fType.insert(ele);
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.AlternateFieldType);
    fAltType.insert(ele);
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfMineTypes);
    ele.setAttribute(DIS_TAGS.Value,fNumMineTypes.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfFixObjTypes);
    ele.setAttribute(DIS_TAGS.Value,fNumOfFixObjTypes.toString());
    ele1.appendChild(ele);

    fLocation.insert(ele1);

    fOrientation.insert(ele1);

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

            if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                extractHeader(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.WorldCoordinate) )
                     fLocation.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EulerAngle) )
                     fOrientation.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.Force) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fForce = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfPerimeterPts) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumPerimeterPts = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfMineTypes) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumMineTypes = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfFixObjTypes) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumOfFixObjTypes = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.AlternateFieldType) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityType);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fAltType.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.FieldType) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityType);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fType.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MinefieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldStatePdu