/* */
package iwt.mine;                                   

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
*
*<pre>
*struct cpduMinefieldState_REV {
*   DIS203_PDU_HEADER_RECORD  fHeader;
*   DIS203_ENTITY_ID_RECORD   fMinefieldId;
*   unsigned                  _unused_1 : 1;
*   unsigned                  fSeqNum   : 15;
*   DIS203_FORCE_ID_TYPE      fForceId;
*   unsigned char             fNumPerimeterPts;   // num pts, mines, fixed objs
*   DIS203_ENTITY_TYPE_RECORD           fType;
*   unsigned short                      fNumMineTypes;
*   DIS203_WORLD_COORDINATES_RECORD     fLocation;
*   DIS203_EULER_ANGLES_RECORD          fOrientation;
*   EMfAppearance_REV              fAppearance;
*   unsigned               fProtocolMode :  2;
*   unsigned               _unused_2     : 14;
*   //   TEntCoord                              fPerimeter[i];
*   //   DIS203_ENTITY_TYPE_RECORD           fMineTypes[j];
*
*   int      GetStructSize();         // without variable sized arrays
*};
*</pre>
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MineFieldStateRevPdu extends ProtocolDataUnit 
{

   /**     
   */
   protected EntityID          fMineFieldId;        // 6 bytes    

   /**
   */
   protected UnsignedInt       fSeqNum;              // 4 byte
   
   /**
   */
   protected UnsignedByte      fForce;              // 1 byte

   /**
   */
   protected UnsignedByte      fNumPerimeterPts;    // 1 byte               

   /**
   */
   protected EntityType        fType;               // 8 bytes     

   /**
   */
   protected UnsignedShort     fNumMineTypes;       // 2 bytes

   /**
   */
   protected WorldCoordinate   fLocation;      // 24 bytes

   /**
   */
   protected EulerAngle        fOrientation;   // 12 byte
 
   /**
   */
   protected EMAppearance      fAppearance;   // 12 byte

   /**
   */
   protected UnsignedInt       fProtocolMode;   // 4 bytes
     
   /**
   *Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 64 bytes</code>
   */
   public final static int     sizeOf = 67;                // size of object as written to wire

/**
 *Default constructor--fills with zeros for all values.
 */
public MineFieldStateRevPdu()
{
    super.setPduType(PDUTypeField.MINEFIELDSTATE); // 190

    fMineFieldId  = new EntityID();
    fForce        = new UnsignedByte(0); 
    fNumPerimeterPts = new UnsignedByte(0);
    fType            = new EntityType();
    
    fNumMineTypes    = new UnsignedShort(0);
    
    fLocation        = new WorldCoordinate(0, 0, 0);
    fOrientation     = new EulerAngle(0, 0, 0);

    return;
}


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MineFieldStateRevPdu   newMineFielStatePdu = new MineFieldStateRevPdu();

    newMineFielStatePdu.setMineFieldEntityID(this.getMineFieldEntityID());
    newMineFielStatePdu.setSequenceNumber(this.getSequenceNumber());
    newMineFielStatePdu.setForce(this.getForce());
    newMineFielStatePdu.setNumPerimeterPts(this.getNumPerimeterPts());
    newMineFielStatePdu.setFieldType(this.getFieldType());

    newMineFielStatePdu.setNumMineTypes(this.getNumMineTypes());
 
    newMineFielStatePdu.setLocationInWorldCoordinate(this.getLocationInWorldCoordinate());

    newMineFielStatePdu.setOrientation(this.getOrientation());
    
    newMineFielStatePdu.setEMAppearance(this.getEMAppearance());

    newMineFielStatePdu.setProtocolMode(this.getProtocolMode());

    return newMineFielStatePdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    
    UnsignedInt padding1 = new UnsignedInt(1);
    UnsignedInt padding2 = new UnsignedInt(14);    
    
    fMineFieldId.serialize(outputStream);
    padding1.serialize(outputStream); 
    fSeqNum.serialize(outputStream);          
    fForce.serialize(outputStream);          
    fNumPerimeterPts.serialize(outputStream);   
    fType.serialize(outputStream);              
    fNumMineTypes.serialize(outputStream);     
    fLocation.serialize(outputStream);          
    fOrientation.serialize(outputStream);        
    fAppearance.serialize(outputStream);          
    fProtocolMode.serialize(outputStream); 
    padding2.serialize(outputStream);       

    return;
  }

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info

    UnsignedInt padding1 = new UnsignedInt(1);
    UnsignedInt padding2 = new UnsignedInt(1);    
    
    fMineFieldId.deSerialize(inputStream);
    padding1.deSerialize(inputStream); 
    fSeqNum.deSerialize(inputStream);          
    fForce.deSerialize(inputStream);          
    fNumPerimeterPts.deSerialize(inputStream);   
    fType.deSerialize(inputStream);              
    fNumMineTypes.deSerialize(inputStream);     
    fLocation.deSerialize(inputStream);          
    fOrientation.deSerialize(inputStream);        
    fAppearance.deSerialize(inputStream);          
    fProtocolMode.deSerialize(inputStream); 
    padding2.deSerialize(inputStream);       

  }// 

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Minefield State");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield State PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0)
      superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineFieldId.printValues(indentLevel, printStream);
    printStream.println(indent + "Force: "+fForce.intValue());
    printStream.println(indent + "NumPerimeterPts: "+fNumPerimeterPts.intValue());
    
    fType.printValues(indentLevel, printStream);
    
    printStream.println(indent + "NumMineTypes: "+fNumMineTypes.intValue());

    fLocation.printValues(indentLevel, printStream);
    fOrientation.printValues(indentLevel, printStream);
    fAppearance.printValues(indentLevel, printStream);
    printStream.println(indent + "ProtocolMode "+fProtocolMode.intValue());
    return;
  }

  //Accessor methods
  public EntityID getMineFieldEntityID()
  { return (EntityID)fMineFieldId.clone();}

  public void setMineFieldEntityID(EntityID _fMineFieldId)
  {fMineFieldId = _fMineFieldId; }
 
  public UnsignedInt getSequenceNumber()
  { return (UnsignedInt)fSeqNum.clone();}

  public void setSequenceNumber(UnsignedInt _fSeqNum)
  { fSeqNum = _fSeqNum;}
 
  public UnsignedByte getForce()
  { return (UnsignedByte)fForce.clone();}

  public void setForce(UnsignedByte _fForce)
  { fForce = _fForce;}

  public UnsignedByte getNumPerimeterPts()
  { return (UnsignedByte)fNumPerimeterPts.clone();}

  public void setNumPerimeterPts(UnsignedByte _fNumPerimeterPts)
  { fNumPerimeterPts = _fNumPerimeterPts;}

  public EntityType getFieldType()
  { return (EntityType)fType.clone();}

  public void setFieldType(EntityType _fType)
  { fType = _fType;}

  public UnsignedShort getNumMineTypes()
  { return (UnsignedShort)fNumMineTypes.clone();}

  public void setNumMineTypes(UnsignedShort _fNumMineTypes)
  { fNumMineTypes = _fNumMineTypes;}

  public WorldCoordinate getLocationInWorldCoordinate()
  { return (WorldCoordinate)fLocation.clone();}

  public void setLocationInWorldCoordinate(WorldCoordinate pWorldCoordinate)
  { fLocation = pWorldCoordinate;}

  public void setLocationInWorldCoordinate(double pX, double pY, double pZ)
  { fLocation = new WorldCoordinate(pX, pY, pZ); }
  
  public EulerAngle getOrientation()
  { return (EulerAngle)fOrientation.clone();}

  public void setOrientation(EulerAngle pWorldCoordinate)
  { fOrientation = pWorldCoordinate;}

  public void setOrientation(float pX, float pY, float pZ)
  { fOrientation = new EulerAngle(pX, pY, pZ); }

  public EMAppearance getEMAppearance()
  { return (EMAppearance)fAppearance.clone();}

  public void setEMAppearance(EMAppearance _fAppearance)
  { fAppearance = _fAppearance;}

  public UnsignedInt getProtocolMode()
  { return (UnsignedInt)fProtocolMode.clone();}

  public void setProtocolMode(UnsignedInt _fProtocolMode)
  { fProtocolMode = _fProtocolMode;}

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
    Text            txt1;

    //System.out.println("Entering Minefield State PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    ele = outDoc.createElement(DIS_TAGS.MineFieldID);
    ele1.appendChild(ele);
    fMineFieldId.insert(ele);
   
    ele = outDoc.createElement(DIS_TAGS.SequenceNumber);
    ele.setAttribute(DIS_TAGS.Value,fSeqNum.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.Force);
    ele.setAttribute(DIS_TAGS.Value,fForce.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfPerimeterPts);
    ele.setAttribute(DIS_TAGS.Value,fNumPerimeterPts.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.FieldType);
    fType.insert(ele);
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumberOfMineTypes);
    ele.setAttribute(DIS_TAGS.Value,fNumMineTypes.toString());
    ele1.appendChild(ele);

    fLocation.insert(ele1);

    fOrientation.insert(ele1);

    fAppearance.insert(ele1);

    ele = outDoc.createElement(DIS_TAGS.ProtocolMode);
    ele.setAttribute(DIS_TAGS.Value,fProtocolMode.toString());
    ele1.appendChild(ele);

  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

            if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                extractHeader(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.WorldCoordinate) )
                     fLocation.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EulerAngle) )
                     fOrientation.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.Force) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fForce = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfPerimeterPts) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumPerimeterPts = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfMineTypes) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumMineTypes = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.ProtocolMode) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fProtocolMode = new UnsignedInt(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
           
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.SequenceNumber) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fSeqNum = new UnsignedInt(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.FieldType) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityType);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fType.extract(bNode);

                     }// end of if

                 }// end of if

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineFieldID) )

                 {
                     aList = ((Element)aNode).getElementsByTagName(DIS_TAGS.EntityID);

                     if( aList!=null && aList.getLength()>0 )
                     {
                        Node bNode = nodes.item(0);
                        if( bNode!=null && 
                            bNode.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE ) 
                            fMineFieldId.extract(bNode);

                     }// end of if

                 }// end of if
                       
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EMAppearance) )
                     fAppearance.extract(aNode);

        }// end of if
        
        i++;

     }// end of while

  }// end of read()

} // end of class MineFieldStatePdu
