/*
*/
package iwt.mine; 
                                  
import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;                                         // utility stuff we need
import java.io.*;                                           // input/output for serialization

import iwt.*;
import iwt.util.*;
import iwt.util.hexConvert;
import org.w3c.dom.*;

/**
*
*<pre>
*struct cpduCpMine {
*   DIS203_PDU_HEADER_RECORD    fHeader;
*   DIS203_ENTITY_ID_RECORD     mine_set_id;
*   TSequence                      q_sequence;
*   //   unsigned short                 quiescent : 1;
*   //   unsigned short                 sequence : 15;
*   DIS203_ENTITY_TYPE_RECORD   munition;
*   DIS203_FUZE_TYPE            fuze;   // unint16
*   unsigned short                 num_mines;
*   DIS203_WORLD_COORDINATES_RECORD   ref_location;
*  
*   TMineDataFlag                  data_flags;  // DisPdu203.h
*   //
*   // struct TMineDataFlag
*   // {
*   //    unsigned int orientation_flag  :  1;
*   //    unsigned int burial_depth_flag :  1;
*   //    unsigned int temperature_flag  :  1;
*   //    unsigned int _unused_          : 29;
*   //  };
*   //
*   //   unsigned int                   orientation_flag : 1;
*   //   unsigned int                   burial_depth_flag : 1;
*   //   unsigned int                   temperature_flag : 1;
*   //   unsigned int                   _unused_ : 29;
*
*   DIS203_VECTOR_RECORD          locations[ 1 // num_mines // ];
*  
*   // optional
*   //   DIS203_EULER_ANGLES_RECORD  orientations[num_mines];
*   //   float                          burial_depths[num_mines];
*   //   float                          temperatures[num_mines];
*};
* </pre>
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MinePdu extends ProtocolDataUnit implements IWTDebug
{

   /**     */
   protected EntityID          fMineSetId;        // 6 bytes    
 
   /**     */
   protected TSequence         fQSequence;        // 4 bytes    
  
   /**     */
   protected EntityType        fMunition;         // 6 bytes    
 
   /**
   */
   protected UnsignedShort      fFuzeType;          // 1 byte

   /**
   */
   protected UnsignedShort      fnum_mines;       // 2 byte               

   /**
   */
   protected WorldCoordinate   fRefLocation;      // 24 bytes

   /**
   */
   protected MineDataFlag      fMineDataFlag;       // 1 bytes
   
   /** */
   protected VectorRecord[]    fLocations;          // 1 byte  
             
   /** */
   protected EulerAngle[]      fOrientations;       // 1 byte               
             
   /** */
   protected float[]      bDepth;       // 1 byte               
             
   /** */
   protected float[]      bThermal;       // 1 byte               

   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 24 bytes</code>
   */
   public final static int     sizeOf = 24;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MinePdu()
   {
      super.setPduType(PDUTypeField.MINE); // 189
      fMineSetId       = new EntityID();
      fQSequence       = new TSequence();
      fMunition        = new EntityType(); 
      fnum_mines       = new UnsignedShort(0);
      fRefLocation     = new WorldCoordinate(0,0,0);
      fLocations       = null;
      fOrientations    = null;
      bDepth           = null;
      bThermal         = null;
      fFuzeType        = new UnsignedShort(0);
      fMineDataFlag    = new MineDataFlag();
 
      return;
   }


  public Object clone()
  {
    int i;
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MinePdu   newPdu = new MinePdu();

    newPdu.setMineFieldEntityID(this.getMineEntityID());
    newPdu.setQSequence(this.getQSequence());
    newPdu.setMunition(this.getMunition());
    newPdu.setFuzeType(this.getFuzeType());
    newPdu.setNumOfMines(this.getNumOfMines());
    newPdu.setRefLocation(this.getRefLocation());
    newPdu.setMineDataFlag(this.getMineDataFlag());
  
    if(fLocations!=null) 
       for(i=0;i<fnum_mines.intValue();i++) 
        newPdu.setLocationAt(i,this.getLocationAt(i));

    if(this.getMineDataFlag().getOrientationFlag())
      for(i=0;i<fnum_mines.intValue();i++) 
        newPdu.setOrientationAt(i,this.getOrientationAt(i));

    if(this.getMineDataFlag().getBurialDepthFlag())
      for(i=0;i<fnum_mines.intValue();i++) 
        newPdu.setBurialDepthAt(i,this.getBurialDepthAt(i));

    if(this.getMineDataFlag().getTemperature())
      for(i=0;i<fnum_mines.intValue();i++) 
        newPdu.setTemperatureAt(i,this.getTemperatureAt(i));

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    int i=0;
    super.serialize(outputStream);      // write out header info
  
      //System.out.println(" before ");
    fMineSetId.serialize(outputStream); 
      //System.out.println(" fMineSetId"); 
    fQSequence.serialize(outputStream);
      //System.out.println(" fQSequence");     
    fMunition.serialize(outputStream);         
      //System.out.println(" fMunition");    
    fFuzeType.serialize(outputStream);         
      //System.out.println(" fFuzeType");
    fnum_mines.serialize(outputStream);   
      //System.out.println(" fnum_mines");
    fRefLocation.serialize(outputStream);              
      //System.out.println(" fRefLocation");
    fMineDataFlag.serialize(outputStream);           
      //System.out.println(" fMineDataFlag");
    //fLocations.serialize(outputStream); 
      //System.out.println(" fLocations");
    if(fLocations!=null) 
       for(i=0;i<fnum_mines.intValue();i++) 
           fLocations[i].serialize(outputStream); 
   
    if(fMineDataFlag.getOrientationFlag())
      for(i=0;i<fnum_mines.intValue();i++) 
          fOrientations[i].serialize(outputStream); 

    if(fMineDataFlag.getBurialDepthFlag())
      try
      {
        for(i=0;i<fnum_mines.intValue();i++) 
         outputStream.writeFloat(bDepth[i]);

      } catch(java.io.IOException ioe) { throw new RuntimeException(""); }

    if(fMineDataFlag.getTemperature())
      try
      {
        for(i=0;i<fnum_mines.intValue();i++) 
         outputStream.writeFloat(bThermal[i]);

      } catch(java.io.IOException ioe) { throw new RuntimeException(""); }

    return;

  }// end of serialize()

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    int i=0;

    super.deSerialize(inputStream);     // read in all the header info
   
    fMineSetId.deSerialize(inputStream);
     if(MDEBUG) System.out.println(" fMineSetId");  
     if(MDEBUG) fMineSetId.printValues(2,System.out);
    fQSequence.deSerialize(inputStream);
     if(MDEBUG) System.out.println(" fQSequence");  
     if(MDEBUG) fQSequence.printValues(2,System.out);
    fMunition.deSerialize(inputStream);         
     if(MDEBUG) System.out.println(" fMunition");  
     if(MDEBUG) fMunition.printValues(2,System.out);
    fFuzeType.deSerialize(inputStream); 
     if(MDEBUG) System.out.print(" fFuzeType ..: "+fFuzeType.intValue());        
     if(MDEBUG) System.out.println(" "+hexConvert.toHexStr(fFuzeType.shortValue()));        
    fnum_mines.deSerialize(inputStream);
     if(MDEBUG) System.out.print(" fnum_mines .: "+fnum_mines.intValue());          
     if(MDEBUG) System.out.println(" "+hexConvert.toHexStr(fnum_mines.shortValue()));        
    setNumOfMines(fnum_mines);
    fRefLocation.deSerialize(inputStream);              
     if(MDEBUG) System.out.println(" reference Location");        
     if(MDEBUG) fRefLocation.printValues(2,System.out);
    fMineDataFlag.deSerialize(inputStream);           
     if(MDEBUG) fMineDataFlag.printValues(2,System.out);

    if(fLocations!=null) 
       for(i=0;i<fnum_mines.intValue();i++) 
       {
           if(MDEBUG) System.out.println(" Location["+i+"]");        
           fLocations[i].deSerialize(inputStream); 
           if(MDEBUG) fLocations[i].printValues(2,System.out);
       }

    if(getMineDataFlag().getOrientationFlag())
      for(i=0;i<fnum_mines.intValue();i++) 
          fOrientations[i].deSerialize(inputStream); 

    if(fMineDataFlag.getBurialDepthFlag())
      try
      {
        for(i=0;i<fnum_mines.intValue();i++) 
         bDepth[i] = inputStream.readFloat();

      } catch(java.io.IOException ioe) { throw new RuntimeException(""); }

    if(fMineDataFlag.getTemperature())
      try
      {
        for(i=0;i<fnum_mines.intValue();i++) 
         bThermal[i] = inputStream.readFloat();

      } catch(java.io.IOException ioe) { throw new RuntimeException(""); }



  }// end of deSerialize()

  public int length() { return sizeOf; }

  public String PDUName(){ return new String("Mine");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0, i;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Mine PDU-");

    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0) superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    fMineSetId.printValues(indentLevel, printStream);
    fQSequence.printValues(indentLevel, printStream);
    fMunition.printValues(indentLevel, printStream);

    printStream.println(indent + "Fuze Type: "+fFuzeType);
    printStream.println(indent + "Number Of Mines: "+fnum_mines);
  
    fRefLocation.printValues(indentLevel, printStream);
  
    fMineDataFlag.printValues(indentLevel, printStream);

    //fLocations.printValues(indentLevel, printStream);
    if(fLocations!=null) 
    {  
       printStream.println(indent + "<<**Mine Locations");
       for(i=0;i<fnum_mines.intValue();i++) 
           fLocations[i].printValues(indentLevel+1, printStream);
       printStream.println(indent + "Mine Locations**>>");

    }// end of if

    if(fMineDataFlag.getOrientationFlag() && fOrientations!=null) 
    {  
       printStream.println(indent + "<<**Mine Orientations");
       for(i=0;i<fnum_mines.intValue();i++) 
           fOrientations[i].printValues(indentLevel+1, printStream);
       printStream.println(indent + "Mine Orientations**>>");

    }// end of if

    if(fMineDataFlag.getBurialDepthFlag() && bDepth!=null) 
    {  
       printStream.println(indent + "<<**Mine Depth");
       for(i=0;i<fnum_mines.intValue();i++) 
           printStream.println(indent + "depth["+i+"] :"+bDepth[i]);
       printStream.println(indent + "Mine Depth**>>");

    }// end of if

    if(fMineDataFlag.getBurialDepthFlag() && bThermal!=null) 
    {  
       printStream.println(indent + "<<**Mine Thermal");
       for(i=0;i<fnum_mines.intValue();i++) 
           printStream.println(indent + "depth["+i+"] :"+bThermal[i]);
       printStream.println(indent + "Mine Thermal**>>");

    }// end of if

    return;
  }
 
  //Accessor methods
  public EntityID getMineEntityID()
  { return (EntityID)fMineSetId.clone();}

  public void setMineFieldEntityID(EntityID _fMineSetId)
  { fMineSetId = _fMineSetId; }

  public TSequence getQSequence()
  { return (TSequence)fQSequence.clone();}

  public void setQSequence(TSequence _fQSequence)
  { fQSequence = _fQSequence; }
  
  public EntityType getMunition()
  { return (EntityType)fMunition.clone();}

  public void setMunition(EntityType _fMunition)
  { fMunition = _fMunition; }
 
  public UnsignedShort getNumOfMines()
  { return (UnsignedShort)fnum_mines.clone();}

  public void setNumOfMines(UnsignedShort _fnum_mines)
  { 
    fnum_mines = _fnum_mines; 
    fLocations=null;
    if(fnum_mines.intValue()>0)
    { fLocations = new VectorRecord[fnum_mines.intValue()]; 
      for(int i=0;i<fnum_mines.intValue();i++) 
          fLocations[i]=new VectorRecord();
      fOrientations = new EulerAngle[fnum_mines.intValue()]; 
      for(int i=0;i<fnum_mines.intValue();i++) 
          fOrientations[i]=new EulerAngle();
      bDepth = new float[fnum_mines.intValue()]; 
      bThermal = new float[fnum_mines.intValue()]; 
    }
  }
 
  public WorldCoordinate getRefLocation()
  { return (WorldCoordinate)fRefLocation.clone();}

  public void setRefLocation(WorldCoordinate _fRefLocation)
  { fRefLocation = _fRefLocation; }
 
  public EulerAngle getOrientationAt(int i)
  { 
     EulerAngle ret = null;
     if(i>=0&&i<fnum_mines.intValue()&&fOrientations!=null&&i<fOrientations.length)
        ret = (EulerAngle)fOrientations[i].clone();
     return ret;
  }

  public void setOrientationAt(int i, EulerAngle _fRefLocation)
  { 
     if(i>=0&&i<fnum_mines.intValue()&&fOrientations!=null&&i<fOrientations.length)
        fOrientations[i] = _fRefLocation;
  }
  public float getBurialDepthAt(int i)
  { 
     float ret = 0;
     if(i>=0&&i<fnum_mines.intValue()&&bDepth!=null&&i<bDepth.length)
        ret = bDepth[i];
     return ret;
  }

  public void setBurialDepthAt(int i, float _fDepth)
  { 
     if(i>=0&&i<fnum_mines.intValue()&&bDepth!=null&&i<bDepth.length)
        bDepth[i] = _fDepth;
  }

  public float getTemperatureAt(int i)
  { 
     float ret = 0;
     if(i>=0&&i<fnum_mines.intValue()&&bThermal!=null&&i<bThermal.length)
        ret = bThermal[i];
     return ret;
  }

  public void setTemperatureAt(int i, float _fThermal)
  { 
     if(i>=0&&i<fnum_mines.intValue()&&bThermal!=null&&i<bThermal.length)
        bThermal[i] = _fThermal;
  }

  public MineDataFlag getMineDataFlag()
  { return (MineDataFlag)fMineDataFlag.clone();}

  public void setMineDataFlag(MineDataFlag _fMineDataFlag)
  { fMineDataFlag = _fMineDataFlag; }

  /*
  * Returns the iTH Mine's Location 
  */
  public VectorRecord getLocationAt(int i)
  { 
     VectorRecord ret = null;
     if(i>=0&&i<fnum_mines.intValue()&&fLocations!=null&&i<fLocations.length)
        ret = (VectorRecord)fLocations[i].clone();
     return ret;

  }// end of getLocationAt()

  public void setLocationAt(int i, VectorRecord _fLocations)
  { 
     if(i>=0&&i<fnum_mines.intValue()&&fLocations!=null&&i<fLocations.length)
        fLocations[i] = _fLocations; 

  }// end of setLocations()

  public UnsignedShort getFuzeType()
  { return (UnsignedShort)fFuzeType.clone();}

  public void setFuzeType(UnsignedShort _fFuzeType)
  { fFuzeType = _fFuzeType; }

  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;

    //System.out.println("Entering Minefield Response PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    fMineSetId.insert(ele1);
    fQSequence.insert(ele1);
    fMunition.insert(ele1);
    
    ele = outDoc.createElement(DIS_TAGS.FuzeType);
    ele.setAttribute(DIS_TAGS.Value,fFuzeType.toString());
    ele1.appendChild(ele);
    
    ele = outDoc.createElement(DIS_TAGS.NumberOfMines);
    ele.setAttribute(DIS_TAGS.Value,fnum_mines.toString());
    ele1.appendChild(ele);

    //fLocations.insert(ele1);
    if(fLocations!=null) 
       for(int i=0;i<fnum_mines.intValue();i++) fLocations[i].insert(ele);

    fRefLocation.insert(ele1);
    
    fMineDataFlag.insert(ele1);


  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0,j;

     if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

                 if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                     extractHeader(aNode);
                
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityID) )
                     fMineSetId.extract(aNode);
                          
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.TSequence) )
                     fQSequence.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityType) )
                     fMunition.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumberOfMines) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fnum_mines = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                     
                     setNumOfMines(fnum_mines);
 
                     NodeList bList = ((Element)aNode).getElementsByTagName(DIS_TAGS.VectorRecord);
                     if( bList!=null && bList.getLength()>0)
                        for(j=0;j<bList.getLength();j++)
                            fLocations[j].extract(bList.item(j));

                 }

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.FuzeType) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fFuzeType = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.WorldCoordinate) )
                     fRefLocation.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.MineDataFlag) )
                     fMineDataFlag.extract(aNode);
           
            /*else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.VectorRecord) )
                     fLocations.extract(aNode);*/
                                   
            
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MinePdu
