/*
*/
package iwt.mine; 
                                  
import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;            // General-purpose utilities
import mil.navy.nps.disEnumerations.*; // Enumerations for DIS

import java.lang.*;
import java.util.*;                    // utility stuff we need
import java.io.*;                      // input/output for serialization

import iwt.*;
import iwt.util.hexConvert;
import org.w3c.dom.*;

/**
*
*<pre>
*struct cpduCpMinefield {
*   DIS203_PDU_HEADER_RECORD    fHeader;
*   DIS203_ENTITY_ID_RECORD     minefield_id;
*   TSequence                      q_sequence;
*   //   unsigned short                 quiescent : 1;
*   //   unsigned short                 sequence : 15;
*   unsigned short                 appearance;
*   unsigned char                  num_vertices;
*   unsigned char                  force_id;
* 
*   //      present in PDU as sent, dynamically sized
*   DIS203_WORLD_COORDINATES_RECORD        vertices[ 1 // num_vertices //];
*
*};
*</pre>
*
* @see ProtocolDataUnit
* @see PduElement 
* @see SerializationInterface
* @see DetonationPdu
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class MinefieldPdu extends ProtocolDataUnit implements IWTDebug
{

   /**     */
   protected EntityID          fMineSetId;        // 6 bytes    
 
   /**     */
   protected TSequence         fQSequence;        // 4 bytes    
   
   /**
   */
   protected UnsignedShort     fAppearance;       // 2 byte                
   
   /**
   */
   protected UnsignedByte      fNumOfVertices;    // 1 byte                
    
   /**
   */
   protected UnsignedByte      fForceID;          // 1 byte                
 
   /**
   */
   protected WorldCoordinate[]   fRefLocationVertices;    // 12 bytes

   /**
   * Constant value--size of Fire PDU with header. Here:
   *<code>sizeOf = 14 bytes</code>
   */
   public final static int     sizeOf = 14;                // size of object as written to wire

   /**
   *Default constructor--fills with zeros for all values.
   */
   public MinefieldPdu()
   {
      super.setPduType(PDUTypeField.MINEFIELD); // 189
      fMineSetId           = new EntityID();
      fQSequence           = new TSequence();
      fAppearance          = new UnsignedShort(0);     
      fNumOfVertices       = new UnsignedByte(0);
      fForceID             = new UnsignedByte(0);
      fRefLocationVertices = null;
      return;
   }


  public Object clone()
  {
    // make a copy of the object. This requires a deep copy, so we don't have two
    // objects sharing pointers to the same data.

    MinefieldPdu   newPdu = new MinefieldPdu();

    newPdu.setMinefieldEntityID(this.getMinefieldEntityID());
    newPdu.setQSequence(this.getQSequence());
    newPdu.setAppearance(this.getAppearance());
    newPdu.setNumOfVertices(this.getNumOfVertices());
    newPdu.setForceID(this.getForceID());
    
    if(fNumOfVertices.intValue()>0)
     for(int i=0;i<fNumOfVertices.intValue();i++)
         newPdu.setRefLocationVerticeAt(i,this.getRefLocationVerticeAt(i));

    return newPdu;

  }// end of clone()

  /**
  */
  public void serialize(DataOutputStream outputStream)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    super.serialize(outputStream);      // write out header info
    fMineSetId.serialize(outputStream);  
    fQSequence.serialize(outputStream);
    fAppearance.serialize(outputStream);         
    fNumOfVertices.serialize(outputStream);   
    fForceID.serialize(outputStream);              
               
    if(fNumOfVertices.intValue()>0)
     for(int i=0;i<fNumOfVertices.intValue();i++)
         fRefLocationVertices[i].serialize(outputStream);           
    return;

  }// end of serialize()

  /**
  */
  public void deSerialize(DataInputStream inputStream)
  {
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    
    super.deSerialize(inputStream);     // read in all the header info

     if(MDEBUG) System.out.println(" before **deSerialize()** ");
    fMineSetId.deSerialize(inputStream); 
     if(MDEBUG) System.out.println(" fMineSetId"); 
    fQSequence.deSerialize(inputStream);
    fAppearance.deSerialize(inputStream);         
     if(MDEBUG) 
         System.out.println(" fAppearance"+hexConvert.toHexStr(fAppearance.shortValue())); 
    fNumOfVertices.deSerialize(inputStream); 
    setNumOfVertices(fNumOfVertices);  
    fForceID.deSerialize(inputStream);              
    if(fNumOfVertices.intValue()>0)
     for(int i=0;i<fNumOfVertices.intValue();i++)
     {
         fRefLocationVertices[i].deSerialize(inputStream);           
         if(MDEBUG) fRefLocationVertices[i].printValues(1,System.out);

     }
     if(MDEBUG) System.out.println(" end **deSerialize()** ");
  }// end of deSerialize()

  public int length() { return sizeOf; }

  public String PDUName() { return new String("Minefield ");}

  public void printValues(int indentLevel, PrintStream printStream)
  {
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;
    int     superclassIndent = indentLevel;     // level of indentation for our superclass

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println();
    printStream.println("Minefield PDU-");


    // ugly wart: get the superclass over to the left a couple pixels, if we have any to spare,
    // so the header info will be indented a bit less.

    if(superclassIndent > 0) superclassIndent -= 1;

    super.printValues(superclassIndent, printStream);

    printStream.println(indent + "MineSetID");
    fMineSetId.printValues(indentLevel+1, printStream);

    printStream.println(indent + "TQSequence");
    fQSequence.printValues(indentLevel+1, printStream);

    printStream.println(indent + "Appearance: "+fAppearance.intValue());
    printStream.println(indent + "Number Of Vertices: "+fNumOfVertices.intValue());
    printStream.println(indent + "ForceID: "+fForceID.intValue());    

    if( fNumOfVertices.intValue()>0 )
    {
      printStream.println(indent + "<<**Vertices");
      for(int i=0;i<fNumOfVertices.intValue();i++)    
        fRefLocationVertices[i].printValues(indentLevel+1, printStream);
      printStream.println(indent + "Vertices**>>");    
    }//end of if

    return;
  }
  
  //Accessor methods
  public EntityID getMinefieldEntityID()
  { return (EntityID)fMineSetId.clone();}

  public void setMinefieldEntityID(EntityID _fMineSetId)
  { fMineSetId = _fMineSetId; }

  public TSequence getQSequence()
  { return (TSequence)fQSequence.clone();}

  public void setQSequence(TSequence _fQSequence)
  { fQSequence = _fQSequence; }
  
  public UnsignedShort getAppearance()
  { return (UnsignedShort)fAppearance.clone();}

  public void setAppearance(UnsignedShort _fAppearance)
  { fAppearance = _fAppearance; }

  public UnsignedByte getNumOfVertices()
  { return (UnsignedByte)fNumOfVertices.clone();}

  public void setNumOfVertices(UnsignedByte _fNumOfVertices)
  { 
    fNumOfVertices = _fNumOfVertices; 
  
    if( fNumOfVertices.intValue()>0)
    {  fRefLocationVertices = new WorldCoordinate[fNumOfVertices.intValue()];
       for(int i=0;i<fNumOfVertices.intValue();i++)
           fRefLocationVertices[i] = new WorldCoordinate();
    }// end of if
  
  }// end of 

  public UnsignedByte getForceID()
  { return (UnsignedByte)fForceID.clone();}

  public void setForceID(UnsignedByte _fForceID)
  { fForceID = _fForceID; }

  public WorldCoordinate getRefLocationVerticeAt(int i)
  { 
    WorldCoordinate ret = null;
    if( i>=0&&fNumOfVertices!=null&&i<fNumOfVertices.intValue()&&
        fRefLocationVertices!=null&&i<fRefLocationVertices.length)
        ret = (WorldCoordinate)fRefLocationVertices[i].clone(); 

    return ret;

  }// end of getRefLocationVerticeAt()

  public void setRefLocationVerticeAt(int i, WorldCoordinate _fRefLocationVertices)
  { 
    if( i>=0&&fNumOfVertices!=null&&i<fNumOfVertices.intValue()&&
        fRefLocationVertices!=null&&i<fRefLocationVertices.length)
        fRefLocationVertices[i] = _fRefLocationVertices;

  }// end of setRefLocationVerticeAt()
  
  public void
  write(org.w3c.dom.Document outDoc) 
  {
    insert(outDoc);

  }// end of write()

  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {
  }// end of read()

  public void
  insert(org.w3c.dom.Node outNode) 
  {

    Element         ele;
    Element         ele1;
 
    //System.out.println("Entering Minefield Response PDU serialization");
    Document outDoc = outNode.getOwnerDocument();
    ele1 = outDoc.createElement(iwt.DIS_TAGS.PDU);
    outNode.appendChild(ele1);

    // write out header info
    insertHeader(ele1);

    fMineSetId.insert(ele1);
    fQSequence.insert(ele1);

    ele = outDoc.createElement(DIS_TAGS.Appearance);
    ele.setAttribute(DIS_TAGS.Value,fAppearance.toString());
    ele1.appendChild(ele);

    ele = outDoc.createElement(DIS_TAGS.NumOfVertices);
    ele.setAttribute(DIS_TAGS.Value,fNumOfVertices.toString());
    ele1.appendChild(ele);

    for(int i=0;i<fNumOfVertices.intValue();i++) 
        fRefLocationVertices[i].insert(ele);

    ele = outDoc.createElement(DIS_TAGS.ForceID);
    ele.setAttribute(DIS_TAGS.Value,fForceID.toString());
    ele1.appendChild(ele);
  
  }// end of insert()

  /**
  */
  public void
  extract(org.w3c.dom.Node inNode) throws InvalidDocumentType
  {

     Element         ele;
     NodeList        nodes;
     NodeList        aList;
     int i=0;

     if( !inNode.getNodeName().equals(iwt.DIS_TAGS.PDU) )
        throw new InvalidDocumentType();

     nodes = inNode.getChildNodes();

     while( i<nodes.getLength() )
     {
        Node aNode = nodes.item(i);

        if( aNode!=null )
        {

                 if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.DIS_PDU_Header) )
                     extractHeader(aNode);
                
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.EntityID) )
                     fMineSetId.extract(aNode);
                          
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.TSequence) )
                     fQSequence.extract(aNode);

            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.Appearance) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fAppearance = new UnsignedShort(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
         
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.NumOfVertices) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fNumOfVertices = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                     
                 }
                                         
            else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                     aNode.getNodeName().equals(iwt.DIS_TAGS.ForceID) )
                 {
                     String val1 = ((Element)aNode).getAttribute(DIS_TAGS.Value);
                     try
                     {  fForceID = new UnsignedByte(Integer.parseInt(val1));
                     } catch(NumberFormatException nfe) {} 
                 }
                        
           /*else if( aNode.getNodeType() == Node.ELEMENT_NODE &&
                    aNode.getNodeName().equals(iwt.DIS_TAGS.WorldCoordinate) )
                    fRefLocationVertices.extract(aNode);*/
                                   
  
        }// end of if

        i++;

     }// end of while

  }// end of read()

} // end of class MinefieldPdu
