
package iwt.mine;

import mil.navy.nps.dis.*;         
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import org.w3c.dom.*;

/** 
*
* <pre>
* struct TSequence {
* unsigned short quiescent : 1;
* unsigned short sequence  : 15;
* };
* </pre>
*
* @see PduElement 
* @see SerializationInterface
* 
* @author Hasan Timucin Ozdemir
* @date January 26, 1999
*
*/
public class TSequence extends PduElement
{
   /** */
   boolean   bquiescent;

   /** */
   short     bsequence;

   /**
   * Constant value--size of a TSequence when written out; 
   * here :<code>sizeOf = 2 bytes</code>.
   */
   public final int sizeOf = 2;   // object is 2 bytes long 

   /**
   *Constructs an new TSequence
   */
   public TSequence()
   {
     // default constructor
     bquiescent = false; 
     bsequence  = 0; 
   }

   /**
   *
   * @param _quiescent
   * @param _sequence
   */
   public TSequence(boolean _quiescent, short _sequence)
   {  bquiescent = _quiescent; bsequence = _sequence; }

   /** */
   public Object clone()
   {
     TSequence   newTSequence = new TSequence(bquiescent, bsequence);

     return newTSequence;
   }

   /** */
   public void serialize(DataOutputStream outputStream)
   {
     UnsignedShort data = new UnsignedShort((bquiescent?0x00008000:0)+bsequence);
     data.serialize(outputStream);
     
   }// end of serialize()
  
   /** */
   public void deSerialize(DataInputStream pInputStream)
   {
     UnsignedShort data = new UnsignedShort(0);
     data.deSerialize(pInputStream);
     bsequence = (short)(0x00007fff&data.intValue());
     int a = (0x00008000&data.intValue());
     bquiescent = (a>0) ;

   }// end of deSerialize()

   /** */
   public int length() { return sizeOf;}

   /** */
   public void printValues(int indentLevel, PrintStream printStream)
   {
     // print the values of the object out, with correct level of
     // indentation on the page.

     char[]  indent = new char[indentLevel];
     int     idx = 0;
     String  spacing;

     for(idx = 0; idx < indentLevel; idx++) indent[idx] = ' ';

     spacing = new String(indent);

     printStream.println(indent + " quiescent: " + bquiescent);
     printStream.println(indent + "  sequence: " + bsequence);

   }// end of printValues()

   // Accessor methods
   public void setQuiescent()    { bquiescent = true; }

   public void resetQuiescent()  { bquiescent = false; }

   public boolean getQuiescent() { return bquiescent;}

   public short   getSequence()  { return bsequence;}

   public void setSequence(short _sequence) { bsequence = _sequence;}

   /**   */
   public void 
   write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

   /**
   */
   public void 
   insert(org.w3c.dom.Node outNode)
   {
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode);

   }// end of insert()

   /**
   */
   public void 
   insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
   {   

     org.w3c.dom.Element ele = outDoc.createElement(iwt.DIS_TAGS.TSequence);
     ele.setAttribute(iwt.DIS_TAGS.Quiescent,(bquiescent?"true":"false"));
     ele.setAttribute(iwt.DIS_TAGS.Sequence,""+bsequence);
     outNode.appendChild(ele);

  }// end of insertToDocument()

  /**
  */
  public void 
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(iwt.DIS_TAGS.TSequence);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {
    if( rt!=null && rt.getNodeName().equals(iwt.DIS_TAGS.TSequence) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.Quiescent);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(iwt.DIS_TAGS.Sequence);

       String v1 = a1.getValue(); 
       String v2 = a2.getValue();
      
       if(v1.equals("true")) bquiescent = true;
       else  bquiescent = false;

       try
       {
         bsequence = (short)Integer.parseInt(v2);

       } catch(NumberFormatException nfe) {}

    } else throw new InvalidDocumentType();

  }// end of read()

} // end of class TSequence
