
package iwt.mine;

import mil.navy.nps.dis.*;         
import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import iwt.util.*;
import iwt.util.hexConvert;
import org.w3c.dom.*;

/** 
* 
*
* @see PduElement 
* @see SerializationInterface
*
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class VectorRecord extends PduElement implements IWTDebug
{
    /**
     *First Vector Component, along X axis
     */
    protected float  x;     // x-position

    /**
     *Second Vector  Component, along Y axis
     */
    protected float  y;     // y-position

    /**
     *Third Vector Component, along Z axis
     */
    protected float  z;     // z-position

    /**
     *Constant value--size of a Vector Record when written out; 
     * here :<code>sizeOf = 12 bytes</code>.
     */
    public final int sizeOf = 12;   // object is 12 bytes long 

/**
 *Constructs an new Vector
 */
public VectorRecord()
{
    // default constructor

    x = 0; 
    y = 0;
    z = 0;

    return;
}

/**
 *Constructs a new VectorRecord with coordinate values passed in parameters.
 *
 *@param pX the first coordinate in the cartesian coordinate system
 *@param pY the second coordinate in the cartesian coordinate system
 *@param pZ the third coordinate in the cartesian coordinate system
 */
public VectorRecord(float pX, float pY, float pZ)
{
    x = pX;
    y = pY;
    z = pZ;

    return;
}

public Object clone()
{
 VectorRecord   newVectorRecord = new VectorRecord(x, y, z);

 return newVectorRecord;
}

public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    //super.serialize(outputStream);        // No need to call super, since no ivars are in our direct superclass
    
    try
     {
        outputStream.writeFloat(x);
        outputStream.writeFloat(y);
        outputStream.writeFloat(z);
     }
    catch (IOException ioError)
        {
            throw new 
                RuntimeException("Exception in VectorRecord. Error writing to wire.");
        }
}

public void deSerialize(DataInputStream pInputStream)
{
    try
     {
        x = pInputStream.readFloat();  
        if(MDEBUG) System.out.println("x : "+x+" "+hexConvert.toHexStr(x));
        y = pInputStream.readFloat();  
        if(MDEBUG) System.out.println("y : "+y+" "+hexConvert.toHexStr(y));
        z = pInputStream.readFloat();  
        if(MDEBUG) System.out.println("z : "+z+" "+hexConvert.toHexStr(z));
        
     }
     catch (IOException ioError)
        {
            throw new 
                RuntimeException("Exception in VectorRecord. Error reading from wire.");
        }

}

public int length()
{
    return sizeOf;          // EntityTypes are this long, always.
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println(indent + "VectorRecord x: "   + x);
    printStream.println(indent + "VectorRecord y: "   + y);
    printStream.println(indent + "VectorRecord z: "   + z);

    return;
}

  // Accessor methods
  public void setValues(float pX, float pY, float pZ)
  { x = pX; y = pY; z = pZ; }

  public float getX() {   return x;}

  public void setX(float pX) {x = pX;}

  public float getY() {   return y;}

  public void setY(float pY) {y = pY;}

  public float getZ() {   return z;}

  public void setZ(float pZ) {z = pZ; }

  /**
  */
  public void 
  write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

  /**
  */
  public void 
  insert(org.w3c.dom.Node outNode)
  {
    org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
    insertToDocument(outDoc,outNode);

  }// end of insert()

  /**
  */
  public void 
  insertToDocument( org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode )
  {
    org.w3c.dom.Element ele = outDoc.createElement(DIS_TAGS.VectorRecord);
    ele.setAttribute(DIS_TAGS.X,Float.toString(x));
    ele.setAttribute(DIS_TAGS.Y,Float.toString(y));
    ele.setAttribute(DIS_TAGS.Z,Float.toString(z));

    outNode.appendChild(ele);

  }// end of insertToDocument()

  /**
  */
  public void 
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(DIS_TAGS.VectorRecord);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(DIS_TAGS.VectorRecord) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.X);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Y);
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Z);
       String v1, v2, v3;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();

       try
       {
         x = Float.valueOf(v1).floatValue();

       } catch(NumberFormatException nfe) {}

       try
       {
         y = Float.valueOf(v2).floatValue();

       } catch(NumberFormatException nfe) {}

       try
       {
         z = Float.valueOf(v3).floatValue();

       } catch(NumberFormatException nfe) {}

    } else throw new InvalidDocumentType();

  }// end of read()

} // end of class VectorRecord
