package jdis;

import java.io.*;
import java.util.*;
import java.awt.Color;

import mil.navy.nps.disEnumerations.*;


/**
*
* @author H. Timucin Ozdemir
* @date   January 26, 1999
*
*/
public class JDISConstants
{    
   public static String DEFAULT_MCAST = "230.0.0.1";
   public static int DEFAULT_MPORT = 4446;
   public static int DEFAULT_COLLECTUDPPORT = 3030;

   public static String OPTION_MCAST = "-mcast";
   public static String OPTION_MPORT = "-mport";
   public static String OPTION_COLLECTUDPPORT = "-cport";
   
   public static String DEFAULT_SERVER = "http://127.0.0.1";
   public static int DEFAULT_SERVER_PORT = 4004;

   public static int DEFAULT_Frequency = 10;
   public static int DEFAULT_PckFrequency = 3;

   public static String OPTION_NSERVER = "-nserver";
   public static String OPTION_NPORT   = "-nport";

   public static String DEFAULT_INTERACTION_CLASS = "Communication";
   public static String DEFAULT_PARAMETER_NAME    = "Message";
   public static String DEFAULT_FEDERATION_NAME   = "HelloWorld";

   public static byte DIS203_PROTOCOL_VERSION_1_0 = (byte)1;
   public static byte DIS203_PROTOCOL_VERSION_IEEE_1278_1993 = (byte)2;
   public static byte DIS203_PROTOCOL_VERSION_2_0 = (byte)3;
   public static byte DIS203_PROTOCOL_VERSION_CURRENT = DIS203_PROTOCOL_VERSION_2_0;

   public static String DEFAULT_LOGGER_SERVER = "http://gandalf.npac.syr.edu:4004";

   public static String DEFAULT_LOGGER_URI = "/servlet/PduLogger";

   public static String DEFAULT_LOGGER_URI_ACTION = "action=save";

   public static String DEFAULT_BROWSE_PATH = "..";
   public static String DEFAULT_PLAY_FILE   = "test_knox1.pdu";
   public static String DEFAULT_SAVE_FILE   = "t1.pdu";

   public static int LAND = 1;

   /**
   */
   public static final void 
   load(String fName)
   {
      try
      {
        FileInputStream in = new FileInputStream(fName);
        Properties def = new Properties();

        def.load(in);
        update(def);

      } catch(Exception e) {}

   }// end of load

   /**
   */
   public static final void 
   update(Properties def)
   {
      if(def!=null)
      {
        //def.list(System.out);

        // HTTP Server settings for HLA
        String val = def.getProperty("nserver");
        if(val!=null) JDISConstants.DEFAULT_SERVER = val;


        val = def.getProperty("nport");
        if(val!=null) 
        { 
           try
           { int v = Integer.parseInt(val); 
             JDISConstants.DEFAULT_SERVER_PORT = v;
           } catch(NumberFormatException nfe) {} 

        }// end of if 

        // Multicast Channel settings for DIS
        val = def.getProperty("mcast");
        if(val!=null) JDISConstants.DEFAULT_MCAST = val;

        val = def.getProperty("mport");
        if(val!=null) 
        { 
           try
           { int v = Integer.parseInt(val); 
             JDISConstants.DEFAULT_MPORT = v;
           } catch(NumberFormatException nfe) {} 

        }// end of if 

        // UDP Port address for collecting CMS Performance stats
        val = def.getProperty("cport");
        if(val!=null) 
        { 
           try
           { int v = Integer.parseInt(val); 
             JDISConstants.DEFAULT_COLLECTUDPPORT = v;
           } catch(NumberFormatException nfe) {} 

        }// end of if 

        // DB Server address
        val = def.getProperty("dbserver");
        if(val!=null) JDISConstants.DEFAULT_LOGGER_SERVER=val;

        // DB Package Size
        val = def.getProperty("dbpackagesize");
        if(val!=null) 
        { 
           try
           { int v = Integer.parseInt(val); 
             JDISConstants.DEFAULT_PckFrequency = v;
           } catch(NumberFormatException nfe) {} 

        }// end of if 

        // Play Frequency 
        val = def.getProperty("playfrequency");
        if(val!=null) 
        { 
           try
           { int v = Integer.parseInt(val); 
             JDISConstants.DEFAULT_Frequency = v;
           } catch(NumberFormatException nfe) {} 

        }// end of if 

        // DIS HLA Federation Name (****NOT*** FEDERATE**** )
        val = def.getProperty("disfedexname");
        if(val!=null) JDISConstants.DEFAULT_FEDERATION_NAME=val;

        // DIS HLA Interaction Class Name
        val = def.getProperty("disinteraction");
        if(val!=null) JDISConstants.DEFAULT_INTERACTION_CLASS=val;

        // DIS HLA Parameter Name
        val = def.getProperty("disparameter");
        if(val!=null) JDISConstants.DEFAULT_PARAMETER_NAME=val;

        //
        val = def.getProperty("browsepath");
        if(val!=null) JDISConstants.DEFAULT_BROWSE_PATH=val;

        //
        val = def.getProperty("playfile");
        if(val!=null) JDISConstants.DEFAULT_PLAY_FILE=val;

        // 
        val = def.getProperty("savefile");
        if(val!=null) JDISConstants.DEFAULT_SAVE_FILE=val;

      }// end of if

   }// end of update

   public static Color
   getColor(int pduType)
   {
      Color ret = new Color(0,0,0);

      switch(pduType)
      {
         case PDUTypeField.ENTITYSTATE: ret = Color.cyan; break;
         case PDUTypeField.FIRE       : ret = Color.red; break;
         case PDUTypeField.DETONATION : ret = Color.green; break;
         case PDUTypeField.ACKNOWLEDGE: ret = Color.orange; break;
         case PDUTypeField.STARTRESUME: ret = Color.lightGray ; break;
         case PDUTypeField.STOPFREEZE : ret = Color.yellow; break;
         case PDUTypeField.MINE       : ret = Color.magenta; break;
         case PDUTypeField.MINEFIELD  : ret = Color.pink; break;
         case PDUTypeField.TRANSMITTER: ret = Color.blue; break;
         default:
                  ret = new Color((pduType*4)%256,(pduType%4)*64,(pduType%3)*85);
      }// end of switch()

      return ret;
      
   }// end of getColor()

}// end of JDISConstants
