package jdis;

/**
* @author H. Timucin Ozdemir
*/

import java.awt.*;
import java.awt.image.*;

public class 
PDUGrapher extends java.awt.Panel 
{
    int pduType[];
    int start;
    int stop;
    Image offscreen;
    Graphics offgraphics;
    Dimension offscreensize;
    int size=0;
    Dimension myDim = new Dimension(250,250);

    /**
    */
    public
    PDUGrapher(int _size)
    {
        size = _size;
        pduType = new int[(size+1)];
        start = 0;
        stop = 0;
        setBackground(Color.white);
        setSize(myDim);
    }

    /**
    */
    public Dimension
    getPreferredSize()
    {
        return  myDim;

    }// end of getPreferredSize()



    /**
    */
    public void
    addPoint(int pduFx)
    {
       int rstop = stop;

       if(stop<start) rstop += (size+1);

       int num_of_ele = rstop - start ;

       if( (rstop-start) >= size) start = (start+1) % (size+1);

       pduType[stop%(size+1)] = pduFx;
       stop = (stop+1) % (size+1);

       //System.out.print(" start "+start+" stop "+stop+" "+" rstop "+rstop);
       //System.out.println(" "+fx+" num_of_ele : "+num_of_ele+" ");
       //System.out.print(" "+fx+" num_of_ele : "+num_of_ele+" ");
       
       repaint();

    }

    /**
    */
    double f(double x) 
    {
       return (Math.cos(x/5) + Math.sin(x/7) + 2) * size().height / 4;
    }

    /**
    * Finds the Min-Max of the data points
    */
    IntStatRec
    findMax()
    {
       IntStatRec ret = new IntStatRec();
       int rstop = stop;
       if(stop<start) rstop += (size+1);
       int num_of_ele = rstop - start ;
       ret.min = ret.max=pduType[start];

       for (int i = 1 ; i < num_of_ele ; i++) 
       {
          ret.max = Math.max(ret.max,pduType[(i+start)%(size+1)]);
          ret.min = Math.min(ret.min,pduType[(i+start)%(size+1)]);

       }// end of if

       ret.min=0;

       return ret;

    }// end of findMax()

    /**
    */
    public void 
    drawStringVertical(Graphics g, String str, int x, int y)
    {
       
       FontMetrics fm = g.getFontMetrics();
       int strWidth = fm.charWidth('M') * str.length();
       Dimension d = new Dimension(strWidth+10,fm.getHeight()+3);

       Image offscr = createImage(d.width, d.height);
       Graphics offgraph = offscr.getGraphics();
       offgraph.setFont(g.getFont());

       offgraph.drawString(str,d.width/2,d.height/2);

       int[] pixelsOrig = new int[d.width * d.height]; 
       int[] pixelsRes  = new int[d.width * d.height]; 

       PixelGrabber pg = new PixelGrabber(offscr, 0, 0, d.width, d.height, pixelsOrig, 0, d.width);
       try 
       {
            pg.grabPixels();
       } catch (InterruptedException e) 
         {
            System.err.println("interrupted waiting for pixels!");
            return;
         }
        /** JDK1.0.2 does not compile this 
        if ((pg.getStatus() & ImageObserver.ABORT) != 0) 
        {
            System.err.println("image fetch aborted or errored");
            return;
        }
        */

        for (int j = 0; j < d.height; j++) 
            for (int i = 0; i < d.width; i++)  
                 pixelsRes[i*d.height+j] = pixelsOrig[j * d.width + i];

        Image img = createImage(new MemoryImageSource(d.height, d.width, pixelsRes, 0, d.height));
 
        g.drawImage(img,x,y,null); 

    }// end of drawStringVertical()


    /**
    *
    *    xLeft,
    */
    public void 
    drawAxis0(Graphics g, IntStatRec ymax, int xmax, int xPlus, int YMinus) 
    {
       Color curr = g.getColor();
       int x,y;
       int xLeft = xPlus-1; 
       int yUp = YMinus-210; 
       int xRight = xPlus+211; 
       int yDown = YMinus+1; 
       FontMetrics fm = g.getFontMetrics();
       g.setColor(Color.black);
       g.drawLine(xLeft,yDown,xRight,yDown);
       g.drawLine(xLeft,yDown,xLeft,yUp);

       //drawStringVertical(g, String.valueOf(ymax.max), xLeft-5, yUp+10);
       //drawStringVertical(g, String.valueOf(ymax.min), xLeft-5, yDown-10);
       g.drawString(String.valueOf(ymax.max),xLeft-10,yUp+10);
       g.drawString(String.valueOf(ymax.min),xLeft-10,yDown+10);

       x = xRight - fm.stringWidth(String.valueOf(xmax))/2;
       y = yDown  + fm.getHeight();
       g.drawString(String.valueOf(xmax),x,y);
       g.setColor(curr);
    }

    /**
    *
    *    xLeft,
    */
    public void 
    drawAxis(Graphics g, IntStatRec ymax, int xmax, int xPlus, int YMinus) 
    {
       Color curr = g.getColor();
       int x,y;
       int xLeft = xPlus-1; 
       int yUp = YMinus-210; 
       int xRight = xPlus+211; 
       int yDown = YMinus+1; 
       FontMetrics fm = g.getFontMetrics();
       g.setColor(Color.black);
       //g.drawLine(xLeft,yDown,xRight,yDown);
       //g.drawLine(xLeft,yDown,xLeft,yUp);
       g.fillRect(xLeft-1,yDown-1,xRight-xLeft,3);
       g.fillRect(xLeft-1,yUp-1,3,yDown-yUp);

       //drawStringVertical(g, String.valueOf(ymax.max), xLeft-5, yUp+10);
       //drawStringVertical(g, String.valueOf(ymax.min), xLeft-5, yDown-10);
       g.drawString(String.valueOf(ymax.max),xLeft-10,yUp+10);
       g.drawString(String.valueOf(ymax.min),xLeft-10,yDown+10);

       x = xRight - fm.stringWidth(String.valueOf(xmax))/2;
       y = yDown  + fm.getHeight();
       g.drawString(String.valueOf(xmax),x,y);
       g.setColor(curr);
    }

    /**
    */
    public void 
    drawGraph0(Graphics g, int xPlus, int YMinus) 
    {
        Dimension d = new Dimension(250,220);

        int rstop = stop;
        if(stop<start) rstop += (size+1);
        int num_of_ele = rstop - start ;

        IntStatRec maxFx = findMax();

        float xDelta = 200f/size;
        float yDelta = 200f/(maxFx.max-maxFx.min);
        float normYfrom;
        float normYto;

        g.setColor(getBackground());
        g.fillRect(0, 0, d.width, d.height);
        g.setColor(Color.red);
        int val_X;
        int val_X_1;

        //for (int x = 0 ; x < size().width ; x++) 
        val_X_1 = pduType[start];
        //normYfrom = (vals[0]-maxFx.min)*yDelta;
        normYfrom = (val_X_1-maxFx.min)*yDelta;
        for (int x = 1 ; x < num_of_ele ; x++) 
        {
            val_X = pduType[(x+start)%(size+1)];
            //normYto = (vals[x]-maxFx.min)*yDelta;
            normYto = yDelta*(val_X-maxFx.min);
            g.drawLine((int)((x-1)*xDelta)+xPlus, (int)(YMinus-normYfrom), 
                       (int)(xDelta*x)+xPlus, (int)(YMinus-normYto));
            normYfrom = normYto;

            if(Math.abs(val_X-val_X_1)>0.1*(maxFx.max-maxFx.min))
            {
              g.setColor(Color.green);
              g.drawOval((int)(xDelta*x)+xPlus, (int)(YMinus-normYto),2,2);
              g.setColor(Color.red);
            }
            val_X_1 = val_X;
        }

        drawAxis(g,maxFx,size,xPlus,YMinus);

    }// end of drawGraph0()

    /**
    */
    public void 
    drawGraph(Graphics g, int xPlus, int YMinus) 
    {
        Dimension d = new Dimension(250,220);

        int rstop = stop;
        if(stop<start) rstop += (size+1);
        int num_of_ele = rstop - start ;

        IntStatRec maxFx = findMax();

        float xDelta = 200f/size;
        float yDelta = 200f/(maxFx.max-maxFx.min);
        float normYfrom;
        float normYto;

        g.setColor(getBackground());
        g.fillRect(0, 0, d.width, d.height);
        g.setColor(Color.red);
        int val_X;
        int val_X_1;

        //for (int x = 0 ; x < size().width ; x++) 
        val_X_1 = pduType[start];
        //normYfrom = (vals[0]-maxFx.min)*yDelta;
        normYfrom = (val_X_1-maxFx.min)*yDelta;
        for (int x = 1 ; x < num_of_ele ; x++) 
        {
            val_X = pduType[(x+start)%(size+1)];
            //normYto = (vals[x]-maxFx.min)*yDelta;
            normYto = (val_X-maxFx.min)*yDelta;

            //g.drawLine((int)((x-1)*xDelta)+xPlus, (int)(YMinus-normYfrom), 
            //           (int)(xDelta*x)+xPlus, (int)(YMinus-normYto));

            int y0 = (int)(YMinus-normYfrom);
            int yH = YMinus-y0;
            g.setColor(new Color((val_X_1*4)%256,(val_X_1%4)*64,(val_X_1%3)*85));
            g.fill3DRect((int)((x-1)*xDelta)+xPlus, y0, (int)xDelta, yH, true);

            normYfrom = normYto;

            /*if(Math.abs(val_X-val_X_1)>0.1*(maxFx.max-maxFx.min))
            {
              g.setColor(Color.green);
              g.drawOval((int)(xDelta*x)+xPlus, (int)(YMinus-normYto),2,2);
              g.setColor(Color.red);
            }*/

            val_X_1 = val_X;
        }

        drawAxis(g,maxFx,size,xPlus,YMinus);

    }// end of drawGraph()

    /**
    */
    public void 
    update(Graphics g) 
    {
        //Dimension d = size();
        Dimension d = new Dimension(240,240);

        if ((offscreen == null) || (d.width != offscreensize.width) ||
            (d.height != offscreensize.height)) 
        {
            offscreen = createImage(d.width, d.height);
            offscreensize = d;
            offgraphics = offscreen.getGraphics();
            offgraphics.setFont(getFont());
        }

        offgraphics.setColor(getBackground());
        offgraphics.fillRect(0, 0, d.width, d.height);

        //drawGraph(g); 
        drawGraph(offgraphics,20,220); 
        g.drawImage(offscreen, 0, 0, null );

    }// end of update()

    /**
    */
    public void 
    paint(Graphics g) 
    {
      update(g);
    }

}
