package jdis;

import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.filechooser.*;

import sgui.*;
import jdis.*;
import worker.*;
import comm.*;
import mil.navy.nps.dis.*;

/**
*
* 
* @author Hasan Timucin Ozdemir
* @date   January 26, 1999 (Update February 8, 1999)
*
*/
public class 
SMain extends JFrame 
      implements MonitorControl, ActionListener, ItemListener, VisConstants 
{

    JTextArea output;
    JScrollPane scrollPane;
    String newline = System.getProperty("line.separator");

    JRadioButtonMenuItem aPlay;
    JRadioButtonMenuItem aStop;
    JRadioButtonMenuItem aResume;
    JRadioButtonMenuItem aRcvSave;
    JRadioButtonMenuItem aRcvDisp;
    JRadioButtonMenuItem aRcvSaveDB;
    JRadioButtonMenuItem aExpFlDB;
    JRadioButtonMenuItem aBridge;

    // State
    String aPlayFileName = null;
    RunnerObject  aFilePlayer = null;
    RunnerObject  aPDUSniffer = null;
    RunnerObject  aFileSaver  = null;

    String  mCastAddress   = JDISConstants.DEFAULT_MCAST;
    int     gPortNumber    = JDISConstants.DEFAULT_MPORT;
    int     collectUdpPort = JDISConstants.DEFAULT_COLLECTUDPPORT; 
    int     nPortNumber    = JDISConstants.DEFAULT_SERVER_PORT;
    String  nServer        = JDISConstants.DEFAULT_SERVER; 

    String  logServer     = JDISConstants.DEFAULT_LOGGER_SERVER;
    String  logUri        = JDISConstants.DEFAULT_LOGGER_URI; 
    String  logCmd        = JDISConstants.DEFAULT_LOGGER_URI_ACTION;

    MulticastSocket socket;
    InetAddress     igroup;

    HLAandDISPane   hladisGUI;

    boolean isDIS = true;

    PDUPublisher  disPublisher = null;
    PDUPublisher  hlaPublisher = null;
    PDUPublisher  currentPublisher = null;
    PDUPublisher  httpPublisher = null;

    SubscriberAdmin  disSubAdmin = null;
    SubscriberAdmin  hlaSubAdmin = null;
    SubscriberAdmin  currentSubAdmin = null;

    int freq  = JDISConstants.DEFAULT_Frequency;
    int psize = JDISConstants.DEFAULT_PckFrequency;

    EntityViewer eView = null;

    String   dBrowsePath = JDISConstants.DEFAULT_BROWSE_PATH ;
    String   dPlayFile   = JDISConstants.DEFAULT_PLAY_FILE ;
    String   dSaveFile   = JDISConstants.DEFAULT_SAVE_FILE ;

    JFileChooser fc = new JFileChooser(new File(JDISConstants.DEFAULT_BROWSE_PATH));
    static final boolean DEBUG = false;

    /**
    */
    public SMain( String _mCast, int _gPNum, int _cUdp, String _nServer, 
                  int  _nPortNumber )
    {
       mCastAddress   = _mCast;
       gPortNumber    = _gPNum;
       collectUdpPort = _cUdp;
       nServer        = _nServer;
       nPortNumber    = _nPortNumber;

       fc.removeChoosableFileFilter(fc.getAcceptAllFileFilter());
       fc.addChoosableFileFilter(new SPDUFileFilter());
       fc.setApproveButtonText(" Play ");
       fc.setApproveButtonToolTipText(" Plays the PDU file. ");
       fc.setDialogTitle(" Select the PDU file for replay. ");

       // Define DIS Components
       disPublisher = new MulticastPDUPublisher(_mCast,_gPNum);
       disSubAdmin  = new MulticastPDUSubAdmin(_mCast,_gPNum);

       // Define HLA Components
       hlaPublisher = new DISHLAPublisher(nServer,nPortNumber);
       hlaSubAdmin  = new DISHLASubscriberAdmin();
       ((DISHLASubscriberAdmin)hlaSubAdmin).setBoss((DISHLAPublisher)hlaPublisher);

       httpPublisher = new HTTPPDUPublisher(logServer,logUri,logCmd);

       // Update Current Components
       currentSubAdmin  = disSubAdmin;
       currentPublisher = disPublisher;

       // Print Report about the assumptions
       if(DEBUG)
       {  System.out.println(" Multicast Address .....:"+mCastAddress);
          System.out.println(" Multicast Port number .:"+gPortNumber);
          System.out.println(" Stat UDP Port .........:"+collectUdpPort);
       }// end of if
     
       JMenuBar menuBar;
       JMenu menu, submenu;
       JMenuItem menuItem;
       JRadioButtonMenuItem rbMenuItem;
       JCheckBoxMenuItem cbMenuItem;

       addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                SMain.this.doExit();
            }
       });

       setTitle(sgui.VisConstants.MainFrameTitle);

       //Add regular components to the window, using the default BorderLayout.
       Container contentPane = getContentPane();
       output = new JTextArea(5, 30);
       output.setEditable(false);
       scrollPane = new JScrollPane(output);
       contentPane.add(scrollPane, BorderLayout.CENTER);

       //Create the menu bar.
       menuBar = new JMenuBar();
       setJMenuBar(menuBar);

        //Build the first menu.
        menu = new JMenu("A Menu");
        menuBar.add(menu);

        //a group of JMenuItems
        /*menuItem = new JMenuItem("A text-only menu item");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem("Both text and icon", 
                                 new ImageIcon("images/middle.gif"));
        menuItem.addActionListener(this);
        menu.add(menuItem);
        menuItem = new JMenuItem(new ImageIcon("images/middle.gif"));
        menuItem.addActionListener(this);
        menu.add(menuItem);*/

        //a group of radio button menu items


        //a group of check box menu items
        ButtonGroup group = new ButtonGroup();
        rbMenuItem = new JRadioButtonMenuItem(sgui.VisConstants.DISTxt);
        group.add(rbMenuItem);
        rbMenuItem.setSelected(true);
        rbMenuItem.addActionListener(this);
        menu.add(rbMenuItem);

        rbMenuItem = new JRadioButtonMenuItem(sgui.VisConstants.HLATxt);
        group.add(rbMenuItem);
        rbMenuItem.addActionListener(this);
        menu.add(rbMenuItem);
        
        //a Menu Item for OpenFile and Close File
        menu.addSeparator();
        menuItem = new JMenuItem(sgui.VisConstants.DISSetTxt);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem(sgui.VisConstants.HLASetTxt);
        menuItem.addActionListener(this);
        menu.add(menuItem);

        menuItem = new JMenuItem(sgui.VisConstants.LogSetTxt);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        

        //a submenu for Exit
        menu.addSeparator();
        menuItem = new JMenuItem(sgui.VisConstants.ExitTxt);
        menuItem.addActionListener(this);
        menu.add(menuItem);
        //menu.addSeparator();
      
        /*menuItem = new JMenuItem("Add");
        menuItem.addActionListener(this);
        menu.add(menuItem);
        */

        //Build second menu in the menu bar.
        menu = new JMenu("Action");
        menuBar.add(menu);

        //a group of radio button menu items
        group = new ButtonGroup();

        aPlay = new JRadioButtonMenuItem(sgui.VisConstants.PlayTxt);
        group.add(aPlay);
        aPlay.addActionListener(this);
        menu.add(aPlay);

        aStop = new JRadioButtonMenuItem(sgui.VisConstants.StopTxt);
        aStop.setSelected(true);
        group.add(aStop);
        aStop.addActionListener(this);
        menu.add(aStop);

        aRcvSave = new JRadioButtonMenuItem(sgui.VisConstants.RcvSaveTxt);
        group.add(aRcvSave);
        aRcvSave.addActionListener(this);
        menu.add(aRcvSave);

        aRcvDisp = new JRadioButtonMenuItem(sgui.VisConstants.RcvDispTxt);
        group.add(aRcvDisp);
        aRcvDisp.addActionListener(this);
        menu.add(aRcvDisp);

        aRcvSaveDB = new JRadioButtonMenuItem(sgui.VisConstants.RcvSaveDBTxt);
        group.add(aRcvSaveDB);
        aRcvSaveDB.addActionListener(this);
        menu.add(aRcvSaveDB);

        aExpFlDB = new JRadioButtonMenuItem(sgui.VisConstants.ExportDBTxt);
        group.add(aExpFlDB);
        aExpFlDB.addActionListener(this);
        menu.add(aExpFlDB);

        aBridge = new JRadioButtonMenuItem(sgui.VisConstants.BridgeTxt);
        group.add(aBridge);
        aBridge.addActionListener(this);
        menu.add(aBridge);

        //a group of radio button menu items
        menu.addSeparator();
        group = new ButtonGroup();

        /*rbMenuItem = new JRadioButtonMenuItem(sgui.VisConstants.RealTimeTxt);
        group.add(rbMenuItem);
        rbMenuItem.addActionListener(this);
        menu.add(rbMenuItem);
        */

        rbMenuItem = new JRadioButtonMenuItem(sgui.VisConstants.SetFrequencyTxt);
        rbMenuItem.setSelected(true);
        group.add(rbMenuItem);
        rbMenuItem.addActionListener(this);
        menu.add(rbMenuItem);

        //
        hladisGUI = new HLAandDISPane(this);
        getContentPane().add(hladisGUI);

    }// end of CONSTRUCTOR

    /**
    */
    public void 
    actionPerformed(ActionEvent e) 
    {
        JMenuItem source = (JMenuItem)(e.getSource());
    
        try
        {
             eView = (jdis.EntityViewer)hladisGUI.getHLAPanel().getEntityViewer();

             if(source.getText().equals(sgui.VisConstants.ExitTxt)) doExit();

        else if( source.getText().equals(sgui.VisConstants.DISTxt) || 
                 source.getText().equals(sgui.VisConstants.HLATxt)) 
             { 
                if(currentSubAdmin!=null) 
                { currentSubAdmin.stopCollect(); }
                if(currentPublisher!=null) 
                { currentPublisher.stopPublishing(); }

                if(source.getText().equals(sgui.VisConstants.DISTxt) && !isDIS) 
                { 
                  isDIS = true;
                  currentPublisher = disPublisher;   
                  currentSubAdmin  = disSubAdmin;

                }
                else if(source.getText().equals(sgui.VisConstants.HLATxt) && isDIS) 
                     { 
                         isDIS = false;
                         currentPublisher = hlaPublisher;   
                         currentSubAdmin  = hlaSubAdmin;
                     }// end of if

             }

        else if(source.getText().equals("Add"))
             {
               hladisGUI.getHLAPanel().getEntityViewer().addEntityRec(new EntityRecord(new EntityID(1,1,1),20.0,20.0,20.0));
               
             }

        else if( source.getText().equals(sgui.VisConstants.PlayTxt) ||
                 source.getText().equals(sgui.VisConstants.ExportDBTxt) )

             {

                reset(true);

                int time = 0;

                do
                { 
                   fc.setDialogTitle(" Select the PDU file for replay. ");
                   fc.setApproveButtonText(" Play ");
                   fc.setApproveButtonToolTipText(" Plays the PDU file. ");
                   fc.setSelectedFile(new File(dPlayFile));
                   int returnVal = fc.showOpenDialog(this);

                   if (returnVal == JFileChooser.APPROVE_OPTION) 
                   {
                      File file = fc.getSelectedFile();
                      aPlayFileName = file.getPath(); //+"/"+file.getName();
                   } 

                   if( aPlayFileName==null || aPlayFileName.trim().length()==0)
                   {
                       JOptionPane.showMessageDialog(this,"File is not selected");
                       aPlayFileName = null;
                   }
                   else
                    {
                       File aFile = new File(aPlayFileName);
                       if( !aFile.exists() || !aFile.canRead() || 
                           aFile.isDirectory() )
                       {
                          String mess = new String("File is not readable");
                          if(!aFile.exists())
                              mess = new String("File does not exist!");
                          else if(aFile.isDirectory())
                              mess = new String("It is a directory!");
                          JOptionPane.showMessageDialog(this,mess);
                          aPlayFileName = null;
                       }
                       else time=3;
                    }

                } while( time++<0 );

                if( aPlayFileName!=null )
                {
                  dPlayFile=aPlayFileName;
                  //PDUPublisherMUX pubMUX = new PDUPublisherMUX();
                  //pubMUX.register(new PDUPublishToEV((jdis.EntityViewer)hladisGUI.getHLAPanel().getEntityViewer()) );

                  if(source.getText().equals(sgui.VisConstants.ExportDBTxt))
                  {

                     GetPackageSize dialog = new GetPackageSize(this,psize);
                     dialog.pack();
                     dialog.setLocationRelativeTo(this);
                     dialog.show();

                     setPackageSize(dialog.getPackageSize());
                     ((HTTPPDUPublisher)httpPublisher).setPackageSize(getPackageSize()); 
                     //pubMUX.register(httpPublisher);
                     aFilePlayer = new PDUFromFile2(aPlayFileName,httpPublisher,getFrequency(),(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher());
                     //aFilePlayer = new PDUFromFile2(aPlayFileName,pubMUX,getFrequency(),(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher());
                  }
                  else 
                    { //pubMUX.register(currentPublisher);
                      //aFilePlayer = new PDUFromFile2(aPlayFileName,pubMUX,getFrequency(),(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher());
                      aFilePlayer = new PDUFromFile2(aPlayFileName,
                                        currentPublisher,
      new PDUPublishToEV((jdis.EntityViewer)hladisGUI.getHLAPanel().getEntityViewer()),
                                         getFrequency(),
              (jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher());
                      currentPublisher.startPublishing();
                    }

                  System.out.println("Playing: " + aPlayFileName + "." + newline);
                  aFilePlayer.start();

                }// end of if 

             }

        else if(source.getText().equals(sgui.VisConstants.StopTxt)) reset(false);

        else if(source.getText().equals(sgui.VisConstants.RcvSaveTxt))
             {
                reset(true);

                String aSaveFileName = null; 
                int times = 0;

                do
                {
                   fc.setSelectedFile(new File(dSaveFile));
                   int returnVal = fc.showSaveDialog(this);

                   if (returnVal == JFileChooser.APPROVE_OPTION) 
                   {
                     File file = fc.getSelectedFile();
                     aSaveFileName = file.getPath(); //+"/"+file.getName();
                   } 

                   if( aSaveFileName==null || aSaveFileName.trim().length()==0)
                   {
                       JOptionPane.showMessageDialog(this,"File is not selected");
                       aSaveFileName = null;
                   }
                   else
                    {
                       File aFile = new File(aSaveFileName);
                       if( aFile.exists() && aFile.isDirectory() )
                       {
                          String mess = new String("File does not exist!");
                          if( aFile.isDirectory() )
                              mess = new String("It is a directory!");

                          JOptionPane.showMessageDialog(this,mess);
                          aSaveFileName = null;
                       }
                       else if( aFile.exists() )
                       {
                          String ystr = "Yes";
                          String nstr = "No";
                          Object[] options = { ystr, nstr } ;
                          int n = JOptionPane.showOptionDialog(this,
                                    "Would you like to overwrite?","",
                                    JOptionPane.YES_NO_OPTION,
                                    JOptionPane.QUESTION_MESSAGE,
                                    null, options, ystr);

                          if ( n == JOptionPane.YES_OPTION)  times = 3;  
                          else  aSaveFileName = null; 

                       } 
                       else times=3;

                    }

                } while( times++<0 ); 

                if( aSaveFileName!=null )
                {

                  PDUSaver2 aSaver = new PDUSaver2(currentSubAdmin,aSaveFileName,(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher()); 

                  System.out.println("Playing: " + aPlayFileName + 
                                     "." + newline);
                  currentSubAdmin.startCollect();

                }// end of if 

             }

        else if(source.getText().equals(sgui.VisConstants.RcvDispTxt))
             {
                reset(true);

                PDUConsumer2 aSniffer = new PDUConsumer2(currentSubAdmin,
                                     new PDUPublishToEV((jdis.EntityViewer)hladisGUI.getHLAPanel().getEntityViewer()),(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher());

                currentSubAdmin.startCollect();

             }

        else if(source.getText().equals(sgui.VisConstants.DISSetTxt))
             {
                 DISSettings dialog = new DISSettings(this,mCastAddress,gPortNumber);
                 dialog.pack();
                 dialog.setLocationRelativeTo(this);
                 dialog.show();

                 updateDISSettings( dialog.getMulticastAddress(),
                                    dialog.getMulticastPortNumber());

            }

        else if(source.getText().equals(sgui.VisConstants.HLASetTxt))
             {
                 HLASettings dialog = new HLASettings(this,nServer,nPortNumber);
                 dialog.pack();
                 dialog.setLocationRelativeTo(this);
                 dialog.show();

                 updateHLASettings( dialog.getServerAddress(),
                                    dialog.getServerPortNumber());

            }

        else if(source.getText().equals(sgui.VisConstants.LogSetTxt))
             {
                 HttpSettings dialog = new HttpSettings(this,logServer,logUri,logCmd);
                 dialog.pack();
                 dialog.setLocationRelativeTo(this);
                 dialog.show();

                 updateHttpSettings( dialog.getServerAddress(), 
                                     dialog.getLogUri(),
                                     dialog.getLogCmd() );

            }

        else if(source.getText().equals(sgui.VisConstants.SetFrequencyTxt))
             {

                 GetFrequency dialog = new GetFrequency(this,freq);
                 dialog.pack();
                 dialog.setLocationRelativeTo(this);
                 dialog.show();

                 setFrequency(dialog.getFrequency());
                 if( aFilePlayer!=null &&
                     aFilePlayer instanceof PDUFromFile2) 
                     ((PDUFromFile2)aFilePlayer).setFrequency(freq);

            }

             // Receives from the communication channel (HLA/DIS) and
             // Submits to the Database
        else if(source.getText().equals(sgui.VisConstants.RcvSaveDBTxt))
             {

                reset(true);
                if(DEBUG) System.out.println(" Publish Logger!!");

                PDUConnector aConnect = new PDUConnector(currentSubAdmin,httpPublisher,(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher()); 

                currentSubAdmin.startCollect();

             }

             // Receives from DIS and publishes to HLA
        else if(source.getText().equals(sgui.VisConstants.BridgeTxt))
             {
                reset(true);
                if(DEBUG) System.out.println(" DIS->HLA Bridge!!");

                hlaPublisher.startPublishing();
                currentPublisher = hlaPublisher;

                PDUConnector aConnect = new PDUConnector(disSubAdmin,hlaPublisher,(jdis.PushPDUType)hladisGUI.getDISPanel().getPDUGrapher()); 

                currentSubAdmin = disSubAdmin;
                currentSubAdmin.startCollect();

             }

     } catch(Exception lastresort) { System.out.print(" Glup! "); }

    }// end of actionPerformed()

    /**
    */
    final void 
    reset(boolean refresh) 
    {
       if(refresh && eView!=null) eView.refresh();
       if(aFileSaver!=null)       { aFileSaver.stop(); aFileSaver=null;}
       if(aFilePlayer!=null)      { aFilePlayer.stop(); aFilePlayer=null;}
       if(currentSubAdmin!=null)  { currentSubAdmin.stopCollect(); }
       if(currentPublisher!=null) { currentPublisher.stopPublishing(); }

    }// end of reset()

    /**
    */
    public void 
    itemStateChanged(ItemEvent e) 
    {
        JMenuItem source = (JMenuItem)(e.getSource());

    }// end of itemStateChanged()

    /**
    *  Returns just the class name -- no package info.
    */
    protected String 
    getClassName(Object o) 
    {
        String classString = o.getClass().getName();
        int dotIndex = classString.lastIndexOf(".");
        return classString.substring(dotIndex+1);

    }// end of getClassName()

    /**
    * Needs update???
    */
    public static void 
    printUsage()
    {
          System.out.println();
          System.out.println(" java SMain -mcast <p1> -mport <p2> -cport <p3>");
          System.out.println("      <p1> : Multicast Group Address ( 230.0.0.1 i s default)");
          System.out.println("      <p2> : Multicast Port ( 4446 is default)");
          System.out.println("      <p3> : Stat Collecting UDP Port Number ( 3030 is default)");
          System.out.println("   purpose : Performance analysis tool for CMS ");
          System.out.println();

    }// end of printUsage()

    /**
    */
    public void 
    doExit() 
    {
        // Remove Comm Ports
        disPublisher.closeConnection();
        disSubAdmin.closeConnection();
        hlaPublisher.closeConnection();
        hlaSubAdmin.closeConnection();

        // Remove Windows 
        toBack();
        dispose();

        // Exit 
        System.exit(0);

    }// end of doExit()

    public void setFrequency(int _fre) 
    { if(_fre>0 && _fre<100) freq = _fre; } 

    public int  getFrequency() { return freq ; } 

    public void setPackageSize(int _psize) 
    { if(_psize>0 && _psize<1000) psize = _psize; } 

    public int  getPackageSize() { return psize ; } 

    /**
    * This method updates the DIS settings if they are changed.
    */
    public void  
    updateDISSettings(String multicastAddress, int multicastPort) 
    {  

       if( !multicastAddress.equals(mCastAddress) || gPortNumber!=multicastPort )
       {
           disPublisher.reconnect(multicastAddress,multicastPort);
           disSubAdmin.reconnect(multicastAddress,multicastPort);
           mCastAddress = multicastAddress;
           gPortNumber = multicastPort;

       }// end of if

    }// end of updateDISSettings() 

    /**
    * This method updates the HLA settings if they are changed.
    */
    public void  
    updateHLASettings(String nameServerAddress, int nameServerPort) 
    {  

       if( !nameServerAddress.equals(nServer) || nameServerPort!=nPortNumber )
       {
           hlaPublisher.reconnect(nameServerAddress,nameServerPort);
           hlaSubAdmin.reconnect(nameServerAddress,nameServerPort);
           nServer = nameServerAddress;
           nPortNumber = nameServerPort;

       }// end of if

    }// end of updateHLASettings() 

    /**
    * This method updates the HTTP settings if they are changed.
    */
    public void  
    updateHttpSettings(String nameServerAddress, String _logUri, String _logCmd) 
    {  

       if( !nameServerAddress.equals(logServer) || !_logUri.equals(logUri) ||
           !_logCmd.equals(logCmd) )
       {
           ((HTTPPDUPublisher)httpPublisher).update(nameServerAddress,_logUri,_logCmd);
           logServer = nameServerAddress;
           logUri = _logUri;
           logCmd = _logCmd;
       }// end of if

    }// end of updateHttpSettings() 

    /**
    */
    public static void 
    main(String args[]) 
    {
        JDISConstants.load("default.properties");

        String  mCastAddress   = JDISConstants.DEFAULT_MCAST;
        int     gPortNumber    = JDISConstants.DEFAULT_MPORT;
        int     collectUdpPort = JDISConstants.DEFAULT_COLLECTUDPPORT;         
        int     nPortNumber    = JDISConstants.DEFAULT_SERVER_PORT;
        String  nServer        = JDISConstants.DEFAULT_SERVER; 

        int     dummy;
        boolean giveHelp=false;

        // Handle Command-line options if any
        int i=0;

        Properties cmdLine = new Properties();
        ByteArrayOutputStream cmdOut = new ByteArrayOutputStream();
        PrintWriter cmdPWr = new PrintWriter(cmdOut);

        while( i<args.length )
        {
               if(args[i].startsWith("-")) 
                  cmdPWr.println(args[i].substring(1)+"="+args[++i]);
          else if(args[i].equals("-help")||args[i].equals("-h")) giveHelp=true;  
          i++;

        }// end of for
        cmdPWr.flush();

        ByteArrayInputStream cmdIn = new ByteArrayInputStream(cmdOut.toByteArray());
        try
        { 
          cmdLine.load(cmdIn);
          JDISConstants.update(cmdLine);

        } catch(IOException ioe) {}

        mCastAddress   = JDISConstants.DEFAULT_MCAST;
        gPortNumber    = JDISConstants.DEFAULT_MPORT;
        collectUdpPort = JDISConstants.DEFAULT_COLLECTUDPPORT;         
        nPortNumber    = JDISConstants.DEFAULT_SERVER_PORT;
        nServer        = JDISConstants.DEFAULT_SERVER; 

        if(giveHelp) SMain.printUsage();
        else
         {
            SMain window = new SMain(mCastAddress,gPortNumber,collectUdpPort,nServer,nPortNumber);

            window.setTitle(VisConstants.MainFrameTitle);
            window.setSize(420, 460);
            window.setVisible(true);
         }

    }// end of main()

}// end of Class SMain
