/*
 File:		AwtMulticastRelayClient.java
 CVS Info:	$Id: AwtMulticastRelayClient.java,v 1.1 1998/02/09 19:00:02 mcgredo Exp $
 Compiler:	jdk1.1.6
 */

package mil.navy.nps.awt;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

import netscape.security.*;     // to enable netscape to read from random datagram sockets

/**
A quick applet that does local relay work for multicast. This is a bit better than a 
plain command line application. Lots of verbose and silly J++ comments included below.

 *@version 1.0
 *@author Don McGregor <A HREF="http://www.stl.nps.navy.mil/~mcgredo">http://www.stl.nps.navy.mil/~mcgredo</A>
 */

public class AwtMulticastRelayClient extends Applet implements ActionListener, ItemListener
{
  public static final String DEFAULT_SERVER = "devo.stl.nps.navy.mil";
  public static final String DEFAULT_SERVER_PORT = "8010";
  public static final String DEFAULT_MCAST_ADDRESS = "224.2.181.145";
  public static final String DEFAULT_MCAST_PORT = "62040";
  public static final String DEFAULT_LOCAL_UNICAST = "8006";
  public static final int    MAX_PACKET_SIZE = 1500;
  public static final boolean DEBUG = true;
  public static final int     SOCKET_TIMEOUT = 40000;           // 40 sec timeout


	// STANDALONE APPLICATION SUPPORT:
	//		m_fStandAlone will be set to true if applet is run standalone
	boolean m_fStandAlone = false;

  // The server we connect to over unicast
  TextField       mcastServer;
  TextField       mcastServerPort;

  // Local parameters: the mcast address we transmit on locally, (if mcast),
  // the port number we transmit on, and a count of how many packets we've
  // relayed. This is mostly eye candy to convice people it's working.

  TextField      sendAddress;
  TextField      port;                // used for both mcast and unicast destinations
  TextField      localUnicastPort;    // what port to send out on ON THIS MACHINE, if using unicast
  TextField      packetCount;

   // In the SocketStyleRegion panel
  CheckboxGroup   socketTypeRadio;        // Radio for how to send info--unicast or multicast
  Checkbox        multicastCheckbox;      // checkbox selection for multicast
  Checkbox        unicastCheckbox;        // Checkbox selection for unicast

  boolean         sending = false;

  // Buttons for starting & stopping
  Button          startButton;
  Button          stopButton;

  // Sockets for receiving & sending
  DatagramSocket  unicastSocket;
  MulticastSocket multicastSocket;

/**
Debugging output aid
*/
public static void debug(String pDebugString)
{
  if(DEBUG)
    System.out.println(pDebugString);
}


  /**
    Main constructor, which lays out fields in the AWT. 
  */
  public AwtMulticastRelayClient()
	{
    Panel serverRegion = new Panel();           // panel for server data
    Panel clientRegion = new Panel();           // client data region
    Panel checkboxRegion = new Panel();         // region for checkbox
    Panel clientTextfieldRegion = new Panel();  // Region for client textfield data
    Panel buttonRegion = new Panel();           // region that holds buttons


    GridLayout serverRegionLayout = new GridLayout(2,2);
    GridLayout clientRegionLayout = new GridLayout(1,1);
    GridLayout checkboxRegionLayout = new GridLayout(2,2);
    GridLayout clientTextfieldRegionLayout = new GridLayout(4,2);


    // The text fields for the server data portion 
    serverRegion.setLayout(serverRegionLayout);
    serverRegion.add(new Label("Server Name:"));
    mcastServer = new TextField(DEFAULT_SERVER, 20);
    serverRegion.add(mcastServer);

    serverRegion.add(new Label("Server Port:"));
    mcastServerPort = new TextField(DEFAULT_SERVER_PORT,20);
    serverRegion.add(mcastServerPort);

    add(serverRegion);

    // Client data region: a checkbox and a few fields.

		// Add a radio button, unicast or multicast
     socketTypeRadio = new CheckboxGroup();
     checkboxRegion.setLayout(checkboxRegionLayout);
     multicastCheckbox = new Checkbox("Multicast", socketTypeRadio, true);
     checkboxRegion.add(multicastCheckbox);
     unicastCheckbox = new Checkbox("Unicast", socketTypeRadio, false);
     checkboxRegion.add(unicastCheckbox);

     // Call us when the user hits the radio button so we can enable & disable fields
     unicastCheckbox.addItemListener(this);
     multicastCheckbox.addItemListener(this);

     clientRegion.add(checkboxRegion);

     // Add a bunch of text fields for client data
     clientTextfieldRegion.setLayout(clientTextfieldRegionLayout);

     // mcast address to send to locally
     clientTextfieldRegion.add(new Label("Address:"));
     sendAddress = new TextField(DEFAULT_MCAST_ADDRESS, 10);
     clientTextfieldRegion.add(sendAddress);

     // The port to send to localy, could be unicast or mcast
     clientTextfieldRegion.add(new Label("Port:"));
     port = new TextField(DEFAULT_MCAST_PORT, 10);
     clientTextfieldRegion.add(port);

     // eye candy: how many packets relayed
     clientTextfieldRegion.add(new Label("Packets Relayed:"));
     packetCount = new TextField("0", 10);
     clientTextfieldRegion.add(packetCount);

    clientRegion.add(clientTextfieldRegion);
    add(clientRegion);

    // Start & stop buttons
    startButton = new  Button("Start");
    startButton.addActionListener(this);
    buttonRegion.add(startButton);

    stopButton  = new Button("Stop");
    stopButton.disable();
    stopButton.addActionListener(this);
    buttonRegion.add(stopButton);

    add(buttonRegion);
	}

	public static void main(String args[])
	{
		// Create Toplevel Window to contain applet AwtMulticastRelayClient
		AwtMulticastRelayClientFrame frame = new AwtMulticastRelayClientFrame("MulticastRelayClient");

		// Must show Frame before we size it so insets() will return valid values
		frame.show();
        frame.hide();
		frame.resize(frame.insets().left + frame.insets().right  + 320,
					 frame.insets().top  + frame.insets().bottom + 240);

		// The following code starts the applet running within the frame window.
		// It also calls GetParameters() to retrieve parameter values from the
		// command line, and sets m_fStandAlone to true to prevent init() from
		// trying to get them from the HTML page.
		AwtMulticastRelayClient applet_MulticastRelayClient = new AwtMulticastRelayClient();

		frame.add("Center", applet_MulticastRelayClient);
		applet_MulticastRelayClient.m_fStandAlone = true;
		applet_MulticastRelayClient.init();
		applet_MulticastRelayClient.start();
        frame.show();
	}


	// APPLET INFO SUPPORT:
	//		The getAppletInfo() method returns a string describing the applet's
	// author, copyright date, or miscellaneous information.
    //--------------------------------------------------------------------------
	public String getAppletInfo()
	{
		return "Name: AwtMulticastRelayClient\r\n" +
		       "Author: Don McGregor\r\n" +
		       "GNU Copyleft applies";
	}


	// The init() method is called by the AWT when an applet is first loaded or
	// reloaded.  Override this method to perform whatever initialization your
	// applet needs, such as initializing data structures, loading images or
	// fonts, creating frame windows, setting the layout manager, or adding UI
	// components.
    //--------------------------------------------------------------------------
	public void init()
	{
        // If you use a ResourceWizard-generated "control creator" class to
        // arrange controls in your applet, you may want to call its
        // CreateControls() method from within this method. Remove the following
        // call to resize() before adding the call to CreateControls();
        // CreateControls() does its own resizing.
        //----------------------------------------------------------------------
		resize(320, 240);

		// TODO: Place additional initialization code here
	}

	// Place additional applet clean up code here.  destroy() is called when
	// when you applet is terminating and being unloaded.
	//-------------------------------------------------------------------------
	public void destroy()
	{
		// TODO: Place applet cleanup code here
	}

	// AwtMulticastRelayClient Paint Handler
	//--------------------------------------------------------------------------
	public void paint(Graphics g)
	{
		
	}

	//		The start() method is called when the page containing the applet
	// first appears on the screen. The AppletWizard's initial implementation
	// of this method starts execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void start()
	{
		// TODO: Place additional applet start code here
	}
	
	//		The stop() method is called when the page containing the applet is
	// no longer on the screen. The AppletWizard's initial implementation of
	// this method stops execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void stop()
	{
	}

  /** 
  Does the dirty work of actually relaying the packets. Connects up to
  the server, reads the packets, and spits them back out again on this side.
  */

public void relayPackets()
{
    byte   outBuff[]  = new byte[MAX_PACKET_SIZE]; 
    byte   inBuff[]   = new byte[MAX_PACKET_SIZE];
    DatagramPacket inDgram=null, outDgram=null;

    String connectCommand = new String("connect");            //connect command 
    byte   conbuf[] = new byte[connectCommand.length()];     // byte buffer for connect command
    String disconnectCommand = new String("disconnect");
    
    InetAddress  serverAddress = null;
    int          serverPortValue = 0;
    String       serverAddressValue = null;
    InetAddress  destMcastAddress = null;
    InetAddress  destUnicastAddress = null;
    String       destUnicastAddressValue = null;
    int          destPortValue = 0;
    int          packetSentCount = 0;

    boolean      usingMulticast = true;

    // THIS IS NETSCAPE SPECIFIC. Switch on all access for sockets.

    PrivilegeManager privilegeManager;

    privilegeManager = PrivilegeManager.getPrivilegeManager();
    privilegeManager.enablePrivilege("UniversalMulticast");
    privilegeManager.enablePrivilege("UniversalConnect");
    privilegeManager.enablePrivilege("UniversalAccept");
    privilegeManager.enablePrivilege("UniversalListen");

    // unicast or multicast?
    if(socketTypeRadio.getSelectedCheckbox() == multicastCheckbox)
      usingMulticast = true;
    else
      usingMulticast = false; 
    
    try  
    { 
       // Create a unicast socket for sending commands and relaying packets
       unicastSocket = new DatagramSocket(); 
       debug("Client unicast port is:  " + unicastSocket.getLocalPort()); 

       destPortValue = Integer.valueOf(port.getText()).intValue();

       // If we're mcast, we need a second socket for sending out on that.

       if(usingMulticast)
       {
          destMcastAddress = InetAddress.getByName(sendAddress.getText());
          multicastSocket = new MulticastSocket(destPortValue);
          multicastSocket.joinGroup(destMcastAddress);
       }
       else
       {
         destUnicastAddressValue = sendAddress.getText();
         if(destUnicastAddressValue.equalsIgnoreCase("localhost"))
          destUnicastAddress = InetAddress.getLocalHost();
         else
          destUnicastAddress = InetAddress.getByName(serverAddressValue);
       }

       // Look up server address and server port number

      serverAddressValue = mcastServer.getText();
      if(serverAddressValue.equalsIgnoreCase("localhost"))
        serverAddress = InetAddress.getLocalHost();
      else
        serverAddress = InetAddress.getByName(serverAddressValue);

      serverPortValue = Integer.valueOf(mcastServerPort.getText()).intValue();

    }  
    catch (java.io.IOException e)  
    { 
      System.err.println("Could not create datagram socket."); 
    } 
    catch(NumberFormatException nfe)
    {
      System.out.println("bad format for port, " + port.getText());
    }

    // Send a command to the server to connect us. First, find the address
    // of the server

    // Hmmm....packets are unreliable. What happens if dropped on floor? Seems to
    // work for now, anyway.

    try
    {
      conbuf = connectCommand.getBytes();     
	    outDgram = new DatagramPacket(conbuf, conbuf.length, serverAddress, serverPortValue); 
	    unicastSocket.send(outDgram);  //send connect 
    }
    catch(IOException ioe)
    {
      System.out.println("Unable to send connect command");
    }
	 
    // Loop until they hit the stop button, relaying packets.
    while(sending) 
	    { 
          try
          { 
            // give the server 30 seconds to respond then exit 
            unicastSocket.setSoTimeout(SOCKET_TIMEOUT); //40 second server response timeout            
            inDgram = new DatagramPacket(inBuff, inBuff.length); 
          
            unicastSocket.receive(inDgram); 
            debug("got a packet from the server"); 
          } 
          catch(SocketException se)
          {
            System.out.println("Can't set timeout on socket");
          }
          catch (IOException e) 
          { 
            System.out.println("*************************************************************"); 
            System.out.println("Server did not respond, try later or check with server admin"); 
            System.out.println("*************************************************************"); 
            System.out.println(); 
            
            break;
	       } 
	      
	      inBuff = inDgram.getData(); 
	      System.arraycopy(inBuff, 0, outBuff, 0, inBuff.length);  
 
        try
        {
          if(usingMulticast)
          {
	         outDgram = new DatagramPacket(outBuff, outBuff.length, destMcastAddress,  destPortValue);   
            multicastSocket.send(outDgram);
          }
          else
          {
            outDgram = new DatagramPacket(outBuff, outBuff.length, destUnicastAddress,  destPortValue);
            unicastSocket.send(outDgram);
          }
          packetSentCount++;
          packetCount.setText(Integer.toString(packetSentCount));
        }
        catch(IOException ioe)
        {
          System.out.println("couldn't relay packet");
        }
	     
    }//endwhile 

    try
    {
       //  disconnect cleanly from server.
		  byte[] disbuf = new byte[disconnectCommand.length()]; 
		  disbuf = disconnectCommand.getBytes();      
		  outDgram = new DatagramPacket(disbuf, disbuf.length, serverAddress, serverPortValue); 
		  unicastSocket.send(outDgram); 
		  unicastSocket.close(); 
      if(usingMulticast)
       multicastSocket.leaveGroup(destMcastAddress);
      sending = false;
      startButton.enable();
      stopButton.disable();
    }
    catch(IOException ioe)
    {
     System.out.println("Couldn't disconnect");
    }
	  
      }//end McastClientConnect 
   

/**
  ActionListener for the start and stop buttons. This uses
  the AWT 1.1 event model, and implemens the ActionListener
  interface.
*/

public void actionPerformed(ActionEvent ae)
{
  Object source = ae.getSource();

  if(source == startButton)
  {
    RelayThread relayThread = new RelayThread(this);
    sending = true;
    relayThread.start();
    startButton.disable();
    stopButton.enable();

  }
  if(source == stopButton)    // Stop button hit
  {
    sending = false;
    startButton.enable();
    stopButton.disable();
  }
  
}   // end of actionListener

/**
itemStateChanged is the implementation of the interface ItemListener,
which is used by checkboxes. The checkboxes add us as a listener, and
we get their events. This is based on the Java 1.1 event model.
*/

public void itemStateChanged(ItemEvent e) // interface for ItemListener
{
  Object source = e.getSource();

  if(source == unicastCheckbox)
  {
    sendAddress.setText("localhost");
    port.setText(DEFAULT_LOCAL_UNICAST);
  }
  if(source == multicastCheckbox)
  {
    sendAddress.setText(DEFAULT_MCAST_ADDRESS);
    port.setText(DEFAULT_MCAST_PORT);
  }
} // end of itemStateChanged

	// TODO: Place additional applet code here

}
