//******************************************************************************
// AwtPduViewer.java:	Applet
//
//******************************************************************************

package mil.navy.nps.awt;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;          // DIS Library

/**

  AwtPduViewer is an applet that reads Pdus and displays their values on the
  screen. It is configured with a socket, which allows the applet to read
  data from the wire; it uses the package mil.navy.nps.dis, which implements
  the DIS protocol. <P>

  This file was generated by Microsquish J++, which is something of an
  experiment. We'll see how it works out. It's also my first applet that
  isn't completely trivial, so be a little gentle.<P>

  This is compliant with the JDK 1.02 spec; it makes use of deprecated aspects
  of that API rather than use JDK 1.1. This is primarily to get it working with
  Netscape 3.01. 1.1 browser implementations should be compatible, at least for
  the next release.<P>

  Netscape bitches & moans about doing a thread.suspend(), complaining about
  the possibility of deadlock. Not sure what this is all about, but it seems
  to work.<P>

  This takes two parameters from the HTML file, "PduSocket" and"MulticastGroup".
  The first is the socket the applet listens on. The second is a multicast group,
  a feature that is not implemented right now since Netscape doesn't do multicast.<P>

  Testing: sent ESPdus to host (madison) from both madison and pinafore using
  mil.navy.nps.dis.Benchmark. 


  HISTORY<P>

  10Apr97 DMcG  New<BR>

  @author Don McGregor (<A HREF="http://www.stl.nps.navy.mil/~mcgredo"> http://www.stl.nps.navy.mil/~mcgredo</A>)<P>
  @version 1.0 
  @see UpdateThread
  @see TextAreaStream

 */

public class AwtPduViewer extends Applet implements Runnable
{
    // Members for applet parameters
    // <type>       <MemberVar>    = <Default Value>

	private    String    m_PduSocket       = "3111";
	private    String    m_MulticastGroup  =  null;          // no default value; unicast by default

  // Parameter names.  To change a name of a parameter, you need only make
	// a single change.  Simply modify the value of the parameter string below.

	private final  String  PARAM_PduSocket      = "PduSocket";
	private final  String  PARAM_MulticastGroup = "MulticastGroup";

  private  NetworkMonitor     networkMonitor = null;      // Reads packets & stores them up
  private  Vector             pduPool        = null;      // data structure that keeps list of arrived Pdus
  private  UpdateThread       updateThread   = null;      // thread that updates screen display

  private  TextArea           textArea    = null;         // scrolling text area where data is displayed
  private  List               list        = null;         // List of Pdus to choose from
  private  Button             startButton = null;         // start button
  private  Button             stopButton  = null;         // stop button
  private  Button             flushButton = null;         // flush button
  private  PrintStream        textAreaPrintStream = null; // print stream, sent to text area above

	public AwtPduViewer()
	{
		
    pduPool = new Vector();
    updateThread = new UpdateThread(this);
	}

	public String getAppletInfo()
	{
		return "Name: AwtPduViewer\r\n" +
		       "Author: Don McGregor\r\n" +
		       "GNU Copyleft 1997";
	}

	// The getParameterInfo() method returns an array of strings describing
	// the parameters understood by this applet.

	public String[][] getParameterInfo()
	{
		String[][] info =
		{
			{ PARAM_PduSocket, "String", "Socket we listen to for Pdus" },
			{ PARAM_MulticastGroup, "String", "Multicast Group (Java 1.1 only)" },
		};
		return info;		
	}

  /**
     Applet initialization code
  */

	public void init()
	{
		
		String          param;                  // a parameter passed into the applet
    TextAreaStream  textAreaStream  = null;
    Thread          aThread         = null;


    // Parameter retrieval: socket to listen on
		param = getParameter(PARAM_PduSocket);
		if (param != null)
			m_PduSocket = param;
    System.out.println(param);

    // Parameter retrieval: multicast group. If no multicast
    // group is passed in, the value is set to null; if one IS passed in, we instantiate
    // a multicast NetworkMonitor. This feature is Java 1.1-specific.

		param = getParameter(PARAM_MulticastGroup);
		if (param != null)
			m_MulticastGroup = param;
     else
       m_MulticastGroup = null;

       System.out.println("m_PduSocket = "        + m_PduSocket +
                          ", m_MulticastGroup = " + m_MulticastGroup);
 
	resize(320, 400);

    // List box, contains scrolling display of arrived Pdus
    list = new List(10, false);     

    Dimension dim = new Dimension(20,40);

    list.resize(dim);
    add(list);

    System.out.println("list size is " + list.size().width + " " + list.size().height);

    // scrolling text area to display output (lines, width)
    textArea = new TextArea(13,45);
    add(textArea);

    // Start and stop buttons to turn on or off socket. Initially, the
    // socket and thread are configured to be in the reading state, so
    // the start button is off, the stop button is on, and the flush 
    // button is always on.

    startButton = new Button("Start");
    startButton.disable();
    add(startButton);

    stopButton = new Button("Stop");
    add(stopButton);

    flushButton = new Button("Flush Pdus");
    add(flushButton);

    // Wrap an output stream around the TextArea, so that we can
    // send stuff streaming to the display with a printf().

    textAreaStream        = new TextAreaStream(textArea);
    textAreaPrintStream   = new PrintStream(textAreaStream); 

    // The threads need to be kicked off from here, since apparently
    // run() is called before init(), and we need the parameters passed
    // in to create the correct socket for the network monitor, and the
    // update thread needs the network monitor to be initialized. Sigh.

    // Create a socket and start reading from it. If we passed in a multicast
    // group as a parameter, create a multicast socket; otherwise, just read
    // from a unicast socket. Note that multicast will NOT work with Visual J++
    // 1.0, which is off the JDK 1.0 spec; you need Java 1.1.

    if(m_MulticastGroup == null)
      networkMonitor = new NetworkMonitor(Integer.valueOf(m_PduSocket).intValue());
     else
      networkMonitor = new NetworkMonitor(m_MulticastGroup, Integer.valueOf(m_PduSocket).intValue());

    aThread = new Thread(networkMonitor);
    networkMonitor.setThread(aThread);
    aThread.start();

    // Start up the thread that updates the display
    updateThread.start();

		return;
	}

	// Place additional applet clean up code here.  destroy() is called when
	// when you applet is terminating and being unloaded.
	//-------------------------------------------------------------------------
	public void destroy()
	{
		// TODO: Place applet cleanup code here
	}

    // ANIMATION SUPPORT:
    //		Draws the next image, if all images are currently loaded
    //--------------------------------------------------------------------------
	private void displayImage(Graphics g)
	{

	}

	// AwtPduViewer Paint Handler
	//--------------------------------------------------------------------------
	public void paint(Graphics g)
	{
	
	}

	//		The start() method is called when the page containing the applet
	// first appears on the screen. The AppletWizard's initial implementation
	// of this method starts execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void start()
	{
		
	}
	
	//		The stop() method is called when the page containing the applet is
	// no longer on the screen. The AppletWizard's initial implementation of
	// this method stops execution of the applet's thread.
	//--------------------------------------------------------------------------
	public void stop()
	{
		
	}

	// THREAD SUPPORT
	//		The run() method is called when the applet's thread is started. If
	// your applet performs any ongoing activities without waiting for user
	// input, the code for implementing that behavior typically goes here. For
	// example, for an applet that performs animation, the run() method controls
	// the display of images.
	//--------------------------------------------------------------------------
	public void run()
  {
    
    // Threads are started from the init() method in this case, since
    // the networkMonitor needs parameters passed in to work, updateThread
    // needs networkMonitor, and init() is called after run().

		repaint();
  }


  /**
  Updates the display by querying the NetworkMonitor, returning any Pdus that
  have arrived since the last time we asked, and updating the display. This
  is kicked off by its own thread, so it runs asyncronously from the app.
  the display is updated every X seconds.

  We need to lock access to pduPool, since this is global data used elsewhere
  in the applet in another thread. Not sure about the requirement to lock list, 
  the GUI element that displays the scrolling list. This is the only place we 
  do a _write_ to it, so I think it's OK.
  */

public void updateDisplay()
{
  Vector      receivedPdus  = null;             // list of Pdus we get back from network monitor;
                                                // those that have arrived since the last time we asked
  Enumeration pduEnumeration;                   // Loop through list
  String      PduNameAndTime = null;            // string that has name of Pdu & timestamp
  String      time = null;                      // string representation of 32-bit timestamp on Pdu

  while(true)     // loop endlessly
  {

    receivedPdus = networkMonitor.receivedPdus();
    System.out.println("got " + receivedPdus.size() + " Pdus this cycle");

    // Add the new Pdus to the existing pool of Pdus and to the end of the
    // GUI list, and place the name of the Pdu type in the display list. This 
    // could probably be faster, but it's OK for now.

    pduEnumeration = receivedPdus.elements();
    while(pduEnumeration.hasMoreElements())
    {
      ProtocolDataUnit  aPdu = (ProtocolDataUnit)pduEnumeration.nextElement();

      // lock access to pduPool, sincee this may be used elsewhere in the program
      // at the same time.

      synchronized(pduPool)
      {
        pduPool.addElement(aPdu);
      }

      // add the name of the Pdu (eg, "Entity State", "Fire", etc) and the
      // 32-bit timestamp to the scrolling list

      PduNameAndTime = aPdu.PDUName();
      time = String.valueOf(aPdu.getTimestamp().longValue());
      PduNameAndTime = PduNameAndTime.concat(new String(" "));
      PduNameAndTime = PduNameAndTime.concat(time);  
      list.addItem(PduNameAndTime);
    }

    this.repaint();

    
    try
    {
      updateThread.sleep(5000);    // sleep for five seconds
    }
    catch (InterruptedException intException)
    {
      throw new 
            RuntimeException("Exception in AwtPduViewer; fitful sleep");
    }
  


  }

}

public boolean action(Event event, Object what)
  {
   
  // if the event target is a button, check to see if it's one
  // of our three. If so, do the correct thing for that button.

    if (event.target instanceof Button)
      {
        Button b = (Button) event.target;

        // Start button; start up reading threads, turn off button, toggle on "stop" button.
        if(b.getLabel() == "Start")
        {
          startButton.enable(false);
          stopButton.enable(true);
          networkMonitor.resumeReading();
          updateThread.resume();
          
          this.repaint();
        }

        // Stop button; suspend threads, turn off button, toggle on "start" button.
        if(b.getLabel() == "Stop")
        {
          startButton.enable(true);
          stopButton.enable(false);
          networkMonitor.suspendReading();
          updateThread.suspend();
          this.repaint();
        }

        // Flush Pdu button hit--remove all the Pdus from the list, clear out
        // GUI display.

        if(b.getLabel() == "Flush Pdus")
        {
          int   textLength = -1;
          
          // Lock access to pduPool, since this is also used in another thread,
          // and we don't want two people writing/modifiying it.
          synchronized(pduPool)
          {
            pduPool = new Vector(10);   // everyting should be GC'd.
          }

          // Ditto for the GUI list; don't want to be adding to it in the updateDisplay
          // method at the same time we're flushing it.
          synchronized(list)
          {
            list.clear();
          }

          // wipe out all the data in the TextArea
          textArea.selectAll();
          textLength = textArea.getSelectionEnd();

          // Deprecated; change for 1.1
          textArea.replaceText(new String(""), 0, textLength);
        }
    }
  return true;
}

public boolean handleEvent(Event evt)
{
  // This is deprecated in the Java 1.1 spec. Should use processEvent()
  // instead. It handles a selection hit on the scrolling GUI list.

  int               itemIndex = -1;     // index into list of the selected item
  ProtocolDataUnit  aPdu      = null;   // Selected Pdu in list

  // if this is a select-item-from-list event, and if the list
  // being selected from is the list of Pdus, process the event.

  if(evt.id == Event.LIST_SELECT)
  {
    if(evt.target.equals(list))
    {
      // find the selected item in the list, then print the values to the
      // text area. The existing data in the text area is wiped out.
      int   textLength = -1;

      textArea.selectAll();
      textLength = textArea.getSelectionEnd();

      // Deprecated; change for 1.1
      textArea.replaceText(new String(""), 0, textLength);

      // Print the new data
      itemIndex = list.getSelectedIndex();
      aPdu = (ProtocolDataUnit)pduPool.elementAt(itemIndex);
      aPdu.printValues(5,textAreaPrintStream);
      
    }
  }
  else
    return super.handleEvent(evt);

  return true;

}

}   // end of class AwtPduViewer
