
package mil.navy.nps.bridge;


import java.io.*;
import java.lang.*;
import java.net.*;

/*

   Bridge

  This is a workaround for browser security that can go away once we get signed
  applets and figure out how to use them. Some people want to
  write to arbitrary sockets on other machines, which is a Bad Thing
  from a security standpoint for an applet. However, it's OK for a stand-alone
  java program to do this. This goes for multicast in applets as well, since
  applets cannot bypass the sandbox by doing reads or writes to multicast
  sockets.

  So the idea here is that the applet writes packets to a Java process on
  the same machine. Since the java process isn't in an applet, it can
  write where ever it pleases. It simply passes on the packets to the
  machine we wanted to go to in the first place. 

  There are four instances here I can think of: Reading unicast and writing
  unicast, reading multicast and writing unicast, reading unicast and writing
  multicast, and reading and writing multicast. This breaks down into four
  subclasses of the Bridge class. I thought about putting it all into one
  class, but that seemed kind of ugly--all sorts of unrelated stuff in one
  class. On the other hand, there isn't a lot of common code to be factored
  out into this abstract class, so it's not clear this approach is a win,
  either. There's gotta be a better way.

  This is an abstract class that implements the Runnable interface. To use
  a subclass, create an instance and then start it in a thread:

  BridgeUniToUni  uniBridge = new BridgeUniToUni(...)
  Thread          aThread   = new Thread(uniBridge);

  aThread.start();

  This requires Java 1.1, since it looks for the multicast sockets in the
  standard Java 1.1 place.

  Note that the use of datagrams makes this bridging operation UNRELIABLE.
  If packets come in too fast, there is no guarantee that all packets will
  be bridged. Not sure about the semantics of multicast here.

    HISTORY

	19Aug97	DMcG			Made "multicast" terminology consistent


   @author Don McGregor (<A HREF="http://www.stl.nps.navy.mil/~mcgredo"> http://www.stl.nps.navy.mil/~mcgredo</A>)<P>
   @version 1.0 
   
*/



public abstract class Bridge extends Object implements Runnable
{
  protected String      destinationMachineName  = null;           // machine we're sending to
  protected int         destinationSocketNumber = -1;             // socket number on remote machine
  protected InetAddress destMachineInet         = null;           // Inet address; cached to save DNS server hits

  protected static final int MAX_PACKET_SIZE    = 1500;           // MTU, more or less. May cause probs with FDDI,
                                                                  // but just about all DIS packets are smaller than MTU.
  protected static final boolean DEBUG          = true;          // prints out debugging info

  public abstract void run();                                     // starts bridging; implemented by subclasses

public Bridge ()
{
	// default consructor for multicast, when we don't need to worry about
	// destination machines and all that.
	
	return;
}

public  Bridge(String   pDestinationMachine,                      // NAME of machine we're sending to
               int      pDestinationSocketNumber)                 // socket number on that machine
{
  destinationMachineName  = pDestinationMachine;
  destinationSocketNumber = pDestinationSocketNumber;

  // find this once, then keep it. Saves a bunch of lookups.
    try
    {
      destMachineInet    = InetAddress.getByName(destinationMachineName);
    }
    catch(UnknownHostException unkHost)
    {
      throw new RuntimeException("can't find hostname" + destinationMachineName + "in DNS/NIS/host tables");
    }

    return;
  }

}
