/*
  BridgeHarness

  A test harness for the various subclasses of Bridge[Unicast|Multicast]To[Unicast|Multicast].

  Though it has been known to actually work at times.

  This is a harness that instantiates the correct bridge subclass based on the
  arguments passed in.

  readerWriter is a special case that threads both a reader and a writer which connect
  to the default unicast browser sockets used by espduReadTransform and espduWriteTransform

      HISTORY

	20Aug97	DMcG			Made "multicast" terminology consistent


*/

package mil.navy.nps.bridge;

import java.net.*;

class BridgeHarness extends Object
{

public static void main(String args[])
{
  // Verbose diagnostic output
	
  boolean		DEBUG = true;
  
  // The input socket. Type, number, and (optional) multicast address.

  String  paramInputSocketType              = null;         // unicast or multicast
  String  paramInputAddress                 = "localhost";  // default
  String  paramInputPort                    = null;

  // The output socket. Type, number, and (optional) multicast address.

  String paramOutputSocketType              = null;         // unicast or multicast
  String paramOutputAddress                 = null;
  String paramOutputPort                    = null;

  // The destination Address

  String            paramDestAddress    = null;
  String            paramDestPort       = "localhost";  // default
  int               idx;

  Thread            bridgeThread        = null;
  Bridge            aBridge             = null;       // abstract class; instantiated as a concrete subclass

  InetAddress 		localAddress 		= null;
  String            multicastAddress    = null;
  String            multicastPort       = null;
  boolean           readerWriter        = false;



  // pick up parameters from command line

  // Error checking. Can't use getParameter, since this isn't an applet.

  for(idx = 0; idx < args.length; idx++)
        {

	// echo back the command line parameters
        	
      if(args[idx].equalsIgnoreCase("-inputSockType"))
      {
        paramInputSocketType = args[idx+1];
        System.out.println("got command line arg of -inputSockType with value of " + paramInputSocketType);
      }

      if(args[idx].equalsIgnoreCase("-inputPort"))
      {
        paramInputPort = args[idx+1];
        System.out.println("got command line arg of -inputPort with value of " + paramInputPort);
      }

      if(args[idx].equalsIgnoreCase("-inputAddress"))
      {
        paramInputAddress = args[idx+1];
        System.out.println("got command line arg of -inputAddress with value of " + paramInputAddress);
      }

      if(args[idx].equalsIgnoreCase("-outputSockType"))
      {
        paramOutputSocketType = args[idx+1];
        System.out.println("got command line arg of -outputSockType with value of " + paramOutputSocketType);
      }

      if(args[idx].equalsIgnoreCase("-outputAddress"))
      {
        paramOutputAddress = args[idx+1];
        System.out.println("got command line arg of -outputAddress with value of " + paramOutputAddress);
      }

    if(args[idx].equalsIgnoreCase("-outputPort"))
                {
                        paramOutputPort = args[idx+1];
                        System.out.println("got command line arg of -outputPort with value of " + paramOutputPort);
                }

    if(args[idx].equalsIgnoreCase("-destAddress"))
                {
                        paramDestAddress = args[idx+1];
                        System.out.println("got command line arg of -destAddress with value of " + paramDestAddress);
                }

    if(args[idx].equalsIgnoreCase("-destPort"))
                {
                        paramDestPort = args[idx+1];
                        System.out.println("got command line arg of -destPort with value of " + paramDestPort);
                }

	if(args[idx].equalsIgnoreCase("-readerWriter"))
                     {
                        multicastAddress = args[idx+1];
                        multicastPort    = args[idx+2];
                        idx = idx + 2;
                        readerWriter = true;
                        System.out.println("got default readerWriter command line arguments " + multicastAddress + ", " + multicastPort);
                     }
                
    if((args[idx].equalsIgnoreCase("-h")) || (args[idx].equalsIgnoreCase("-?") ) )
                {
                        // Provide some help from the command line. 

      System.out.println("Command line options: ");
      System.out.println("  -inputPortType <unicast|multicast>  listen on multicast or unicast socket (required)");
      System.out.println("  -inputAddress <mcast Address>  multicast Address (required if mcast socket specified)");
      System.out.println("  -inputPort <Port no>              local Port to listen on");
      System.out.println("  -outputSockType <unicast|multicast> listen on multicast or unicast socket (required)");
      System.out.println("  -outputAddress <mcast Address> multicast Address (required if mcast socket specifed)");
      System.out.println("  -outputPort <Port no>             local Port to write on");
      System.out.println("  -destAddress <desination Address>   Address to send to");
      System.out.println("  -destPort <Port no>               remote Port to write to");
                        return;
                }
   }  // end of for indexing along command line arguments

   if (readerWriter == true)  // launch both reader and writer, no error checks
   {
    try {
  	    localAddress = InetAddress.getLocalHost ();
  		paramInputAddress = localAddress.getHostAddress ();
        System.out.println("InputAddress localhost = " + paramInputAddress);
  	}
  	catch (UnknownHostException e) {
        System.out.println("InputAddress localhost failed, couldn't find this computer's address.");
  	}
  	if(DEBUG)
  	{
  	 System.out.println("BridgeHarness: Creating multicast-to-unicast bridge");
  	 System.out.println("  input address:       " + multicastAddress);
  	 System.out.println("  input port:          " + multicastPort);
  	 System.out.println("  output Port:         " + 8005);
  	 System.out.println("  destination address: " + paramInputAddress);
  	 System.out.println("  destination port:    " + 8006);
  	}
  	 
    aBridge = new BridgeMulticastToUnicast( Integer.valueOf(multicastPort).intValue(),
                                            multicastAddress,
                                            8005,
                                            paramInputAddress,
                                            8006);
    bridgeThread = new Thread(aBridge);
    bridgeThread.start();

  if(DEBUG)
  	{
  	 System.out.println("BridgeHarness: Creating unicast-to-multicast bridge");
  	 System.out.println("  input port:          " + 8007);
  	 System.out.println("  destination address: " + multicastAddress);
  	 System.out.println("  destination port:    " + multicastPort);
  	}
  	
  	aBridge = new BridgeUnicastToMulticast(8007,
  	                                       multicastAddress,
  	                                       Integer.valueOf(multicastPort).intValue());

  	bridgeThread = new Thread(aBridge);
  	bridgeThread.start();
  }
       
else {  // all options other than readerWriter


  // alright, alright, some error checking. We _have_ to have some parameters.

  // Have to secify socket type, multicast or unicast

  if(paramInputSocketType == null)
  {
    System.out.println("Must specify multicast or unicast input socket type");
    return;
  }

  if(paramOutputSocketType == null)
  {
    System.out.println("Must specify multicast or unicast output socket types");
    return;
  }

  // Have to specify input and output Port numbers for the bridge

  if(paramInputPort == null)
  {
    System.out.println("must specify bridge InputPort");
    return;
  }

  if(paramDestPort == null)
  {
    System.out.println("must specify bridge DestPort");
    return;
  }

  if((paramOutputPort == null) && (paramOutputSocketType.equalsIgnoreCase("unicast")))
  {
    System.out.println("must specify bridge OutputPort for unicast output");
    return;
  }

  // Have to specify where it's going to go--Address name and Port number

  if(((paramDestAddress == null) || (paramDestPort == null)) && (paramOutputSocketType.equalsIgnoreCase("unicast")))
  {
    System.out.println("must specify destination Address and destination Port number");
    return;
  }

  // If either of the sockets was specified as multicast, check to make sure we got
  // a multicast Address for that socket, too.

  if((paramInputAddress == null) && (paramInputSocketType.equalsIgnoreCase("multicast")))
  {
    System.out.println("If you specify the input socket to be multicast, you must also specify a input Address");
    return;
  }

  if((paramDestAddress == null) && (paramOutputSocketType.equalsIgnoreCase("multicast")))
  {
    System.out.println("If you specify the output socket to be multicast, you must also specify a destination Address");
    return;
  }

  if (paramInputAddress.equalsIgnoreCase ("localhost"))
  {
  	try {
  	    localAddress = InetAddress.getLocalHost ();
  		paramInputAddress = localAddress.getHostAddress ();
        System.out.println("InputAddress localhost = " + paramInputAddress);
  	}
  	catch (UnknownHostException e) {
        System.out.println("InputAddress localhost failed, couldn't find this computer's address.");
  	}
  }

  if (paramDestAddress.equalsIgnoreCase ("localhost"))
  {
  	try {
  	    localAddress = InetAddress.getLocalHost ();
  		paramDestAddress = localAddress.getHostAddress ();
        System.out.println("DestAddress localhost = " + paramDestAddress);
  	}
  	catch (UnknownHostException e) {
        System.out.println("DestAddress localhost failed, couldn't find this computer's address.");
  	}
  }

  // Create bridge of correct type and run it.

  // unicast to unicast

  if((paramInputSocketType.equalsIgnoreCase("unicast")) && (paramOutputSocketType.equalsIgnoreCase("unicast")))
  {
  	if(DEBUG)
  	{
  	 System.out.println("BridgeHarness: Creating unicast-to-unicast bridge");
  	 System.out.println("  input port:          " + paramInputPort);
  	 System.out.println("  output port:         " + paramOutputPort);
  	 System.out.println("  destination Address: " + paramDestAddress);
  	 System.out.println("  destination Port:    " + paramDestPort);
  	}
  	             
  	 
    aBridge = new BridgeUnicastToUnicast( Integer.valueOf(paramInputPort).intValue(),
                                          Integer.valueOf(paramOutputPort).intValue(),
                                          paramDestAddress,
                                          Integer.valueOf(paramDestPort).intValue());
    bridgeThread = new Thread(aBridge);
    bridgeThread.start();
  }


  // multicast to unicast

  if((paramInputSocketType.equalsIgnoreCase("multicast")) && (paramOutputSocketType.equalsIgnoreCase("unicast")))
  {
  	if(DEBUG)
  	{
  	 System.out.println("BridgeHarness: Creating multicast-to-unicast bridge");
  	 System.out.println("  input address:       " + paramInputAddress);
  	 System.out.println("  input port:          " + paramInputPort);
  	 System.out.println("  output Port:         " + paramOutputPort);
  	 System.out.println("  destination address: " + paramDestAddress);
  	 System.out.println("  destination port:    " + paramDestPort);
  	}
  	 
    aBridge = new BridgeMulticastToUnicast( Integer.valueOf(paramInputPort).intValue(),
                                            paramInputAddress,
                                            Integer.valueOf(paramOutputPort).intValue(),
                                            paramDestAddress,
                                            Integer.valueOf(paramDestPort).intValue());
    bridgeThread = new Thread(aBridge);
    bridgeThread.start();
  }

  // unicast to multicast
  
  if((paramInputSocketType.equalsIgnoreCase("unicast")) && (paramOutputSocketType.equalsIgnoreCase("multicast")))
  {
  	
  if(DEBUG)
  	{
  	 System.out.println("BridgeHarness: Creating unicast-to-multicast bridge");
  	 System.out.println("  input port:          " + paramInputPort);
  	 System.out.println("  destination address: " + paramDestAddress);
  	 System.out.println("  destination port:    " + paramDestPort);
  	}
  	
  	aBridge = new BridgeUnicastToMulticast(Integer.valueOf(paramInputPort).intValue(),
  	                                       paramDestAddress,
  	                                       Integer.valueOf(paramDestPort).intValue());

  	bridgeThread = new Thread(aBridge);
  	bridgeThread.start();
  }
  
}  // end else readerWriter
  	
  // we have to sleep now, so the program doesn't exit and kill the bridge threads we
  // launched above. Kill with ctrl-C.

  while(true)
  {
    try
    {
      Thread.sleep(10000);          // go to sleep
    }
    catch(InterruptedException interruptedException)
    {
      throw new
        RuntimeException("Exceptional sleep ");
    }
  }


} // end of main

} // end of class
