package mil.navy.nps.bridge;

import java.io.*;
import java.net.*;


/*
   BridgeUnicastToMulticast

   This is a subclass of the Bridge abstract class, which implements datagram
   bridging between a unicast socket and a multicast socket. The program simply 
   accepts packets on a unicast socket, and repeats them out on a multicast
   socket.

    HISTORY

	19Aug97	DMcG			Made "multicast" terminology consistent


   @author Don McGregor (<A HREF="http://www.stl.nps.navy.mil/~mcgredo"> http://www.stl.nps.navy.mil/~mcgredo</A>)<P>
   @version 1.0 
   
*/

public class BridgeUnicastToMulticast extends Bridge implements Runnable
{

  private MulticastSocket outputSocket             = null;   // socket we write on 
  private DatagramSocket  inputSocket              = null;   // socket we read on

  private int             inputSocketNumber        = -1;     // input socket number 
  private String          multicastAddress         = null;   // multicast Address
  private int             multicastPort            = -1  ;   // multicast port

  private InetAddress     inetAddress             = null;


 public BridgeUnicastToMulticast( 
        int     pInputSocketNumber,                         // INPUT: socket to listen on
        String  pMulticastAddress,                          // INPUT: multicast Address to join
		int		pMulticastPort)								// INPUT: multicast port                     
{

   // superclass initialization

   //super(pDestMachineName, pDestSocketNumber);

  multicastPort  = pMulticastPort;
  multicastAddress = pMulticastAddress;

  System.out.println ("BridgeUnicastToMulticast  unicast port    = " + pInputSocketNumber);
  System.out.println ("BridgeUnicastToMulticast multicastAddress = " + multicastAddress);
  System.out.println ("BridgeUnicastToMulticast multicastPort    = " + multicastPort);

  try  // unicast first
    {
        inputSocket  = new DatagramSocket(pInputSocketNumber);
     }
  catch(SocketException sockEx)
    {
      throw new RuntimeException("couldn't create unicast socket");
    }
  catch(IOException ioEx)
    {
      throw new RuntimeException("problem with unicast socket");
    }
  
  
  try // multicast second
    {
        outputSocket = new MulticastSocket(pMulticastPort);

        inetAddress = InetAddress.getByName (pMulticastAddress);
		
        outputSocket.joinGroup(inetAddress);
    }
  // unkHostException has to be caught first, since it's more specific than an
  // IOException.
  catch(UnknownHostException unkHostException)
    {
      throw new RuntimeException("problem with joining multicast Address " + multicastAddress);
    }
  catch(SocketException sockEx)
    {
      throw new RuntimeException("couldn't create multicast sockets");
    }
  catch(IOException ioEx)
    {
      throw new RuntimeException("problem with multicast socket");
    }
  
  
   return;
 }

 public void run()
 {
   // listen on a multicast socket, repeat it on a unicast socket.

  DatagramPacket  inDgram   = null;
  DatagramPacket  outDgram  = null;
  byte            ibuff[]   = null;
  byte            obuff[]   = null;

  while(true)
  {
    ibuff   = new byte[MAX_PACKET_SIZE];
    obuff   = new byte[MAX_PACKET_SIZE];
    inDgram = new DatagramPacket(ibuff, ibuff.length);
    

    if(DEBUG)
      System.out.println("reading...");

    // Read a datagram into the new buffer.
    try
    {
      inputSocket.receive(inDgram);
    }
    catch(IOException ioe)
    {
      throw new RuntimeException("error reading from socket");
    }

    if(DEBUG)
      System.out.println("writing...");

    // create a new outgoing datagram from the incoming datagram's data. Note that
    // we have to be careful about length here; if they don't match up on the sending
    // and receiving side, we'll run into problems. The "solution" for now is to always
    // use datagrams of length MAX_PACKET_SIZE. Need to find a better way around this,
    // but don't have time to look at it right now.

    System.arraycopy(ibuff, 0, obuff, 0, ibuff.length);

    outDgram = new DatagramPacket(obuff,                      // data from incoming datagram
                                  obuff.length,               // length of data
                                  inetAddress,       		  // multicast Address we're sending to
                                  multicastPort);             // socket we're writing to

    if(DEBUG)
    {
      System.out.println("outDgram length = " + outDgram.getLength());
      System.out.println("outDgram mcast Address number = " + multicastAddress);
    }

    try
    {
      outputSocket.send(outDgram);
    }
    catch(IOException ioe)
    {
      throw new RuntimeException("error sending to output socket");
    }

  } // end of endless loop

} // end of run()

} // end of class

