
package mil.navy.nps.bridge;

import java.io.*;
import java.net.*;


/*
   BridgeUnicastToUnicast

   This is a sublcass of the Bridge abstract class, which implements datagram
   bridging between two unicast sockets. The program simply accepts packets on
   one socket, and repeats them out on the other.

   @author Don McGregor (<A HREF="http://www.stl.nps.navy.mil/~mcgredo"> http://www.stl.nps.navy.mil/~mcgredo</A>)<P>
   @version 1.0 
   
*/

public class BridgeUnicastToUnicast extends Bridge implements Runnable
{

  private DatagramSocket  inputSocket             = null;   // socket we listen on 
  private DatagramSocket  outputSocket            = null;   // socket we write to

  private int             inputSocketNumber       = -1;     // input socket number 
  private int             outputSocketNumber      = -1;     // output socket number


  
 public BridgeUnicastToUnicast( 
        int     pInputSocketNumber,                         // INPUT: socket to listen on
        int     pOutputSocketNumber,                        // INPUT: socket we write on
        String  pDestMachineName,                           // INPUT: name of machine we're sending to
        int     pDestSocketNumber)                          // INPUT: socket to write to on foreign machine
                        
{

   // superclass initialization

   super(pDestMachineName, pDestSocketNumber);

  // Create a couple sockets

    try
    {
        inputSocket  = new DatagramSocket(pInputSocketNumber);
        outputSocket = new DatagramSocket(pOutputSocketNumber);
    }
    catch(SocketException sockEx)
    {
      throw new RuntimeException("couldn't create one or both sockets");
    }

    inputSocketNumber       = pInputSocketNumber;
    outputSocketNumber      = pOutputSocketNumber;
    
    return;
 }

 public void run()
  {
    // Go to it. Listen for a datagram packet, get it, and send it off.
    // You can probably optimize the snot out of this naive implementation.
  
  DatagramPacket  inDgram   = null;
  DatagramPacket  outDgram  = null;
  byte            ibuff[]   = null;
  byte            obuff[]   = null;
  
  while(true)
  {
    ibuff   = new byte[MAX_PACKET_SIZE];
    obuff   = new byte[MAX_PACKET_SIZE];
    inDgram = new DatagramPacket(ibuff, ibuff.length);

    if(DEBUG)
      System.out.println("reading...");

    // Read a datagram into the new buffer.
    try
    {
      inputSocket.receive(inDgram);
    }
    catch(IOException ioe)
    {
      throw new RuntimeException("error reading from socket");
    }

    if(DEBUG)
      System.out.println("writing...");

    // create a new outgoing datagram from the incoming datagram's data. Note that
    // we have to be careful about length here; if they don't match up on the sending
    // and receiving side, we'll run into problems. The "solution" for now is to always
    // use datagrams of length MAX_PACKET_SIZE. Need to find a better way around this.

    System.arraycopy(ibuff, 0, obuff, 0, ibuff.length);

    outDgram = new DatagramPacket(obuff,                      // data from incoming datagram
                                  obuff.length,               // length of data
                                  destMachineInet,            // machine we're sending to
                                  destinationSocketNumber);   // socket on machine we're sending to
    try
    {
      outputSocket.send(outDgram);
    }
    catch(IOException ioe)
    {
      throw new RuntimeException("error sending to output socket");
    }

  } // end of endless loop

} // end of bridge

} // end of class BridgeUnicastToUnicast
