
/* Author: Mike McCarthy
 * File: MulticastRelayClient.java
 * Last update: 9 Jan., 1998
 * Program Invocation: command line entry: java MulticastClient <#serverhostname#> 
 *                     <#serverportnumber#>
 * Description: The client class code that connects with a Multicast Relay Server.
 * This is really just a primitive example model of how a client DIS-
 * Java-VRML application would implement a connection to a unicast server
 * port to access multicast data. This would be necessary if a user/participant
 * was to say, call in on a modem from home.
 */
 

package mil.navy.nps.bridge;

import mil.navy.nps.dis.*;
import java.io.*;
import java.net.*;
import java.util.*;

public class MulticastRelayClient extends Object {
 
  DatagramSocket socket = null;
  DatagramPacket packet;
  DatagramPacket inpacket;
  byte[] outbuff = new byte[256];
  byte[] inbuff = new byte[1500];
  ProtocolDataUnit aPdu;
  
  /**************************************************************************
   main method that takes clients input of multicast server name and port
   that clint wants to connect to. Creates and instance of client class
   and calls methods to mange server connect and datagram processing
   *************************************************************************/

  public static void main(String[] args) { 
    
    int port;
    InetAddress address;

    if (args.length != 2) 
      {
        System.out.println("Usage: java MulticastClient <hostname> <port#>");
        return;
      }
    try{
      
      port = Integer.parseInt(args[1]);
      address = InetAddress.getByName(args[0]);
      
      MulticastRelayClient  mcastClient = new MulticastRelayClient();
      

      mcastClient.mcastClientConnect(port, address); //call connect method
    
    }//endtry
    
    catch(IOException e)
      {
        System.out.println("IO error");
      }
    
  }//end main
  
  
    public void mcastClientConnect(int connect_port, InetAddress connect_address)
      { 
        
        String connect = new String("connect");        //set up connect request
        byte[] conbuf = new byte[connect.length()];
        conbuf = connect.getBytes();     
      
      try 
        {
          // bind to the socket
          socket = new DatagramSocket();
          System.out.println("Client unicast port is:  " + socket.getLocalPort());
        } 
      catch (java.io.IOException e) 
        {
          System.err.println("Could not create datagram socket.");
        }

      if (socket != null) {
        try 
          {
            packet = new DatagramPacket(conbuf, conbuf.length, connect_address, connect_port);
            socket.send(packet);  //send connect
            
            

            // create the thread instance that monitors client for disconnect
            Runnable ot = new  ClientStatusThread(socket, connect_address, connect_port); 
            Thread   csThread = new Thread(ot);
            csThread.setPriority(Thread.MIN_PRIORITY); //required for IRIX to properly schedule
            csThread.start();
         
       
          while(true)
            
            { // give the server 20 seconds to respond then exit
              socket.setSoTimeout(20000); //20 second server response timeout           
              inpacket = new DatagramPacket(inbuff, 1500);
              try{
                socket.receive(inpacket);
              }

              catch (IOException e) {

                System.out.println("*************************************************************");
                System.out.println("Server did not respond, try later or check with server admin");
                System.out.println("*************************************************************");
                System.out.println();
                System.exit(1);
              }
                
              System.out.println("Multicast data received from Server: " + packet.getAddress().toString());

              aPdu = ProtocolDataUnit.datagramToPdu(inpacket); //convert datagram to PDU
                
              System.out.println("PDU Name is: "  + aPdu.PDUName() ); //verify conversion
              System.out.println(" ");
              
              
            }//endwhile
         
          }//end try
      
        

        catch (IOException e) {
          System.err.println("IOException:  " + e);
          e.printStackTrace();
        }//end catch

      }//endif
      
      }//end McastClientConnect
  
 
  
  
}//end MulticastRelayClient Class
        
/************************************************************************
 the client status thread class which monitors keyboard input for client
 command to dissconnect from server
 ***********************************************************************/                                      

class ClientStatusThread implements Runnable
{
  static  String disconnect = new String("disconnect");  
  DatagramPacket packet;
  DatagramSocket the_socket;
  InetAddress    the_address;
  int            the_port;
  

  public ClientStatusThread(DatagramSocket in_socket, InetAddress con_address, int con_port)
    {
      System.out.println("Monitor Thread has started ");
      this.setSocket(in_socket);
      this.setAddress(con_address);
      this.setPort(con_port);
     
    }
  
  void setSocket(DatagramSocket temp_socket) //bunch of interface methods to get socket and
    {                                        //datagram objects instances
      the_socket = temp_socket;
    }

  void setAddress(InetAddress temp_address)
    {
      the_address = temp_address;
    }

  void setPort(int temp_port)
    {
      the_port = temp_port;
    }

  public void run()
    { 
        byte[] disbuf = new byte[disconnect.length()];
        disbuf = disconnect.getBytes();     
        
        try
          {
            Thread.sleep(100);          // go to sleep so don't block
          }
        catch(InterruptedException interruptedException)
          {
              throw new
                RuntimeException("Exceptional sleep ");
          }
        
        try
          {
            String command, instring;
            command = "quit";
            DataInputStream dis = new DataInputStream(System.in);
            //BufferedReader br = new BufferedReader(dis);
            instring = dis.readLine();
            System.out.println("data in = : " + instring);
            
            if(instring.equals("quit"))
              {  
                packet = new DatagramPacket(disbuf, disbuf.length, the_address, the_port);
                System.out.println("Sending Disconnect to server and Exiting program.");
                the_socket.send(packet);
                dis.close();
                System.exit(0);
              }
            }
        catch(IOException e)
          {
            System.out.println("error");
          }
       
        
        //      }//end while true
    }//end run
  
}//end class




