/* Author:       Mike McCarthy
 * File:         MulticastRelayServer Class 
 * Last Updated: 6 January, 1998
 *
 * Invocation:   command line entry, "java MulticastRelayServer"
 * This class defines a server that creates a multicast group connection and
 * relays multicast datagrams to unicast Clients.  The server waits for
 * clients to request multicast data via a threaded method.  Once a client
 * subscribes, multicast datagrams are translated into unicast datagrams
 * and sent to the client. 
 */ 


package mil.navy.nps.bridge;

import java.io.*; 
import java.net.*; 
import java.util.*; 
 
class MulticastRelayServer extends Object 
{   
   //class declarations 
  protected Vector client_address_vec = new Vector(); //maintain a vector table
  protected Vector client_port_vec = new Vector();    //of client IPs and ports
   
  protected static int defaultPort = 8010;            //default server port   
  protected int unicastPort; 
  protected DatagramSocket socket = null; 
 
  private   byte[] connectbuff = new byte[CLIENT_REQUEST_BUFSIZE]; //client input buffer 
  private   byte[] outbuff = new byte[DATAGRAM_BUFSIZE];           //output buffer to client 
  protected DatagramPacket connectpacket; 
 
  protected DatagramPacket packet;      
  protected InetAddress address;  //client address
  protected int port;             //client port

  private   int multicastport = 62040;  
  private   String multicastgroup = "224.2.181.145"; 
  private   DatagramPacket mpacket; 
  private   byte[] multicastbuff = new byte[DATAGRAM_BUFSIZE]; 
                               
  protected MulticastSocket multicastsocket;  
 
  protected DatagramPacket outDgram;   
 
  private   Integer port_obj;   //Integer object for port in Vector
 
  private static int CLIENT_REQUEST_BUFSIZE = 256; 
  private static int DATAGRAM_BUFSIZE = 1500;




  // main method- creates instance of server and client
  // listen thread
  public static void main(String args[]) 
    { 
      MulticastRelayServer            mcastServer = new MulticastRelayServer(); 
      ClientListenThread     clThread = new ClientListenThread(mcastServer); 
      clThread.start(); 
      mcastServer.mcastListen(); 
    }
 

  // the multicast listen method which joins the specified 
  // multicast group
  public void mcastListen() 
    { //instance of multicast datagram packet 
      mpacket = new DatagramPacket(multicastbuff, multicastbuff.length);         
      
      try{ // lets try to join
        multicastsocket = new MulticastSocket(multicastport); 
        multicastsocket.joinGroup(InetAddress.getByName(multicastgroup)); 
        multicastsocket.receive(mpacket); //check for multicast input 
      
        //Server debug printout
        if (mpacket != null) 
          System.out.println("Received data from: " + mpacket.getAddress().toString() + 
                             ":" + mpacket.getPort() + " with length: " + 
                             mpacket.getLength()); 
      } 
      
      catch(UnknownHostException unkHostException) 
        { 
          throw new RuntimeException("problem joining mcast group " + multicastgroup); 
        } 
      
      catch(SocketException sockEx) 
        { 
          throw new RuntimeException("couldn't create inputSocket"); 
        } 
      
      catch(IOException ioEx) 
        { 
          throw new RuntimeException("problem with input multicast socket"); 
        } 
      
      
      
      while(true) // infinite loop to send multicast to unicast clients
        {
          try{ 
            multicastsocket.receive(mpacket); //get multicastpackets

             //Server debug printout
             if (mpacket != null) 
               System.out.println("Received data from: " + mpacket.getAddress().toString() + 
                             ":" + mpacket.getPort() + " with length: " + 
                             mpacket.getLength()); 
             
             //copy the mulicastbuffer to unicast buffer- this is necessary
             System.arraycopy(multicastbuff, 0, outbuff, 0, multicastbuff.length); 
            
             // loop thru the client vector table, synchonize read because
             // the write method could have the Vector
             for(int ix = 0;ix < client_address_vec.size(); ix++) 
               {   
                 synchronized(client_address_vec)
                   { 
                     address = (InetAddress)client_address_vec.elementAt(ix); 
                   } 
                 synchronized(client_port_vec)
                   {     
                     Integer port_Integer  = (Integer)client_port_vec.elementAt(ix); 
                     port = port_Integer.intValue();
                     System.out.println("debug port is: " + port);
                   }
                 
                 outDgram = new DatagramPacket(outbuff,outbuff.length,address,port);    
                 socket.send(outDgram);          
               } //endfor
           } //endtry
           catch(IOException ioEx) 
             { 
               throw new RuntimeException("problem with output unicast socket"); 
             } 
           
        } //end while 
      
    } //end Mcastlisten()


  /* clientListen method
   * loops and reads client requests and store
   * client info in Vector table
   */
  
  public void clientListen() 
    { 
      // while(true){//loop infinite so we don't miss client request      
        try { 
          unicastPort = defaultPort; 
          socket = new DatagramSocket(unicastPort); 
          System.out.println("MulticastServer listening on unicastPort: " + socket.getLocalPort()); 
        }  
        catch (java.io.IOException e) { 
          System.err.println("Could not create datagram socket."); 
        } 
         while(true){//loop infinite so we don't miss client request
        try 
          {                     
            byte[] connectbuff = new byte[CLIENT_REQUEST_BUFSIZE]; //client input buffer 
            byte[] outbuff = new byte[DATAGRAM_BUFSIZE];           //output buffer to client 
            
            // listen for unicast clients 
            connectpacket = new DatagramPacket(connectbuff, CLIENT_REQUEST_BUFSIZE); 
            socket.receive(connectpacket); 
            
            String s = new String(connectpacket.getData(),0,connectpacket.getLength());


            




            if (s.equals("connect"))    //valid request
              {
                System.out.println("Connect string = " + s);
                
                address = connectpacket.getAddress(); //client address set 
                port =    connectpacket.getPort();    //client port 
                
                Integer port_obj; 
                port_obj = new Integer(port); 
            
                synchronized(client_address_vec)  //sync for write
                  { 
                    client_address_vec.addElement(address); 
                  } 
                
                synchronized(client_port_vec)  
                  { 
                    client_port_vec.addElement(port_obj); 
                  } 
              }//endif
            
            if (s.equals("disconnect")) //valid disconnect
              {
                System.out.println("DisConnect string = " + s);

                address = connectpacket.getAddress(); //client address set 
                port =    connectpacket.getPort();    //client port 
                
                Integer port_obj; 
                port_obj = new Integer(port); 
            
                synchronized(client_address_vec)  //sync for write
                  { 
                    client_address_vec.removeElement(address); 
                  } 
                
                synchronized(client_port_vec)  
                  { 
                    client_port_vec.removeElement(port_obj); 
                  }

              }
            
            
          }//endtry  
        catch (IOException e) { 
          System.err.println("IOException:  " + e); 
          e.printStackTrace(); 
        } 
        
      }//endwhile
    }//end clientListen() 
}//endclass MulticastRelayServer 

/* ClientListenThread class
 * Creates an instance of serverclass and calls client
 * listen method
 */

class ClientListenThread extends Thread 
{ 
  MulticastRelayServer  mcastServerInstance; 
  
  public ClientListenThread(MulticastRelayServer mcastServer) 
    { 
      mcastServerInstance = mcastServer; 
    } 
  
  public void run() 
    { 
      mcastServerInstance.clientListen(); 
    } 
}//end ClientListenThread class
