/*
 File:		EntityType.java
 CVS Info:	$Id: EntityType.java,v 1.2 1998/01/27 18:44:11 mcgredo Exp $
 Compiler:	jdk 1.1.6 
 */

package mil.navy.nps.dis;                                       // package to which we belong

import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.util.*;                                             // utility stuff we need
import java.io.*;                                               // input/output for serialization

import iwt.*;
import org.w3c.dom.*;

/**
 *@version 1.0
 *@author <a href="mailto:mcgredo@stl.nps.navy.mil"> Don McGregor</a> (<a href="http://www.stl.nps.navy.mil/~mcgredo">http://www.stl.nps.navy.mil/~mcgredo</a>)
 *
 *<dt><b>Location:</b>
 *<dd><a href="http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/EntityType.java">
 *  http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/EntityType.java</a>
 *
 *<dt><b>Hierarchy Diagram:</b>
 *<dd><a href="images/PduClassHierarchy.jpg"><IMG SRC="images/PduClassHierarchyButton.jpg" ALIGN=ABSCENTER></a>
 *
 *<dt><b>Summary:</b>
 *<dd>This record shall specify the kind of entity, the country of design, the domain, 
 *  the specific identification of the entity, and any extra information necessary 
 *  for describing the entity. <p>
 *<dt><b>Explanation:</b>
 *<dd>The type of entity in a DIS exercise shall be specified by an Entity Type record.
 *  Fields not used shall contain the value zero.<p>
 *
 *<dt><b>History:</b>
 *<dd>		2Dec96	/Don McGregor		/New
 *<dd>  	10Mar97	/Don McGregor		/Cleaned up for javadoc
 *<dd>  	16Apr97	/Don McGregor		/PrintStream passed to printValues
 *<dd>		8Dec97  /Ronan Fauglas		/changes for documentation templates + complements in documentation<br>
 *<dd>		8Dec97  /Ronan Fauglas		/changed entityKind to kind
 *
 *<dt><b>References:</b>
 *<dd>		DIS Data Dictionary :<A HREF="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/28.htm">
 *  		Entity Type Record</A><P>
 *<dd>		DIS specification : IEEE 1278.1, 5.3.16
 *
 *@see PduElement 
 */
public class EntityType extends PduElement
{
    /**
     *Entity kind: munition, life form, environmental...
     *For now, entity kind is : 0-9
     *
     *<dl>
     *<dt><b>Value:</b>
     *Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/22.htm"> 
     *			Entity Kind Field</a>			
     *<dd>	see Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedByte      kind;                     // kind of entity


    /**
     *This field shall specify the domain in which the equipment operates 
     *(for example, subsurface, surface, land, etc.) except for munition entities. 
     *For Munition entities this field shall specify the domain of the target 
     *(for example the munition might be surface-to-air, so the domain would be anti-air).
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/23.htm"> 
     *			Entity Domain Field</a>			
     *<dd>	see Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedByte      domain;                         // subsurface, surface, etc.

    /**
     *This field shall specify the country to which the design of the entity is
     *attributed. a 16 bits enumeration.
     *
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/d.htm"> 
     *			Country Field</a>			
     *<dd>	See Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedShort     country;                        // country to which design is attributed

    /**
     *This field shall specify the main category that describes the entity. 
     *The enumerations of category depend upon both the Kind and Domain. 
     *
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/24.htm"> 
     *			Entity Category Field</a>			
     *<dd>	See Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedByte      category;                       // main category that describes entityt

    /**
     *This field shall specify a particular subcategory to which the entity
     *belongs based on the category and the country. 
     *
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/25.htm"> 
     *			Entity Subcategory Field</a>			
     *<dd>	See Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedByte      subCategory;                    // subcategory, based on category field

    /**
     *This field shall specify specific information about an entity based upon
     *the subcategory field to which it belongs. 
     *
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration; see references below for information.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/26.htm"> 
     *			Entity Specific Field</a>			
     *<dd>	See Section 4 in EBV-DOC
     *</dl>
     */
    protected UnsignedByte      specific;                       // specific info, based on subCategory field

    /**
     *This field shall specify extra information required to describe a particular entity. 
     *The contents of this field shall depend on the type of entity represented. 
     *
     *<dl>
     *<dt><b>Value:</b>
     *  Enumeration.
     *<dt><b>References:</b>
     *<dd>	DIS Data Dictionary: <a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/27.htm"> 
     *			Entity Extra Field</a>			
     *</dl>
     */
    protected UnsignedByte      extra;                          // extra info, depending on type of entity

    /**
     *Constant value--size of an EntityType as written out to the wire. Here:
     *<code>sizeOf = 8 bytes</code>
     */
    public final static int     sizeOf = 8;                     // size of default object AS WRITTEN TO WIRE


/**
 *Default constructor--fills with zeros for all values.
 */
public EntityType()                                             // default constructor
{

    kind  = new UnsignedByte(0);
    domain      = new UnsignedByte(0);
    country     = new UnsignedShort(0);
    category    = new UnsignedByte(0);
    subCategory = new UnsignedByte(0);
    specific    = new UnsignedByte(0);
    extra       = new UnsignedByte(0);

    return;
}

public Object clone()
{
 EntityType newEntityType = new EntityType();

 newEntityType.setKind(this.getKind());
 newEntityType.setDomain(this.getDomain());
 newEntityType.setCountry(this.getCountry());
 newEntityType.setCategory(this.getCategory());
 newEntityType.setSubCategory(this.getSubCategory());
 newEntityType.setSpecific(this.getSpecific());
 newEntityType.setExtra(this.getExtra());

 return newEntityType;
}

public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    //super.serialize(outputStream);        // No need to call super, since no ivars are in our direct superclass
    
    kind.serialize(outputStream);
    domain.serialize(outputStream);
    country.serialize(outputStream);
    category.serialize(outputStream);
    subCategory.serialize(outputStream);
    specific.serialize(outputStream);
    extra.serialize(outputStream);

    return;
}


public void deSerialize(DataInputStream inputStream)
{
    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    // no need to call super, since there are no ivars in the superclass.

    kind.deSerialize(inputStream);
    domain.deSerialize(inputStream);
    country.deSerialize(inputStream);
    category.deSerialize(inputStream);
    subCategory.deSerialize(inputStream);
    specific.deSerialize(inputStream);
    extra.deSerialize(inputStream);

    return;
}

public int length()
{
    return sizeOf;          // EntityTypes are this long, always.
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println(indent + "kind: "  + kind.intValue());
    printStream.println(indent + "domain: "      + domain.intValue());
    printStream.println(indent + "country: "     + country.intValue());
    printStream.println(indent + "category: "    + category.intValue());
    printStream.println(indent + "subCategory: " + subCategory.intValue());
    printStream.println(indent + "specific: "    + specific.intValue());
    printStream.println(indent + "extra: "       + extra.intValue());

    return;
}

    //Accessor methods

public UnsignedByte getKind()
{   return (UnsignedByte)kind.clone();
}
public void setKind(UnsignedByte pKind)
{kind = pKind;
}
public void setKind(int pKind)
{ kind = new UnsignedByte(pKind);
}

public UnsignedByte getDomain()
{   return (UnsignedByte)domain.clone();
}
public void setDomain(UnsignedByte pDomain)
{domain = pDomain;
}
public void setDomain(int pDomain)
{ domain = new UnsignedByte(pDomain);
}

public UnsignedShort getCountry()
{   return (UnsignedShort)country.clone();
}
public void setCountry(UnsignedShort pCountry)
{country = pCountry;
}
public void setCountry(int pCountry)
{ country = new UnsignedShort(pCountry);
}

public UnsignedByte getCategory()
{   return (UnsignedByte)category.clone();
}
public void setCategory(UnsignedByte pCategory)
{category = pCategory;
}
public void setCategory(int pCategory)
{ category = new UnsignedByte(pCategory);
}

public UnsignedByte getSubCategory()
{   return (UnsignedByte)subCategory.clone();
}
public void setSubCategory(UnsignedByte pSubCategory)
{subCategory = pSubCategory;
}
public void setSubCategory(int pSubCategory)
{ subCategory = new UnsignedByte(pSubCategory);
}

public UnsignedByte getSpecific()
{   return (UnsignedByte)specific.clone();
}
public void setSpecific(UnsignedByte pSpecific)
{specific = pSpecific;
}
public void setSpecific(int pSpecific)
{ specific = new UnsignedByte(pSpecific);
}

public UnsignedByte getExtra()
{   return (UnsignedByte)extra.clone();
}
public void setExtra(UnsignedByte pExtra)
{extra = pExtra;
}
public void setExtra(int pExtra)
{ extra = new UnsignedByte(pExtra);
}


  /**
  */
  public void 
  write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

  /**
  */
  public void 
  insert(org.w3c.dom.Node outNode) 
  { 
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode); 

  }// end of insert()

  /**
  */
  public void 
  insertToDocument(org.w3c.dom.Document outDoc,org.w3c.dom.Node outNode)
  {
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    org.w3c.dom.Element ele = outDoc.createElement(DIS_TAGS.EntityType);
    ele.setAttribute(DIS_TAGS.Kind,kind.toString());
    ele.setAttribute(DIS_TAGS.Domain,domain.toString());
    ele.setAttribute(DIS_TAGS.Country,country.toString());
    ele.setAttribute(DIS_TAGS.Category,category.toString());
    ele.setAttribute(DIS_TAGS.Subcategory,subCategory.toString());
    ele.setAttribute(DIS_TAGS.Specific,specific.toString());
    ele.setAttribute(DIS_TAGS.Extra,extra.toString());

    outNode.appendChild(ele); 

  }// end of write()

  /**
  */
  public void 
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    // order is important here, since we need to read in the same order as
    // specified by the DIS standard.
    // no need to call super, since there are no ivars in the superclass.

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(DIS_TAGS.EntityType);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);
    extract(rt);

  }// end of read()

  /**
  */
  public void 
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(DIS_TAGS.EntityType) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Kind); 
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Domain); 
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Country); 
       org.w3c.dom.Attribute a4 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Category); 
       org.w3c.dom.Attribute a5 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Subcategory); 
       org.w3c.dom.Attribute a6 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Specific); 
       org.w3c.dom.Attribute a7 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Extra); 
       String v1, v2, v3, v4, v5, v6, v7;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();
       v4 = a4.getValue();
       v5 = a5.getValue();
       v6 = a6.getValue();
       v7 = a7.getValue();

       try
       {
         kind = new UnsignedByte(Integer.parseInt(v1)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         domain = new UnsignedByte(Integer.parseInt(v2)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         country = new UnsignedShort(Integer.parseInt(v3)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         category = new UnsignedByte(Integer.parseInt(v4)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         subCategory = new UnsignedByte(Integer.parseInt(v5)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         specific = new UnsignedByte(Integer.parseInt(v6)); 

       } catch(NumberFormatException nfe) {} 

       try
       {
         extra = new UnsignedByte(Integer.parseInt(v7)); 

       } catch(NumberFormatException nfe) {} 

    } else throw new InvalidDocumentType();

  }// end of read()
  
} // end of class EntityType
