/*
 File:		EulerAngle.java
 CVS Info:	$Id: EulerAngle.java,v 1.2 1998/01/27 18:44:12 mcgredo Exp $
 Compiler:	jdk 1.1.6 
 */

package mil.navy.nps.dis;

import mil.navy.nps.util.*;         // General-purpose utilities
import mil.navy.nps.disEnumerations.*;         // Enumerations for DIS

import java.lang.*;
import java.util.*;
import java.io.*;

import iwt.*;
import org.w3c.dom.*;

/**
 *@version 1.0
 *@author <a href="mailto:mcgredo@stl.nps.navy.mil"> Don McGregor</a> (<a href="http://www.stl.nps.navy.mil/~mcgredo">http://www.stl.nps.navy.mil/~mcgredo</a>)
 *
 *<dt><b>Location:</b>
 *<dd><a href="http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/EulerAngle.java">
 *  http://www.stl.nps.navy.mil/dis-java-vrml/mil/navy/nps/dis/EulerAngle.java</a>
 *
 *<dt><b>Hierarchy Diagram:</b>
 *<dd><a href="images/PduClassHierarchy.jpg"><IMG SRC="images/PduClassHierarchyButton.jpg" ALIGN=ABSCENTER></a>
 *
 *<dt><b>Summary:</b>
 *<dd>Orientation of a simulated entity shall be specified by the Euler Angles Record. 
 *  This record shall specify three angles which are specifed with
 *  respect to the entities coordinate system. The three angles shall be represented in radians
 *  
 *<dt><b>Explanation:</b>
 *<dd>Describes the orientation of an object, in psi, theta, phi 32 bit
 *  floating point terms. There are a number of classes that use the
 *  same basic layout of 3 32-bit numbers (velocity, angular velocity,
 *  EntityPosition, etc.) but they're split out into separate classes
 *  in the interests of type safety fascism. <P>
 * "Don't ask, don't tell" does NOT apply here. We can ask any entity
 * about its orientation.<P>
 *
 *<dt><b>History:</b>
 *<dd>		06Mar97	/Don McGregor		/New
 *<dd>		16Apr97	/Don McGregor		/PrintStream passed to printValues
 *<dd>		12Aug97 /Don McGregor		/elaborated printValues
 *<dd>		8Dec97	/Ronan Fauglas		/changes for documentation templates + complements in documentation
 *<dd>		11Dec97	/Ronan Fauglas		/changes access methods names from "variable()" to "getVariable()"
 *
 *<dt><b>References:</b>
 *<dd>		DIS Data Dictionary :<a href="http://stds.sc.ist.ucf.edu/dis/dis-dd/pdu/35.htm"> 
 *			Euler Angles Record</a>
 *<dd>		DIS specification : IEEE 1278.1, Section 5.3.17,1.3.2
 *
 *
 *@see PduElement 
 *@see SerializationInterface
 */
public class EulerAngle extends PduElement
{
    /**
     *The first angle of rotation (around the "Z" axis) in radians.
     */
    protected float  psi;     // psi-axis

    /**
     *The second angle of rotation (around the "Y" axis) in radians.
     */
    protected float  theta;   // theta-axis

    /**
     *The second angle of rotation (around the "X" axis) in radians.
     */
    protected float  phi;     // phi-axis

    /**
     *Constant value--size of an EulerAngle record when written out; here :<code>sizeOf = 12 bytes</code>.
     */
    public final int sizeOf = 12;   // object is 12 bytes long 


/**
 *Constructs an new Entity Identifier, with initial values initiated to 0.
 */
public EulerAngle()
{
    // default constructor

    psi   = 0; 
    theta = 0;
    phi   = 0;

    return;
}

/**
 *Constructs an new EulerAngle Record, with variables values passed by parameters.
 *
 *@param pPsi the rotation about z by angle
 *@param pTheta the rotation about y by angle
 *@param pPhi the rotation about x by angle
 */
public EulerAngle(float pPsi, float pTheta, float pPhi)
{
    psi   = pPsi;
    theta = pTheta;
    phi   = pPhi;

    return;
}

public Object clone()
{

 EulerAngle   newEulerAngle = new EulerAngle(psi, theta, phi);

 return newEulerAngle;   
}

public void serialize(DataOutputStream outputStream)
{
    // write out the data to an output stream. Order is important here, since
    // it needs to conform to the DIS standard.

    //super.serialize(outputStream);        // No need to call super, since no ivars are in our direct superclass
    
    try
     {
        outputStream.writeFloat(psi);
        outputStream.writeFloat(theta);
        outputStream.writeFloat(phi);
     }
    catch (IOException ioError)
        {
            throw new 
                RuntimeException("Exception in EulerAngle. Error writing to wire.");
        }
}

public void deSerialize(DataInputStream pInputStream)
{
    try
     {
        psi   = pInputStream.readFloat();
        theta = pInputStream.readFloat();
        phi   = pInputStream.readFloat();
     }
     catch (IOException ioError)
        {
            throw new 
                RuntimeException("Exception in EulerAngle. Error reading from wire.");
        }

}

public int length()
{
    return sizeOf;          // EntityTypes are this long, always.
}

public void printValues(int indentLevel, PrintStream printStream)
{
    // print the values of the object out, with correct level of
    // indentation on the page.

    char[]  indent = new char[indentLevel];
    int     idx = 0;
    String  spacing;

    for(idx = 0; idx < indentLevel; idx++)
        indent[idx] = ' ';

    spacing = new String(indent);

    printStream.println(indent + "EulerAngle psi: "   + psi);
    printStream.println(indent + "EulerAngle theta: "   + theta);
    printStream.println(indent + "EulerAngle phi: "   + phi);

    return;
}

// Accessor methods

public void setValues(float pPsi, float pTheta, float pPhi)
{ psi   = pPsi;
  theta = pTheta;
  phi   = pPhi;
}

  
public float getPsi()
{   return psi;
}
public void setPsi(float pPsi)
{psi = pPsi;
}

public float getTheta()
{   return theta;
}
public void setTheta(float pTheta)
{theta = pTheta;
}

public float getPhi()
{   return phi;
}
public void setPhi(float pPhi)
{phi = pPhi;
}

  /**
  */
  public void
  write(org.w3c.dom.Document outDoc) { insertToDocument(outDoc,outDoc); }

  /**
  */
  public void
  insert(org.w3c.dom.Node outNode) 
  { 
     org.w3c.dom.Document outDoc = outNode.getOwnerDocument();
     insertToDocument(outDoc,outNode); 

  }// insert()

  /**
  */
  public void
  insertToDocument(org.w3c.dom.Document outDoc, org.w3c.dom.Node outNode)
  {
    org.w3c.dom.Element ele = outDoc.createElement(DIS_TAGS.EulerAngle);
    ele.setAttribute(DIS_TAGS.Psi,Float.toString(psi));
    ele.setAttribute(DIS_TAGS.Theta,Float.toString(theta));
    ele.setAttribute(DIS_TAGS.Phi,Float.toString(phi));

    outNode.appendChild(ele);

  }// end of insertToDocument()

  /**
  */
  public void
  read(org.w3c.dom.Document inDoc) throws InvalidDocumentType
  {

    org.w3c.dom.NodeList nodes = inDoc.getElementsByTagName(DIS_TAGS.EulerAngle);
    if( nodes.getLength()<1 ) throw new InvalidDocumentType();

    org.w3c.dom.Node rt = nodes.item(0);

    extract(rt);

  }// end of read()

  /**
  */
  public void
  extract(org.w3c.dom.Node rt) throws InvalidDocumentType
  {

    if( rt!=null && rt.getNodeName().equals(DIS_TAGS.EulerAngle) &&
        rt.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE )
    {
       org.w3c.dom.NamedNodeMap attribs = rt.getAttributes();
       org.w3c.dom.Attribute a1 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Psi);
       org.w3c.dom.Attribute a2 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Theta);
       org.w3c.dom.Attribute a3 = (org.w3c.dom.Attribute) attribs.getNamedItem(DIS_TAGS.Phi);
       String v1, v2, v3;

       v1 = a1.getValue();
       v2 = a2.getValue();
       v3 = a3.getValue();

       try
       {
         psi = Float.valueOf(v1).floatValue();

       } catch(NumberFormatException nfe) {}

       try
       {
         theta = Float.valueOf(v2).floatValue();

       } catch(NumberFormatException nfe) {}

       try
       {
         phi = Float.valueOf(v3).floatValue();

       } catch(NumberFormatException nfe) {}

    } else throw new InvalidDocumentType();

  }// end of extract()

} // end of class EulerAngle
