// **********************************************************************
//   EXECUTIVE SUMMARY
// Module Name: NIU.java
// Description: Definition of the NIU applet
// Author: Kent A. Watsen, http://www.mbay.net/~watsen
// Author: Don McGregor, http://www.stl.nps.navy.mil/~mcgredo
// **********************************************************************


package mil.navy.nps.eaiDemoAuv;


// **********************************************************************
//   IMPORTS
// **********************************************************************


// Standard classes
import java.awt.*;
import java.util.*;
import java.applet.*;
import java.net.*;
import java.io.*;

// NPS library classes
import mil.navy.nps.dis.*;

// **********************************************************************
//   CLASS
// **********************************************************************


public class NIU extends Applet implements Runnable
{
  private static final int MAX_DATAGRAM_SIZE = 1500;

  private Applet         worldApplet        = null;
  private int            sendSocketNumber   = -1;
  private DatagramSocket sendSocket         = null;
  private InetAddress    sendDestHost       = null;
  private int            sendDestSocket     = -1;
  private boolean        amSending          = false;
  private int            listenSocketNumber = -1;
  private DatagramSocket listenSocket       = null;
  private Vector         listenBuffer       = null;
  private boolean        amListening        = false;

int pduCount = 0;

  public String getName()
    {
    return "NIU";
    }

  public void init()
    {
    // Retrieve passed values
    String tmpParam = null;

    // test and configure listener
    tmpParam = getParameter("listenSocketNumber");
    if (tmpParam != null && tmpParam.length() != 0)
      {
      amListening = true;
      listenSocketNumber = (Integer.valueOf(tmpParam)).intValue();
      System.out.println("Listening to socket number  " + listenSocketNumber);
      System.out.println("NIU configured for listening");
      }

    // test and configure sender
    tmpParam = getParameter("sendSocketNumber");
    if (tmpParam != null && tmpParam.length() != 0)
      {
      sendSocketNumber = (Integer.valueOf(tmpParam)).intValue();
      System.out.println("Sending to socket number " + sendSocketNumber);
      }
    tmpParam = getParameter("sendDestHost");
    if (tmpParam != null && tmpParam.length() != 0)
      {
      try {sendDestHost = InetAddress.getByName(tmpParam); }
      catch (UnknownHostException e) { System.out.println("Error finding sendDestHost address"); }
      System.out.println("Sending to host: " + tmpParam);
      }
    tmpParam = getParameter("sendDestSocket");
    if (tmpParam != null && tmpParam.length() != 0)
      {
      sendDestSocket = (Integer.valueOf(tmpParam)).intValue();
      System.out.println("Sending to socket # " + sendDestSocket);
      }
    if (sendSocketNumber != -1 && sendDestHost != null && sendDestSocket != -1)
      {
      amSending = true;
      System.out.println("NIU configured for sending, Dude");
      }

    // Get a handle to the World
    while (worldApplet == null)
      {
      worldApplet = getAppletContext().getApplet("World");
      }
    //System.out.println("world name = " + ((World)worldApplet).getName() );

    // initialize for sender
    if (amSending == true)
      {
      try { sendSocket = new DatagramSocket(sendSocketNumber); }
      catch (SocketException e) {System.out.println("new sendSocket Error");}
      System.out.println("Sending socket opened");
//      sendBuffer = new Vector();
      }

    // initialize listener
    if (amListening == true)
      {
      try { listenSocket = new DatagramSocket(listenSocketNumber); }
      catch (SocketException e) {System.out.println("new listenSocket Error");}
      System.out.println("Listening socket opened");
      listenBuffer = new Vector();

      Thread runner = new Thread(this);
      runner.setPriority(Thread.MIN_PRIORITY);
      runner.start();
      }
    }

  public void run()
    {
    DatagramPacket datagramPacket = null;
    byte           dataBuffer[]   = null; 

    while(true)
      {
      // create a new datagram object, then read it from the wire.

      dataBuffer = new byte[MAX_DATAGRAM_SIZE];
      datagramPacket = new DatagramPacket(dataBuffer, dataBuffer.length);

      try { listenSocket.receive(datagramPacket); }
      catch (IOException e) { System.out.println("listenSocket.receive Error"); }

pduCount++;
if (pduCount % 100 == 0) {System.out.println(pduCount + " PDUs received");}


      // we have to be careful about access to the datagram holding pen
      synchronized(listenBuffer)
        {
        listenBuffer.addElement(datagramPacket);
        }
      }
    }

  // Returns a Vector of all the PDUs received since we last asked.
  // Creates a new datagram buffer for the read thread, then takes the
  // old datagram buffer for its own nefarious purposes.
  public Vector getPDUs()
    {
    Vector           locListenBuffer = null; // just a ptr to keep the old buffer from being GC'd.
    Vector           pduBuffer       = null; // vector to hold inflated PDU objects
    Vector           newlistenBuffer = null;
    DatagramPacket   datagram        = null;
    ProtocolDataUnit pdu             = null;

    if (!amListening)
      {
      System.out.println("NIU not initialized to listen to a socket");
      return null;
      }

    // Try to make the changeover fast, with minimal locked time
    locListenBuffer = listenBuffer;
    newlistenBuffer = new Vector(); 
    synchronized(listenBuffer)
      {
      listenBuffer = newlistenBuffer;	
      }

    // now translate all the datagrams into PDUs.
    pduBuffer = new Vector();
    for (int i=0; i<locListenBuffer.size(); i++)
      {
      datagram = (DatagramPacket)locListenBuffer.elementAt(i);
      pdu = ProtocolDataUnit.datagramToPdu(datagram);
      if(pdu != null)
        {
        pduBuffer.addElement(pdu);
        }
      }
    locListenBuffer.removeAllElements(); // garbage collector will destroy vector
    return pduBuffer;
    }

  public void sendPDU(ProtocolDataUnit pdu)
    {
    ByteArrayOutputStream oStream          = null;
    DataOutputStream      dataOutputStream = null;
    DatagramPacket        dgramPacket      = null;

    //System.out.println("attempting to send a PDU");

    if (!amSending)
      {
      System.out.println("NIU not initialized to send to a socket");
      return;
      }

    oStream = new ByteArrayOutputStream();
    dataOutputStream = new DataOutputStream(oStream);

    //System.out.println("preparing to serialize a PDU");

    pdu.serialize(dataOutputStream);

    //System.out.println("serialized PDU");

    dgramPacket = new DatagramPacket(oStream.toByteArray(), oStream.size(), sendDestHost, sendDestSocket);
    
    System.out.println("Created PDU in datagram packet");

    try { sendSocket.send(dgramPacket); }
    catch (IOException e) { System.out.println("Error sending packet"); }

    System.out.println("datagram sent to socket");

    }
}

