// **********************************************************************
//   EXECUTIVE SUMMARY
// Module Name: Ownship.java
// Description: Definition of the Ownship class
// Author: Mike McCarthy based on 
// Kent A. Watsen, http://www.mbay.net/~watsen original Ownship.java and 
// Tim Holliday's AUV design work
// **********************************************************************

package mil.navy.nps.eaiDemoAuv;


// **********************************************************************
//   IMPORTS
// **********************************************************************


// Standard classes
import java.*;
import java.awt.*;
import java.util.*;

// EAI classes
import vrml.external.Node;
import vrml.external.Browser;
import vrml.external.field.*;
import vrml.external.exception.*;

import netscape.javascript.JSObject;

// NPS library stuff
import mil.navy.nps.dis.*;
import mil.navy.nps.math.*;


// **********************************************************************
//   CLASS
// **********************************************************************


public class Ownship extends Object
  {
  static final String vrmlString    = "Inline{url \"martins_auv.wrl\"}";
  static final float  initPos[]     = {0f, 5f, 0f}; // x, y, z
  static final float  initPosDt[]   = {0f, 0f, 0f}; // aka: velocity
  static final float  initAng[]     = {0f, 0f, 0f}; // heading, pitch, roll
  static final float  initAngDt[]   = {0f, 0f, 0f}; // aka: angular rates
  public static final int    UP            = 0;
  public static final int    DOWN          = 1;
  public static final int    LEFT          = 2;
  public static final int    RIGHT         = 3;
  static final float  SCREW_COEF    = 0.16f;
  static final float  LIFT_COEF     = 2.82f;
  static final float  DRAG          = 20;
  static final float  GRAVITY       = -9.8f;
  static final float  MASS          = 450.0f;
  static final float  VOLUME        = .2f;
  //static final float  DTIME         = .1f;



  static final float  aDeg          = ((float)Math.PI)/180.0f;
  NIU                 niu           = null; // for sending pdus
  Node                myNode[]      = null;
  Node                myTransform[] = null;
  EventInSFVec3f      myPos         = null;
  EventInSFRotation   myRot         = null;
  Quaternion          currQuat       = null;
  Quaternion          tmpQuat       = null;
  Quaternion          tmpQuat1       = null;
  Quaternion          tmpQuat2       = null;
  int                 currFrame     = 0;
  float               speed         = 0;

  private float               weight        = MASS * GRAVITY;
  private float               bouancy       = 0f;
  private float               ang_accel     = 0f;
  private float               ang_veloc     = 0f;
  private float               accel         = 0f;
  private float               accelz        = 0f;
  private float               velocz        = 0f;

  private int                 ordered_depth = 5;
  private int                 ordered_rudder  = 0;
  private int                 ordered_turns = 0;
  private float               ordered_planes= 0;
  private boolean             transiting    = false;



  float               rot[]         = null; // axis, angle (radians)
  float               pos[]         = null; // x, y, z
  float               posDt[]       = null; // aka: velocity
  float               ang[]         = null; // heading, pitch, roll
  float               angDt[]       = null; // aka: angular rates
  float               drPos[]       = null; // x, y, z
  float               drPosDt[]     = null; // aka: velocity
  float               drAng[]       = null; // heading, pitch, roll
  float               drAngDt[]     = null; // aka: angular rates
  EntityStatePdu      espdu         = null;
  float               pduTimer      = 0.0f;

  public Ownship (Browser browser, NIU _niu, String tmpParam)
    {
    short entityID[] = null;

    System.out.println("in constructor for Ownship");

    // use passed browser now
    myNode = browser.createVrmlFromString(vrmlString);
    myTransform = browser.createVrmlFromString("Transform {}" );

    // save reference to NIU doe later use
    niu = _niu;

    // use tmpParam now
    entityID = new short[3];
    entityID[0] = -1;
    entityID[1] = -1;
    entityID[2] = -1;
    if (tmpParam != null && tmpParam.length() != 0)
      {
      StringTokenizer st = null;
      st = new StringTokenizer(tmpParam, " ,");  // skip 'space' & ',' 
      try
        {
        entityID[0] = (short)(Integer.valueOf(st.nextToken())).intValue();
        entityID[1] = (short)(Integer.valueOf(st.nextToken())).intValue();
        entityID[2] = (short)(Integer.valueOf(st.nextToken())).intValue();
        }
      catch (NoSuchElementException e)
        {
        System.out.println("bad ownship entityID passed");
        }
      }
    espdu = new EntityStatePdu();
    espdu.setEntityID(entityID[0], entityID[1], entityID[2]);

    System.out.println("entity ID is set to " + entityID[0] + ", " + entityID[1] + ", " + entityID[2]);


    EventInMFNode tmpAddChildren = (EventInMFNode) myTransform[0].getEventIn("addChildren");
    tmpAddChildren.setValue(myNode);

    myPos = (EventInSFVec3f) myTransform[0].getEventIn("set_translation");
    myRot = (EventInSFRotation) myTransform[0].getEventIn("set_rotation");

    // hi-fi vars
    pos     = new float[3]; // x, y, z
    posDt   = new float[3]; // aka: velocity
    ang     = new float[3]; // heading, pitch, roll
    angDt   = new float[3]; // aka: angular rates

    // lo-fi vars
    drPos   = new float[3]; // x, y, z
    drPosDt = new float[3]; // aka: velocity
    drAng   = new float[3]; // heading, pitch, roll
    drAngDt = new float[3]; // aka: angular rates

    // these are used for hpr->axis/angle conversions
    tmpQuat  = new Quaternion();
    rot     = new float[4]; // axis/angle

    // now set init vars
    System.arraycopy(initPos, 0, pos, 0, 3);
    System.arraycopy(initPosDt, 0, posDt, 0, 3);
    System.arraycopy(initAng, 0, ang, 0, 3);
    System.arraycopy(initAngDt, 0, angDt, 0, 3);

    // now convert hpr (ang) to axis/angle (rot)
    tmpQuat.setEulers(ang);
    tmpQuat.getAxisAngle(rot);

    // finally, push values to vrml
    myPos.setValue(pos);
    myRot.setValue(rot);

    currQuat  = new Quaternion();
    currQuat.setAxisAngle(rot);
    myPos.setValue(pos);
    myRot.setValue(rot);

    tmpQuat1  = new Quaternion();
    tmpQuat2  = new Quaternion();
    tmpQuat2.setAxisAngle(rot);


    System.out.println("exiting constructor, haven't yet sent PDU");

    // let the world know about it
    sendPDU();

    System.out.println("just sent PDU");
    }


  public void cycle()
    {
    
    float f[] = {0f, 0f, -0.1f};
    float   dt                = 0.1f;  // assume 10Hz
    boolean thresholdExceeded = false;
    Vec3f   forward           = new Vec3f();


   
      if (pos[1] > 49.5) {
         bouancy = -weight*5/6;
      }
        else {
          bouancy = -weight;
        }

    accel = (ordered_turns * SCREW_COEF - speed * DRAG)/MASS;
    accelz = -(speed * (ordered_planes* 10  * LIFT_COEF) + weight + bouancy)/MASS;
    ang_accel = (-ordered_rudder * speed * LIFT_COEF - ang_veloc * 5*DRAG)/MASS;

    speed  = speed  + accel  * dt;
    velocz = velocz + accelz * dt;
    ang_veloc = ang_veloc + ang_accel * dt;

    // update the current turn based on desired turn
    tmpQuat1.setAxisAngle(0f, 1f, 0f, ang_veloc * dt / 2);
    tmpQuat2.postMult(tmpQuat1);

    // save this in the current quaternion
    tmpQuat2.getQuat(rot);
    currQuat.setQuat(rot);
  
    // update distance traveled based on specified speed
    f[2] = speed * dt;

    // update depth based on desired depth and current depth
    if ((pos[1] > ((float) ordered_depth+1))) {
      ordered_planes = +5f;   // go deep
      transiting = true;
    }
    else if ((pos[1] > ((float) ordered_depth+5))) {
      ordered_planes = +20f;  // go shallow
      transiting = true;
    }
    else if ((pos[1] < ((float) ordered_depth-1))) {
      ordered_planes = -5f;  // go shallow
      transiting = true;
    }
    else if ((pos[1] < ((float) ordered_depth-5))) {
      ordered_planes = -20f;  // go shallow
      transiting = true;
    }
    else if ((pos[1] < ((float) ordered_depth+1) && (pos[1] > ((float) ordered_depth-1)))) {
      transiting = false;
      ordered_planes = 0f;  // zero out angle
    }
    
    //calculate the new auv position and rotation vectors
    currQuat.xform(f);
    pos[0] = pos[0] + f[0];

    // update vertical distance travelled
    pos[1] = pos[1] + f[1] + accelz * dt; 
    pos[2] = pos[2] + f[2];
    
    currQuat.getAxisAngle(rot);
    // currQuat.getEulers(ang);   //new shit
    
    currQuat.setEulers(ang);
    //tmpQuat.getAxisAngle(rot);
   
    // now apply any changes in pos/rot
    myPos.setValue(pos);
    myRot.setValue(rot);

    // set angle (heading based on transformed quat
    //ang[0] = -rot[3];
    
   // myRot.setValue(ang);  //new
 
  
/*
    //set velocity vector = speed*direction
    forward.set(0.0f, 0.0f, -speed);
    tmpQuat.setEulers(ang);
    tmpQuat.xform(forward);
    posDt[0] = forward.get(0);
    posDt[1] = forward.get(1);
    posDt[2] = forward.get(2);

    // now update first order equations
    pos[0] = pos[0] + dt*posDt[0]; // x
    pos[1] = pos[1] + dt*posDt[1]; // y
    //pos[2] = pos[2] + dt*posDt[2]; // z 
    ang[0] = ang[0] + dt*angDt[0]; // h
    ang[1] = ang[1] + dt*angDt[1]; // p
    ang[2] = ang[2] + dt*angDt[2]; // r   */

    // enforced caps on euler angles
    if (ang[0] < -180.0f) ang[0] = 360.0f - ang[0];
    if (ang[0] >  180.0f) ang[0] = ang[0] - 360.0f;
    if (ang[1] < -89.0f) { ang[1] = -89.0f; angDt[1] = 0.0f; }
    if (ang[1] >  89.0f) { ang[1] =  89.0f; angDt[1] = 0.0f; }
    if (ang[2] < -180.0f) ang[2] = 360.0f - ang[2];
    if (ang[2] >  180.0f) ang[2] = ang[2] - 360.0f;

    // now convert hpr (ang) to axis/angle (rot)
   // tmpQuat.setEulers(ang);
   //  tmpQuat.getAxisAngle(rot);

    // finally, push values to vrml
    myPos.setValue(pos);
    myRot.setValue(rot); 

    // deadreckon lo-res model
    drPos[0] = drPos[0] + dt*drPosDt[0]; // x
    drPos[1] = drPos[1] + dt*drPosDt[1]; // y
    drPos[2] = drPos[2] + dt*drPosDt[2]; // z
    drAng[0] = drAng[0] + dt*drAngDt[0]; // h
    drAng[1] = drAng[1] + dt*drAngDt[1]; // p
    drAng[2] = drAng[2] + dt*drAngDt[2]; // r

    // send pdu if threshold exceeded
    pduTimer = pduTimer + dt;
    if (pduTimer > 5.0f) // 5 sec update
      {
      pduTimer = 0.0f;
      sendPDU();
      return;
      }
    if (Math.abs(pos[0]-drPos[0]) > 2.0f) // 2 meters
      {
      sendPDU();
      return;
      }
    if (Math.abs(pos[1]-drPos[1]) > 2.0f) // 2 meters
      {
      sendPDU();
      return;
      }
    if (Math.abs(pos[2]-drPos[2]) > 2.0f) // 2 meters
      {
      sendPDU();
      return;
      }
    if (Math.abs(ang[0]-drAng[0]) > aDeg) // 1 degree
      {
      sendPDU();
      return;
      }
    if (Math.abs(ang[1]-drAng[1]) > aDeg) // 1 degree
      {
      sendPDU();
      return;
      }
    if (Math.abs(ang[2]-drAng[2]) > aDeg) // 1 degree
      {
      sendPDU();
      return;
      }

    // no pdu to send
    niu.setBackground(Color.lightGray);
    }

  public void sendPDU()
    {
    niu.setBackground(Color.red);

    // first set dr* vars from hi-fi vars
    System.arraycopy(pos, 0, drPos, 0, 3);
    System.arraycopy(posDt, 0, drPosDt, 0, 3);
    System.arraycopy(ang, 0, drAng, 0, 3);
    System.arraycopy(angDt, 0, drAngDt, 0, 3);

    // now fill in espdu
    espdu.setEntityLocationX(pos[0]);
    espdu.setEntityLocationY(pos[1]);
    espdu.setEntityLocationZ(pos[2]);
    espdu.setEntityLinearVelocityX(posDt[0]);
    espdu.setEntityLinearVelocityY(posDt[1]);
    espdu.setEntityLinearVelocityZ(posDt[2]);
    espdu.setEntityOrientationPsi(ang[0]);   // h
    espdu.setEntityOrientationTheta(ang[1]); // p
    espdu.setEntityOrientationPhi(ang[2]);   // r
    espdu.setEntityAngularVelocityY(angDt[0]); // h
    espdu.setEntityAngularVelocityX(angDt[1]); // p
    espdu.setEntityAngularVelocityZ(angDt[2]); // r

    // send it now
    niu.sendPDU(espdu);
    }

// provide access to private variables of depth, speed and rudder
  public void update_depth(int depth)  {
    ordered_depth = 50 - depth;
  }

  public void update_rudder(int angle) {
    ordered_rudder = angle;
  }
  
  public void update_speed(int turns) {
    ordered_turns = turns;
  }


 


  public void dispose()
    {
    // nothing to do just yet
    }


  public Node[] getNode()
    {
    return myTransform;
    }

  }

