// **********************************************************************
//   EXECUTIVE SUMMARY
// Module Name: World.java
// Description: Definition of the world applet
// Author: Mike McCarthy, http://dubhe.cc.nps.navy.mil/~mpmccart
// As adapted from Kent Watsen's http://www.mbay.net/~watsen original World.java
// and implementing an AUV world designed by Tim Holliday
// **********************************************************************


package mil.navy.nps.eaiDemoAuv;


// **********************************************************************
//   IMPORTS
// **********************************************************************


// Standard classes
import java.awt.*;
import java.util.*;
import java.applet.*;

// EAI classes
import vrml.external.Node;
import vrml.external.Browser;
import vrml.external.field.*;
import vrml.external.exception.*;

import netscape.javascript.JSObject;

// NPS library classes
import mil.navy.nps.dis.*;		
import mil.navy.nps.Math.*;


// **********************************************************************
//   CLASS
// **********************************************************************


public class World extends Applet implements EventOutObserver
  {
  private boolean                     DEBUG           = true;
  private NIU                         niu             = null;
  private Browser                     browser         = null;
  private Node                        rootNode        = null;
  private Node                        globalClock     = null;
  private EventInMFNode               addChildren     = null;
  private EventInMFNode               removeChildren  = null;
  private EventOutSFFloat             globalTick      = null;
  private float                       frameNum        = 0f;
  private TextField                   frameField      = null;
  private Entity                      entity          = null;
  private Ownship                     ownship         = null;
  private Class                       espduClass      = null;

  private Label     depthLabel;
  private Scrollbar depthbar;

  private Label     speedLabel;
  private Scrollbar speedbar;
 
  private Label     rudderLabel;
  private Scrollbar rudderbar;
		

  int pduCount = 0;

  public String getName()
    {
    return "World";
    }

  public void init()
    {
    System.out.println("initializing world");

    // Create the AWT stuff
     // Group the controls into 2 sets North and South
    Panel pN = new Panel();
    Panel pS = new Panel();

    // These components are modified versions of the old boid buttons
    pN.add(new Button("Add Ownship"));
    pN.add(new Button("Remove Ownship"));
    pN.add(new Label("Framerate = "));
    frameField = new TextField("", 3);
    frameField.setEditable(false);
    pN.add(frameField);

    // These are layed out by JAVA's AWT in a grid format
    pS.setLayout(new GridLayout(1,6));

    pS.add(depthLabel = new Label("Depth 45"));
    pS.add(depthbar = new Scrollbar(Scrollbar.HORIZONTAL));
    depthbar.setValues( 45, 0, 0, 50);
    depthbar.setPageIncrement(5);

    pS.add(speedLabel = new Label("  Speed 0"));
    pS.add(speedbar = new Scrollbar(Scrollbar.HORIZONTAL));
    speedbar.setValues( 0, 0, 0, 700);
    speedbar.setPageIncrement(50);
  

    pS.add(rudderLabel = new Label("  Rudder 0"));
    pS.add(rudderbar = new Scrollbar(Scrollbar.HORIZONTAL));
    rudderbar.setValues( 0, 0, -30, 30);
    rudderbar.setPageIncrement(10);
  
    add("North", pN);
    add("South", pS);

    // Get a handle to the NIU
    while (niu == null)
      {
      niu = (NIU)(getAppletContext().getApplet("NIU"));
      }
    System.out.println("niu name is maybe= " + niu.getName() );

    // trick for getting a handle to the browser plugin
    JSObject win = JSObject.getWindow(this);
    JSObject doc = (JSObject) win.getMember("document");
    JSObject embeds = (JSObject) doc.getMember("embeds");
    browser = (Browser) embeds.getSlot(0);

    if(DEBUG)
    {
      if((win == null) || (doc == null) || (embeds == null))
        System.out.println("Some sort of failure with Javascript");
      if(browser == null)
        System.out.println("null browser");
    }
      
    // Get the VaderTransform and GlobalClock nodes...
    try
      {
      rootNode = browser.getNode("RootNode");
      globalClock = browser.getNode("GlobalClock");
      }
    catch (InvalidNodeException e)
      {
      System.out.println("Invalid Node Exception");
      }

    // Get EventIns of the rootNode
    addChildren = (EventInMFNode) rootNode.getEventIn("addChildren");
    removeChildren = (EventInMFNode) rootNode.getEventIn("removeChildren");

    // Get the EventOut for GlobalClock's tick
    globalTick = (EventOutSFFloat) globalClock.getEventOut("fraction_changed");
 
    // Set up a callback for globalTick
    globalTick.advise(this, new Integer(1));

    // get handle to espduClass
    try
      {
      espduClass = Class.forName("mil.navy.nps.dis.EntityStatePdu");
      }
    catch (ClassNotFoundException e)
      {
      System.out.println("espduClass not found");
      }
    }


  public void callback(EventOut who, double when, Object which)
    {
    Integer          whichNum  = (Integer) which;
    Vector           pduVector   = null;
    ProtocolDataUnit pdu         = null;
    EntityID         entityID    = null;
    EntityStatePdu   espdu       = null;

    // First set framerate field
    if (whichNum.intValue() == 1)
      {
      frameNum++;
      if (frameNum % 10 == 0)
        {
        frameField.setText(String.valueOf(browser.getCurrentFrameRate()));
        }


      // Pick off and set PDUs from NIU
      pduVector = niu.getPDUs();
      for (int i=0; i<pduVector.size(); i++)
        {
        pdu = (ProtocolDataUnit)pduVector.elementAt(i);
        if (pdu.getClass() == espduClass)
          {
          espdu = (EntityStatePdu)pdu;
          entityID = espdu.getEntityID();

          // check if entity already exists
          entity = Entity.get(entityID);
          if (entity == null) // first time seeing this entity
            {
            entity = new Entity(browser, entityID);
            addChildren.setValue(entity.getNode());
            }

          // go ahead and set pdu now
          entity.setESPDU(espdu);
          }
        }
      pduVector.removeAllElements(); // garbage collector will destroy Vector

      // let each entity update itself
      for (int i=Entity.getNum(); i>0; i--)  // backwards 'cause delete shifts list
        {
        entity = Entity.get(i-1);

        if (entity.getPduTime() > 10.0) // 10 sec timeout
          {
          removeChildren.setValue(entity.getNode());
          entity.dispose();
          }
        else
          {
          entity.cycle();
          }
        }

      // Finally, let the AUV update itself
      ownship.cycle();
      }
    }


  public boolean handleEvent(Event event)
    {
	if(event.id == Event.SCROLL_ABSOLUTE
		|| event.id == Event.SCROLL_LINE_DOWN
		|| event.id == Event.SCROLL_LINE_UP
		|| event.id == Event.SCROLL_PAGE_DOWN
		|| event.id == Event.SCROLL_PAGE_UP)
	{ depthLabel.setText("Depth " + depthbar.getValue());
        ownship.update_depth(depthbar.getValue());
	  speedLabel.setText("  Speed " + speedbar.getValue());
        ownship.update_speed(speedbar.getValue());
	  rudderLabel.setText("  Rudder " + rudderbar.getValue());
        ownship.update_rudder(rudderbar.getValue());
        }
    return super.handleEvent(event);
    }



  public boolean action(Event event, Object what)
    {
    if (event.target instanceof Button)
      {
      Button b = (Button) event.target;
      if (b.getLabel() == "Add Ownship")
        {
        System.out.println("Got ownship hit");
        if (ownship == null) // can only have one!
          {
          System.out.println("ownship is null");
          ownship = new Ownship(browser, niu, getParameter("ownshipEntityID"));
          System.out.println("got a new ownship instantiated");
          addChildren.setValue(ownship.getNode());
          System.out.println("ownship added to addChildren");
          }
         else
           System.out.println("ownship not null, no new ship added");
        }
      else if (b.getLabel() == "Remove Ownship")
        {
        if (ownship != null)
          {
          removeChildren.setValue(ownship.getNode());
          ownship.dispose();
          ownship = null;
          }  
        }
      }
    return true;
    }
  }


