// **********************************************************************
// 
// Script Name:	EntityState.java
//
// Description:	Redefinition of the Entity class
//              Modified for Java via Script node, cleanup is possible
//
// Author:	Kent A. Watsen, http://www.mbay.net/~watsen
// Updates:     Don Brutzman
//
// Revised:	15 February 98
//
// VRML:	EspduReadTransform.wrl  and EspduReadTransformExample.wrl
//   		EspduWriteTransform.wrl and EspduWriteTransformExample.wrl
//
// Notes:	need to fix the dt timer update
//
// **********************************************************************
// **********************************************************************
//   IMPORT
// **********************************************************************

package mil.navy.nps.javaViaScriptNode;

// Standard classes
import java.*;
import java.util.*;

// NPS DIS library
import mil.navy.nps.dis.*;
import mil.navy.nps.math.*;


// **********************************************************************
//   CLASS
// **********************************************************************

public class EntityState extends Object
{
  public static boolean DEBUG         = true;

  static Hashtable    entityContainer = null;
  float               position[]      = null; // x, y, z
  float               rotation[]      = null; // axis, angle (radians)
  float               eulers[]        = null; // roll about X, pitch about Y, yaw about Z
  Quaternion          tmpQuat         = null;
  EntityID            entityID        = null;
  EntityStatePdu      espdu           = null;
  float               pduTime         = 0.0f;

public EntityState (EntityID new_entityID)
{
    entityID = new_entityID;  // first set passed args we wish to keep
    
	if (DEBUG) System.out.println ("EntityState constructor:  entityID = " + entityID);

    if (entityContainer == null) // now create & add our entity container
	{
		entityContainer = new Hashtable(10); // big enough??
	}
	entityContainer.put (entityID, this); // this object handles this entity,
                                          // not called unless not found in hash table
    position = new float[3];
    rotation = new float[4];
    eulers   = new float[3];
    tmpQuat  = new Quaternion(); // allocate up front so runtime efficient
}

	public void dispose()
	{
		if (DEBUG) System.out.println
		           ("EntityState destructor:  entityID = " + entityID);

		entityContainer.remove(entityID);  // remove entity from hashtable
	}

	public void setESPDU (EntityStatePdu new_espdu)
	{
    	espdu = new_espdu;

		if (DEBUG) System.out.println ("EntityState setESPDU:  espdu = " + espdu);

    	pduTime = 0.0f;

	    position[0] = (float) espdu.getEntityLocationX();
    	position[1] = (float) espdu.getEntityLocationY();
    	position[2] = (float) espdu.getEntityLocationZ();
    	// !!! note order wacky
    	eulers[2]   = (float) espdu.getEntityOrientationPhi();   // roll
    	eulers[1]   = (float) espdu.getEntityOrientationTheta(); // pitch
    	eulers[0]   = (float) espdu.getEntityOrientationPsi();   // heading
	}

	public EntityStatePdu getESPDU()
	{
 	   return espdu;
	}

	public static int numberEntities ()
	{
    	if (entityContainer == null) return 0;
    	else                         return entityContainer.size();
	}

	public static EntityState get (int index)
	{
    	Enumeration enumeration = null;
   		EntityState entity      = null;

    	if ((entityContainer == null)	||
    		(index < 0) 				||
       		(index >= entityContainer.size()))
		{
			if (DEBUG) System.out.println
				("EntityState get:  index = " + index + ", not found, " +
				  entityContainer.size() + " elements total.");
       		return null;
		}
		enumeration = entityContainer.elements();
		for (int i=0; i<=index; i++)
	 	{
	      entity = (EntityState) enumeration.nextElement();
	  	}
		if (DEBUG) System.out.println
				("EntityState get:  index = " + index + ", entity = " + entity);
    	return entity;
	}

	public static EntityState get (EntityID new_entityID)
	{
 		if (entityContainer == null)
		{
			if (DEBUG) System.out.println
				("EntityState get:  new_entityID = " + new_entityID + ", entityContainer == null");
			return null;
		}
		else
		{
			if (DEBUG) System.out.println
				("EntityState get:  new_entityID = " + new_entityID);
	   		return (EntityState) entityContainer.get(new_entityID);
		}
	}

	public float getPduTime ()
	{
		if (DEBUG) System.out.println
				("EntityState getPduTime:  pduTime = " + pduTime);
		return pduTime;
	}
}

