/*-----------------------------------------------------------------------

  Class:       MulticastPduSender
  Author:      Mike McCarthy
  Last Update: 6 Jan. 1998

  Program Invocation:  This class is instantiated by other classes
                       
  When this class is instantiated, its methods are used to output 
  PDUs on the specified Multicast group at a 1/10Hz rate. 

--------------------------------------------------------------------------*/

package mil.navy.nps.testing;

import mil.navy.nps.dis.*;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

public class MulticastPduSender {

   byte     	outputBuffer[]  = new byte[1500];// buffer to hold datagram data
   InetAddress 	group; 
   MulticastSocket s ;                   // socket to spit it out on
   DatagramPacket          dgramPacket;  // one datagram packet
   ByteArrayOutputStream   outputStream;
   DataOutputStream        dataOutputStream;
   byte                    streamArray[];
   int                     pduSentCount = 0;


   public static final byte DIS203_PROTOCOL_VERSION_1_0 = (byte)1;
   public static final byte DIS203_PROTOCOL_VERSION_IEEE_1278_1993 = (byte)2;
   public static final byte DIS203_PROTOCOL_VERSION_2_0 = (byte)3;
   public static final byte DIS203_PROTOCOL_VERSION_CURRENT = DIS203_PROTOCOL_VERSION_2_0;
   
   public MulticastPduSender()
   {try{
   //group = InetAddress.getByName("224.2.181.145");
   group = InetAddress.getByName("230.0.0.1");
   s = new MulticastSocket(62040);
   s.joinGroup(group);}

   catch (UnknownHostException e) {
	   System.err.println(e);
	}

	
        catch(IOException ioException) {
            System.out.println("sent a total of " + pduSentCount + "packets");
        }

}



   public void SendMPDU (ProtocolDataUnit pdu)
   {
   
   
   pdu.setProtocolVersion(DIS203_PROTOCOL_VERSION_CURRENT);
   pdu.setExerciseID(1);
   outputStream = new ByteArrayOutputStream();
   dataOutputStream = new DataOutputStream(outputStream);
   pdu.serialize(dataOutputStream);
   streamArray = outputStream.toByteArray();
   System.out.println("data serialized to " + streamArray[0] + " and " + streamArray[1]);
   dgramPacket = new DatagramPacket(outputStream.toByteArray(), 
                                         outputStream.size(), 
                                         group, 
                                         4446);


	try{
	
	{
        s.send(dgramPacket, (byte)127);
	}
           
	pduSentCount++;
           if(pduSentCount % 100 == 0)
            {
            System.out.println("sent " + pduSentCount + " Pdus");
            }

	
	 }//end try

	catch (SocketException se) {
	   System.err.println(se);
	}

	catch (UnknownHostException e) {
	   System.err.println(e);
	}

	
        catch(IOException ioException) {
            System.out.println("sent a total of " + pduSentCount + "packets");
        }
          

	
    
   }	//end method


}//endclass

