/**
*/
package util;

import java.io.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;
import dishelper.*;

/**
*
* Gathers PDUs from the given multicast address, produces fictituos 
* processing time values and sends back the originating host of the 
* message, dumps given number of pdus to the given file.
*
* @author H. Timucin Ozdemir
* @date  October 17, 1998 ( Last Updated October 23, 1998 )
*
*/
public class MulticastClient2 
{

    public static final String DEFAULT_MCAST = "230.0.0.1";
    public static final int DEFAULT_MPORT = 4446;
    public static final int DEFAULT_COLLECTUDPPORT = 3030;
    public static final String DEFAULT_OFILE = "dump.pdu";
    public static final int DEFAULT_LIMIT = 50;

    public static final String OPTION_MCAST = "-mcast";
    public static final String OPTION_MPORT = "-mport";
    public static final String OPTION_COLLECTUDPPORT = "-cport";
    public static final String OPTION_OFILE = "-ofile";
    public static final String OPTION_LIMIT = "-npdu";

    public static void printESPDU(EntityStatePdu pdu)
    {
 
       /*System.out.print(" x: "+pdu.getEntityLocationX());
       System.out.print(" y: "+pdu.getEntityLocationY());
       System.out.print(" z: "+pdu.getEntityLocationZ());
       System.out.print(" lvx: "+pdu.getEntityLinearVelocityX());
       System.out.print(" lvy: "+pdu.getEntityLinearVelocityY());
       System.out.print(" lvz: "+pdu.getEntityLinearVelocityZ());
       System.out.print(" Psi: "+pdu.getEntityOrientationPsi());
       System.out.print(" The: "+pdu.getEntityOrientationTheta());
       System.out.print(" Phi: "+pdu.getEntityOrientationPhi());
       */

       System.out.print(" avx: "+pdu.getEntityAngularVelocityX());
       System.out.print(" avy: "+pdu.getEntityAngularVelocityY());
       System.out.print(" avz: "+pdu.getEntityAngularVelocityZ());

       System.out.println(" Mrk: "+pdu.getMarking());
   

    }// end of printESPDU()

    /**
    */
    public static void 
    collect( String mCastAddress, int gPortNumber, String oFile, 
             int limit, int collectUdpPort  ) throws IOException
    {
        int i;
        MulticastSocket socket = new MulticastSocket(gPortNumber);
        InetAddress address = InetAddress.getByName(mCastAddress);
        socket.joinGroup(address);

        DatagramSocket socketOut = new DatagramSocket();

        DatagramPacket packet;
        Random  rnd = new Random();

        PrintWriter pDump = new PrintWriter(new FileOutputStream(oFile));
    
        // get a few quotes
        for (i = 0; i < limit; i++) 
        {
            byte[] buf = new byte[256];
            packet = new DatagramPacket(buf, buf.length);
            socket.receive(packet);

            String received = new String(packet.getData());
            ProtocolDataUnit pdu = ProtocolDataUnit.datagramToPdu(packet);
            if( pdu!=null ) 
            {
               //pdu.printValues(2,pDump);
               if( pdu instanceof EntityStatePdu ) 
               {
                       MulticastClient2.printESPDU((EntityStatePdu)pdu);
                   ((EntityStatePdu)pdu).getEntityID().printValues(2,System.out);
               }
               float tm = rnd.nextFloat()*200f+10f;
               byte[] buf0 = CDRCoderEncoder.toByte(tm);
               DatagramPacket packet2 = new DatagramPacket(buf0, buf0.length, packet.getAddress(), collectUdpPort);
               socketOut.send(packet2);

            }// end of if

            System.out.println("Quote of the Moment: " + received);

        }// end of for

        socket.leaveGroup(address);
        socket.close();

    }// end of collect()

    /**
    */
    public static void 
    main(String[] args) throws IOException 
    {
        String  mCastAddress   = MulticastClient2.DEFAULT_MCAST;
        int     gPortNumber    = MulticastClient2.DEFAULT_MPORT;
        int     collectUdpPort = MulticastClient2.DEFAULT_COLLECTUDPPORT;
        String  oFile          = MulticastClient2.DEFAULT_OFILE;
        int     limit          = MulticastClient2.DEFAULT_LIMIT;

        int     dummy;
        boolean giveHelp=false;

        
        // Handle Command-line options if any
        int i=0;
        while( i<args.length )
        {

          if(args[i].equals(MulticastClient2.OPTION_MCAST))
             mCastAddress = args[++i];
          else
            if(args[i].equals(MulticastClient2.OPTION_MPORT))
            {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>10) gPortNumber = dummy;
               } catch( NumberFormatException nfe)
                { gPortNumber = MulticastClient2.DEFAULT_MPORT; }
            }
          else
            if(args[i].equals(MulticastClient2.OPTION_COLLECTUDPPORT))
            {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>10) collectUdpPort = dummy;
               } catch( NumberFormatException nfe)
                { collectUdpPort = MulticastClient2.DEFAULT_COLLECTUDPPORT; }
            }
          if(args[i].equals(MulticastClient2.OPTION_LIMIT)&&i<args.length-1)
            {
               try
               {  dummy = Integer.parseInt(args[++i]);
                  if(dummy>0) limit=dummy;
               } catch( NumberFormatException nfe)
                { limit = MulticastClient2.DEFAULT_LIMIT; }
            }
          else
            if(args[i].equals(MulticastClient2.OPTION_OFILE)&&i<args.length-1)
               oFile = args[++i];
          else if(args[i].equals("-help")||args[i].equals("-h")) giveHelp=true;

          i++;

        }// end of for

       if(giveHelp)
       {
         System.out.println();
         System.out.println(" java MulticastClient2 [-mcast <address>] [-mport <prNumber>] [-help] ");
         System.out.println("                       [-cport <udpNumber>] [-npdu <num>] [-ofile <fname>]");
         System.out.println();
         System.out.println("   <address> : Multicast Group Address ( 230.0.0.1 is default)");
         System.out.println("  <prNumber> : Multicast Port ( 4446 is default) ");
         System.out.println(" <udpNumber> : Stat Sending UDP Port Number ( 3030 is default)");
         System.out.println("       <num> : Number of pdus to collect.(50)");
         System.out.println("     <fname> : Output filename. (dump.pdu)");
         System.out.println("     purpose : Collects given number of PDUs from group, sends back");
         System.out.println("               a fictituous proccessing time info to the originating");
         System.out.println("               machine and keeps received pdus into the given file.");
         System.out.println();
       } else MulticastClient2.collect(mCastAddress,gPortNumber,oFile,limit,collectUdpPort);

    }// end of main()

}// end of MulticastClient2



