/**
* @date  October 16, 1998 ( Last Upadated on October 16, 1998 )
*
*/
package util;

import java.io.*;
import java.net.*;
import java.util.*;
import mil.navy.nps.dis.*;

import dishelper.*;

/**
*
* @author H. Timucin Ozdemir
* @date  October 16, 1998 ( Last Updated on October 23, 1998 )
*
*/
public class MulticastClientC 
{

    public static final String DEFAULT_MCAST       = "230.0.0.1";
    public static final int DEFAULT_MPORT          = 4446;
    public static final int DEFAULT_COLLECTUDPPORT = 3030;

    public static final String OPTION_MCAST          = "-mcast";
    public static final String OPTION_MPORT          = "-mport";
    public static final String OPTION_COLLECTUDPPORT = "-cport";

    /**
    */
    public static void 
    printESPDU(EntityStatePdu pdu)
    {
 
       System.out.print(" x: "+pdu.getEntityLocationX());
       System.out.print(" y: "+pdu.getEntityLocationY());
       System.out.print(" z: "+pdu.getEntityLocationZ());
       System.out.print(" lvx: "+pdu.getEntityLinearVelocityX());
       System.out.print(" lvy: "+pdu.getEntityLinearVelocityY());
       System.out.print(" lvz: "+pdu.getEntityLinearVelocityZ());
       System.out.print(" Psi: "+pdu.getEntityOrientationPsi());
       System.out.print(" The: "+pdu.getEntityOrientationTheta());
       System.out.print(" Phi: "+pdu.getEntityOrientationPhi());
       System.out.print(" avx: "+pdu.getEntityAngularVelocityX());
       System.out.print(" avy: "+pdu.getEntityAngularVelocityY());
       System.out.print(" avz: "+pdu.getEntityAngularVelocityZ());
       System.out.println(" Mrk: "+pdu.getMarking());

    }// end of printESPDU()

    /**
    */
    public static void 
    collect(String mCastAddress, int gPortNumber, int collectUdpPort ) throws IOException
    {
       PDUOracle oracle = new PDUOracle();
      MulticastSocket socket = new MulticastSocket(gPortNumber);
      InetAddress address = InetAddress.getByName(mCastAddress);
      socket.joinGroup(address);

      DatagramSocket socketOut = new DatagramSocket();

      DatagramPacket packet;
      Random rnd = new Random();

      while(true)
      {

        byte[] buf = new byte[256];
        packet = new DatagramPacket(buf, buf.length);
        socket.receive(packet);

        String received = new String(packet.getData());
        ProtocolDataUnit pdu = ProtocolDataUnit.datagramToPdu(packet);

        if( pdu!=null ) 
        {
            //if( pdu instanceof EntityStatePdu ) 
            //        MulticastClient2.printESPDU((EntityStatePdu)pdu);
            System.out.println("Quote of the Moment: " + oracle.getPDUName(pdu.getPduType().shortValue()));

            float tm   = rnd.nextFloat()*200f+10f;
            short pnum = (short)((rnd.nextInt()%8)+1);
            if(pnum<0) pnum = (short)(pnum*-1);
            if(pnum==0) pnum=(short)1; 
            byte[] buf01 = CDRCoderEncoder.toByte(tm/pnum);
            byte[] buf02 = CDRCoderEncoder.toByte(pnum);
            byte[] buf0 = new byte[6];
            int i;
            for(i=0;i<4;i++) buf0[i]=buf01[i]; 
            for(i=0;i<2;i++) buf0[i+4]=buf02[i]; 
               
            DatagramPacket packet2 = new DatagramPacket(buf0, buf0.length, packet.getAddress(), collectUdpPort);
            socketOut.send(packet2);

        }// end of if

      }// end of while

    }// end of collect()

    public static void 
    main(String[] args) throws IOException 
    {

       String  mCastAddress   = MulticastClientC.DEFAULT_MCAST;
       int     gPortNumber    = MulticastClientC.DEFAULT_MPORT;
       int     collectUdpPort = MulticastClientC.DEFAULT_COLLECTUDPPORT;
       int     dummy;
       boolean giveHelp=false;

       // Handle Command-line options if any
       int i=0;
       while( i<args.length )
       {

         if(args[i].equals(MulticastClientC.OPTION_MCAST)) 
           mCastAddress = args[++i];
         else
           if(args[i].equals(MulticastClientC.OPTION_MPORT))
           {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>10) gPortNumber = dummy;
               } catch( NumberFormatException nfe)
                { gPortNumber = MulticastClientC.DEFAULT_MPORT; }
           }
         else
           if(args[i].equals(MulticastClientC.OPTION_COLLECTUDPPORT))
           {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>10) collectUdpPort = dummy;
               } catch( NumberFormatException nfe)
                { collectUdpPort = MulticastClientC.DEFAULT_COLLECTUDPPORT; }
           }
         else if(args[i].equals("-help")||args[i].equals("-h")) giveHelp=true;

         i++;

      }// end of while

      if(giveHelp)
      {
         System.out.println(); 
         System.out.println(" java MulticastClientC [-mcast <address>] [-mport <portNumber>] [-cport <UDPportNumber>] [-help] "); 
         System.out.println("       <address> : Multicast Group Address ( 230.0.0.1 i s default)");
         System.out.println("    <portNumber> : Multicast Port ( 4446 is default)");
         System.out.println(" <UDPportNumber> : Stat Collecting UDP Port Number ( 3030 is default)");
         System.out.println("         purpose : Consumes the PDUs on the multicast group and sends ");
         System.out.println("                   a fictitious processing time into to the corresponding ");
         System.out.println("                   UDP port of the machine which sended this message");
         System.out.println(); 

      } else MulticastClientC.collect(mCastAddress,gPortNumber,collectUdpPort);

    }// end of main

}// end of MulticastClientC
