/**
* @date  October 16, 1998 ( Last Upadated on October 16, 1998 )
*
*/
package util;
import java.io.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;


/**
*
* @author H. Timucin Ozdemir
* @date  October 16, 1998 ( Last Updated on October 16, 1998 )
*
*/
public class 
MulticastPDUServer 
{

    public static final String DEFAULT_MCAST       = "230.0.0.1";
    public static final int DEFAULT_MPORT          = 4446;

    public static final String OPTION_MCAST = "-mcast";
    public static final String OPTION_MPORT = "-mport";
    public static final String OPTION_HELP  = "-help";
    public static final String OPTION_CYCLE = "-period";

    public static void 
    generate(String mCastAddress, int gPortNumber, int sleepTime) throws IOException 
    {

      int     dummy;
      boolean stay=true;

      System.out.println(" Multicast Address .....:"+mCastAddress);
      System.out.println(" Multicast Port number .:"+gPortNumber);
      System.out.println(" Generation Period .....:"+sleepTime+" msec");

      MulticastSocket socket = new MulticastSocket(gPortNumber);
      InetAddress address = InetAddress.getByName(mCastAddress);
      socket.joinGroup(address);

      DatagramSocket socketOut = new DatagramSocket();

      DatagramPacket packet;
      Random rnd = new Random();

      while(stay)
      {
        ProtocolDataUnit pdu = null;

        int pType = rnd.nextInt()%3;
        if( pType==0  ) pdu = new EntityStatePdu();
        else if( pType==1  ) pdu = new FirePdu();
        else if( pType==2  ) pdu = new DetonationPdu();
        
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        DataOutputStream dataOutputStream = new DataOutputStream(outputStream);
        if(pdu!=null)
        { 
           pdu.serialize(dataOutputStream);
        
           packet = new DatagramPacket(outputStream.toByteArray(), 
                                     outputStream.size(), address, gPortNumber);

           try
           {
             socket.send(packet, (byte)127);
             System.out.println(" I send : "+(pType+1));

           } catch (SocketException se) { System.err.println(se); }
             catch (UnknownHostException e) { System.err.println(e); }
             catch(IOException ioException) { }
             catch(NullPointerException nullExp) { }

        } // end of if

        try { Thread.sleep(sleepTime); }
        catch(java.lang.InterruptedException ie) { stay=false;}

      }// end of while

    }// end of generate()

    public static void 
    main(String[] args) throws IOException 
    {

       String  mCastAddress   = MulticastClientC.DEFAULT_MCAST;
       int     gPortNumber    = MulticastClientC.DEFAULT_MPORT;
       int     dummy;
       int     sleepTime=100;
       boolean giveHelp=false;
       boolean stay=true;

       // Handle Command-line options if any
       int i=0;
       while( i<args.length )
       {

         if(args[i].equals(MulticastPDUServer.OPTION_MCAST)) 
            mCastAddress = args[++i];
         else
           if(args[i].equals(MulticastPDUServer.OPTION_MPORT))

           {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>10) gPortNumber = dummy;
               } catch( NumberFormatException nfe)
                { gPortNumber = MulticastPDUServer.DEFAULT_MPORT; }
           }
        else
           if(args[i].equals(MulticastPDUServer.OPTION_CYCLE))
           {
               try
               { dummy = Integer.parseInt(args[++i]);
                 if(dummy>0) sleepTime = dummy;
               } catch( NumberFormatException nfe)
                { sleepTime = 100; }
           }
         else if(args[i].equals("-help")||args[i].equals("-h")) giveHelp=true;

         i++;

      }// end of while

      if(giveHelp)
      {
         System.out.println();
         System.out.println(" java MulticastPDUServer [-mcast <address>] [-mport <portnumber>] [-help] [-period  <cyle_length_in_msec]");
         System.out.println("             <address> : Multicast Group Address ( 230.0.0.1 is default)");
         System.out.println("          <portnumber> : Multicast Port ( 4446 is default)");
         System.out.println(" <cyle_length_in_msec> : In each x msec, it produces a PDU.");
         System.out.println("               purpose : Random PDU generator. ");
         System.out.println();
      }
      else MulticastPDUServer.generate(mCastAddress,gPortNumber,sleepTime); 

    }// end of main

}// end of MulticastPDUServer

