/**
* @date  October 19, 1998 ( Last Updated on October 19, 1998 )
*
*/
package util;

import java.io.*;
import java.awt.*;

import java.io.*;
import java.net.*;
import java.util.*;

import mil.navy.nps.dis.*;
import comm.*;


/**
*
* Listens a Multicast port and spits out the collected date.
*
* @author H. Timucin Ozdemir
* @date  October 19, 1998 ( Last Updated on October 19, 1998 )
*
*/
public class PduDumper 
{

    public static final String DEFAULT_MCAST = "230.0.0.1";
    public static final int DEFAULT_MPORT = 4446;
    public static final String DEFAULT_OFILE = "out.pdu";
    public static final int DEFAULT_LIMIT = 50;

    public static final String OPTION_MCAST = "-mcast";
    public static final String OPTION_MPORT = "-mport";
    public static final String OPTION_OFILE = "-ofile";
    public static final String OPTION_LIMIT = "-npdu";

    String  grAddress = DEFAULT_MCAST;
    int     grPort    = DEFAULT_MPORT;
    String  oFName    = DEFAULT_OFILE;

    /**
    */
    public 
    PduDumper(String _grAddress, int _grPort, String _oFName) 
    { 
       if(_grAddress!=null) grAddress = _grAddress;
       if(_oFName!=null)    oFName = _oFName;
       if(_grPort>10)       grPort = _grPort;

    }// end of CONSTUCTOR

    /**
    */
    public void 
    conductExperiment(int limit) 
    {
       try
       {
       MulticastSocket socket = new MulticastSocket(grPort);
       InetAddress address = InetAddress.getByName(grAddress);
       socket.joinGroup(address);

       DatagramPacket packet;

       DataOutputStream pDump = new DataOutputStream(new FileOutputStream(oFName));
    
       // get a few quotes
       int i=0;
       while( i<limit )
       //for (int i = 0; i < limit; i++) 
       {

          byte[] buf = new byte[1500];
          packet = new DatagramPacket(buf, buf.length);
          socket.receive(packet);

          String received = new String(packet.getData());
          ProtocolDataUnit pdu = ProtocolDataUnit.datagramToPdu(packet);
          if(pdu!=null) 
          { 
            pdu.serialize(pDump); i++; 
            if( pdu instanceof EntityStatePdu) 
                System.out.println("*ESPDU received! ");
            else if( pdu instanceof DetonationPdu) 
                System.out.println("*Detonation received! ");
            else if( pdu instanceof FirePdu ) 
                System.out.println("*Fire received! ");
          }
          else System.out.println("Quote of the Moment: " + PDUReadWrite.getPDUType(packet.getData()));

        }// end of for

        socket.leaveGroup(address);

        socket.close();

        pDump.close(); 
        } catch(java.io.IOException ioe) {}

    }// end of conductExperiment()

    /**
    */
    public static void 
    printESPDU(EntityStatePdu pdu)
    {
 
       System.out.print(" x: "+pdu.getEntityLocationX());
       System.out.print(" y: "+pdu.getEntityLocationY());
       System.out.print(" z: "+pdu.getEntityLocationZ());
       System.out.print(" lvx: "+pdu.getEntityLinearVelocityX());
       System.out.print(" lvy: "+pdu.getEntityLinearVelocityY());
       System.out.print(" lvz: "+pdu.getEntityLinearVelocityZ());
       System.out.print(" Psi: "+pdu.getEntityOrientationPsi());
       System.out.print(" The: "+pdu.getEntityOrientationTheta());
       System.out.print(" Phi: "+pdu.getEntityOrientationPhi());
       System.out.print(" avx: "+pdu.getEntityAngularVelocityX());
       System.out.print(" avy: "+pdu.getEntityAngularVelocityY());
       System.out.print(" avz: "+pdu.getEntityAngularVelocityZ());
       System.out.println(" Mrk: "+pdu.getMarking());
   
    }// end of printESPDU()

    public static void 
    main(String[] args) throws IOException 
    {
       String mCastAddress   = PduDumper.DEFAULT_MCAST;
       int    gPortNumber    = PduDumper.DEFAULT_MPORT;
       String oFile          = PduDumper.DEFAULT_OFILE; 
       int    limit          = PduDumper.DEFAULT_LIMIT;
       int    dummy;
       boolean giveHelp = false;

       // Handle Command-line options if any
       int i=0;
       while(i<args.length)
       {
          
          if(args[i].equals(PduDumper.OPTION_MCAST)&&i<args.length-1) 
             mCastAddress = args[++i];
          else
            if(args[i].equals(PduDumper.OPTION_MPORT)&&i<args.length-1) 
            { 
               try
               {  dummy = Integer.parseInt(args[++i]); 
                  if(dummy>10) gPortNumber=dummy;
               } catch( NumberFormatException nfe) 
                { gPortNumber = PduDumper.DEFAULT_MPORT; }
            }
          else
            if(args[i].equals(PduDumper.OPTION_LIMIT)&&i<args.length-1) 
            { 
               try
               {  dummy = Integer.parseInt(args[++i]); 
                  if(dummy>0) limit=dummy;
               } catch( NumberFormatException nfe) 
                { limit = PduDumper.DEFAULT_LIMIT; }
            }
          else
            if(args[i].equals(PduDumper.OPTION_OFILE)&&i<args.length-1) 
               oFile = args[++i];
          //else if(args[i].equals("-help")||args[i].equals("-h")) giveHelp=true;

          i++;

       }// end of while

       if(giveHelp)
       {
          System.out.println();
          System.out.println(" java PduDumper -mcast <p1> -mport <p2> -ofile <p3> -npdu <p4>");
          System.out.println("      <p1> : Multicast Group Address ( 230.0.0.1 is default)");
          System.out.println("      <p2> : Multicast Port ( 4446 is default)");
          System.out.println("      <p3> : Output file name ( out.pdu is default)");
          System.out.println("      <p4> : Number of PDUs to collect ( 50 is default)");
          System.out.println("   purpose : Collects PDUs from given address and ");
          System.out.println("           : dumps them into a file for playing later.");
          System.out.println();
       }
       else
        {
           // Print Report about the assumptions
           System.out.println(" Multicast Address .....:"+mCastAddress);
           System.out.println(" Multicast Port number .:"+gPortNumber);
           System.out.println(" Output File ...........:"+oFile);
           System.out.println(" Max Number Of PDUs ....:"+limit);

           PduDumper listener = new PduDumper(mCastAddress,gPortNumber,oFile);
           listener.conductExperiment(limit);

           /*EntityStatePdu espdu = new EntityStatePdu();
           DataOutputStream out = new DataOutputStream(new FileOutputStream(oFile));
           PDUReadWrite.putPDU(out,espdu);
           out.close();
           */
        }

    }// end of main

}// end of PduDumper
