/**
*/
package util;
import java.net.*;
import java.io.*;
import mil.navy.nps.dis.*;

import comm.*;

/**
*
*  (<length-of-pdu-info><pdu-info>)*
*  <length-of-pdu-info> 4 byte big endian integer value gives the length of
*                       the following <pdu-info> package.
* 
* Reads the PDU input file and produces the frequency information about
* each type of pdu and total number of pdus in the file.
*
* It reports the unresolved PDU information to the unk/ directory in a file
* in which only <pdu-info> part is stored.
* 
* @author H. Timucin Ozdemir
* @date   October 19, 1998 (Last Updated October 22, 1998)
*
*/
public class PkgReader
{
   /**
   */
   public static void
   read(String fileName)
   {

      int i=0, n=0;
      int[] frequency = new int[256];
      int[] ufrequency = new int[256];
      int[] pfrequency = new int[256];
      byte[] buf = new byte[5500];

      for(i=0;i<256;i++)  pfrequency[i]=ufrequency[i]=frequency[i]=0;

      int numOfPdus = 0;
      int notRecognized = 0;
      boolean prequired=false;

      try
      {
         DataInputStream fin = new DataInputStream(new FileInputStream(fileName));
         int limit=100,k=0;
         int a;

         //System.out.println(" read ");
         while( (limit=fin.readInt())>0 ) 
         {

            fin.readFully(buf,0,limit);

            PushbackInputStream in = new  PushbackInputStream(new ByteArrayInputStream(buf,0,limit),30);

            ProtocolDataUnit pdu = null;

            pdu = PDUReadWrite.getPDU(in);
            n++;
            prequired=false;
    
            if(pdu!=null) 
            {    
                  //pdu.printValues(2,System.out);
                  System.out.println(" PDU Type: "+pdu.getPduType());
                  numOfPdus++;
                  frequency[pdu.getPduType().shortValue()]++;

                  ByteArrayOutputStream bout = new ByteArrayOutputStream();
                  DataOutputStream      dout = new DataOutputStream(bout);
                  pdu.serialize(dout);

                  prequired=(bout.toByteArray().length!=limit ) ;
                  if(!prequired)
                  {
                     byte[] ac = bout.toByteArray();
                     for(int h=0;h<limit&&!prequired;h++) 
                         prequired=(ac[h]!=buf[h]); 
                  }

                  if(prequired)
                  {
                     // If something is fishy, then record it for further
                     // investigation
                     pfrequency[pdu.getPduType().shortValue()]++;

                     String fN = new String( "unk/pck"+pdu.getPduType()+"_"+
                                             numOfPdus+".unk"); 
                     DataOutputStream      dout1 = new DataOutputStream(new FileOutputStream(fN));

                     dout1.write(buf,0,limit);
                     dout1.close(); 
                  }


            }
            else 
              { notRecognized++; 
                int t = PDUReadWrite.getPDUType(buf);
                if(t>=0&&t<256) 
                { ufrequency[t]++; 

                  DataOutputStream      dout1 = new DataOutputStream(new FileOutputStream("unk/pck"+t+"_"+n+".unk"));

                  dout1.write(buf,0,limit);
                  dout1.close(); 
                }
              }

         }// end of while

      } catch(java.io.FileNotFoundException fnf) {}
        catch(java.io.IOException ioe) {}

      int totRecognized = 0;
      System.out.println(" There are "+numOfPdus+" pdus in the file! ");

      for(i=0;i<256;i++)
         if(frequency[i]>0) 
         { System.out.println("F["+i+"] : "+frequency[i]);
           totRecognized+=frequency[i]; }

      System.out.print(" There are "+totRecognized+" recognized");
      System.out.print(" and "+notRecognized+" unrecognized");
      System.out.println(" total of "+(notRecognized+totRecognized)+" pdus");

      for(i=0;i<256;i++)
         if(ufrequency[i]>0) 
          System.out.println("~F["+i+"] : "+ufrequency[i]);

      for(i=0;i<256;i++)
         if(pfrequency[i]>0) 
          System.out.println("P["+i+"] : "+pfrequency[i]);

   }// end of main

   /**
   */
   public static void
   main(String[] args)
   {
      String fileName = "in.pdu";
      int i=0;
      int[] frequency = new int[256];
      boolean giveHelp=false;

      while(i<args.length)
      {
        if( args[i].equals("-ifile") && i<(args.length-1) )
            fileName =  args[++i];
        else if( args[i].equals("-help") ) giveHelp=true;
        i++;
      }// end of while

      if(giveHelp)
      {
        System.out.println();
        System.out.println(" java PkgReader [-ifile <filename>] [-help] ");
        System.out.println("   ifile : gives the name of pdu input file.(default in.pdu) ");
        System.out.println("    help : ? ");
        System.out.println(" purpose : Reads the PDU info from file and produces statistics. ");
        System.out.println();
      }
      else PkgReader.read(fileName);
      
   }// end of main

}// end of PkgReader()
