/**
* @date  October 17, 1998 ( Last Updated on October 17, 1998 )
*
*/
package cmsper.gui;
import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import cmsper.*;

/**
*
* @author H. Timucin Ozdemir
* @date  October 16, 1998 ( Last Updated on October 17, 1998 )
*
*/
public class 
CMSMonitor extends Applet implements ActionListener, ItemListener
{
    SelectPanel leftPanel;
    FormPanel   centerPanel;
    CMSTester   tester;
    CMSProber   sCollector;
    GraphPanel  graph;
    PDUConsumer pduGrapher;
    boolean     pduGrapherOn = false;

    String mCastAddress   = monitor.DEFAULT_MCAST;
    int    gPortNumber    = monitor.DEFAULT_MPORT;
    int    collectUdpPort = monitor.DEFAULT_COLLECTUDPPORT; 

    Panel  cards; 
    String currFormName;

    public static final String PDU_FORM = "PDUFORM";
    public static final String FPLAY_FORM = "FILEPDUFORM";
    public static final String GRAPH_FORM = "Graphics";
    MulticastSocket socket;
    InetAddress     group;

    /**
    */
    public CMSMonitor(Frame _fr, String _mCast, int _gPNum, int _cUdp) 
           throws java.io.IOException, java.net.UnknownHostException 
    {

      if(_fr!=null) registerFrame(_fr);

      if(_mCast!=null) mCastAddress   = _mCast;
      if(_gPNum>10)    gPortNumber    = _gPNum;
      if(_cUdp>10)     collectUdpPort = _cUdp;

      socket = new MulticastSocket(gPortNumber);
      group  = InetAddress.getByName(mCastAddress);
      socket.joinGroup(group);

      /*
      centerPanel = new FormPanel(_fr,mCastAddress,gPortNumber);
      tester      = new CMSTester(centerPanel);
      graph       = new GraphPanel();
      sCollector  = new CMSProber(collectUdpPort,graph);
      leftPanel   = new SelectPanel(sCollector,mCastAddress,gPortNumber);
      pduGrapher  = new PDUConsumer(mCastAddress,gPortNumber,graph);
      */

      centerPanel = new FormPanel(_fr,mCastAddress,gPortNumber);
      tester      = new CMSTester(centerPanel,socket,group);
      graph       = new GraphPanel();
      sCollector  = new CMSProber(collectUdpPort,graph);
      leftPanel   = new SelectPanel(sCollector,mCastAddress,gPortNumber,socket,group);
      pduGrapher  = new PDUConsumer(socket,graph);
   
      centerPanel.setTester(tester);
      tester.setPeer(sCollector);
  
      cards = new Panel();
      cards.setLayout(new CardLayout());

      currFormName = PDU_FORM;
      cards.add(PDU_FORM, centerPanel);
      cards.add(FPLAY_FORM, leftPanel);
      cards.add(GRAPH_FORM, new MFGPanel());

      Panel dummy1 = new Panel();
      dummy1.setLayout(new FlowLayout());
      dummy1.add(cards);

      Panel dummy2 = new Panel();
      dummy2.setLayout(new BorderLayout());

      setLayout( new GridLayout(1,2));

      add(cards);
      add(graph);

      validate();

      //pduGrapher.start();

    }// end of CONSTRUCTOR()

    /**
    */
    void registerFrame(Frame _fr)
    {
       MenuBar mBar  = _fr.getMenuBar();
       Menu    test  = mBar.getMenu(0);
       MenuItem fPlay     = test.getItem(3);

       fPlay.addActionListener(this);

       MenuItem espdu      = test.getItem(0);
       MenuItem detonpdu   = test.getItem(1);
       MenuItem firepdu    = test.getItem(2);
       MenuItem pduReceive = test.getItem(4);
       MenuItem graphics = test.getItem(5);

       espdu.addActionListener(this);
       detonpdu.addActionListener(this);
       firepdu.addActionListener(this);
       ((CheckboxMenuItem)pduReceive).addItemListener(this);
       graphics.addActionListener(this);

    }// end of registerFrame()

    /**
    */
    public void
    itemStateChanged(ItemEvent evt)
    {
       System.out.println(" I received "+evt.paramString());
       String label = (String) evt.getItem();

       if( label.equals("Receive PDUs") )
       {
          if( evt.getStateChange()==ItemEvent.SELECTED )
          { pduGrapher.start(); pduGrapherOn=true; }
          else { pduGrapher.stop(); pduGrapherOn=false; }

       }
 
    }// end of itemStateChanged()

    /**
    */
    public void
    actionPerformed(ActionEvent evt)
    {
       System.out.println(" I received "+evt.getActionCommand());

       if( evt.getActionCommand().equals("Play File") )
                handleCard(FPLAY_FORM);
       else
       if( evt.getActionCommand().equals(FormPanel.DETONATION_FORM) ||
           evt.getActionCommand().equals(FormPanel.ESPDU_FORM) ||
           evt.getActionCommand().equals(FormPanel.FIRE_FORM) )
                handleCard(PDU_FORM);
      else if( evt.getActionCommand().equals("Receive PDUs") )
           {
               if(!pduGrapherOn) 
               { pduGrapher.start(); pduGrapherOn=true; }
               else { pduGrapher.stop(); pduGrapherOn=false; }
           }
      else
       if( evt.getActionCommand().equals(GRAPH_FORM) ) handleCard(GRAPH_FORM);

    }// end of itemStateChanged()

    void handleCard(String _newFormName)
    {
       if(!currFormName.equals(_newFormName))
       {
          currFormName = _newFormName;
          CardLayout cl = (CardLayout)cards.getLayout();
          cl.show(cards,currFormName);

       }// end of if

    }// end of handleCard)

}// end of Class CMSMonitor
